=== modified file 'NEWS'
--- NEWS	2017-07-23 19:01:03 +0000
+++ NEWS	2017-07-23 22:06:41 +0000
@@ -1,4 +1,2 @@
 The NEWS file has been moved and split into multiple files (one per release
 series).  The NEWS files are now found in doc/en/release-notes/.
-
- * Boo

=== modified file 'breezy/__init__.py'
--- breezy/__init__.py	2017-06-09 01:09:37 +0000
+++ breezy/__init__.py	2017-06-13 23:55:52 +0000
@@ -141,14 +141,27 @@
     The use of intern() may defer breakage is but is not enough, the string
     object should be secure against module reloading and during teardown.
     """
+    is_py3 = sys.version_info > (3,)
     try:
         import ctypes
         old_ptr = ctypes.c_void_p.in_dll(ctypes.pythonapi,
             "Py_FileSystemDefaultEncoding")
+        if is_py3:
+            has_enc = ctypes.c_int.in_dll(ctypes.pythonapi,
+                "Py_HasFileSystemDefaultEncoding")
+            as_utf8 = ctypes.PYFUNCTYPE(
+                ctypes.POINTER(ctypes.c_char), ctypes.py_object)(
+                    ("PyUnicode_AsUTF8", ctypes.pythonapi))
     except (ImportError, ValueError):
         return # No ctypes or not CPython implementation, do nothing
-    new_ptr = ctypes.cast(ctypes.c_char_p(intern(new_enc)), ctypes.c_void_p)
-    old_ptr.value = new_ptr.value
+    if is_py3:
+        new_enc = sys.intern(new_enc)
+        enc_ptr = as_utf8(new_enc)
+        has_enc.value = 1
+    else:
+        new_enc = intern(new_enc)
+        enc_ptr = ctypes.c_char_p(new_enc)
+    old_ptr.value = ctypes.cast(enc_ptr, ctypes.c_void_p).value
     if sys.getfilesystemencoding() != new_enc:
         raise RuntimeError("Failed to change the filesystem default encoding")
     return new_enc

=== modified file 'breezy/_bencode_pyx.pyx'
--- breezy/_bencode_pyx.pyx	2017-06-10 01:43:31 +0000
+++ breezy/_bencode_pyx.pyx	2017-06-27 01:29:30 +0000
@@ -18,38 +18,51 @@
 
 from __future__ import absolute_import
 
+from cpython.bytes cimport (
+    PyBytes_CheckExact,
+    PyBytes_FromStringAndSize,
+    PyBytes_AS_STRING,
+    PyBytes_GET_SIZE,
+    )
+from cpython.long cimport (
+    PyLong_CheckExact,
+    )
+from cpython.int cimport (
+    PyInt_CheckExact,
+    PyInt_FromString,
+    )
+from cpython.tuple cimport (
+    PyTuple_CheckExact,
+    )
+from cpython.list cimport (
+    PyList_CheckExact,
+    PyList_Append,
+    )
+from cpython.dict cimport (
+    PyDict_CheckExact,
+    )
+from cpython.bool cimport (
+    PyBool_Check,
+    )
+from cpython.mem cimport (
+    PyMem_Free,
+    PyMem_Malloc,
+    PyMem_Realloc,
+    )
 
-cdef extern from "stddef.h":
-    ctypedef unsigned int size_t
+from libc.stdlib cimport (
+    strtol,
+    )
+from libc.string cimport (
+    memcpy,
+    )
 
 cdef extern from "Python.h":
-    ctypedef int  Py_ssize_t
-    int PyInt_CheckExact(object o)
-    int PyLong_CheckExact(object o)
-    int PyString_CheckExact(object o)
-    int PyTuple_CheckExact(object o)
-    int PyList_CheckExact(object o)
-    int PyDict_CheckExact(object o)
-    int PyBool_Check(object o)
-    object PyString_FromStringAndSize(char *v, Py_ssize_t len)
-    char *PyString_AS_STRING(object o) except NULL
-    Py_ssize_t PyString_GET_SIZE(object o) except -1
-    object PyInt_FromString(char *str, char **pend, int base)
+    # There is no cython module for ceval.h for some reason
     int Py_GetRecursionLimit()
     int Py_EnterRecursiveCall(char *)
     void Py_LeaveRecursiveCall()
 
-    int PyList_Append(object, object) except -1
-
-cdef extern from "stdlib.h":
-    void free(void *memblock)
-    void *malloc(size_t size)
-    void *realloc(void *memblock, size_t size)
-    long strtol(char *, char **, int)
-
-cdef extern from "string.h":
-    void *memcpy(void *dest, void *src, size_t count)
-
 cdef extern from "python-compat.h":
     int snprintf(char* buffer, size_t nsize, char* fmt, ...)
 
@@ -78,12 +91,12 @@
         """Initialize decoder engine.
         @param  s:  Python string.
         """
-        if not PyString_CheckExact(s):
-            raise TypeError("String required")
+        if not PyBytes_CheckExact(s):
+            raise TypeError("bytes required")
 
         self.text = s
-        self.tail = PyString_AS_STRING(s)
-        self.size = PyString_GET_SIZE(s)
+        self.tail = PyBytes_AS_STRING(s)
+        self.size = PyBytes_GET_SIZE(s)
         self._yield_tuples = int(yield_tuples)
 
     def decode(self):
@@ -166,13 +179,13 @@
             raise ValueError('leading zeros are not allowed')
         D_UPDATE_TAIL(self, next_tail - self.tail + 1)
         if n == 0:
-            return ''
+            return b''
         if n > self.size:
             raise ValueError('stream underflow')
         if n < 0:
             raise ValueError('string size below zero: %d' % n)
 
-        result = PyString_FromStringAndSize(self.tail, n)
+        result = PyBytes_FromStringAndSize(self.tail, n)
         D_UPDATE_TAIL(self, n)
         return result
 
@@ -210,7 +223,7 @@
                 if self.tail[0] < c'0' or self.tail[0] > c'9':
                     raise ValueError('key was not a simple string.')
                 key = self._decode_string()
-                if lastkey >= key:
+                if lastkey is not None and lastkey >= key:
                     raise ValueError('dict keys disordered')
                 else:
                     lastkey = key
@@ -260,7 +273,7 @@
         self.size = 0
         self.tail = NULL
 
-        p = <char*>malloc(maxsize)
+        p = <char*>PyMem_Malloc(maxsize)
         if p == NULL:
             raise MemoryError('Not enough memory to allocate buffer '
                               'for encoder')
@@ -269,15 +282,14 @@
         self.tail = p
 
     def __dealloc__(self):
-        free(self.buffer)
+        PyMem_Free(self.buffer)
         self.buffer = NULL
         self.maxsize = 0
 
-    def __str__(self):
+    def to_bytes(self):
         if self.buffer != NULL and self.size != 0:
-            return PyString_FromStringAndSize(self.buffer, self.size)
-        else:
-            return ''
+            return PyBytes_FromStringAndSize(self.buffer, self.size)
+        return b''
 
     cdef int _ensure_buffer(self, int required) except 0:
         """Ensure that tail of CharTail buffer has enough size.
@@ -293,7 +305,7 @@
         new_size = self.maxsize
         while new_size < self.size + required:
             new_size = new_size * 2
-        new_buffer = <char*>realloc(self.buffer, <size_t>new_size)
+        new_buffer = <char*>PyMem_Realloc(self.buffer, <size_t>new_size)
         if new_buffer == NULL:
             raise MemoryError('Cannot realloc buffer for encoder')
 
@@ -308,32 +320,32 @@
         """
         cdef int n
         self._ensure_buffer(INT_BUF_SIZE)
-        n = snprintf(self.tail, INT_BUF_SIZE, "i%de", x)
+        n = snprintf(self.tail, INT_BUF_SIZE, b"i%de", x)
         if n < 0:
             raise MemoryError('int %d too big to encode' % x)
         E_UPDATE_TAIL(self, n)
         return 1
 
     cdef int _encode_long(self, x) except 0:
-        return self._append_string(''.join(('i', str(x), 'e')))
+        return self._append_string(b'i%de' % x)
 
     cdef int _append_string(self, s) except 0:
         cdef Py_ssize_t n
-        n = PyString_GET_SIZE(s)
+        n = PyBytes_GET_SIZE(s)
         self._ensure_buffer(n)
-        memcpy(self.tail, PyString_AS_STRING(s), n)
+        memcpy(self.tail, PyBytes_AS_STRING(s), n)
         E_UPDATE_TAIL(self, n)
         return 1
 
     cdef int _encode_string(self, x) except 0:
         cdef int n
         cdef Py_ssize_t x_len
-        x_len = PyString_GET_SIZE(x)
+        x_len = PyBytes_GET_SIZE(x)
         self._ensure_buffer(x_len + INT_BUF_SIZE)
-        n = snprintf(self.tail, INT_BUF_SIZE, '%d:', x_len)
+        n = snprintf(self.tail, INT_BUF_SIZE, b'%d:', x_len)
         if n < 0:
             raise MemoryError('string %s too big to encode' % x)
-        memcpy(<void *>(self.tail+n), PyString_AS_STRING(x), x_len)
+        memcpy(<void *>(self.tail+n), PyBytes_AS_STRING(x), x_len)
         E_UPDATE_TAIL(self, n + x_len)
         return 1
 
@@ -355,10 +367,8 @@
         self.tail[0] = c'd'
         E_UPDATE_TAIL(self, 1)
 
-        keys = x.keys()
-        keys.sort()
-        for k in keys:
-            if not PyString_CheckExact(k):
+        for k in sorted(x):
+            if not PyBytes_CheckExact(k):
                 raise TypeError('key in dict should be string')
             self._encode_string(k)
             self.process(x[k])
@@ -372,14 +382,14 @@
         if Py_EnterRecursiveCall("encode"):
             raise RuntimeError("too deeply nested")
         try:
-            if PyString_CheckExact(x):
+            if PyBytes_CheckExact(x):
                 self._encode_string(x)
-            elif PyInt_CheckExact(x):
+            elif PyInt_CheckExact(x) and x.bit_length() < 32:
                 self._encode_int(x)
             elif PyLong_CheckExact(x):
                 self._encode_long(x)
             elif (PyList_CheckExact(x) or PyTuple_CheckExact(x)
-                  or StaticTuple_CheckExact(x)):
+                  or isinstance(x, StaticTuple)):
                 self._encode_list(x)
             elif PyDict_CheckExact(x):
                 self._encode_dict(x)
@@ -397,4 +407,4 @@
     """Encode Python object x to string"""
     encoder = Encoder()
     encoder.process(x)
-    return str(encoder)
+    return encoder.to_bytes()

=== modified file 'breezy/_export_c_api.h'
--- breezy/_export_c_api.h	2009-10-12 21:44:27 +0000
+++ breezy/_export_c_api.h	2017-06-27 00:10:52 +0000
@@ -15,7 +15,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-
 /* This file contains helper functions for exporting a C API for a CPython
  * extension module.
  */
@@ -43,30 +42,27 @@
 _export_function(PyObject *module, char *funcname, void *func, char *signature)
 {
     PyObject *d = NULL;
-    PyObject *c_obj = NULL;
+    PyObject *capsule = NULL;
 
-    /* (char *) is because python2.4 declares this api as 'char *' rather than
-     * const char* which it really is.
-     */
-    d = PyObject_GetAttrString(module, (char *)_C_API_NAME);
+    d = PyObject_GetAttrString(module, _C_API_NAME);
     if (!d) {
         PyErr_Clear();
         d = PyDict_New();
         if (!d)
             goto bad;
         Py_INCREF(d);
-        if (PyModule_AddObject(module, (char *)_C_API_NAME, d) < 0)
+        if (PyModule_AddObject(module, _C_API_NAME, d) < 0)
             goto bad;
     }
-    c_obj = PyCObject_FromVoidPtrAndDesc(func, signature, 0);
-    if (!c_obj)
+    capsule = PyCapsule_New(func, signature, 0);
+    if (!capsule)
         goto bad;
-    if (PyDict_SetItemString(d, funcname, c_obj) < 0)
+    if (PyDict_SetItemString(d, funcname, capsule) < 0)
         goto bad;
     Py_DECREF(d);
     return 0;
 bad:
-    Py_XDECREF(c_obj);
+    Py_XDECREF(capsule);
     Py_XDECREF(d);
     return -1;
 }

=== modified file 'breezy/_import_c_api.h'
--- breezy/_import_c_api.h	2009-10-12 21:44:27 +0000
+++ breezy/_import_c_api.h	2017-06-27 00:10:52 +0000
@@ -44,36 +44,28 @@
                             void **func, const char *signature)
 {
     PyObject *d = NULL;
-    PyObject *c_obj = NULL;
-    const char *desc = NULL;
+    PyObject *capsule = NULL;
+    void *pointer;
 
-    /* (char *) because Python2.4 defines this as (char *) rather than
-     * (const char *)
-     */
-    d = PyObject_GetAttrString(module, (char *)_C_API_NAME);
+    d = PyObject_GetAttrString(module, _C_API_NAME);
     if (!d) {
         // PyObject_GetAttrString sets an appropriate exception
         goto bad;
     }
-    c_obj = PyDict_GetItemString(d, funcname);
-    if (!c_obj) {
+    capsule = PyDict_GetItemString(d, funcname);
+    if (!capsule) {
         // PyDict_GetItemString does not set an exception
         PyErr_Format(PyExc_AttributeError,
             "Module %s did not export a function named %s\n",
             PyModule_GetName(module), funcname);
         goto bad;
     }
-    desc = (char *)PyCObject_GetDesc(c_obj);
-    if (!desc || strcmp(desc, signature) != 0) {
-        if (desc == NULL) {
-            desc = "<null>";
-        }
-        PyErr_Format(PyExc_TypeError,
-            "C function %s.%s has wrong signature (expected %s, got %s)",
-                PyModule_GetName(module), funcname, signature, desc);
+    pointer = PyCapsule_GetPointer(capsule, signature);
+    if (!pointer) {
+	// PyCapsule_GetPointer sets an error with a little context
         goto bad;
     }
-    *func = PyCObject_AsVoidPtr(c_obj);
+    *func = pointer;
     Py_DECREF(d);
     return 0;
 bad:

=== modified file 'breezy/_patiencediff_c.c'
--- breezy/_patiencediff_c.c	2010-12-02 09:46:15 +0000
+++ breezy/_patiencediff_c.c	2017-06-13 02:58:21 +0000
@@ -641,11 +641,7 @@
 
     res = PyList_New(nmatches);
     for (i = 0; i < nmatches; i++) {
-#if PY_VERSION_HEX < 0x02050000
-        item = Py_BuildValue("ii", matches[nmatches - i - 1].a, matches[nmatches - i - 1].b);
-#else
         item = Py_BuildValue("nn", matches[nmatches - i - 1].a, matches[nmatches - i - 1].b);
-#endif
         if (item == NULL)
             goto error;
         if (PyList_SetItem(res, i, item) != 0)
@@ -680,13 +676,8 @@
     struct hashtable hashtable;
     struct matching_blocks matches;
 
-#if PY_VERSION_HEX < 0x02050000
-    if (!PyArg_ParseTuple(args, "OOiiiiOi", &aseq, &bseq, &alo, &blo,
-                          &ahi, &bhi, &answer, &maxrecursion))
-#else
     if (!PyArg_ParseTuple(args, "OOnnnnOi", &aseq, &bseq, &alo, &blo,
                           &ahi, &bhi, &answer, &maxrecursion))
-#endif
         return NULL;
 
     hashtable.table = NULL;
@@ -722,13 +713,8 @@
 
     for (i = 0; i < matches.count; i++) {
         for (j = 0; j < matches.matches[i].len; j++) {
-#if PY_VERSION_HEX < 0x02050000
-            item = Py_BuildValue("ii", matches.matches[i].a + j,
-                                 matches.matches[i].b + j);
-#else
             item = Py_BuildValue("nn", matches.matches[i].a + j,
                                  matches.matches[i].b + j);
-#endif
             if (item == NULL)
                 goto error;
             if (PyList_Append(answer, item) != 0)
@@ -854,23 +840,14 @@
     }
 
     for (i = 0; i < matches.count; i++) {
-#if PY_VERSION_HEX < 0x02050000
-        item = Py_BuildValue("iii", matches.matches[i].a,
-                             matches.matches[i].b, matches.matches[i].len);
-#else
         item = Py_BuildValue("nnn", matches.matches[i].a,
                              matches.matches[i].b, matches.matches[i].len);
-#endif
         if (item == NULL)
             goto error;
         if (PyList_SetItem(answer, i, item) != 0)
             goto error;
     }
-#if PY_VERSION_HEX < 0x02050000
-    item = Py_BuildValue("iii", self->asize, self->bsize, 0);
-#else
     item = Py_BuildValue("nnn", self->asize, self->bsize, 0);
-#endif
     if (item == NULL)
         goto error;
     if (PyList_SetItem(answer, i, item) != 0)
@@ -960,11 +937,7 @@
             tag = OP_INSERT;
 
         if (tag != -1) {
-#if PY_VERSION_HEX < 0x02050000
-            item = Py_BuildValue("siiii", opcode_names[tag], i, ai, j, bj);
-#else
             item = Py_BuildValue("snnnn", opcode_names[tag], i, ai, j, bj);
-#endif
             if (item == NULL)
                 goto error;
             if (PyList_Append(answer, item) != 0)
@@ -975,11 +948,7 @@
         j = bj + matches.matches[k].len;
 
         if (matches.matches[k].len > 0) {
-#if PY_VERSION_HEX < 0x02050000
-            item = Py_BuildValue("siiii", opcode_names[OP_EQUAL], ai, i, bj, j);
-#else
             item = Py_BuildValue("snnnn", opcode_names[OP_EQUAL], ai, i, bj, j);
-#endif
             if (item == NULL)
                 goto error;
             if (PyList_Append(answer, item) != 0)
@@ -1137,13 +1106,8 @@
         /* end the current group and start a new one whenever
            there is a large range with no changes. */
         if (tag == OP_EQUAL && i2 - i1 > nn) {
-#if PY_VERSION_HEX < 0x02050000
-            item = Py_BuildValue("siiii", opcode_names[tag],
-                                  i1, MIN(i2, i1 + n), j1, MIN(j2, j1 + n));
-#else
             item = Py_BuildValue("snnnn", opcode_names[tag],
                                   i1, MIN(i2, i1 + n), j1, MIN(j2, j1 + n));
-#endif
             if (item == NULL)
                 goto error;
             if (PyList_Append(group, item) != 0)
@@ -1156,11 +1120,7 @@
             i1 = MAX(i1, i2 - n);
             j1 = MAX(j1, j2 - n);
         }
-#if PY_VERSION_HEX < 0x02050000
-        item = Py_BuildValue("siiii", opcode_names[tag], i1, i2, j1 ,j2);
-#else
         item = Py_BuildValue("snnnn", opcode_names[tag], i1, i2, j1 ,j2);
-#endif
         if (item == NULL)
             goto error;
         if (PyList_Append(group, item) != 0)

=== modified file 'breezy/_simple_set_pyx.pxd'
--- breezy/_simple_set_pyx.pxd	2010-02-17 17:11:16 +0000
+++ breezy/_simple_set_pyx.pxd	2017-06-21 22:25:57 +0000
@@ -23,9 +23,7 @@
      eg. SimpleSet.add(key) => saved_key and SimpleSet[key] => saved_key
 """
 
-cdef extern from "Python.h":
-    ctypedef struct PyObject:
-        pass
+from cpython.object cimport PyObject
 
 
 cdef public api class SimpleSet [object SimpleSetObject, type SimpleSet_Type]:
@@ -74,8 +72,8 @@
                             # so we manage it manually
 
     cdef PyObject *_get(self, object key) except? NULL
-    cdef object _add(self, key)
-    cdef int _discard(self, key) except -1
+    cpdef object add(self, key)
+    cpdef bint discard(self, key) except -1
     cdef int _insert_clean(self, PyObject *key) except -1
     cdef Py_ssize_t _resize(self, Py_ssize_t min_unused) except -1
 

=== modified file 'breezy/_simple_set_pyx.pyx'
--- breezy/_simple_set_pyx.pyx	2017-06-10 00:09:13 +0000
+++ breezy/_simple_set_pyx.pyx	2017-06-21 22:25:57 +0000
@@ -18,31 +18,25 @@
 
 from __future__ import absolute_import
 
-cdef extern from "python-compat.h":
-    pass
-
-cdef extern from "Python.h":
-    ctypedef unsigned long size_t
-    ctypedef long (*hashfunc)(PyObject*) except -1
-    ctypedef object (*richcmpfunc)(PyObject *, PyObject *, int)
-    ctypedef int (*visitproc)(PyObject *, void *)
-    ctypedef int (*traverseproc)(PyObject *, visitproc, void *)
-    int Py_EQ
-    void Py_INCREF(PyObject *)
-    void Py_DECREF(PyObject *)
-    ctypedef struct PyTypeObject:
-        hashfunc tp_hash
-        richcmpfunc tp_richcompare
-        traverseproc tp_traverse
-
-    PyTypeObject *Py_TYPE(PyObject *)
-    # Note: *Don't* use hash(), Pyrex 0.9.8.5 thinks it returns an 'int', and
-    #       thus silently truncates to 32-bits on 64-bit machines.
-    long PyObject_Hash(PyObject *) except -1
-        
-    void *PyMem_Malloc(size_t nbytes)
-    void PyMem_Free(void *)
-    void memset(void *, int, size_t)
+from cpython.object cimport (
+    hashfunc,
+    Py_EQ,
+    PyObject_Hash,
+    PyTypeObject,
+    Py_TYPE,
+    richcmpfunc,
+    traverseproc,
+    visitproc,
+    )
+from cpython.mem cimport (
+    PyMem_Malloc,
+    PyMem_Free,
+    )
+from cpython.ref cimport (
+    Py_INCREF,
+    Py_DECREF,
+    )
+from libc.string cimport memset
 
 
 # Dummy is an object used to mark nodes that have been deleted. Since
@@ -61,11 +55,9 @@
 _NotImplemented = NotImplemented
 
 
-cdef int _is_equal(PyObject *this, long this_hash, PyObject *other) except -1:
+cdef int _is_equal(object this, long this_hash, object other) except -1:
     cdef long other_hash
 
-    if this == other:
-        return 1
     other_hash = PyObject_Hash(other)
     if other_hash != this_hash:
         return 0
@@ -204,13 +196,14 @@
         """
         cdef size_t i, n_lookup
         cdef long the_hash
-        cdef PyObject **table, **slot
+        cdef PyObject **table
+        cdef PyObject **slot
         cdef Py_ssize_t mask
 
         mask = self._mask
         table = self._table
 
-        the_hash = PyObject_Hash(key)
+        the_hash = PyObject_Hash(<object>key)
         i = the_hash
         for n_lookup from 0 <= n_lookup <= <size_t>mask: # Don't loop forever
             slot = &table[i & mask]
@@ -236,7 +229,9 @@
         :return: The new size of the internal table
         """
         cdef Py_ssize_t new_size, n_bytes, remaining
-        cdef PyObject **new_table, **old_table, **slot
+        cdef PyObject **new_table
+        cdef PyObject **old_table
+        cdef PyObject **slot
 
         new_size = DEFAULT_SIZE
         while new_size <= min_used and new_size > 0:
@@ -269,7 +264,7 @@
         slot = old_table
         while remaining > 0:
             if slot[0] == NULL: # unused slot
-                pass 
+                pass
             elif slot[0] == _dummy: # dummy slot
                 remaining = remaining - 1
             else: # active slot
@@ -279,22 +274,18 @@
         PyMem_Free(old_table)
         return new_size
 
-    def add(self, key):
+    cpdef object add(self, key):
         """Similar to set.add(), start tracking this key.
-        
+
         There is one small difference, which is that we return the object that
         is stored at the given location. (which is closer to the
         dict.setdefault() functionality.)
         """
-        return self._add(key)
-
-    cdef object _add(self, key):
-        cdef PyObject **slot, *py_key
-        cdef int added
-
-        py_key = <PyObject *>key
-        if (Py_TYPE(py_key).tp_richcompare == NULL
-            or Py_TYPE(py_key).tp_hash == NULL):
+        cdef PyObject **slot
+        cdef bint added
+
+        if (Py_TYPE(key).tp_richcompare == NULL
+            or Py_TYPE(key).tp_hash == NULL):
             raise TypeError('Types added to SimpleSet must implement'
                             ' both tp_richcompare and tp_hash')
         added = 0
@@ -302,15 +293,15 @@
         assert self._used < self._mask
         slot = _lookup(self, key)
         if (slot[0] == NULL):
-            Py_INCREF(py_key)
+            Py_INCREF(key)
             self._fill = self._fill + 1
             self._used = self._used + 1
-            slot[0] = py_key
+            slot[0] = <PyObject *>key
             added = 1
         elif (slot[0] == _dummy):
-            Py_INCREF(py_key)
+            Py_INCREF(key)
             self._used = self._used + 1
-            slot[0] = py_key
+            slot[0] = <PyObject *>key
             added = 1
         # No else: clause. If _lookup returns a pointer to
         # a live object, then we already have a value at this location.
@@ -324,23 +315,18 @@
         # so we can still return it
         return retval
 
-    def discard(self, key):
+    cpdef bint discard(self, key) except -1:
         """Remove key from the set, whether it exists or not.
 
         :return: False if the item did not exist, True if it did
         """
-        if self._discard(key):
-            return True
-        return False
-
-    cdef int _discard(self, key) except -1:
-        cdef PyObject **slot, *py_key
+        cdef PyObject **slot
 
         slot = _lookup(self, key)
         if slot[0] == NULL or slot[0] == _dummy:
             return 0
         self._used = self._used - 1
-        Py_DECREF(slot[0])
+        Py_DECREF(<object>slot[0])
         slot[0] = _dummy
         # PySet uses the heuristic: If more than 1/5 are dummies, then resize
         #                           them away
@@ -399,7 +385,6 @@
         if self.set is not None and self._used == self.set._used:
             return self.len
         return 0
-    
 
 
 cdef api SimpleSet SimpleSet_New():
@@ -461,12 +446,12 @@
     cdef size_t i, n_lookup
     cdef Py_ssize_t mask
     cdef long key_hash
-    cdef PyObject **table, **slot, *cur, **free_slot, *py_key
+    cdef PyObject **table
+    cdef PyObject **slot
+    cdef PyObject *cur
+    cdef PyObject **free_slot
 
-    py_key = <PyObject *>key
-    # Note: avoid using hash(obj) because of a bug w/ pyrex 0.9.8.5 and 64-bit
-    #       (it treats hash() as returning an 'int' rather than a 'long')
-    key_hash = PyObject_Hash(py_key)
+    key_hash = PyObject_Hash(key)
     i = <size_t>key_hash
     mask = self._mask
     table = self._table
@@ -481,13 +466,13 @@
                 return free_slot
             else:
                 return slot
-        if cur == py_key:
+        if cur == <PyObject *>key:
             # Found an exact pointer to the key
             return slot
         if cur == _dummy:
             if free_slot == NULL:
                 free_slot = slot
-        elif _is_equal(py_key, key_hash, cur):
+        elif _is_equal(key, key_hash, <object>cur):
             # Both py_key and cur belong in this slot, return it
             return slot
         i = i + 1 + n_lookup
@@ -519,8 +504,8 @@
         This may be the same object, or it may be an equivalent object.
         (consider dict.setdefault(key, key))
     """
-    return _check_self(self)._add(key)
-    
+    return _check_self(self).add(key)
+
 
 cdef api int SimpleSet_Contains(object self, object key) except -1:
     """Is key present in self?"""
@@ -535,7 +520,7 @@
     :return: 1 if there was an object present, 0 if there was not, and -1 on
         error.
     """
-    return _check_self(self)._discard(key)
+    return _check_self(self).discard(key)
 
 
 cdef api PyObject *SimpleSet_Get(SimpleSet self, object key) except? NULL:

=== modified file 'breezy/_static_tuple_c.c'
--- breezy/_static_tuple_c.c	2017-05-21 18:10:28 +0000
+++ breezy/_static_tuple_c.c	2017-06-27 00:56:24 +0000
@@ -26,12 +26,6 @@
 #include "_static_tuple_c.h"
 #include "_export_c_api.h"
 
-/* Pyrex 0.9.6.4 exports _simple_set_pyx_api as
- * import__simple_set_pyx(), while Pyrex 0.9.8.5 and Cython 0.11.3 export them
- * as import_breezy___simple_set_pyx(). As such, we just #define one to be
- * equivalent to the other in our internal code.
- */
-#define import__simple_set_pyx import_breezy___simple_set_pyx
 #include "_simple_set_pyx_api.h"
 
 #if defined(__GNUC__)
@@ -242,11 +236,14 @@
                 " should not have a NULL entry.");
             return 0;
         }
-        if (PyString_CheckExact(obj)
+        if (PyBytes_CheckExact(obj)
             || StaticTuple_CheckExact(obj)
             || obj == Py_None
             || PyBool_Check(obj)
+#if PY_MAJOR_VERSION >= 3
+#else
             || PyInt_CheckExact(obj)
+#endif
             || PyLong_CheckExact(obj)
             || PyFloat_CheckExact(obj)
             || PyUnicode_CheckExact(obj)
@@ -314,8 +311,12 @@
     if (tuple_repr == NULL) {
         return NULL;
     }
+#if PY_MAJOR_VERSION >= 3
+    result = PyUnicode_FromFormat("StaticTuple%U", tuple_repr);
+#else
     result = PyString_FromFormat("StaticTuple%s",
                                  PyString_AsString(tuple_repr));
+#endif
     return result;
 }
 
@@ -362,7 +363,7 @@
 {
     PyObject *vt;
     PyObject *result = NULL;
-    
+
     vt = StaticTuple_as_tuple((StaticTuple *)v);
     if (vt == NULL) {
         goto done;
@@ -427,14 +428,22 @@
     } else if (w == Py_None) {
         // None is always less than the object
         switch (op) {
-        case Py_NE:case Py_GT:case Py_GE:
+        case Py_NE:
+#if PY_MAJOR_VERSION >= 3
+#else
+        case Py_GT:case Py_GE:
+#endif
             Py_INCREF(Py_True);
             return Py_True;
-        case Py_EQ:case Py_LT:case Py_LE:
+        case Py_EQ:
+#if PY_MAJOR_VERSION >= 3
+#else
+        case Py_LT:case Py_LE:
+#endif
             Py_INCREF(Py_False);
             return Py_False;
-    default: // Should never happen
-        return Py_NotImplemented;
+        default: // Should only happen on Python 3
+            return Py_NotImplemented;
         }
     } else {
         /* We don't special case this comparison, we just let python handle
@@ -478,7 +487,7 @@
     vlen = v_st->size;
     wlen = w_st->size;
     min_len = (vlen < wlen) ? vlen : wlen;
-    string_richcompare = PyString_Type.tp_richcompare;
+    string_richcompare = PyBytes_Type.tp_richcompare;
     for (i = 0; i < min_len; i++) {
         PyObject *result = NULL;
         v_obj = StaticTuple_GET_ITEM(v_st, i);
@@ -487,7 +496,7 @@
             /* Shortcut case, these must be identical */
             continue;
         }
-        if (PyString_CheckExact(v_obj) && PyString_CheckExact(w_obj)) {
+        if (PyBytes_CheckExact(v_obj) && PyBytes_CheckExact(w_obj)) {
             result = string_richcompare(v_obj, w_obj, Py_EQ);
         } else if (StaticTuple_CheckExact(v_obj) &&
                    StaticTuple_CheckExact(w_obj))
@@ -547,7 +556,7 @@
         return Py_True;
     }
     /* It is some other comparison, go ahead and do the real check. */
-    if (PyString_CheckExact(v_obj) && PyString_CheckExact(w_obj))
+    if (PyBytes_CheckExact(v_obj) && PyBytes_CheckExact(w_obj))
     {
         return string_richcompare(v_obj, w_obj, op);
     } else if (StaticTuple_CheckExact(v_obj) &&
@@ -679,6 +688,8 @@
     return obj;
 }
 
+#if PY_MAJOR_VERSION >= 3
+#else
 static PyObject *
 StaticTuple_slice(StaticTuple *self, Py_ssize_t ilow, Py_ssize_t ihigh)
 {
@@ -692,6 +703,21 @@
     Py_DECREF(as_tuple);
     return result;
 }
+#endif
+
+static PyObject *
+StaticTuple_subscript(StaticTuple *self, PyObject *key)
+{
+    PyObject *as_tuple, *result;
+
+    as_tuple = StaticTuple_as_tuple(self);
+    if (as_tuple == NULL) {
+        return NULL;
+    }
+    result = PyTuple_Type.tp_as_mapping->mp_subscript(as_tuple, key);
+    Py_DECREF(as_tuple);
+    return result;
+}
 
 static int
 StaticTuple_traverse(StaticTuple *self, visitproc visit, void *arg)
@@ -707,10 +733,10 @@
 static PyObject *
 StaticTuple_sizeof(StaticTuple *self)
 {
-	Py_ssize_t res;
+    Py_ssize_t res;
 
-	res = _PyObject_SIZE(&StaticTuple_Type) + (int)self->size * sizeof(void*);
-	return PyInt_FromSsize_t(res);
+    res = _PyObject_SIZE(&StaticTuple_Type) + (int)self->size * sizeof(void*);
+    return PyInt_FromSsize_t(res);
 }
 
 
@@ -759,29 +785,36 @@
     0,                              /* nb_or */
     0,                              /* nb_coerce */
 };
-    
+
 
 static PySequenceMethods StaticTuple_as_sequence = {
     (lenfunc)StaticTuple_length,            /* sq_length */
     0,                              /* sq_concat */
     0,                              /* sq_repeat */
     (ssizeargfunc)StaticTuple_item,         /* sq_item */
+#if PY_MAJOR_VERSION >= 3
+#else
     (ssizessizeargfunc)StaticTuple_slice,   /* sq_slice */
+#endif
     0,                              /* sq_ass_item */
     0,                              /* sq_ass_slice */
     0,                              /* sq_contains */
-};
-
-/* TODO: Implement StaticTuple_as_mapping.
- *       The only thing we really want to support from there is mp_subscript,
- *       so that we could support extended slicing (foo[::2]). Not worth it
- *       yet, though.
- */
+#if PY_MAJOR_VERSION >= 3
+    0,                              /* sq_inplace_concat */
+    0,                              /* sq_inplace_repeat */
+#endif
+};
+
+
+static PyMappingMethods StaticTuple_as_mapping = {
+    (lenfunc)StaticTuple_length,            /* mp_length */
+    (binaryfunc)StaticTuple_subscript,      /* mp_subscript */
+    0,                                      /* mp_ass_subscript */
+};
 
 
 PyTypeObject StaticTuple_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0,                                           /* ob_size */
+    PyVarObject_HEAD_INIT(NULL, 0)
     "breezy._static_tuple_c.StaticTuple",        /* tp_name */
     sizeof(StaticTuple),                         /* tp_basicsize */
     sizeof(PyObject *),                          /* tp_itemsize */
@@ -793,7 +826,7 @@
     (reprfunc)StaticTuple_repr,                  /* tp_repr */
     &StaticTuple_as_number,                      /* tp_as_number */
     &StaticTuple_as_sequence,                    /* tp_as_sequence */
-    0,                                           /* tp_as_mapping */
+    &StaticTuple_as_mapping,                     /* tp_as_mapping */
     (hashfunc)StaticTuple_hash,                  /* tp_hash */
     0,                                           /* tp_call */
     0,                                           /* tp_str */
@@ -887,72 +920,32 @@
 }
 
 
-static int
-_workaround_pyrex_096(void)
-{
-    /* Work around an incompatibility in how pyrex 0.9.6 exports a module,
-     * versus how pyrex 0.9.8 and cython 0.11 export it.
-     * Namely 0.9.6 exports import__simple_set_pyx and tries to
-     * "import _simple_set_pyx" but it is available only as
-     * "import breezy._simple_set_pyx"
-     * It is a shame to hack up sys.modules, but that is what we've got to do.
-     */
-    PyObject *sys_module = NULL, *modules = NULL, *set_module = NULL;
-    int retval = -1;
-
-    /* Clear out the current ImportError exception, and try again. */
-    PyErr_Clear();
-    /* Note that this only seems to work if somewhere else imports
-     * breezy._simple_set_pyx before importing breezy._static_tuple_c
-     */
-    set_module = PyImport_ImportModule("breezy._simple_set_pyx");
-    if (set_module == NULL) {
-        goto end;
-    }
-    /* Add the _simple_set_pyx into sys.modules at the appropriate location. */
-    sys_module = PyImport_ImportModule("sys");
-    if (sys_module == NULL) {
-        goto end;
-    }
-    modules = PyObject_GetAttrString(sys_module, "modules");
-    if (modules == NULL || !PyDict_Check(modules)) {
-        goto end;
-    }
-    PyDict_SetItemString(modules, "_simple_set_pyx", set_module);
-    /* Now that we have hacked it in, try the import again. */
-    retval = import_breezy___simple_set_pyx();
-end:
-    Py_XDECREF(set_module);
-    Py_XDECREF(sys_module);
-    Py_XDECREF(modules);
-    return retval;
-}
-
-
-PyMODINIT_FUNC
-init_static_tuple_c(void)
+PYMOD_INIT_FUNC(_static_tuple_c)
 {
     PyObject* m;
 
     StaticTuple_Type.tp_getattro = PyObject_GenericGetAttr;
-    if (PyType_Ready(&StaticTuple_Type) < 0)
-        return;
+    if (PyType_Ready(&StaticTuple_Type) < 0) {
+        return PYMOD_ERROR;
+    }
 
-    m = Py_InitModule3("_static_tuple_c", static_tuple_c_methods,
-                       "C implementation of a StaticTuple structure");
-    if (m == NULL)
-      return;
+    PYMOD_CREATE(m, "_static_tuple_c",
+                 "C implementation of a StaticTuple structure",
+                 static_tuple_c_methods);
+    if (m == NULL) {
+      return PYMOD_ERROR;
+    }
 
     Py_INCREF(&StaticTuple_Type);
     PyModule_AddObject(m, "StaticTuple", (PyObject *)&StaticTuple_Type);
-    if (import_breezy___simple_set_pyx() == -1
-        && _workaround_pyrex_096() == -1)
-    {
-        return;
+    if (import_breezy___simple_set_pyx() == -1) {
+        return PYMOD_ERROR;
     }
     setup_interned_tuples(m);
     setup_empty_tuple(m);
     setup_c_api(m);
+
+    return PYMOD_SUCCESS(m);
 }
 
 // vim: tabstop=4 sw=4 expandtab

=== modified file 'breezy/annotate.py'
--- breezy/annotate.py	2017-06-04 18:09:30 +0000
+++ breezy/annotate.py	2017-07-21 13:20:17 +0000
@@ -41,7 +41,11 @@
     errors,
     osutils,
     )
-from .config import extract_email_address
+from .config import (
+    NoEmailInUsername,
+    NoWhoami,
+    extract_email_address,
+    )
 from .repository import _strip_NULL_ghosts
 from .revision import (
     CURRENT_REVISION,
@@ -82,7 +86,7 @@
         current_rev.parent_ids = tree.get_parent_ids()
         try:
             current_rev.committer = branch.get_config_stack().get('email')
-        except errors.NoWhoami:
+        except NoWhoami:
             current_rev.committer = 'local user'
         current_rev.message = "?"
         current_rev.timestamp = round(time.time(), 3)
@@ -188,10 +192,10 @@
         revision_id_to_revno[CURRENT_REVISION] = (
             "%d?" % (branch.revno() + 1),)
         revisions[CURRENT_REVISION] = current_rev
-    revision_ids = [o for o in revision_ids if
-                    repository.has_revision(o)]
-    revisions.update((r.revision_id, r) for r in
-                     repository.get_revisions(revision_ids))
+    revisions.update(
+            entry for entry in
+            repository.iter_revisions(revision_ids)
+            if entry[1] is not None)
     for origin, text in annotations:
         text = text.rstrip('\r\n')
         if origin == last_origin:
@@ -212,7 +216,7 @@
             author = rev.get_apparent_authors()[0]
             try:
                 author = extract_email_address(author)
-            except errors.NoEmailInUsername:
+            except NoEmailInUsername:
                 pass        # use the whole name
         yield (revno_str, author, date_str, origin, text)
 

=== modified file 'breezy/branch.py'
--- breezy/branch.py	2017-06-11 18:41:26 +0000
+++ breezy/branch.py	2017-07-15 13:23:08 +0000
@@ -800,7 +800,7 @@
                 try:
                     url = url.encode('ascii')
                 except UnicodeEncodeError:
-                    raise errors.InvalidURL(url,
+                    raise urlutils.InvalidURL(url,
                         "Urls must be 7-bit ascii, "
                         "use breezy.urlutils.escape")
             url = urlutils.relative_url(self.base, url)
@@ -1112,7 +1112,7 @@
             parent = urlutils.local_path_to_url(parent.decode('utf8'))
         try:
             return urlutils.join(self.base[:-1], parent)
-        except errors.InvalidURLJoin as e:
+        except urlutils.InvalidURLJoin as e:
             raise errors.InaccessibleParent(parent, self.user_url)
 
     def _get_parent_location(self):
@@ -2259,7 +2259,7 @@
             try:
                 relpath = self.source.user_transport.relpath(normalized)
                 source_is_master = (relpath == '')
-            except (errors.PathNotChild, errors.InvalidURL):
+            except (errors.PathNotChild, urlutils.InvalidURL):
                 source_is_master = False
         if not local and bound_location and not source_is_master:
             # not pulling from master, so we need to update master.

=== modified file 'breezy/bugtracker.py'
--- breezy/bugtracker.py	2017-05-22 00:56:52 +0000
+++ breezy/bugtracker.py	2017-07-10 20:12:25 +0000
@@ -16,10 +16,13 @@
 
 from __future__ import absolute_import
 
-from . import registry
+from . import (
+    errors,
+    registry,
+    )
 from .lazy_import import lazy_import
 lazy_import(globals(), """
-from breezy import errors, urlutils
+from breezy import urlutils
 """)
 
 
@@ -145,6 +148,52 @@
 """
 
 
+class MalformedBugIdentifier(errors.BzrError):
+
+    _fmt = ('Did not understand bug identifier %(bug_id)s: %(reason)s. '
+            'See "brz help bugs" for more information on this feature.')
+
+    def __init__(self, bug_id, reason):
+        self.bug_id = bug_id
+        self.reason = reason
+
+
+class InvalidBugTrackerURL(errors.BzrError):
+
+    _fmt = ("The URL for bug tracker \"%(abbreviation)s\" doesn't "
+            "contain {id}: %(url)s")
+
+    def __init__(self, abbreviation, url):
+        self.abbreviation = abbreviation
+        self.url = url
+
+
+class UnknownBugTrackerAbbreviation(errors.BzrError):
+
+    _fmt = ("Cannot find registered bug tracker called %(abbreviation)s "
+            "on %(branch)s")
+
+    def __init__(self, abbreviation, branch):
+        self.abbreviation = abbreviation
+        self.branch = branch
+
+
+class InvalidLineInBugsProperty(errors.BzrError):
+
+    _fmt = ("Invalid line in bugs property: '%(line)s'")
+
+    def __init__(self, line):
+        self.line = line
+
+
+class InvalidBugStatus(errors.BzrError):
+
+    _fmt = ("Invalid bug status: '%(status)s'")
+
+    def __init__(self, status):
+        self.status = status
+
+
 def get_bug_url(abbreviated_bugtracker_name, branch, bug_id):
     """Return a URL pointing to the canonical web page of the bug identified by
     'bug_id'.
@@ -167,8 +216,8 @@
             tracker = tracker_type.get(abbreviated_bugtracker_name, branch)
             if tracker is not None:
                 return tracker
-        raise errors.UnknownBugTrackerAbbreviation(abbreviated_bugtracker_name,
-                                                   branch)
+        raise UnknownBugTrackerAbbreviation(abbreviated_bugtracker_name,
+                branch)
 
     def help_topic(self, topic):
         return _bugs_help
@@ -203,7 +252,7 @@
         try:
             int(bug_id)
         except ValueError:
-            raise errors.MalformedBugIdentifier(bug_id, "Must be an integer")
+            raise MalformedBugIdentifier(bug_id, "Must be an integer")
 
 
 class UniqueIntegerBugTracker(IntegerBugTracker):
@@ -304,8 +353,7 @@
     def _get_bug_url(self, bug_id):
         """Given a validated bug_id, return the bug's web page's URL."""
         if '{id}' not in self._base_url:
-            raise errors.InvalidBugTrackerURL(self._abbreviation,
-                                              self._base_url)
+            raise InvalidBugTrackerURL(self._abbreviation, self._base_url)
         return self._base_url.replace('{id}', str(bug_id))
 
 

=== modified file 'breezy/builtins.py'
--- breezy/builtins.py	2017-06-11 20:15:04 +0000
+++ breezy/builtins.py	2017-07-21 14:18:24 +0000
@@ -22,7 +22,7 @@
 import os
 import sys
 
-import breezy.bzr.bzrdir
+import breezy.bzr
 
 from . import lazy_import
 lazy_import.lazy_import(globals(), """
@@ -40,6 +40,7 @@
     errors,
     globbing,
     hooks,
+    lazy_regex,
     log,
     merge as _mod_merge,
     merge_directive,
@@ -1411,8 +1412,6 @@
 
     To retrieve the branch as of a particular revision, supply the --revision
     parameter, as in "branch foo/bar -r 5".
-
-    The synonyms 'clone' and 'get' for this command are deprecated.
     """
 
     _see_also = ['checkout']
@@ -1440,19 +1439,12 @@
         Option('bind',
             help="Bind new branch to from location."),
         ]
-    aliases = ['get', 'clone']
 
     def run(self, from_location, to_location=None, revision=None,
             hardlink=False, stacked=False, standalone=False, no_tree=False,
             use_existing_dir=False, switch=False, bind=False,
             files_from=None):
         from breezy import switch as _mod_switch
-        if self.invoked_as in ['get', 'clone']:
-            ui.ui_factory.show_user_warning(
-                'deprecated_command',
-                deprecated_name=self.invoked_as,
-                recommended_name='branch',
-                deprecated_in_version='2.4')
         accelerator_tree, br_from = controldir.ControlDir.open_tree_or_branch(
             from_location)
         if not (hardlink or files_from):
@@ -2073,7 +2065,7 @@
                 'See "help formats".',
                 lazy_registry=('breezy.controldir', 'format_registry'),
                 converter=lambda name: controldir.format_registry.make_controldir(name),
-                value_switches=True,
+                value_switches=False,
                 title="Branch format",
                 ),
          Option('append-revisions-only',
@@ -2152,7 +2144,7 @@
                 url = repository.controldir.root_transport.external_url()
                 try:
                     url = urlutils.local_path_from_url(url)
-                except errors.InvalidURL:
+                except urlutils.InvalidURL:
                     pass
                 self.outf.write(gettext("Using shared repository: %s\n") % url)
 
@@ -2192,7 +2184,7 @@
                                  ' "brz help formats" for details.',
                             lazy_registry=('breezy.controldir', 'format_registry'),
                             converter=lambda name: controldir.format_registry.make_controldir(name),
-                            value_switches=True, title='Repository format'),
+                            value_switches=False, title='Repository format'),
                      Option('no-trees',
                              help='Branches in the repository will default to'
                                   ' not having a working tree.'),
@@ -3199,7 +3191,7 @@
                             'Invalid ignore patterns found. %s',
                             bad_patterns_count) % bad_patterns)
             ui.ui_factory.show_error(msg)
-            raise errors.InvalidPattern('')
+            raise lazy_regex.InvalidPattern('')
         for name_pattern in name_pattern_list:
             if (name_pattern[0] == '/' or
                 (len(name_pattern) > 1 and name_pattern[1] == ':')):
@@ -3556,10 +3548,10 @@
                 tag, bug_id = tokens
             try:
                 yield bugtracker.get_bug_url(tag, branch, bug_id)
-            except errors.UnknownBugTrackerAbbreviation:
+            except bugtracker.UnknownBugTrackerAbbreviation:
                 raise errors.BzrCommandError(gettext(
                     'Unrecognized bug %s. Commit refused.') % fixed_bug)
-            except errors.MalformedBugIdentifier as e:
+            except bugtracker.MalformedBugIdentifier as e:
                 raise errors.BzrCommandError(gettext(
                     "%s\nCommit refused.") % (str(e),))
 
@@ -3872,7 +3864,7 @@
         # display a warning if an email address isn't included in the given name.
         try:
             _mod_config.extract_email_address(name)
-        except errors.NoEmailInUsername as e:
+        except _mod_config.NoEmailInUsername as e:
             warning('"%s" does not seem to contain an email address.  '
                     'This is allowed, but not recommended.', name)
 
@@ -4413,7 +4405,7 @@
                     raise errors.BzrCommandError(gettext(
                         'Cannot use -r with merge directives or bundles'))
                 merger, verified = _mod_merge.Merger.from_mergeable(tree,
-                   mergeable, None)
+                   mergeable)
 
         if merger is None and uncommitted:
             if revision is not None and len(revision) > 0:
@@ -4568,7 +4560,7 @@
         # Merge tags (but don't set them in the master branch yet, the user
         # might revert this merge).  Commit will propagate them.
         other_branch.tags.merge_to(tree.branch.tags, ignore_master=True)
-        merger = _mod_merge.Merger.from_revision_ids(pb, tree,
+        merger = _mod_merge.Merger.from_revision_ids(tree,
             other_revision_id, base_revision_id, other_branch, base_branch)
         if other_path != '':
             allow_pending = False
@@ -4720,7 +4712,7 @@
         # have not yet been seen.
         tree.set_parent_ids(parents[:1])
         try:
-            merger = _mod_merge.Merger.from_revision_ids(None, tree, parents[1])
+            merger = _mod_merge.Merger.from_revision_ids(tree, parents[1])
             merger.interesting_ids = interesting_ids
             merger.merge_type = merge_type
             merger.show_base = show_base
@@ -5216,6 +5208,7 @@
             b.repository.start_write_group()
             try:
                 for revision_id in revision_id_list:
+                    revision_id = cache_utf8.encode(revision_id)
                     b.repository.sign_revision(revision_id, gpg_strategy)
             except:
                 b.repository.abort_write_group()

=== modified file 'breezy/bzr/__init__.py'
--- breezy/bzr/__init__.py	2017-06-10 01:39:46 +0000
+++ breezy/bzr/__init__.py	2017-06-14 22:34:05 +0000
@@ -15,3 +15,339 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from __future__ import absolute_import
+
+from .. import (
+    config,
+    errors,
+    controldir,
+    pyutils,
+    registry,
+    )
+
+class BzrProber(controldir.Prober):
+    """Prober for formats that use a .bzr/ control directory."""
+
+    formats = registry.FormatRegistry(controldir.network_format_registry)
+    """The known .bzr formats."""
+
+    @classmethod
+    def probe_transport(klass, transport):
+        """Return the .bzrdir style format present in a directory."""
+        try:
+            format_string = transport.get_bytes(".bzr/branch-format")
+            # GZ 2017-06-09: Where should format strings get decoded...
+            format_text = format_string.decode("ascii")
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=transport.base)
+        try:
+            first_line = format_text[:format_text.index("\n")+1]
+        except ValueError:
+            first_line = format_text
+        try:
+            cls = klass.formats.get(first_line)
+        except KeyError:
+            raise errors.UnknownFormatError(format=first_line, kind='bzrdir')
+        return cls.from_string(format_text)
+
+    @classmethod
+    def known_formats(cls):
+        result = set()
+        for name, format in cls.formats.items():
+            if callable(format):
+                format = format()
+            result.add(format)
+        return result
+
+
+controldir.ControlDirFormat.register_prober(BzrProber)
+
+
+class RemoteBzrProber(controldir.Prober):
+    """Prober for remote servers that provide a Bazaar smart server."""
+
+    @classmethod
+    def probe_transport(klass, transport):
+        """Return a RemoteBzrDirFormat object if it looks possible."""
+        try:
+            medium = transport.get_smart_medium()
+        except (NotImplementedError, AttributeError,
+                errors.TransportNotPossible, errors.NoSmartMedium,
+                errors.SmartProtocolError):
+            # no smart server, so not a branch for this format type.
+            raise errors.NotBranchError(path=transport.base)
+        else:
+            # Decline to open it if the server doesn't support our required
+            # version (3) so that the VFS-based transport will do it.
+            if medium.should_probe():
+                try:
+                    server_version = medium.protocol_version()
+                except errors.SmartProtocolError:
+                    # Apparently there's no usable smart server there, even though
+                    # the medium supports the smart protocol.
+                    raise errors.NotBranchError(path=transport.base)
+                if server_version != '2':
+                    raise errors.NotBranchError(path=transport.base)
+            from .remote import RemoteBzrDirFormat
+            return RemoteBzrDirFormat()
+
+    @classmethod
+    def known_formats(cls):
+        from .remote import RemoteBzrDirFormat
+        return {RemoteBzrDirFormat()}
+
+
+controldir.ControlDirFormat.register_server_prober(RemoteBzrProber)
+
+# Register bzr formats
+BzrProber.formats.register_lazy(
+    "Bazaar-NG meta directory, format 1\n",
+    __name__ + '.bzrdir', 'BzrDirMetaFormat1')
+BzrProber.formats.register_lazy(
+    "Bazaar meta directory, format 1 (with colocated branches)\n",
+    __name__ + '.bzrdir', 'BzrDirMetaFormat1Colo')
+
+
+def register_metadir(registry, key,
+         repository_format, help, native=True, deprecated=False,
+         branch_format=None,
+         tree_format=None,
+         hidden=False,
+         experimental=False,
+         alias=False, bzrdir_format=None):
+    """Register a metadir subformat.
+
+    These all use a meta bzrdir, but can be parameterized by the
+    Repository/Branch/WorkingTreeformats.
+
+    :param repository_format: The fully-qualified repository format class
+        name as a string.
+    :param branch_format: Fully-qualified branch format class name as
+        a string.
+    :param tree_format: Fully-qualified tree format class name as
+        a string.
+    """
+    if bzrdir_format is None:
+        bzrdir_format = 'breezy.bzr.bzrdir.BzrDirMetaFormat1'
+    # This should be expanded to support setting WorkingTree and Branch
+    # formats, once the API supports that.
+    def _load(full_name):
+        mod_name, factory_name = full_name.rsplit('.', 1)
+        try:
+            factory = pyutils.get_named_object(mod_name, factory_name)
+        except ImportError as e:
+            raise ImportError('failed to load %s: %s' % (full_name, e))
+        except AttributeError:
+            raise AttributeError('no factory %s in module %r'
+                % (full_name, sys.modules[mod_name]))
+        return factory()
+
+    def helper():
+        bd = _load(bzrdir_format)
+        if branch_format is not None:
+            bd.set_branch_format(_load(branch_format))
+        if tree_format is not None:
+            bd.workingtree_format = _load(tree_format)
+        if repository_format is not None:
+            bd.repository_format = _load(repository_format)
+        return bd
+    registry.register(key, helper, help, native, deprecated, hidden,
+        experimental, alias)
+
+register_metadir(controldir.format_registry, 'knit',
+    'breezy.bzr.knitrepo.RepositoryFormatKnit1',
+    'Format using knits.  Recommended for interoperation with bzr <= 0.14.',
+    branch_format='breezy.bzr.fullhistory.BzrBranchFormat5',
+    tree_format='breezy.bzr.workingtree_3.WorkingTreeFormat3',
+    hidden=True,
+    deprecated=True)
+register_metadir(controldir.format_registry, 'dirstate',
+    'breezy.bzr.knitrepo.RepositoryFormatKnit1',
+    help='Format using dirstate for working trees. '
+        'Compatible with bzr 0.8 and '
+        'above when accessed over the network. Introduced in bzr 0.15.',
+    branch_format='breezy.bzr.fullhistory.BzrBranchFormat5',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    hidden=True,
+    deprecated=True)
+register_metadir(controldir.format_registry, 'dirstate-tags',
+    'breezy.bzr.knitrepo.RepositoryFormatKnit1',
+    help='Variant of dirstate with support for tags. '
+        'Introduced in bzr 0.15.',
+    branch_format='breezy.bzr.branch.BzrBranchFormat6',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    hidden=True,
+    deprecated=True)
+register_metadir(controldir.format_registry, 'rich-root',
+    'breezy.bzr.knitrepo.RepositoryFormatKnit4',
+    help='Variant of dirstate with better handling of tree roots. '
+        'Introduced in bzr 1.0',
+    branch_format='breezy.bzr.branch.BzrBranchFormat6',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    hidden=True,
+    deprecated=True)
+register_metadir(controldir.format_registry, 'dirstate-with-subtree',
+    'breezy.bzr.knitrepo.RepositoryFormatKnit3',
+    help='Variant of dirstate with support for nested trees. '
+         'Introduced in 0.15.',
+    branch_format='breezy.bzr.branch.BzrBranchFormat6',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    experimental=True,
+    hidden=True,
+    )
+register_metadir(controldir.format_registry, 'pack-0.92',
+    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack1',
+    help='Pack-based format used in 1.x series. Introduced in 0.92. '
+        'Interoperates with bzr repositories before 0.92 but cannot be '
+        'read by bzr < 0.92. '
+        ,
+    branch_format='breezy.bzr.branch.BzrBranchFormat6',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    deprecated=True,
+    )
+register_metadir(controldir.format_registry, 'pack-0.92-subtree',
+    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack3',
+    help='Pack-based format used in 1.x series, with subtree support. '
+        'Introduced in 0.92. Interoperates with '
+        'bzr repositories before 0.92 but cannot be read by bzr < 0.92. '
+        ,
+    branch_format='breezy.bzr.branch.BzrBranchFormat6',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    hidden=True,
+    deprecated=True,
+    experimental=True,
+    )
+register_metadir(controldir.format_registry, 'rich-root-pack',
+    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack4',
+    help='A variant of pack-0.92 that supports rich-root data '
+         '(needed for bzr-svn and bzr-git). Introduced in 1.0.',
+    branch_format='breezy.bzr.branch.BzrBranchFormat6',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    hidden=True,
+    deprecated=True,
+    )
+register_metadir(controldir.format_registry, '1.6',
+    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack5',
+    help='A format that allows a branch to indicate that there is another '
+         '(stacked) repository that should be used to access data that is '
+         'not present locally.',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    hidden=True,
+    deprecated=True,
+    )
+register_metadir(controldir.format_registry, '1.6.1-rich-root',
+    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack5RichRoot',
+    help='A variant of 1.6 that supports rich-root data '
+         '(needed for bzr-svn and bzr-git).',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    hidden=True,
+    deprecated=True,
+    )
+register_metadir(controldir.format_registry, '1.9',
+    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack6',
+    help='A repository format using B+tree indexes. These indexes '
+         'are smaller in size, have smarter caching and provide faster '
+         'performance for most operations.',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    hidden=True,
+    deprecated=True,
+    )
+register_metadir(controldir.format_registry, '1.9-rich-root',
+    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack6RichRoot',
+    help='A variant of 1.9 that supports rich-root data '
+         '(needed for bzr-svn and bzr-git).',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
+    hidden=True,
+    deprecated=True,
+    )
+register_metadir(controldir.format_registry, '1.14',
+    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack6',
+    help='A working-tree format that supports content filtering.',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat5',
+    hidden=True,
+    deprecated=True,
+    )
+register_metadir(controldir.format_registry, '1.14-rich-root',
+    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack6RichRoot',
+    help='A variant of 1.14 that supports rich-root data '
+         '(needed for bzr-svn and bzr-git).',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat5',
+    hidden=True,
+    deprecated=True,
+    )
+# The following un-numbered 'development' formats should always just be aliases.
+register_metadir(controldir.format_registry, 'development-subtree',
+    'breezy.bzr.groupcompress_repo.RepositoryFormat2aSubtree',
+    help='Current development format, subtree variant. Can convert data to and '
+        'from pack-0.92-subtree (and anything compatible with '
+        'pack-0.92-subtree) format repositories. Repositories and branches in '
+        'this format can only be read by bzr.dev. Please read '
+        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
+        'before use.',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',
+    experimental=True,
+    hidden=True,
+    alias=False, # Restore to being an alias when an actual development subtree format is added
+                 # This current non-alias status is simply because we did not introduce a
+                 # chk based subtree format.
+    )
+register_metadir(controldir.format_registry, 'development5-subtree',
+    'breezy.bzr.knitpack_repo.RepositoryFormatPackDevelopment2Subtree',
+    help='Development format, subtree variant. Can convert data to and '
+        'from pack-0.92-subtree (and anything compatible with '
+        'pack-0.92-subtree) format repositories. Repositories and branches in '
+        'this format can only be read by bzr.dev. Please read '
+        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
+        'before use.',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',
+    experimental=True,
+    hidden=True,
+    alias=False,
+    )
+
+register_metadir(controldir.format_registry, 'development-colo',
+    'breezy.bzr.groupcompress_repo.RepositoryFormat2a',
+    help='The 2a format with experimental support for colocated branches.\n',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',
+    experimental=True,
+    bzrdir_format='breezy.bzr.bzrdir.BzrDirMetaFormat1Colo',
+    )
+
+
+# And the development formats above will have aliased one of the following:
+
+# Finally, the current format.
+register_metadir(controldir.format_registry, '2a',
+    'breezy.bzr.groupcompress_repo.RepositoryFormat2a',
+    help='Format for the bzr 2.0 series.\n'
+        'Uses group-compress storage.\n'
+        'Provides rich roots which are a one-way transition.\n',
+        # 'storage in packs, 255-way hashed CHK inventory, bencode revision, group compress, '
+        # 'rich roots. Supported by bzr 1.16 and later.',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',
+    experimental=False,
+    )
+
+# The following format should be an alias for the rich root equivalent
+# of the default format
+register_metadir(controldir.format_registry, 'default-rich-root',
+    'breezy.bzr.groupcompress_repo.RepositoryFormat2a',
+    branch_format='breezy.bzr.branch.BzrBranchFormat7',
+    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',
+    alias=True,
+    hidden=True,
+    help='Same as 2a.')
+
+
+# The current format that is made on 'bzr init'.
+format_name = config.GlobalStack().get('default_format')
+controldir.format_registry.set_default(format_name)

=== modified file 'breezy/bzr/_btree_serializer_py.py'
--- breezy/bzr/_btree_serializer_py.py	2017-06-08 23:30:31 +0000
+++ breezy/bzr/_btree_serializer_py.py	2017-07-02 14:34:31 +0000
@@ -22,24 +22,24 @@
 from .. import static_tuple
 
 
-def _parse_leaf_lines(bytes, key_length, ref_list_length):
-    lines = bytes.split('\n')
+def _parse_leaf_lines(data, key_length, ref_list_length):
+    lines = data.split(b'\n')
     nodes = []
     as_st = static_tuple.StaticTuple.from_sequence
     stuple = static_tuple.StaticTuple
     for line in lines[1:]:
-        if line == '':
+        if line == b'':
             return nodes
-        elements = line.split('\0', key_length)
+        elements = line.split(b'\0', key_length)
         # keys are tuples
         key = as_st(elements[:key_length]).intern()
         line = elements[-1]
-        references, value = line.rsplit('\0', 1)
+        references, value = line.rsplit(b'\0', 1)
         if ref_list_length:
             ref_lists = []
-            for ref_string in references.split('\t'):
-                ref_list = as_st([as_st(ref.split('\0')).intern()
-                                  for ref in ref_string.split('\r') if ref])
+            for ref_string in references.split(b'\t'):
+                ref_list = as_st([as_st(ref.split(b'\0')).intern()
+                                  for ref in ref_string.split(b'\r') if ref])
                 ref_lists.append(ref_list)
             ref_lists = as_st(ref_lists)
             node_value = stuple(value, ref_lists)
@@ -63,12 +63,12 @@
         # TODO: Consider turning this back into the 'unoptimized' nested loop
         #       form. It is probably more obvious for most people, and this is
         #       just a reference implementation.
-        flattened_references = ['\r'.join(['\x00'.join(reference)
-                                           for reference in ref_list])
+        flattened_references = [b'\r'.join([b'\x00'.join(reference)
+                                            for reference in ref_list])
                                 for ref_list in node[3]]
     else:
         flattened_references = []
-    string_key = '\x00'.join(node[1])
-    line = ("%s\x00%s\x00%s\n" % (string_key,
-        '\t'.join(flattened_references), node[2]))
+    string_key = b'\x00'.join(node[1])
+    line = (b"%s\x00%s\x00%s\n" % (string_key,
+        b'\t'.join(flattened_references), node[2]))
     return string_key, line

=== modified file 'breezy/bzr/branch.py'
--- breezy/bzr/branch.py	2017-06-10 16:40:42 +0000
+++ breezy/bzr/branch.py	2017-06-14 23:29:06 +0000
@@ -234,7 +234,7 @@
 
     @needs_write_lock
     def set_last_revision_info(self, revno, revision_id):
-        if not revision_id or not isinstance(revision_id, basestring):
+        if not revision_id or not isinstance(revision_id, bytes):
             raise errors.InvalidRevisionId(revision_id=revision_id, branch=self)
         revision_id = _mod_revision.ensure_null(revision_id)
         old_revno, old_revid = self.last_revision_info()

=== modified file 'breezy/bzr/btree_index.py'
--- breezy/bzr/btree_index.py	2017-06-10 12:50:32 +0000
+++ breezy/bzr/btree_index.py	2017-07-02 14:36:53 +0000
@@ -17,7 +17,7 @@
 
 """B+Tree indices"""
 
-from __future__ import absolute_import
+from __future__ import absolute_import, division
 
 from ..lazy_import import lazy_import
 lazy_import(globals(), """
@@ -84,7 +84,7 @@
         if self.nodes == 0:
             self.spool = BytesIO()
             # padded note:
-            self.spool.write("\x00" * _RESERVED_HEADER_BYTES)
+            self.spool.write(b"\x00" * _RESERVED_HEADER_BYTES)
         elif self.nodes == 1:
             # We got bigger than 1 node, switch to a temp file
             spool = tempfile.TemporaryFile(prefix='bzr-index-row-')
@@ -323,7 +323,7 @@
                         optimize_for_size=optimize_for_size)
                     internal_row.writer.write(_INTERNAL_FLAG)
                     internal_row.writer.write(_INTERNAL_OFFSET +
-                        str(rows[pos + 1].nodes) + "\n")
+                        b"%d\n" % rows[pos + 1].nodes)
             # add a new leaf
             length = _PAGE_SIZE
             if rows[-1].nodes == 0:
@@ -339,7 +339,7 @@
                 raise errors.BadIndexKey(string_key)
             # this key did not fit in the node:
             rows[-1].finish_node()
-            key_line = string_key + "\n"
+            key_line = string_key + b"\n"
             new_row = True
             for row in reversed(rows[:-1]):
                 # Mark the start of the next node in the node above. If it
@@ -367,7 +367,7 @@
                     optimize_for_size=self._optimize_for_size)
                 new_row.writer.write(_INTERNAL_FLAG)
                 new_row.writer.write(_INTERNAL_OFFSET +
-                    str(rows[1].nodes - 1) + "\n")
+                    b"%d\n" % (rows[1].nodes - 1))
                 new_row.writer.write(key_line)
             self._add_key(string_key, line, rows, allow_optimize=allow_optimize)
 
@@ -435,7 +435,7 @@
             node = row.spool.read(_PAGE_SIZE)
             result.write(node[reserved:])
             if len(node) == _PAGE_SIZE:
-                result.write("\x00" * (reserved - position))
+                result.write(b"\x00" * (reserved - position))
             position = 0 # Only the root row actually has an offset
             copied_len = osutils.pumpfile(row.spool, result)
             if copied_len != (row.nodes - 1) * _PAGE_SIZE:
@@ -626,14 +626,14 @@
     def __init__(self, bytes):
         """Parse bytes to create an internal node object."""
         # splitlines mangles the \r delimiters.. don't use it.
-        self.keys = self._parse_lines(bytes.split('\n'))
+        self.keys = self._parse_lines(bytes.split(b'\n'))
 
     def _parse_lines(self, lines):
         nodes = []
         self.offset = int(lines[1][7:])
         as_st = static_tuple.StaticTuple.from_sequence
         for line in lines[2:]:
-            if line == '':
+            if line == b'':
                 break
             # GZ 2017-05-24: Used to intern() each chunk of line as well, need
             # to recheck performance and perhaps adapt StaticTuple to adjust.
@@ -688,6 +688,9 @@
         self._row_lengths = None
         self._row_offsets = None # Start of each row, [-1] is the end
 
+    def __hash__(self):
+        return id(self)
+
     def __eq__(self, other):
         """Equal when self and other were created with the same parameters."""
         return (
@@ -734,8 +737,7 @@
         pages fit in that length.
         """
         recommended_read = self._transport.recommended_page_size()
-        recommended_pages = int(math.ceil(recommended_read /
-                                          float(_PAGE_SIZE)))
+        recommended_pages = int(math.ceil(recommended_read / _PAGE_SIZE))
         return recommended_pages
 
     def _compute_total_pages_in_index(self):
@@ -752,7 +754,7 @@
             return self._row_offsets[-1]
         # This is the number of pages as defined by the size of the index. They
         # should be indentical.
-        total_pages = int(math.ceil(self._size / float(_PAGE_SIZE)))
+        total_pages = int(math.ceil(self._size / _PAGE_SIZE))
         return total_pages
 
     def _expand_offsets(self, offsets):
@@ -1516,7 +1518,7 @@
                 node = _InternalNode(bytes)
             else:
                 raise AssertionError("Unknown node type for %r" % bytes)
-            yield offset / _PAGE_SIZE, node
+            yield offset // _PAGE_SIZE, node
 
     def _signature(self):
         """The file signature for this index type."""

=== modified file 'breezy/bzr/bzrdir.py'
--- breezy/bzr/bzrdir.py	2017-06-11 14:07:05 +0000
+++ breezy/bzr/bzrdir.py	2017-07-21 14:18:24 +0000
@@ -34,7 +34,6 @@
 import breezy
 from breezy import (
     cleanup,
-    errors,
     fetch,
     graph,
     lockable_files,
@@ -72,10 +71,20 @@
 from .. import (
     config,
     controldir,
+    errors,
     registry,
     )
 
 
+class MissingFeature(errors.BzrError):
+
+    _fmt = ("Missing feature %(feature)s not provided by this "
+            "version of Bazaar or any plugin.")
+
+    def __init__(self, feature):
+        self.feature = feature
+
+
 class BzrDir(controldir.ControlDir):
     """A .bzr control diretory.
 
@@ -1149,11 +1158,11 @@
                 mutter("ignoring optional missing feature %s", name)
                 continue
             elif necessity == "required":
-                raise errors.MissingFeature(name)
+                raise MissingFeature(name)
             else:
                 mutter("treating unknown necessity as require for %s",
                        name)
-                raise errors.MissingFeature(name)
+                raise MissingFeature(name)
 
     @classmethod
     def get_format_string(cls):
@@ -1223,78 +1232,6 @@
                 self.features[name] = necessity
 
 
-class BzrProber(controldir.Prober):
-    """Prober for formats that use a .bzr/ control directory."""
-
-    formats = registry.FormatRegistry(controldir.network_format_registry)
-    """The known .bzr formats."""
-
-    @classmethod
-    def probe_transport(klass, transport):
-        """Return the .bzrdir style format present in a directory."""
-        try:
-            format_string = transport.get_bytes(".bzr/branch-format")
-            # GZ 2017-06-09: Where should format strings get decoded...
-            format_text = format_string.decode("ascii")
-        except errors.NoSuchFile:
-            raise errors.NotBranchError(path=transport.base)
-        try:
-            first_line = format_text[:format_text.index("\n")+1]
-        except ValueError:
-            first_line = format_text
-        try:
-            cls = klass.formats.get(first_line)
-        except KeyError:
-            raise errors.UnknownFormatError(format=first_line, kind='bzrdir')
-        return cls.from_string(format_text)
-
-    @classmethod
-    def known_formats(cls):
-        result = set()
-        for name, format in cls.formats.items():
-            if callable(format):
-                format = format()
-            result.add(format)
-        return result
-
-
-controldir.ControlDirFormat.register_prober(BzrProber)
-
-
-class RemoteBzrProber(controldir.Prober):
-    """Prober for remote servers that provide a Bazaar smart server."""
-
-    @classmethod
-    def probe_transport(klass, transport):
-        """Return a RemoteBzrDirFormat object if it looks possible."""
-        try:
-            medium = transport.get_smart_medium()
-        except (NotImplementedError, AttributeError,
-                errors.TransportNotPossible, errors.NoSmartMedium,
-                errors.SmartProtocolError):
-            # no smart server, so not a branch for this format type.
-            raise errors.NotBranchError(path=transport.base)
-        else:
-            # Decline to open it if the server doesn't support our required
-            # version (3) so that the VFS-based transport will do it.
-            if medium.should_probe():
-                try:
-                    server_version = medium.protocol_version()
-                except errors.SmartProtocolError:
-                    # Apparently there's no usable smart server there, even though
-                    # the medium supports the smart protocol.
-                    raise errors.NotBranchError(path=transport.base)
-                if server_version != '2':
-                    raise errors.NotBranchError(path=transport.base)
-            from .remote import RemoteBzrDirFormat
-            return RemoteBzrDirFormat()
-
-    @classmethod
-    def known_formats(cls):
-        from .remote import RemoteBzrDirFormat
-        return {RemoteBzrDirFormat()}
-
-
 class BzrDirFormat(BzrFormat, controldir.ControlDirFormat):
     """ControlDirFormat base class for .bzr/ directories.
 
@@ -1745,12 +1682,6 @@
                                   __set_workingtree_format)
 
 
-# Register bzr formats
-BzrProber.formats.register(BzrDirMetaFormat1.get_format_string(),
-    BzrDirMetaFormat1)
-controldir.ControlDirFormat._default_format = BzrDirMetaFormat1()
-
-
 class BzrDirMetaFormat1Colo(BzrDirMetaFormat1):
     """BzrDirMeta1 format with support for colocated branches."""
 
@@ -1775,10 +1706,6 @@
         return BzrDirMeta1(transport, format)
 
 
-BzrProber.formats.register(BzrDirMetaFormat1Colo.get_format_string(),
-    BzrDirMetaFormat1Colo)
-
-
 class ConvertMetaToMeta(controldir.Converter):
     """Converts the components of metadirs."""
 
@@ -1892,9 +1819,6 @@
         return BzrDir.open_from_transport(to_convert.root_transport)
 
 
-controldir.ControlDirFormat.register_server_prober(RemoteBzrProber)
-
-
 class RepositoryAcquisitionPolicy(object):
     """Abstract base class for repository acquisition policies.
 
@@ -1928,7 +1852,7 @@
                 stack_on = urlutils.rebase_url(self._stack_on,
                     self._stack_on_pwd,
                     branch.user_url)
-            except errors.InvalidRebaseURLs:
+            except urlutils.InvalidRebaseURLs:
                 stack_on = self._get_full_stack_on()
         try:
             branch.set_stacked_on_url(stack_on)
@@ -2066,246 +1990,4 @@
         return self._repository, False
 
 
-def register_metadir(registry, key,
-         repository_format, help, native=True, deprecated=False,
-         branch_format=None,
-         tree_format=None,
-         hidden=False,
-         experimental=False,
-         alias=False, bzrdir_format=None):
-    """Register a metadir subformat.
-
-    These all use a meta bzrdir, but can be parameterized by the
-    Repository/Branch/WorkingTreeformats.
-
-    :param repository_format: The fully-qualified repository format class
-        name as a string.
-    :param branch_format: Fully-qualified branch format class name as
-        a string.
-    :param tree_format: Fully-qualified tree format class name as
-        a string.
-    """
-    if bzrdir_format is None:
-        bzrdir_format = BzrDirMetaFormat1
-    # This should be expanded to support setting WorkingTree and Branch
-    # formats, once the API supports that.
-    def _load(full_name):
-        mod_name, factory_name = full_name.rsplit('.', 1)
-        try:
-            factory = pyutils.get_named_object(mod_name, factory_name)
-        except ImportError as e:
-            raise ImportError('failed to load %s: %s' % (full_name, e))
-        except AttributeError:
-            raise AttributeError('no factory %s in module %r'
-                % (full_name, sys.modules[mod_name]))
-        return factory()
-
-    def helper():
-        bd = bzrdir_format()
-        if branch_format is not None:
-            bd.set_branch_format(_load(branch_format))
-        if tree_format is not None:
-            bd.workingtree_format = _load(tree_format)
-        if repository_format is not None:
-            bd.repository_format = _load(repository_format)
-        return bd
-    registry.register(key, helper, help, native, deprecated, hidden,
-        experimental, alias)
-
-register_metadir(controldir.format_registry, 'knit',
-    'breezy.bzr.knitrepo.RepositoryFormatKnit1',
-    'Format using knits.  Recommended for interoperation with bzr <= 0.14.',
-    branch_format='breezy.bzr.fullhistory.BzrBranchFormat5',
-    tree_format='breezy.bzr.workingtree_3.WorkingTreeFormat3',
-    hidden=True,
-    deprecated=True)
-register_metadir(controldir.format_registry, 'dirstate',
-    'breezy.bzr.knitrepo.RepositoryFormatKnit1',
-    help='Format using dirstate for working trees. '
-        'Compatible with bzr 0.8 and '
-        'above when accessed over the network. Introduced in bzr 0.15.',
-    branch_format='breezy.bzr.fullhistory.BzrBranchFormat5',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    hidden=True,
-    deprecated=True)
-register_metadir(controldir.format_registry, 'dirstate-tags',
-    'breezy.bzr.knitrepo.RepositoryFormatKnit1',
-    help='Variant of dirstate with support for tags. '
-        'Introduced in bzr 0.15.',
-    branch_format='breezy.bzr.branch.BzrBranchFormat6',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    hidden=True,
-    deprecated=True)
-register_metadir(controldir.format_registry, 'rich-root',
-    'breezy.bzr.knitrepo.RepositoryFormatKnit4',
-    help='Variant of dirstate with better handling of tree roots. '
-        'Introduced in bzr 1.0',
-    branch_format='breezy.bzr.branch.BzrBranchFormat6',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    hidden=True,
-    deprecated=True)
-register_metadir(controldir.format_registry, 'dirstate-with-subtree',
-    'breezy.bzr.knitrepo.RepositoryFormatKnit3',
-    help='Variant of dirstate with support for nested trees. '
-         'Introduced in 0.15.',
-    branch_format='breezy.bzr.branch.BzrBranchFormat6',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    experimental=True,
-    hidden=True,
-    )
-register_metadir(controldir.format_registry, 'pack-0.92',
-    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack1',
-    help='Pack-based format used in 1.x series. Introduced in 0.92. '
-        'Interoperates with bzr repositories before 0.92 but cannot be '
-        'read by bzr < 0.92. '
-        ,
-    branch_format='breezy.bzr.branch.BzrBranchFormat6',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    deprecated=True,
-    )
-register_metadir(controldir.format_registry, 'pack-0.92-subtree',
-    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack3',
-    help='Pack-based format used in 1.x series, with subtree support. '
-        'Introduced in 0.92. Interoperates with '
-        'bzr repositories before 0.92 but cannot be read by bzr < 0.92. '
-        ,
-    branch_format='breezy.bzr.branch.BzrBranchFormat6',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    hidden=True,
-    deprecated=True,
-    experimental=True,
-    )
-register_metadir(controldir.format_registry, 'rich-root-pack',
-    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack4',
-    help='A variant of pack-0.92 that supports rich-root data '
-         '(needed for bzr-svn and bzr-git). Introduced in 1.0.',
-    branch_format='breezy.bzr.branch.BzrBranchFormat6',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    hidden=True,
-    deprecated=True,
-    )
-register_metadir(controldir.format_registry, '1.6',
-    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack5',
-    help='A format that allows a branch to indicate that there is another '
-         '(stacked) repository that should be used to access data that is '
-         'not present locally.',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    hidden=True,
-    deprecated=True,
-    )
-register_metadir(controldir.format_registry, '1.6.1-rich-root',
-    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack5RichRoot',
-    help='A variant of 1.6 that supports rich-root data '
-         '(needed for bzr-svn and bzr-git).',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    hidden=True,
-    deprecated=True,
-    )
-register_metadir(controldir.format_registry, '1.9',
-    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack6',
-    help='A repository format using B+tree indexes. These indexes '
-         'are smaller in size, have smarter caching and provide faster '
-         'performance for most operations.',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    hidden=True,
-    deprecated=True,
-    )
-register_metadir(controldir.format_registry, '1.9-rich-root',
-    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack6RichRoot',
-    help='A variant of 1.9 that supports rich-root data '
-         '(needed for bzr-svn and bzr-git).',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat4',
-    hidden=True,
-    deprecated=True,
-    )
-register_metadir(controldir.format_registry, '1.14',
-    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack6',
-    help='A working-tree format that supports content filtering.',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat5',
-    hidden=True,
-    deprecated=True,
-    )
-register_metadir(controldir.format_registry, '1.14-rich-root',
-    'breezy.bzr.knitpack_repo.RepositoryFormatKnitPack6RichRoot',
-    help='A variant of 1.14 that supports rich-root data '
-         '(needed for bzr-svn and bzr-git).',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat5',
-    hidden=True,
-    deprecated=True,
-    )
-# The following un-numbered 'development' formats should always just be aliases.
-register_metadir(controldir.format_registry, 'development-subtree',
-    'breezy.bzr.groupcompress_repo.RepositoryFormat2aSubtree',
-    help='Current development format, subtree variant. Can convert data to and '
-        'from pack-0.92-subtree (and anything compatible with '
-        'pack-0.92-subtree) format repositories. Repositories and branches in '
-        'this format can only be read by bzr.dev. Please read '
-        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
-        'before use.',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',
-    experimental=True,
-    hidden=True,
-    alias=False, # Restore to being an alias when an actual development subtree format is added
-                 # This current non-alias status is simply because we did not introduce a
-                 # chk based subtree format.
-    )
-register_metadir(controldir.format_registry, 'development5-subtree',
-    'breezy.bzr.knitpack_repo.RepositoryFormatPackDevelopment2Subtree',
-    help='Development format, subtree variant. Can convert data to and '
-        'from pack-0.92-subtree (and anything compatible with '
-        'pack-0.92-subtree) format repositories. Repositories and branches in '
-        'this format can only be read by bzr.dev. Please read '
-        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
-        'before use.',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',
-    experimental=True,
-    hidden=True,
-    alias=False,
-    )
-
-register_metadir(controldir.format_registry, 'development-colo',
-    'breezy.bzr.groupcompress_repo.RepositoryFormat2a',
-    help='The 2a format with experimental support for colocated branches.\n',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',
-    experimental=True,
-    bzrdir_format=BzrDirMetaFormat1Colo,
-    )
-
-
-# And the development formats above will have aliased one of the following:
-
-# Finally, the current format.
-register_metadir(controldir.format_registry, '2a',
-    'breezy.bzr.groupcompress_repo.RepositoryFormat2a',
-    help='Format for the bzr 2.0 series.\n'
-        'Uses group-compress storage.\n'
-        'Provides rich roots which are a one-way transition.\n',
-        # 'storage in packs, 255-way hashed CHK inventory, bencode revision, group compress, '
-        # 'rich roots. Supported by bzr 1.16 and later.',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',
-    experimental=False,
-    )
-
-# The following format should be an alias for the rich root equivalent 
-# of the default format
-register_metadir(controldir.format_registry, 'default-rich-root',
-    'breezy.bzr.groupcompress_repo.RepositoryFormat2a',
-    branch_format='breezy.bzr.branch.BzrBranchFormat7',
-    tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',
-    alias=True,
-    hidden=True,
-    help='Same as 2a.')
-
-# The current format that is made on 'bzr init'.
-format_name = config.GlobalStack().get('default_format')
-controldir.format_registry.set_default(format_name)
+controldir.ControlDirFormat._default_format = BzrDirMetaFormat1()

=== modified file 'breezy/bzr/dirstate.py'
--- breezy/bzr/dirstate.py	2017-06-11 13:48:12 +0000
+++ breezy/bzr/dirstate.py	2017-06-14 23:29:06 +0000
@@ -2584,7 +2584,7 @@
             return
         if new_id.__class__ != bytes:
             raise AssertionError(
-                "must be a utf8 file_id not %s" % (type(file_id), ))
+                "must be a utf8 file_id not %s" % (type(new_id), ))
         # mark the old path absent, and insert a new root path
         self._make_absent(entry)
         self.update_minimal((b'', b'', new_id), b'd',

=== modified file 'breezy/bzr/fullhistory.py'
--- breezy/bzr/fullhistory.py	2017-06-10 16:40:42 +0000
+++ breezy/bzr/fullhistory.py	2017-06-14 23:29:06 +0000
@@ -43,7 +43,7 @@
 
     @needs_write_lock
     def set_last_revision_info(self, revno, revision_id):
-        if not revision_id or not isinstance(revision_id, basestring):
+        if not revision_id or not isinstance(revision_id, bytes):
             raise errors.InvalidRevisionId(revision_id=revision_id, branch=self)
         revision_id = _mod_revision.ensure_null(revision_id)
         # this old format stores the full history, but this api doesn't

=== modified file 'breezy/bzr/groupcompress_repo.py'
--- breezy/bzr/groupcompress_repo.py	2017-06-11 13:48:12 +0000
+++ breezy/bzr/groupcompress_repo.py	2017-06-22 01:06:22 +0000
@@ -1136,7 +1136,7 @@
             raise AssertionError()
         vf = self.revisions
         if revisions_iterator is None:
-            revisions_iterator = self._iter_revisions(None)
+            revisions_iterator = self.iter_revisions(self.all_revision_ids())
         for revid, revision in revisions_iterator:
             if revision is None:
                 pass

=== modified file 'breezy/bzr/inventory.py'
--- breezy/bzr/inventory.py	2017-06-11 13:48:12 +0000
+++ breezy/bzr/inventory.py	2017-06-14 23:29:06 +0000
@@ -53,6 +53,7 @@
 from ..sixish import (
     bytesintern,
     PY3,
+    text_type,
     viewitems,
     viewvalues,
     )
@@ -667,7 +668,7 @@
                 return
             from_dir = self.root
             yield '', self.root
-        elif isinstance(from_dir, basestring):
+        elif isinstance(from_dir, (str, text_type)):
             from_dir = self[from_dir]
 
         # unrolling the recursive called changed the time from
@@ -751,7 +752,7 @@
             if (specific_file_ids is None or yield_parents or
                 self.root.file_id in specific_file_ids):
                 yield u'', self.root
-        elif isinstance(from_dir, basestring):
+        elif isinstance(from_dir, (str, text_type)):
             from_dir = self[from_dir]
 
         if specific_file_ids is not None:
@@ -843,7 +844,7 @@
 
         Returns None IFF the path is not found.
         """
-        if isinstance(relpath, basestring):
+        if isinstance(relpath, (str, text_type)):
             names = osutils.splitpath(relpath)
         else:
             names = relpath
@@ -2094,7 +2095,7 @@
     def path2id(self, relpath):
         """See CommonInventory.path2id()."""
         # TODO: perhaps support negative hits?
-        if isinstance(relpath, basestring):
+        if isinstance(relpath, (str, text_type)):
             names = osutils.splitpath(relpath)
         else:
             names = relpath

=== modified file 'breezy/bzr/inventorytree.py'
--- breezy/bzr/inventorytree.py	2017-06-11 01:22:16 +0000
+++ breezy/bzr/inventorytree.py	2017-07-14 20:34:39 +0000
@@ -50,7 +50,11 @@
 from ..sixish import (
     viewvalues,
     )
-from ..tree import InterTree, Tree
+from ..tree import (
+    FileTimestampUnavailable,
+    InterTree,
+    Tree,
+    )
 
 
 class InventoryTree(Tree):
@@ -611,7 +615,7 @@
         try:
             revision = self._repository.get_revision(ie.revision)
         except errors.NoSuchRevision:
-            raise errors.FileTimestampUnavailable(self.id2path(file_id))
+            raise FileTimestampUnavailable(self.id2path(file_id))
         return revision.timestamp
 
     def get_file_size(self, file_id):

=== modified file 'breezy/bzr/knitpack_repo.py'
--- breezy/bzr/knitpack_repo.py	2017-06-10 16:40:42 +0000
+++ breezy/bzr/knitpack_repo.py	2017-06-13 02:57:49 +0000
@@ -690,11 +690,7 @@
             for result in self._do_copy_nodes_graph(index_map, writer,
                 write_index, output_lines, pb, readv_group_iter, total_items):
                 yield result
-        except Exception:
-            # Python 2.4 does not permit try:finally: in a generator.
-            pb.finished()
-            raise
-        else:
+        finally:
             pb.finished()
 
     def _do_copy_nodes_graph(self, index_map, writer, write_index,

=== modified file 'breezy/bzr/remote.py'
--- breezy/bzr/remote.py	2017-06-11 20:15:04 +0000
+++ breezy/bzr/remote.py	2017-06-22 01:06:22 +0000
@@ -22,6 +22,7 @@
 from .. import (
     bencode,
     branch,
+    bzr as _mod_bzr,
     config as _mod_config,
     controldir,
     debug,
@@ -490,7 +491,7 @@
                 warning('VFS BzrDir access triggered\n%s',
                     ''.join(traceback.format_stack()))
             self._real_bzrdir = _mod_bzrdir.BzrDir.open_from_transport(
-                self.root_transport, probers=[_mod_bzrdir.BzrProber])
+                self.root_transport, probers=[_mod_bzr.BzrProber])
             self._format._network_name = \
                 self._real_bzrdir._format.network_name()
 
@@ -2536,7 +2537,7 @@
                 mode='r|bz2')
             tmpdir = osutils.mkdtemp()
             try:
-                _extract_tar(tar, tmpdir)
+                tar.extractall(tmpdir)
                 tmp_bzrdir = _mod_bzrdir.BzrDir.open(tmpdir)
                 tmp_repo = tmp_bzrdir.open_repository()
                 tmp_repo.copy_content_into(destination, revision_id)
@@ -2657,42 +2658,29 @@
             yield serializer.read_revision_from_string("".join(chunks))
 
     @needs_read_lock
-    def get_revisions(self, revision_ids):
-        if revision_ids is None:
-            revision_ids = self.all_revision_ids()
-        else:
-            for rev_id in revision_ids:
-                if not rev_id or not isinstance(rev_id, basestring):
-                    raise errors.InvalidRevisionId(
-                        revision_id=rev_id, branch=self)
+    def iter_revisions(self, revision_ids):
+        for rev_id in revision_ids:
+            if not rev_id or not isinstance(rev_id, bytes):
+                raise errors.InvalidRevisionId(
+                    revision_id=rev_id, branch=self)
         try:
             missing = set(revision_ids)
-            revs = {}
             for rev in self._iter_revisions_rpc(revision_ids):
                 missing.remove(rev.revision_id)
-                revs[rev.revision_id] = rev
+                yield (rev.revision_id, rev)
+            for fallback in self._fallback_repositories:
+                if not missing:
+                    break
+                for (revid, rev) in fallback.iter_revisions(missing):
+                    if rev is not None:
+                        yield (revid, rev)
+                        missing.remove(revid)
+            for revid in missing:
+                yield (revid, None)
         except errors.UnknownSmartMethod:
             self._ensure_real()
-            return self._real_repository.get_revisions(revision_ids)
-        for fallback in self._fallback_repositories:
-            if not missing:
-                break
-            for revid in list(missing):
-                # XXX JRV 2011-11-20: It would be nice if there was a
-                # public method on Repository that could be used to query
-                # for revision objects *without* failing completely if one
-                # was missing. There is VersionedFileRepository._iter_revisions,
-                # but unfortunately that's private and not provided by
-                # all repository implementations.
-                try:
-                    revs[revid] = fallback.get_revision(revid)
-                except errors.NoSuchRevision:
-                    pass
-                else:
-                    missing.remove(revid)
-        if missing:
-            raise errors.NoSuchRevision(self, list(missing)[0])
-        return [revs[revid] for revid in revision_ids]
+            for entry in self._real_repository.iter_revisions(revision_ids):
+                yield entry
 
     def supports_rich_root(self):
         return self._format.rich_root_data
@@ -3893,7 +3881,7 @@
         # XXX: These should be returned by the set_last_revision_info verb
         old_revno, old_revid = self.last_revision_info()
         self._run_pre_change_branch_tip_hooks(revno, revision_id)
-        if not revision_id or not isinstance(revision_id, basestring):
+        if not revision_id or not isinstance(revision_id, bytes):
             raise errors.InvalidRevisionId(revision_id=revision_id, branch=self)
         try:
             response = self._call('Branch.set_last_revision_info',
@@ -4130,15 +4118,6 @@
         return self._bzrdir._real_bzrdir
 
 
-def _extract_tar(tar, to_dir):
-    """Extract all the contents of a tarfile object.
-
-    A replacement for extractall, which is not present in python2.4
-    """
-    for tarinfo in tar:
-        tar.extract(tarinfo, to_dir)
-
-
 error_translators = registry.Registry()
 no_context_error_translators = registry.Registry()
 

=== modified file 'breezy/bzr/smart/bzrdir.py'
--- breezy/bzr/smart/bzrdir.py	2017-06-11 20:15:04 +0000
+++ breezy/bzr/smart/bzrdir.py	2017-06-14 22:34:05 +0000
@@ -25,10 +25,12 @@
     repository,
     urlutils,
     )
+from .. import (
+    BzrProber,
+    )
 from ..bzrdir import (
     BzrDir,
     BzrDirFormat,
-    BzrProber,
     )
 from ...controldir import (
     network_format_registry,

=== modified file 'breezy/bzr/smart/server.py'
--- breezy/bzr/smart/server.py	2017-06-11 20:15:04 +0000
+++ breezy/bzr/smart/server.py	2017-07-15 13:23:08 +0000
@@ -372,7 +372,7 @@
             base_url = base_url[len('readonly+'):]
         try:
             return urlutils.local_path_from_url(base_url)
-        except errors.InvalidURL:
+        except urlutils.InvalidURL:
             return None
 
 

=== modified file 'breezy/bzr/versionedfile.py'
--- breezy/bzr/versionedfile.py	2017-06-11 13:48:12 +0000
+++ breezy/bzr/versionedfile.py	2017-06-14 23:29:06 +0000
@@ -603,8 +603,6 @@
 
         Must raise RevisionNotPresent if any of the given versions are
         not present in file history."""
-        if isinstance(version_ids, basestring):
-            version_ids = [version_ids]
         raise NotImplementedError(self.get_ancestry)
 
     def get_ancestry_with_ghosts(self, version_ids):

=== modified file 'breezy/bzr/vf_repository.py'
--- breezy/bzr/vf_repository.py	2017-06-11 01:22:16 +0000
+++ breezy/bzr/vf_repository.py	2017-06-22 01:06:22 +0000
@@ -112,9 +112,6 @@
     """Commit builder implementation for versioned files based repositories.
     """
 
-    # this commit builder supports the record_entry_contents interface
-    supports_record_entry_contents = True
-
     # the default CommitBuilder does not manage trees whose root is versioned.
     _versioned_root = False
 
@@ -129,25 +126,11 @@
         except IndexError:
             basis_id = _mod_revision.NULL_REVISION
         self.basis_delta_revision = basis_id
-        self.new_inventory = Inventory(None)
+        self._new_inventory = None
         self._basis_delta = []
         self.__heads = graph.HeadsCache(repository.get_graph()).heads
         # memo'd check for no-op commits.
         self._any_changes = False
-        # API compatibility, older code that used CommitBuilder did not call
-        # .record_delete(), which means the delta that is computed would not be
-        # valid. Callers that will call record_delete() should call
-        # .will_record_deletes() to indicate that.
-        self._recording_deletes = False
-
-    def will_record_deletes(self):
-        """Tell the commit builder that deletes are being notified.
-
-        This enables the accumulation of an inventory delta; for the resulting
-        commit to be valid, deletes against the basis MUST be recorded via
-        builder.record_delete().
-        """
-        self._recording_deletes = True
 
     def any_changes(self):
         """Return True if any entries were changed.
@@ -234,11 +217,11 @@
         require deserializing the inventory, while we already have a copy in
         memory.
         """
-        if self.new_inventory is None:
-            self.new_inventory = self.repository.get_inventory(
+        if self._new_inventory is None:
+            self._new_inventory = self.repository.get_inventory(
                 self._new_revision_id)
         return inventorytree.InventoryRevisionTree(self.repository,
-            self.new_inventory, self._new_revision_id)
+            self._new_inventory, self._new_revision_id)
 
     def finish_inventory(self):
         """Tell the builder that the inventory is finished.
@@ -246,42 +229,14 @@
         :return: The inventory id in the repository, which can be used with
             repository.get_inventory.
         """
-        if self.new_inventory is None:
-            # an inventory delta was accumulated without creating a new
-            # inventory.
-            basis_id = self.basis_delta_revision
-            # We ignore the 'inventory' returned by add_inventory_by_delta
-            # because self.new_inventory is used to hint to the rest of the
-            # system what code path was taken
-            self.inv_sha1, _ = self.repository.add_inventory_by_delta(
-                basis_id, self._basis_delta, self._new_revision_id,
-                self.parents)
-        else:
-            if self.new_inventory.root is None:
-                raise AssertionError('Root entry should be supplied to'
-                    ' record_entry_contents, as of bzr 0.10.')
-                self.new_inventory.add(InventoryDirectory(ROOT_ID, '', None))
-            self.new_inventory.revision_id = self._new_revision_id
-            self.inv_sha1 = self.repository.add_inventory(
-                self._new_revision_id,
-                self.new_inventory,
-                self.parents
-                )
+        # an inventory delta was accumulated without creating a new
+        # inventory.
+        basis_id = self.basis_delta_revision
+        self.inv_sha1, self._new_inventory = self.repository.add_inventory_by_delta(
+            basis_id, self._basis_delta, self._new_revision_id,
+            self.parents)
         return self._new_revision_id
 
-    def _check_root(self, ie, parent_invs, tree):
-        """Helper for record_entry_contents.
-
-        :param ie: An entry being added.
-        :param parent_invs: The inventories of the parent revisions of the
-            commit.
-        :param tree: The tree that is being committed.
-        """
-        # In this revision format, root entries have no knit or weave When
-        # serializing out to disk and back in root.revision is always
-        # _new_revision_id
-        ie.revision = self._new_revision_id
-
     def _require_root_change(self, tree):
         """Enforce an appropriate root object change.
 
@@ -325,249 +280,11 @@
     def get_basis_delta(self):
         """Return the complete inventory delta versus the basis inventory.
 
-        This has been built up with the calls to record_delete and
-        record_entry_contents. The client must have already called
-        will_record_deletes() to indicate that they will be generating a
-        complete delta.
-
         :return: An inventory delta, suitable for use with apply_delta, or
             Repository.add_inventory_by_delta, etc.
         """
-        if not self._recording_deletes:
-            raise AssertionError("recording deletes not activated.")
         return self._basis_delta
 
-    def record_delete(self, path, file_id):
-        """Record that a delete occured against a basis tree.
-
-        This is an optional API - when used it adds items to the basis_delta
-        being accumulated by the commit builder. It cannot be called unless the
-        method will_record_deletes() has been called to inform the builder that
-        a delta is being supplied.
-
-        :param path: The path of the thing deleted.
-        :param file_id: The file id that was deleted.
-        """
-        if not self._recording_deletes:
-            raise AssertionError("recording deletes not activated.")
-        delta = (path, None, file_id, None)
-        self._basis_delta.append(delta)
-        self._any_changes = True
-        return delta
-
-    def record_entry_contents(self, ie, parent_invs, path, tree,
-        content_summary):
-        """Record the content of ie from tree into the commit if needed.
-
-        Side effect: sets ie.revision when unchanged
-
-        :param ie: An inventory entry present in the commit.
-        :param parent_invs: The inventories of the parent revisions of the
-            commit.
-        :param path: The path the entry is at in the tree.
-        :param tree: The tree which contains this entry and should be used to
-            obtain content.
-        :param content_summary: Summary data from the tree about the paths
-            content - stat, length, exec, sha/link target. This is only
-            accessed when the entry has a revision of None - that is when it is
-            a candidate to commit.
-        :return: A tuple (change_delta, version_recorded, fs_hash).
-            change_delta is an inventory_delta change for this entry against
-            the basis tree of the commit, or None if no change occured against
-            the basis tree.
-            version_recorded is True if a new version of the entry has been
-            recorded. For instance, committing a merge where a file was only
-            changed on the other side will return (delta, False).
-            fs_hash is either None, or the hash details for the path (currently
-            a tuple of the contents sha1 and the statvalue returned by
-            tree.get_file_with_stat()).
-        """
-        if self.new_inventory.root is None:
-            if ie.parent_id is not None:
-                raise errors.RootMissing()
-            self._check_root(ie, parent_invs, tree)
-        if ie.revision is None:
-            kind = content_summary[0]
-        else:
-            # ie is carried over from a prior commit
-            kind = ie.kind
-        # XXX: repository specific check for nested tree support goes here - if
-        # the repo doesn't want nested trees we skip it ?
-        if (kind == 'tree-reference' and
-            not self.repository._format.supports_tree_reference):
-            # mismatch between commit builder logic and repository:
-            # this needs the entry creation pushed down into the builder.
-            raise NotImplementedError('Missing repository subtree support.')
-        self.new_inventory.add(ie)
-
-        # TODO: slow, take it out of the inner loop.
-        try:
-            basis_inv = parent_invs[0]
-        except IndexError:
-            basis_inv = Inventory(root_id=None)
-
-        # ie.revision is always None if the InventoryEntry is considered
-        # for committing. We may record the previous parents revision if the
-        # content is actually unchanged against a sole head.
-        if ie.revision is not None:
-            if not self._versioned_root and path == '':
-                # repositories that do not version the root set the root's
-                # revision to the new commit even when no change occurs (more
-                # specifically, they do not record a revision on the root; and
-                # the rev id is assigned to the root during deserialisation -
-                # this masks when a change may have occurred against the basis.
-                # To match this we always issue a delta, because the revision
-                # of the root will always be changing.
-                if basis_inv.has_id(ie.file_id):
-                    delta = (basis_inv.id2path(ie.file_id), path,
-                        ie.file_id, ie)
-                else:
-                    # add
-                    delta = (None, path, ie.file_id, ie)
-                self._basis_delta.append(delta)
-                return delta, False, None
-            else:
-                # we don't need to commit this, because the caller already
-                # determined that an existing revision of this file is
-                # appropriate. If it's not being considered for committing then
-                # it and all its parents to the root must be unaltered so
-                # no-change against the basis.
-                if ie.revision == self._new_revision_id:
-                    raise AssertionError("Impossible situation, a skipped "
-                        "inventory entry (%r) claims to be modified in this "
-                        "commit (%r).", (ie, self._new_revision_id))
-                return None, False, None
-        # XXX: Friction: parent_candidates should return a list not a dict
-        #      so that we don't have to walk the inventories again.
-        parent_candidate_entries = ie.parent_candidates(parent_invs)
-        head_set = self._heads(ie.file_id, parent_candidate_entries)
-        heads = []
-        for inv in parent_invs:
-            if inv.has_id(ie.file_id):
-                old_rev = inv[ie.file_id].revision
-                if old_rev in head_set:
-                    heads.append(inv[ie.file_id].revision)
-                    head_set.remove(inv[ie.file_id].revision)
-
-        store = False
-        # now we check to see if we need to write a new record to the
-        # file-graph.
-        # We write a new entry unless there is one head to the ancestors, and
-        # the kind-derived content is unchanged.
-
-        # Cheapest check first: no ancestors, or more the one head in the
-        # ancestors, we write a new node.
-        if len(heads) != 1:
-            store = True
-        if not store:
-            # There is a single head, look it up for comparison
-            parent_entry = parent_candidate_entries[heads[0]]
-            # if the non-content specific data has changed, we'll be writing a
-            # node:
-            if (parent_entry.parent_id != ie.parent_id or
-                parent_entry.name != ie.name):
-                store = True
-        # now we need to do content specific checks:
-        if not store:
-            # if the kind changed the content obviously has
-            if kind != parent_entry.kind:
-                store = True
-        # Stat cache fingerprint feedback for the caller - None as we usually
-        # don't generate one.
-        fingerprint = None
-        if kind == 'file':
-            if content_summary[2] is None:
-                raise ValueError("Files must not have executable = None")
-            if not store:
-                # We can't trust a check of the file length because of content
-                # filtering...
-                if (# if the exec bit has changed we have to store:
-                    parent_entry.executable != content_summary[2]):
-                    store = True
-                elif parent_entry.text_sha1 == content_summary[3]:
-                    # all meta and content is unchanged (using a hash cache
-                    # hit to check the sha)
-                    ie.revision = parent_entry.revision
-                    ie.text_size = parent_entry.text_size
-                    ie.text_sha1 = parent_entry.text_sha1
-                    ie.executable = parent_entry.executable
-                    return self._get_delta(ie, basis_inv, path), False, None
-                else:
-                    # Either there is only a hash change(no hash cache entry,
-                    # or same size content change), or there is no change on
-                    # this file at all.
-                    # Provide the parent's hash to the store layer, so that the
-                    # content is unchanged we will not store a new node.
-                    nostore_sha = parent_entry.text_sha1
-            if store:
-                # We want to record a new node regardless of the presence or
-                # absence of a content change in the file.
-                nostore_sha = None
-            ie.executable = content_summary[2]
-            file_obj, stat_value = tree.get_file_with_stat(ie.file_id, path)
-            try:
-                text = file_obj.read()
-            finally:
-                file_obj.close()
-            try:
-                ie.text_sha1, ie.text_size = self._add_text_to_weave(
-                    ie.file_id, text, heads, nostore_sha)
-                # Let the caller know we generated a stat fingerprint.
-                fingerprint = (ie.text_sha1, stat_value)
-            except errors.ExistingContent:
-                # Turns out that the file content was unchanged, and we were
-                # only going to store a new node if it was changed. Carry over
-                # the entry.
-                ie.revision = parent_entry.revision
-                ie.text_size = parent_entry.text_size
-                ie.text_sha1 = parent_entry.text_sha1
-                ie.executable = parent_entry.executable
-                return self._get_delta(ie, basis_inv, path), False, None
-        elif kind == 'directory':
-            if not store:
-                # all data is meta here, nothing specific to directory, so
-                # carry over:
-                ie.revision = parent_entry.revision
-                return self._get_delta(ie, basis_inv, path), False, None
-            self._add_text_to_weave(ie.file_id, '', heads, None)
-        elif kind == 'symlink':
-            current_link_target = content_summary[3]
-            if not store:
-                # symlink target is not generic metadata, check if it has
-                # changed.
-                if current_link_target != parent_entry.symlink_target:
-                    store = True
-            if not store:
-                # unchanged, carry over.
-                ie.revision = parent_entry.revision
-                ie.symlink_target = parent_entry.symlink_target
-                return self._get_delta(ie, basis_inv, path), False, None
-            ie.symlink_target = current_link_target
-            self._add_text_to_weave(ie.file_id, '', heads, None)
-        elif kind == 'tree-reference':
-            if not store:
-                if content_summary[3] != parent_entry.reference_revision:
-                    store = True
-            if not store:
-                # unchanged, carry over.
-                ie.reference_revision = parent_entry.reference_revision
-                ie.revision = parent_entry.revision
-                return self._get_delta(ie, basis_inv, path), False, None
-            ie.reference_revision = content_summary[3]
-            if ie.reference_revision is None:
-                raise AssertionError("invalid content_summary for nested tree: %r"
-                    % (content_summary,))
-            self._add_text_to_weave(ie.file_id, '', heads, None)
-        else:
-            raise NotImplementedError('unknown kind')
-        ie.revision = self._new_revision_id
-        # The initial commit adds a root directory, but this in itself is not
-        # a worthwhile commit.
-        if (self.basis_delta_revision != _mod_revision.NULL_REVISION or
-            path != ""):
-            self._any_changes = True
-        return self._get_delta(ie, basis_inv, path), True, fingerprint
-
     def record_iter_changes(self, tree, basis_revision_id, iter_changes,
         _entry_factory=entry_factory):
         """Record a new tree via iter_changes.
@@ -827,7 +544,6 @@
             inv_delta.append((change[1][0], new_path, change[0], entry))
             if new_path == '':
                 seen_root = True
-        self.new_inventory = None
         # The initial commit adds a root directory, but this in itself is not
         # a worthwhile commit.
         if ((len(inv_delta) > 0 and basis_revision_id != _mod_revision.NULL_REVISION) or
@@ -854,15 +570,6 @@
     # the root entry gets versioned properly by this builder.
     _versioned_root = True
 
-    def _check_root(self, ie, parent_invs, tree):
-        """Helper for record_entry_contents.
-
-        :param ie: An entry being added.
-        :param parent_invs: The inventories of the parent revisions of the
-            commit.
-        :param tree: The tree that is being committed.
-        """
-
     def _require_root_change(self, tree):
         """Enforce an appropriate root object change.
 
@@ -1395,28 +1102,9 @@
         be used by reconcile, or reconcile-alike commands that are correcting
         or testing the revision graph.
         """
-        return self._get_revisions([revision_id])[0]
-
-    @needs_read_lock
-    def get_revisions(self, revision_ids):
-        """Get many revisions at once.
-        
-        Repositories that need to check data on every revision read should 
-        subclass this method.
-        """
-        return self._get_revisions(revision_ids)
-
-    @needs_read_lock
-    def _get_revisions(self, revision_ids):
-        """Core work logic to get many revisions without sanity checks."""
-        revs = {}
-        for revid, rev in self._iter_revisions(revision_ids):
-            if rev is None:
-                raise errors.NoSuchRevision(self, revid)
-            revs[revid] = rev
-        return [revs[revid] for revid in revision_ids]
-
-    def _iter_revisions(self, revision_ids):
+        return self.get_revisions([revision_id])[0]
+
+    def iter_revisions(self, revision_ids):
         """Iterate over revision objects.
 
         :param revision_ids: An iterable of revisions to examine. None may be
@@ -1426,22 +1114,23 @@
         :return: An iterator of (revid, revision) tuples. Absent revisions (
             those asked for but not available) are returned as (revid, None).
         """
-        if revision_ids is None:
-            revision_ids = self.all_revision_ids()
-        else:
+        self.lock_read()
+        try:
             for rev_id in revision_ids:
-                if not rev_id or not isinstance(rev_id, basestring):
+                if not rev_id or not isinstance(rev_id, bytes):
                     raise errors.InvalidRevisionId(revision_id=rev_id, branch=self)
-        keys = [(key,) for key in revision_ids]
-        stream = self.revisions.get_record_stream(keys, 'unordered', True)
-        for record in stream:
-            revid = record.key[0]
-            if record.storage_kind == 'absent':
-                yield (revid, None)
-            else:
-                text = record.get_bytes_as('fulltext')
-                rev = self._serializer.read_revision_from_string(text)
-                yield (revid, rev)
+            keys = [(key,) for key in revision_ids]
+            stream = self.revisions.get_record_stream(keys, 'unordered', True)
+            for record in stream:
+                revid = record.key[0]
+                if record.storage_kind == 'absent':
+                    yield (revid, None)
+                else:
+                    text = record.get_bytes_as('fulltext')
+                    rev = self._serializer.read_revision_from_string(text)
+                    yield (revid, rev)
+        finally:
+            self.unlock()
 
     @needs_write_lock
     def add_signature_text(self, revision_id, signature):
@@ -1973,7 +1662,7 @@
             raise AssertionError()
         vf = self.revisions
         if revisions_iterator is None:
-            revisions_iterator = self._iter_revisions(None)
+            revisions_iterator = self.iter_revisions(self.all_revision_ids())
         for revid, revision in revisions_iterator:
             if revision is None:
                 pass

=== modified file 'breezy/bzr/weave.py'
--- breezy/bzr/weave.py	2017-06-08 23:30:31 +0000
+++ breezy/bzr/weave.py	2017-07-10 22:36:09 +0000
@@ -77,11 +77,11 @@
     errors,
     osutils,
     )
-from ..errors import (WeaveError, WeaveFormatError, WeaveParentMismatch,
-        RevisionAlreadyPresent,
-        RevisionNotPresent,
-        UnavailableRepresentation,
-        )
+from ..errors import (
+    RevisionAlreadyPresent,
+    RevisionNotPresent,
+    UnavailableRepresentation,
+    )
 from ..osutils import dirname, sha, sha_strings, split_lines
 from .. import patiencediff
 from ..revision import NULL_REVISION
@@ -99,6 +99,79 @@
 from .weavefile import _read_weave_v5, write_weave_v5
 
 
+class WeaveError(errors.BzrError):
+
+    _fmt = "Error in processing weave: %(msg)s"
+
+    def __init__(self, msg=None):
+        errors.BzrError.__init__(self)
+        self.msg = msg
+
+
+class WeaveRevisionAlreadyPresent(WeaveError):
+
+    _fmt = "Revision {%(revision_id)s} already present in %(weave)s"
+
+    def __init__(self, revision_id, weave):
+
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave = weave
+
+
+class WeaveRevisionNotPresent(WeaveError):
+
+    _fmt = "Revision {%(revision_id)s} not present in %(weave)s"
+
+    def __init__(self, revision_id, weave):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave = weave
+
+
+class WeaveFormatError(WeaveError):
+
+    _fmt = "Weave invariant violated: %(what)s"
+
+    def __init__(self, what):
+        WeaveError.__init__(self)
+        self.what = what
+
+
+class WeaveParentMismatch(WeaveError):
+
+    _fmt = "Parents are mismatched between two revisions. %(msg)s"
+
+
+class WeaveInvalidChecksum(WeaveError):
+
+    _fmt = "Text did not match its checksum: %(msg)s"
+
+
+class WeaveTextDiffers(WeaveError):
+
+    _fmt = ("Weaves differ on text content. Revision:"
+            " {%(revision_id)s}, %(weave_a)s, %(weave_b)s")
+
+    def __init__(self, revision_id, weave_a, weave_b):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave_a = weave_a
+        self.weave_b = weave_b
+
+
+class WeaveTextDiffers(WeaveError):
+
+    _fmt = ("Weaves differ on text content. Revision:"
+            " {%(revision_id)s}, %(weave_a)s, %(weave_b)s")
+
+    def __init__(self, revision_id, weave_a, weave_b):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave_a = weave_a
+        self.weave_b = weave_b
+
+
 class WeaveContentFactory(ContentFactory):
     """Content factory for streaming from weaves.
 
@@ -528,22 +601,11 @@
 
     def get_ancestry(self, version_ids, topo_sorted=True):
         """See VersionedFile.get_ancestry."""
-        if isinstance(version_ids, basestring):
+        if isinstance(version_ids, bytes):
             version_ids = [version_ids]
         i = self._inclusions([self._lookup(v) for v in version_ids])
         return [self._idx_to_name(v) for v in i]
 
-    def _check_lines(self, text):
-        if not isinstance(text, list):
-            raise ValueError("text should be a list, not %s" % type(text))
-
-        for l in text:
-            if not isinstance(l, basestring):
-                raise ValueError("text line should be a string or unicode, not %s"
-                                 % type(l))
-
-
-
     def _check_versions(self, indexes):
         """Check everything in the sequence of indexes is valid"""
         for i in indexes:
@@ -762,7 +824,7 @@
         expected_sha1 = self._sha1s[int_index]
         measured_sha1 = sha_strings(result)
         if measured_sha1 != expected_sha1:
-            raise errors.WeaveInvalidChecksum(
+            raise WeaveInvalidChecksum(
                     'file %s, revision %s, expected: %s, measured %s'
                     % (self._weave_name, version_id,
                        expected_sha1, measured_sha1))
@@ -839,7 +901,7 @@
             hd = sha1s[version].hexdigest()
             expected = self._sha1s[i]
             if hd != expected:
-                raise errors.WeaveInvalidChecksum(
+                raise WeaveInvalidChecksum(
                         "mismatched sha1 for version %s: "
                         "got %s, expected %s"
                         % (version, hd, expected))
@@ -875,7 +937,7 @@
         this_idx = self._name_map.get(name, -1)
         if this_idx != -1:
             if self._sha1s[this_idx] != other._sha1s[other_idx]:
-                raise errors.WeaveTextDiffers(name, self, other)
+                raise WeaveTextDiffers(name, self, other)
             self_parents = self._parents[this_idx]
             other_parents = other._parents[other_idx]
             n1 = {self._names[i] for i in self_parents}
@@ -1010,7 +1072,7 @@
                     lines = list(difflib.unified_diff(lines, lines_b,
                             wa._weave_name, wb._weave_name))
                     mutter('lines:\n%s', ''.join(lines))
-                    raise errors.WeaveTextDiffers(name, wa, wb)
+                    raise WeaveTextDiffers(name, wa, wb)
         else:
             lines = wb.get_lines(name)
         wr._add(name, lines, [wr._lookup(i) for i in combined_parents[name]])

=== modified file 'breezy/bzr/workingtree.py'
--- breezy/bzr/workingtree.py	2017-06-11 13:48:12 +0000
+++ breezy/bzr/workingtree.py	2017-07-14 20:34:39 +0000
@@ -30,9 +30,9 @@
 """
 
 
-
 from __future__ import absolute_import
 
+import breezy
 import collections
 import errno
 import os
@@ -48,7 +48,9 @@
     cache_utf8,
     conflicts as _mod_conflicts,
     errors,
+    globbing,
     graph as _mod_graph,
+    ignores,
     osutils,
     revision as _mod_revision,
     revisiontree,
@@ -68,8 +70,10 @@
 from .inventorytree import InventoryRevisionTree, MutableInventoryTree
 from ..sixish import (
     BytesIO,
+    text_type,
     )
 from ..trace import mutter
+from ..tree import FileTimestampUnavailable
 from ..workingtree import (
     TreeDirectory,
     TreeFile,
@@ -393,6 +397,50 @@
         finally:
             confile.close()
 
+    def get_ignore_list(self):
+        """Return list of ignore patterns.
+
+        Cached in the Tree object after the first call.
+        """
+        ignoreset = getattr(self, '_ignoreset', None)
+        if ignoreset is not None:
+            return ignoreset
+
+        ignore_globs = set()
+        ignore_globs.update(ignores.get_runtime_ignores())
+        ignore_globs.update(ignores.get_user_ignores())
+        if self.has_filename(breezy.IGNORE_FILENAME):
+            f = self.get_file_byname(breezy.IGNORE_FILENAME)
+            try:
+                ignore_globs.update(ignores.parse_ignore_file(f))
+            finally:
+                f.close()
+        self._ignoreset = ignore_globs
+        return ignore_globs
+
+    def _cleanup(self):
+        self._flush_ignore_list_cache()
+
+    def _flush_ignore_list_cache(self):
+        """Resets the cached ignore list to force a cache rebuild."""
+        self._ignoreset = None
+        self._ignoreglobster = None
+
+    def is_ignored(self, filename):
+        r"""Check whether the filename matches an ignore pattern.
+
+        Patterns containing '/' or '\' need to match the whole path;
+        others match against only the last component.  Patterns starting
+        with '!' are ignore exceptions.  Exceptions take precedence
+        over regular patterns and cause the filename to not be ignored.
+
+        If the file is ignored, returns the pattern which caused it to
+        be ignored, otherwise None.  So this can simply be used as a
+        boolean if desired."""
+        if getattr(self, '_ignoreglobster', None) is None:
+            self._ignoreglobster = globbing.ExceptionGlobster(self.get_ignore_list())
+        return self._ignoreglobster.match(filename)
+
     def read_basis_inventory(self):
         """Read the cached basis inventory."""
         path = self._basis_inventory_name()
@@ -532,7 +580,7 @@
             return os.lstat(self.abspath(path)).st_mtime
         except OSError as e:
             if e.errno == errno.ENOENT:
-                raise errors.FileTimestampUnavailable(path)
+                raise FileTimestampUnavailable(path)
             raise
 
     def _is_executable_from_path_and_stat_from_basis(self, path, stat_result):
@@ -959,7 +1007,7 @@
         if to_dir is None:
             raise TypeError('You must supply a target directory')
         # check destination directory
-        if isinstance(from_paths, basestring):
+        if isinstance(from_paths, (str, text_type)):
             raise ValueError()
         to_abs = self.abspath(to_dir)
         if not osutils.isdir(to_abs):

=== modified file 'breezy/bzr/workingtree_4.py'
--- breezy/bzr/workingtree_4.py	2017-06-11 13:48:12 +0000
+++ breezy/bzr/workingtree_4.py	2017-07-14 20:34:39 +0000
@@ -81,6 +81,7 @@
     )
 from ..transport.local import LocalTransport
 from ..tree import (
+    FileTimestampUnavailable,
     InterTree,
     )
 from ..workingtree import (
@@ -135,7 +136,7 @@
         """See MutableTree._add."""
         state = self.current_dirstate()
         for f, file_id, kind in zip(files, ids, kinds):
-            f = f.strip(b'/')
+            f = f.strip(u'/')
             if self.path2id(f):
                 # special case tree root handling.
                 if f == b'' and self.path2id(f) == ROOT_ID:
@@ -664,7 +665,7 @@
         if not from_paths:
             return result
         state = self.current_dirstate()
-        if isinstance(from_paths, basestring):
+        if isinstance(from_paths, (str, bytes)):
             raise ValueError()
         to_dir_utf8 = to_dir.encode('utf8')
         to_entry_dirname, to_basename = os.path.split(to_dir_utf8)
@@ -1868,7 +1869,7 @@
         try:
             rev = self._repository.get_revision(last_changed_revision)
         except errors.NoSuchRevision:
-            raise errors.FileTimestampUnavailable(self.id2path(file_id))
+            raise FileTimestampUnavailable(self.id2path(file_id))
         return rev.timestamp
 
     def get_file_sha1(self, file_id, path=None, stat_value=None):

=== modified file 'breezy/check.py'
--- breezy/check.py	2017-06-10 00:52:08 +0000
+++ breezy/check.py	2017-06-22 01:06:22 +0000
@@ -181,7 +181,8 @@
 
     def check_revisions(self):
         """Scan revisions, checking data directly available as we go."""
-        revision_iterator = self.repository._iter_revisions(None)
+        revision_iterator = self.repository.iter_revisions(
+            self.repository.all_revision_ids())
         revision_iterator = self._check_revisions(revision_iterator)
         # We read the all revisions here:
         # - doing this allows later code to depend on the revision index.

=== modified file 'breezy/chunk_writer.py'
--- breezy/chunk_writer.py	2011-12-19 13:23:58 +0000
+++ breezy/chunk_writer.py	2017-07-02 13:38:15 +0000
@@ -144,7 +144,7 @@
                                  % (self.bytes_out_len, self.chunk_size))
         nulls_needed = self.chunk_size - self.bytes_out_len
         if nulls_needed:
-            self.bytes_list.append("\x00" * nulls_needed)
+            self.bytes_list.append(b"\x00" * nulls_needed)
         return self.bytes_list, self.unused_bytes, nulls_needed
 
     def set_optimize(self, for_size=True):

=== modified file 'breezy/commands.py'
--- breezy/commands.py	2017-06-05 20:48:31 +0000
+++ breezy/commands.py	2017-07-20 00:00:04 +0000
@@ -37,7 +37,6 @@
     cleanup,
     cmdline,
     debug,
-    errors,
     i18n,
     option,
     osutils,
@@ -51,12 +50,17 @@
 # Compatibility - Option used to be in commands.
 from .option import Option
 from .plugin import disable_plugins, load_plugins, plugin_name
-from . import registry
+from . import errors, registry
 from .sixish import (
     string_types,
     )
 
 
+class BzrOptionError(errors.BzrCommandError):
+
+    _fmt = "Error in command line options"
+
+
 class CommandInfo(object):
     """Information about a command."""
 
@@ -218,6 +222,52 @@
     return plugin_cmds.keys()
 
 
+# Overrides for common mispellings that heuristics get wrong
+_GUESS_OVERRIDES = {
+    'ic': {'ci': 0}, # heuristic finds nick
+    }
+
+
+def guess_command(cmd_name):
+    """Guess what command a user typoed.
+
+    :param cmd_name: Command to search for
+    :return: None if no command was found, name of a command otherwise
+    """
+    names = set()
+    for name in all_command_names():
+        names.add(name)
+        cmd = get_cmd_object(name)
+        names.update(cmd.aliases)
+    # candidate: modified levenshtein distance against cmd_name.
+    costs = {}
+    from . import patiencediff
+    for name in sorted(names):
+        matcher = patiencediff.PatienceSequenceMatcher(None, cmd_name, name)
+        distance = 0.0
+        opcodes = matcher.get_opcodes()
+        for opcode, l1, l2, r1, r2 in opcodes:
+            if opcode == 'delete':
+                distance += l2-l1
+            elif opcode == 'replace':
+                distance += max(l2-l1, r2-l1)
+            elif opcode == 'insert':
+                distance += r2-r1
+            elif opcode == 'equal':
+                # Score equal ranges lower, making similar commands of equal
+                # length closer than arbitrary same length commands.
+                distance -= 0.1 * (l2-l1)
+        costs[name] = distance
+    costs.update(_GUESS_OVERRIDES.get(cmd_name, {}))
+    costs = sorted((value, key) for key, value in costs.iteritems())
+    if not costs:
+        return
+    if costs[0][0] > 4:
+        return
+    candidate = costs[0][1]
+    return candidate
+
+
 def get_cmd_object(cmd_name, plugins_override=True):
     """Return the command object for a command.
 
@@ -227,7 +277,14 @@
     try:
         return _get_cmd_object(cmd_name, plugins_override)
     except KeyError:
-        raise errors.BzrCommandError(gettext('unknown command "%s"') % cmd_name)
+        # No command found, see if this was a typo
+        candidate = guess_command(cmd_name)
+        if candidate is not None:
+            raise errors.BzrCommandError(
+                    gettext('unknown command "%s". Perhaps you meant "%s"')
+                    % (cmd_name, candidate))
+        raise errors.BzrCommandError(gettext('unknown command "%s"')
+                % cmd_name)
 
 
 def _get_cmd_object(cmd_name, plugins_override=True, check_missing=True):

=== modified file 'breezy/commit.py'
--- breezy/commit.py	2017-06-11 13:48:12 +0000
+++ breezy/commit.py	2017-06-20 22:54:06 +0000
@@ -75,6 +75,33 @@
 from .i18n import gettext
 
 
+def filter_excluded(iter_changes, exclude):
+    """Filter exclude filenames.
+
+    :param iter_changes: iter_changes function
+    :param exclude: List of paths to exclude
+    :return: iter_changes function
+    """
+    for change in iter_changes:
+        old_path = change[1][0]
+        new_path = change[1][1]
+
+        new_excluded = (new_path is not None and
+            is_inside_any(exclude, new_path))
+
+        old_excluded = (old_path is not None and
+            is_inside_any(exclude, old_path))
+
+        if old_excluded and new_excluded:
+            continue
+
+        if old_excluded or new_excluded:
+            # TODO(jelmer): Perhaps raise an error here instead?
+            continue
+
+        yield change
+
+
 class NullCommitReporter(object):
     """I report on progress of a commit."""
 
@@ -317,14 +344,6 @@
         self.work_tree.lock_write()
         operation.add_cleanup(self.work_tree.unlock)
         self.parents = self.work_tree.get_parent_ids()
-        # We can use record_iter_changes IFF iter_changes is compatible with
-        # the command line parameters, and the repository has fast delta
-        # generation. See bug 347649.
-        self.use_record_iter_changes = (
-            not self.exclude and 
-            not self.branch.repository._format.supports_tree_reference and
-            (self.branch.repository._format.fast_deltas or
-             len(self.parents) < 2))
         self.pb = ui.ui_factory.nested_progress_bar()
         operation.add_cleanup(self.pb.finished)
         self.basis_revid = self.work_tree.last_revision()
@@ -349,8 +368,6 @@
         if self.config_stack is None:
             self.config_stack = self.work_tree.get_config_stack()
 
-        self._set_specific_file_ids()
-
         # Setup the progress bar. As the number of files that need to be
         # committed in unknown, progress is reported as stages.
         # We keep track of entries separately though and include that
@@ -368,7 +385,6 @@
         self.pb.show_count = True
         self.pb.show_bar = True
 
-        self._gather_parents()
         # After a merge, a selected file commit is not supported.
         # See 'bzr help merge' for an explanation as to why.
         if len(self.parents) > 1 and self.specific_files is not None:
@@ -383,9 +399,6 @@
         self.builder = self.branch.get_commit_builder(self.parents,
             self.config_stack, timestamp, timezone, committer, self.revprops,
             rev_id, lossy=lossy)
-        if not self.builder.supports_record_entry_contents and self.exclude:
-            self.builder.abort()
-            raise errors.ExcludesUnsupported(self.branch.repository)
 
         if self.builder.updates_branch and self.bound_branch:
             self.builder.abort()
@@ -394,7 +407,6 @@
                 "that update the branch")
 
         try:
-            self.builder.will_record_deletes()
             # find the location being committed to
             if self.bound_branch:
                 master_location = self.master_branch.base
@@ -636,42 +648,21 @@
                      old_revno, old_revid, new_revno, self.rev_id,
                      tree_delta, future_tree)
 
-    def _gather_parents(self):
-        """Record the parents of a merge for merge detection."""
-        # TODO: Make sure that this list doesn't contain duplicate
-        # entries and the order is preserved when doing this.
-        if self.use_record_iter_changes:
-            return
-        self.basis_inv = self.basis_tree.root_inventory
-        self.parent_invs = [self.basis_inv]
-        for revision in self.parents[1:]:
-            if self.branch.repository.has_revision(revision):
-                mutter('commit parent revision {%s}', revision)
-                inventory = self.branch.repository.get_inventory(revision)
-                self.parent_invs.append(inventory)
-            else:
-                mutter('commit parent ghost revision {%s}', revision)
-
     def _update_builder_with_changes(self):
         """Update the commit builder with the data about what has changed.
         """
-        exclude = self.exclude
         specific_files = self.specific_files
         mutter("Selecting files for commit with filter %r", specific_files)
 
         self._check_strict()
-        if self.use_record_iter_changes:
-            iter_changes = self.work_tree.iter_changes(self.basis_tree,
-                specific_files=specific_files)
-            iter_changes = self._filter_iter_changes(iter_changes)
-            for file_id, path, fs_hash in self.builder.record_iter_changes(
-                self.work_tree, self.basis_revid, iter_changes):
-                self.work_tree._observed_sha1(file_id, path, fs_hash)
-        else:
-            # Build the new inventory
-            self._populate_from_inventory()
-            self._record_unselected()
-            self._report_and_accumulate_deletes()
+        iter_changes = self.work_tree.iter_changes(self.basis_tree,
+            specific_files=specific_files)
+        if self.exclude:
+            iter_changes = filter_excluded(iter_changes, self.exclude)
+        iter_changes = self._filter_iter_changes(iter_changes)
+        for file_id, path, fs_hash in self.builder.record_iter_changes(
+            self.work_tree, self.basis_revid, iter_changes):
+            self.work_tree._observed_sha1(file_id, path, fs_hash)
 
     def _filter_iter_changes(self, iter_changes):
         """Process iter_changes.
@@ -725,55 +716,6 @@
         # Unversion IDs that were found to be deleted
         self.deleted_ids = deleted_ids
 
-    def _record_unselected(self):
-        # If specific files are selected, then all un-selected files must be
-        # recorded in their previous state. For more details, see
-        # https://lists.ubuntu.com/archives/bazaar/2007q3/028476.html.
-        if self.specific_files or self.exclude:
-            specific_files = self.specific_files or []
-            for path, old_ie in self.basis_inv.iter_entries():
-                if self.builder.new_inventory.has_id(old_ie.file_id):
-                    # already added - skip.
-                    continue
-                if (is_inside_any(specific_files, path)
-                    and not is_inside_any(self.exclude, path)):
-                    # was inside the selected path, and not excluded - if not
-                    # present it has been deleted so skip.
-                    continue
-                # From here down it was either not selected, or was excluded:
-                # We preserve the entry unaltered.
-                ie = old_ie.copy()
-                # Note: specific file commits after a merge are currently
-                # prohibited. This test is for sanity/safety in case it's
-                # required after that changes.
-                if len(self.parents) > 1:
-                    ie.revision = None
-                self.builder.record_entry_contents(ie, self.parent_invs, path,
-                    self.basis_tree, None)
-
-    def _report_and_accumulate_deletes(self):
-        if (isinstance(self.basis_inv, Inventory)
-            and isinstance(self.builder.new_inventory, Inventory)):
-            # the older Inventory classes provide a _byid dict, and building a
-            # set from the keys of this dict is substantially faster than even
-            # getting a set of ids from the inventory
-            #
-            # <lifeless> set(dict) is roughly the same speed as
-            # set(iter(dict)) and both are significantly slower than
-            # set(dict.keys())
-            deleted_ids = set(self.basis_inv._byid.keys()) - \
-               set(self.builder.new_inventory._byid.keys())
-        else:
-            deleted_ids = set(self.basis_inv) - set(self.builder.new_inventory)
-        if deleted_ids:
-            self.any_entries_deleted = True
-            deleted = sorted([(self.basis_tree.id2path(file_id), file_id)
-                for file_id in deleted_ids])
-            # XXX: this is not quite directory-order sorting
-            for path, file_id in deleted:
-                self.builder.record_delete(path, file_id)
-                self.reporter.deleted(path)
-
     def _check_strict(self):
         # XXX: when we use iter_changes this would likely be faster if
         # iter_changes would check for us (even in the presence of
@@ -783,107 +725,6 @@
             for unknown in self.work_tree.unknowns():
                 raise StrictCommitFailed()
 
-    def _populate_from_inventory(self):
-        """Populate the CommitBuilder by walking the working tree inventory."""
-        # Build the revision inventory.
-        #
-        # This starts by creating a new empty inventory. Depending on
-        # which files are selected for commit, and what is present in the
-        # current tree, the new inventory is populated. inventory entries
-        # which are candidates for modification have their revision set to
-        # None; inventory entries that are carried over untouched have their
-        # revision set to their prior value.
-        #
-        # ESEPARATIONOFCONCERNS: this function is diffing and using the diff
-        # results to create a new inventory at the same time, which results
-        # in bugs like #46635.  Any reason not to use/enhance Tree.changes_from?
-        # ADHB 11-07-2006
-
-        specific_files = self.specific_files
-        exclude = self.exclude
-        report_changes = self.reporter.is_verbose()
-        deleted_ids = []
-        # A tree of paths that have been deleted. E.g. if foo/bar has been
-        # deleted, then we have {'foo':{'bar':{}}}
-        deleted_paths = {}
-        # XXX: Note that entries may have the wrong kind because the entry does
-        # not reflect the status on disk.
-        # NB: entries will include entries within the excluded ids/paths
-        # because iter_entries_by_dir has no 'exclude' facility today.
-        entries = self.work_tree.iter_entries_by_dir(
-            specific_file_ids=self.specific_file_ids, yield_parents=True)
-        for path, existing_ie in entries:
-            file_id = existing_ie.file_id
-            name = existing_ie.name
-            parent_id = existing_ie.parent_id
-            kind = existing_ie.kind
-            # Skip files that have been deleted from the working tree.
-            # The deleted path ids are also recorded so they can be explicitly
-            # unversioned later.
-            if deleted_paths:
-                path_segments = splitpath(path)
-                deleted_dict = deleted_paths
-                for segment in path_segments:
-                    deleted_dict = deleted_dict.get(segment, None)
-                    if not deleted_dict:
-                        # We either took a path not present in the dict
-                        # (deleted_dict was None), or we've reached an empty
-                        # child dir in the dict, so are now a sub-path.
-                        break
-                else:
-                    deleted_dict = None
-                if deleted_dict is not None:
-                    # the path has a deleted parent, do not add it.
-                    continue
-            if exclude and is_inside_any(exclude, path):
-                # Skip excluded paths. Excluded paths are processed by
-                # _update_builder_with_changes.
-                continue
-            content_summary = self.work_tree.path_content_summary(path)
-            kind = content_summary[0]
-            # Note that when a filter of specific files is given, we must only
-            # skip/record deleted files matching that filter.
-            if not specific_files or is_inside_any(specific_files, path):
-                if kind == 'missing':
-                    if not deleted_paths:
-                        # path won't have been split yet.
-                        path_segments = splitpath(path)
-                    deleted_dict = deleted_paths
-                    for segment in path_segments:
-                        deleted_dict = deleted_dict.setdefault(segment, {})
-                    self.reporter.missing(path)
-                    self._next_progress_entry()
-                    deleted_ids.append(file_id)
-                    continue
-            # TODO: have the builder do the nested commit just-in-time IF and
-            # only if needed.
-            if kind == 'tree-reference':
-                # enforce repository nested tree policy.
-                if (not self.work_tree.supports_tree_reference() or
-                    # repository does not support it either.
-                    not self.branch.repository._format.supports_tree_reference):
-                    kind = 'directory'
-                    content_summary = (kind, None, None, None)
-                elif self.recursive == 'down':
-                    nested_revision_id = self._commit_nested_tree(
-                        file_id, path)
-                    content_summary = (kind, None, None, nested_revision_id)
-                else:
-                    nested_revision_id = self.work_tree.get_reference_revision(file_id)
-                    content_summary = (kind, None, None, nested_revision_id)
-
-            # Record an entry for this item
-            # Note: I don't particularly want to have the existing_ie
-            # parameter but the test suite currently (28-Jun-07) breaks
-            # without it thanks to a unicode normalisation issue. :-(
-            definitely_changed = kind != existing_ie.kind
-            self._record_entry(path, file_id, specific_files, kind, name,
-                parent_id, definitely_changed, existing_ie, report_changes,
-                content_summary)
-
-        # Unversion IDs that were found to be deleted
-        self.deleted_ids = deleted_ids
-
     def _commit_nested_tree(self, file_id, path):
         "Commit a nested tree."
         sub_tree = self.work_tree.get_nested_tree(file_id, path)
@@ -909,49 +750,6 @@
         except errors.PointlessCommit:
             return self.work_tree.get_reference_revision(file_id)
 
-    def _record_entry(self, path, file_id, specific_files, kind, name,
-        parent_id, definitely_changed, existing_ie, report_changes,
-        content_summary):
-        "Record the new inventory entry for a path if any."
-        # mutter('check %s {%s}', path, file_id)
-        # mutter('%s selected for commit', path)
-        if definitely_changed or existing_ie is None:
-            ie = make_entry(kind, name, parent_id, file_id)
-        else:
-            ie = existing_ie.copy()
-            ie.revision = None
-        # For carried over entries we don't care about the fs hash - the repo
-        # isn't generating a sha, so we're not saving computation time.
-        _, _, fs_hash = self.builder.record_entry_contents(
-            ie, self.parent_invs, path, self.work_tree, content_summary)
-        if report_changes:
-            self._report_change(ie, path)
-        if fs_hash:
-            self.work_tree._observed_sha1(ie.file_id, path, fs_hash)
-        return ie
-
-    def _report_change(self, ie, path):
-        """Report a change to the user.
-
-        The change that has occurred is described relative to the basis
-        inventory.
-        """
-        if (self.basis_inv.has_id(ie.file_id)):
-            basis_ie = self.basis_inv[ie.file_id]
-        else:
-            basis_ie = None
-        change = ie.describe_change(basis_ie, ie)
-        if change in (InventoryEntry.RENAMED,
-            InventoryEntry.MODIFIED_AND_RENAMED):
-            old_path = self.basis_inv.id2path(ie.file_id)
-            self.reporter.renamed(change, old_path, path)
-            self._next_progress_entry()
-        else:
-            if change == gettext('unchanged'):
-                return
-            self.reporter.snapshot_change(change, path)
-            self._next_progress_entry()
-
     def _set_progress_stage(self, name, counter=False):
         """Set the progress stage and emit an update to the progress bar."""
         self.pb_stage_name = name
@@ -974,20 +772,3 @@
         else:
             text = gettext("%s - Stage") % (self.pb_stage_name, )
         self.pb.update(text, self.pb_stage_count, self.pb_stage_total)
-
-    def _set_specific_file_ids(self):
-        """populate self.specific_file_ids if we will use it."""
-        if not self.use_record_iter_changes:
-            # If provided, ensure the specified files are versioned
-            if self.specific_files is not None:
-                # Note: This routine is being called because it raises
-                # PathNotVersionedError as a side effect of finding the IDs. We
-                # later use the ids we found as input to the working tree
-                # inventory iterator, so we only consider those ids rather than
-                # examining the whole tree again.
-                # XXX: Dont we have filter_unversioned to do this more
-                # cheaply?
-                self.specific_file_ids = tree.find_ids_across_trees(
-                    self.specific_files, [self.basis_tree, self.work_tree])
-            else:
-                self.specific_file_ids = None

=== modified file 'breezy/config.py'
--- breezy/config.py	2017-06-11 01:19:16 +0000
+++ breezy/config.py	2017-07-21 13:20:17 +0000
@@ -27,7 +27,6 @@
 email=Your Name <your@email.address>
 check_signatures=require|ignore|check-available(default)
 create_signatures=always|never|when-required(default)
-gpg_signing_command=name-of-program
 log_format=name-of-format
 validate_signatures_in_log=true|false(default)
 acceptable_keys=pattern1,pattern2
@@ -83,15 +82,16 @@
 from .lazy_import import lazy_import
 lazy_import(globals(), """
 import base64
+import errno
 import fnmatch
 import re
+import stat
 
 from breezy import (
     atomicfile,
     controldir,
     debug,
     directory_service,
-    errors,
     lazy_regex,
     library_state,
     lockdir,
@@ -107,6 +107,7 @@
 """)
 from . import (
     commands,
+    errors,
     hooks,
     lazy_regex,
     registry,
@@ -154,6 +155,89 @@
 STORE_GLOBAL = 4
 
 
+class OptionExpansionLoop(errors.BzrError):
+
+    _fmt = 'Loop involving %(refs)r while expanding "%(string)s".'
+
+    def __init__(self, string, refs):
+        self.string = string
+        self.refs = '->'.join(refs)
+
+
+class ExpandingUnknownOption(errors.BzrError):
+
+    _fmt = 'Option "%(name)s" is not defined while expanding "%(string)s".'
+
+    def __init__(self, name, string):
+        self.name = name
+        self.string = string
+
+
+class IllegalOptionName(errors.BzrError):
+
+    _fmt = 'Option "%(name)s" is not allowed.'
+
+    def __init__(self, name):
+        self.name = name
+
+
+class ConfigContentError(errors.BzrError):
+
+    _fmt = "Config file %(filename)s is not UTF-8 encoded\n"
+
+    def __init__(self, filename):
+        self.filename = filename
+
+
+class ParseConfigError(errors.BzrError):
+
+    _fmt = "Error(s) parsing config file %(filename)s:\n%(errors)s"
+
+    def __init__(self, errors, filename):
+        self.filename = filename
+        self.errors = '\n'.join(e.msg for e in errors)
+
+
+class ConfigOptionValueError(errors.BzrError):
+
+    _fmt = ('Bad value "%(value)s" for option "%(name)s".\n'
+            'See ``brz help %(name)s``')
+
+    def __init__(self, name, value):
+        errors.BzrError.__init__(self, name=name, value=value)
+
+
+class NoEmailInUsername(errors.BzrError):
+
+    _fmt = "%(username)r does not seem to contain a reasonable email address"
+
+    def __init__(self, username):
+        self.username = username
+
+
+class NoSuchConfig(errors.BzrError):
+
+    _fmt = ('The "%(config_id)s" configuration does not exist.')
+
+    def __init__(self, config_id):
+        errors.BzrError.__init__(self, config_id=config_id)
+
+
+class NoSuchConfigOption(errors.BzrError):
+
+    _fmt = ('The "%(option_name)s" configuration option does not exist.')
+
+    def __init__(self, option_name):
+        errors.BzrError.__init__(self, option_name=option_name)
+
+
+class NoWhoami(errors.BzrError):
+
+    _fmt = ('Unable to determine your name.\n'
+        "Please, set your name with the 'whoami' command.\n"
+        'E.g. brz whoami "Your Name <name@example.com>"')
+
+
 def signature_policy_from_unicode(signature_string):
     """Convert a string to a signing policy."""
     if signature_string.lower() == 'check-available':
@@ -341,11 +425,11 @@
                 else:
                     name = chunk[1:-1]
                     if name in _ref_stack:
-                        raise errors.OptionExpansionLoop(string, _ref_stack)
+                        raise OptionExpansionLoop(string, _ref_stack)
                     _ref_stack.append(name)
                     value = self._expand_option(name, env, _ref_stack)
                     if value is None:
-                        raise errors.ExpandingUnknownOption(name, string)
+                        raise ExpandingUnknownOption(name, string)
                     if isinstance(value, list):
                         list_value = True
                         chunks.extend(value)
@@ -474,7 +558,7 @@
         $BRZ_EMAIL can be set to override this, then
         the concrete policy type is checked, and finally
         $EMAIL is examined.
-        If no username can be found, errors.NoWhoami exception is raised.
+        If no username can be found, NoWhoami exception is raised.
         """
         v = os.environ.get('BRZ_EMAIL')
         if v:
@@ -487,7 +571,7 @@
         return default_email()
 
     def ensure_username(self):
-        """Raise errors.NoWhoami if username is not set.
+        """Raise NoWhoami if username is not set.
 
         This method relies on the username() function raising the error.
         """
@@ -665,9 +749,9 @@
         try:
             self._parser = ConfigObj(co_input, encoding='utf-8')
         except configobj.ConfigObjError as e:
-            raise errors.ParseConfigError(e.errors, e.config.filename)
+            raise ParseConfigError(e.errors, e.config.filename)
         except UnicodeDecodeError:
-            raise errors.ConfigContentError(self.file_name)
+            raise ConfigContentError(self.file_name)
         # Make sure self.reload() will use the right file name
         self._parser.filename = self.file_name
         for hook in OldConfigHooks['load']:
@@ -796,10 +880,6 @@
         else:
             return None
 
-    def _gpg_signing_command(self):
-        """See Config.gpg_signing_command."""
-        return self._get_user_option('gpg_signing_command')
-
     def _log_format(self):
         """See Config.log_format."""
         return self._get_user_option('log_format')
@@ -843,7 +923,7 @@
         try:
             del section[option_name]
         except KeyError:
-            raise errors.NoSuchConfigOption(option_name)
+            raise NoSuchConfigOption(option_name)
         self._write_config_file()
         for hook in OldConfigHooks['remove']:
             hook(self, option_name)
@@ -1312,10 +1392,6 @@
     def remove_user_option(self, option_name, section_name=None):
         self._get_branch_data_config().remove_option(option_name, section_name)
 
-    def _gpg_signing_command(self):
-        """See Config.gpg_signing_command."""
-        return self._get_safe_value('_gpg_signing_command')
-
     def _post_commit(self):
         """See Config.post_commit."""
         return self._get_safe_value('_post_commit')
@@ -1513,7 +1589,7 @@
         return u'%s <%s>' % (name, email)
     elif email:
         return email
-    raise errors.NoWhoami()
+    raise NoWhoami()
 
 
 def _auto_user_id():
@@ -1598,7 +1674,7 @@
     """
     name, email = parse_username(e)
     if not email:
-        raise errors.NoEmailInUsername(e)
+        raise NoEmailInUsername(e)
     return email
 
 
@@ -1643,6 +1719,9 @@
             self.branch.unlock()
 
 
+_authentication_config_permission_errors = set()
+
+
 class AuthenticationConfig(object):
     """The authentication configuration file based on a ini file.
 
@@ -1655,6 +1734,7 @@
         if _file is None:
             self._filename = authentication_config_filename()
             self._input = self._filename = authentication_config_filename()
+            self._check_permissions()
         else:
             # Tests can provide a string as _file
             self._filename = None
@@ -1672,17 +1752,38 @@
             # encoded, but the values in the ConfigObj are always Unicode.
             self._config = ConfigObj(self._input, encoding='utf-8')
         except configobj.ConfigObjError as e:
-            raise errors.ParseConfigError(e.errors, e.config.filename)
+            raise ParseConfigError(e.errors, e.config.filename)
         except UnicodeError:
-            raise errors.ConfigContentError(self._filename)
+            raise ConfigContentError(self._filename)
         return self._config
 
+    def _check_permissions(self):
+        """Check permission of auth file are user read/write able only."""
+        try:
+            st = os.stat(self._filename)
+        except OSError as e:
+            if e.errno != errno.ENOENT:
+                trace.mutter('Unable to stat %r: %r', self._filename, e)
+            return
+        mode = stat.S_IMODE(st.st_mode)
+        if ((stat.S_IXOTH | stat.S_IWOTH | stat.S_IROTH | stat.S_IXGRP |
+             stat.S_IWGRP | stat.S_IRGRP ) & mode):
+            # Only warn once
+            if (not self._filename in _authentication_config_permission_errors
+                and not GlobalConfig().suppress_warning(
+                    'insecure_permissions')):
+                trace.warning("The file '%s' has insecure "
+                        "file permissions. Saved passwords may be accessible "
+                        "by other users.", self._filename)
+                _authentication_config_permission_errors.add(self._filename)
+
     def _save(self):
         """Save the config file, only tests should use it for now."""
         conf_dir = os.path.dirname(self._filename)
         ensure_config_dir_exists(conf_dir)
-        f = file(self._filename, 'wb')
+        fd = os.open(self._filename, os.O_RDWR|os.O_CREAT, 0o600)
         try:
+            f = os.fdopen(fd, 'wb')
             self._get_config().write(f)
         finally:
             f.close()
@@ -2184,9 +2285,9 @@
             try:
                 conf = ConfigObj(f, encoding='utf-8')
             except configobj.ConfigObjError as e:
-                raise errors.ParseConfigError(e.errors, self._external_url())
+                raise ParseConfigError(e.errors, self._external_url())
             except UnicodeDecodeError:
-                raise errors.ConfigContentError(self._external_url())
+                raise ConfigContentError(self._external_url())
         finally:
             f.close()
         return conf
@@ -2302,7 +2403,7 @@
                 trace.warning('Value "%s" is not valid for "%s"',
                               unicode_value, self.name)
             elif self.invalid == 'error':
-                raise errors.ConfigOptionValueError(self.name, unicode_value)
+                raise ConfigOptionValueError(self.name, unicode_value)
         return converted
 
     def get_override(self):
@@ -2321,7 +2422,9 @@
         for var in self.default_from_env:
             try:
                 # If the env variable is defined, its value is the default one
-                value = os.environ[var].decode(osutils.get_user_encoding())
+                value = os.environ[var]
+                if not PY3:
+                    value = value.decode(osutils.get_user_encoding())
                 break
             except KeyError:
                 continue
@@ -2500,7 +2603,7 @@
         :param option_name: The name to validate.
         """
         if _option_ref_re.match('{%s}' % option_name) is None:
-            raise errors.IllegalOptionName(option_name)
+            raise IllegalOptionName(option_name)
 
     def register(self, option):
         """Register a new option to its name.
@@ -2677,14 +2780,6 @@
     Option('email', override_from_env=['BRZ_EMAIL'], default=default_email,
            help='The users identity'))
 option_registry.register(
-    Option('gpg_signing_command',
-           default='gpg',
-           help="""\
-Program to use use for creating signatures.
-
-This should support at least the -u and --clearsign options.
-"""))
-option_registry.register(
     Option('gpg_signing_key',
            default=None,
            help="""\
@@ -2693,14 +2788,6 @@
 This defaults to the first key associated with the users email.
 """))
 option_registry.register(
-    Option('ignore_missing_extensions', default=False,
-           from_unicode=bool_from_store,
-           help='''\
-Control the missing extensions warning display.
-
-The warning will not be emitted if set to True.
-'''))
-option_registry.register(
     Option('language',
            help='Language to translate messages into.'))
 option_registry.register(
@@ -3149,9 +3236,9 @@
                                          list_values=False)
         except configobj.ConfigObjError as e:
             self._config_obj = None
-            raise errors.ParseConfigError(e.errors, self.external_url())
+            raise ParseConfigError(e.errors, self.external_url())
         except UnicodeDecodeError:
-            raise errors.ConfigContentError(self.external_url())
+            raise ConfigContentError(self.external_url())
 
     def save_changes(self):
         if not self.is_loaded():
@@ -3200,7 +3287,7 @@
             self.load()
         except errors.NoSuchFile:
             # The file doesn't exist, let's pretend it was empty
-            self._load_from_string('')
+            self._load_from_string(b'')
         if section_id in self.dirty_sections:
             # We already created a mutable section for this id
             return self.dirty_sections[section_id]
@@ -3713,11 +3800,11 @@
                     expanded = True
                     name = chunk[1:-1]
                     if name in _refs:
-                        raise errors.OptionExpansionLoop(string, _refs)
+                        raise OptionExpansionLoop(string, _refs)
                     _refs.append(name)
                     value = self._expand_option(name, env, _refs)
                     if value is None:
-                        raise errors.ExpandingUnknownOption(name, string)
+                        raise ExpandingUnknownOption(name, string)
                     chunks.append(value)
                     _refs.pop()
             result = ''.join(chunks)
@@ -4108,7 +4195,7 @@
                 if write_access:
                     self.add_cleanup(br.lock_write().unlock)
                 return br.get_config_stack()
-            raise errors.NoSuchConfig(scope)
+            raise NoSuchConfig(scope)
         else:
             try:
                 (_, br, _) = (
@@ -4133,7 +4220,7 @@
             value = self._quote_multiline(value)
             self.outf.write('%s\n' % (value,))
         else:
-            raise errors.NoSuchConfigOption(name)
+            raise NoSuchConfigOption(name)
 
     def _show_matching_options(self, name, directory, scope):
         name = lazy_regex.lazy_compile(name)
@@ -4178,7 +4265,7 @@
             # Explicitly save the changes
             conf.store.save_changes()
         except KeyError:
-            raise errors.NoSuchConfigOption(name)
+            raise NoSuchConfigOption(name)
 
 
 # Test registries

=== modified file 'breezy/controldir.py'
--- breezy/controldir.py	2017-06-11 00:27:48 +0000
+++ breezy/controldir.py	2017-07-15 13:23:08 +0000
@@ -760,7 +760,7 @@
                 pass
             try:
                 new_t = a_transport.clone('..')
-            except errors.InvalidURLJoin:
+            except urlutils.InvalidURLJoin:
                 # reached the root, whatever that may be
                 raise errors.NotBranchError(path=url)
             if new_t.base == a_transport.base:
@@ -1091,14 +1091,6 @@
             target_format.rich_root_data)
 
     @classmethod
-    def register_format(klass, format):
-        """Register a format that does not use '.bzr' for its control dir.
-
-        """
-        raise errors.BzrError("ControlDirFormat.register_format() has been "
-            "removed in Bazaar 2.4. Please upgrade your plugins.")
-
-    @classmethod
     def register_prober(klass, prober):
         """Register a prober that can look for a control dir.
 

=== modified file 'breezy/diff.py'
--- breezy/diff.py	2017-06-11 00:27:48 +0000
+++ breezy/diff.py	2017-07-14 20:34:39 +0000
@@ -48,6 +48,8 @@
     Registry,
     )
 from .trace import mutter, note, warning
+from .tree import FileTimestampUnavailable
+
 
 DEFAULT_CONTEXT_AMOUNT = 3
 
@@ -484,7 +486,7 @@
     """Returns a timestamp suitable for use in a patch header."""
     try:
         mtime = tree.get_file_mtime(file_id, path)
-    except errors.FileTimestampUnavailable:
+    except FileTimestampUnavailable:
         mtime = 0
     return timestamp.format_patch_date(mtime)
 
@@ -825,7 +827,7 @@
             source.close()
         try:
             mtime = tree.get_file_mtime(file_id)
-        except errors.FileTimestampUnavailable:
+        except FileTimestampUnavailable:
             pass
         else:
             os.utime(full_path, (mtime, mtime))

=== modified file 'breezy/email_message.py'
--- breezy/email_message.py	2017-05-22 00:56:52 +0000
+++ breezy/email_message.py	2017-06-14 23:29:06 +0000
@@ -28,6 +28,9 @@
 
 from . import __version__ as _breezy_version
 from .osutils import safe_unicode
+from .sixish import (
+    text_type,
+    )
 from .smtp_connection import SMTPConnection
 
 
@@ -60,7 +63,7 @@
         self._body = body
         self._parts = []
 
-        if isinstance(to_address, basestring):
+        if isinstance(to_address, (str, text_type)):
             to_address = [ to_address ]
 
         to_addresses = []

=== modified file 'breezy/errors.py'
--- breezy/errors.py	2017-06-10 18:39:27 +0000
+++ breezy/errors.py	2017-07-21 14:18:24 +0000
@@ -259,15 +259,6 @@
         BzrError.__init__(self, branch_url=public_location)
 
 
-class NoHelpTopic(BzrError):
-
-    _fmt = ("No help could be found for '%(topic)s'. "
-        "Please use 'brz help topics' to obtain a list of topics.")
-
-    def __init__(self, topic):
-        self.topic = topic
-
-
 class NoSuchId(BzrError):
 
     _fmt = 'The file id "%(file_id)s" is not present in the tree %(tree)s.'
@@ -351,11 +342,6 @@
         self.not_locked = not_locked
 
 
-class BzrOptionError(BzrCommandError):
-
-    _fmt = "Error in command line options"
-
-
 class BadIndexFormatSignature(BzrError):
 
     _fmt = "%(value)s is not an index of type %(_type)s."
@@ -412,14 +398,6 @@
         self.value = value
 
 
-class BadOptionValue(BzrError):
-
-    _fmt = """Bad value "%(value)s" for option "%(name)s"."""
-
-    def __init__(self, name, value):
-        BzrError.__init__(self, name=name, value=value)
-
-
 class StrictCommitFailed(BzrError):
 
     _fmt = "Commit refused because there are unknown files in the tree"
@@ -511,32 +489,6 @@
     _fmt = 'Permission denied: "%(path)s"%(extra)s'
 
 
-class InvalidURL(PathError):
-
-    _fmt = 'Invalid url supplied to transport: "%(path)s"%(extra)s'
-
-
-class InvalidURLJoin(PathError):
-
-    _fmt = "Invalid URL join request: %(reason)s: %(base)r + %(join_args)r"
-
-    def __init__(self, reason, base, join_args):
-        self.reason = reason
-        self.base = base
-        self.join_args = join_args
-        PathError.__init__(self, base, reason)
-
-
-class InvalidRebaseURLs(PathError):
-
-    _fmt = "URLs differ by more than path: %(from_)r and %(to)r"
-
-    def __init__(self, from_, to):
-        self.from_ = from_
-        self.to = to
-        PathError.__init__(self, from_, 'URLs differ by more than path.')
-
-
 class UnavailableRepresentation(InternalBzrError):
 
     _fmt = ("The encoding '%(wanted)s' is not available for key %(key)s which "
@@ -1314,79 +1266,6 @@
         self.error = error
 
 
-class WeaveError(BzrError):
-
-    _fmt = "Error in processing weave: %(msg)s"
-
-    def __init__(self, msg=None):
-        BzrError.__init__(self)
-        self.msg = msg
-
-
-class WeaveRevisionAlreadyPresent(WeaveError):
-
-    _fmt = "Revision {%(revision_id)s} already present in %(weave)s"
-
-    def __init__(self, revision_id, weave):
-
-        WeaveError.__init__(self)
-        self.revision_id = revision_id
-        self.weave = weave
-
-
-class WeaveRevisionNotPresent(WeaveError):
-
-    _fmt = "Revision {%(revision_id)s} not present in %(weave)s"
-
-    def __init__(self, revision_id, weave):
-        WeaveError.__init__(self)
-        self.revision_id = revision_id
-        self.weave = weave
-
-
-class WeaveFormatError(WeaveError):
-
-    _fmt = "Weave invariant violated: %(what)s"
-
-    def __init__(self, what):
-        WeaveError.__init__(self)
-        self.what = what
-
-
-class WeaveParentMismatch(WeaveError):
-
-    _fmt = "Parents are mismatched between two revisions. %(msg)s"
-
-
-class WeaveInvalidChecksum(WeaveError):
-
-    _fmt = "Text did not match its checksum: %(msg)s"
-
-
-class WeaveTextDiffers(WeaveError):
-
-    _fmt = ("Weaves differ on text content. Revision:"
-            " {%(revision_id)s}, %(weave_a)s, %(weave_b)s")
-
-    def __init__(self, revision_id, weave_a, weave_b):
-        WeaveError.__init__(self)
-        self.revision_id = revision_id
-        self.weave_a = weave_a
-        self.weave_b = weave_b
-
-
-class WeaveTextDiffers(WeaveError):
-
-    _fmt = ("Weaves differ on text content. Revision:"
-            " {%(revision_id)s}, %(weave_a)s, %(weave_b)s")
-
-    def __init__(self, revision_id, weave_a, weave_b):
-        WeaveError.__init__(self)
-        self.revision_id = revision_id
-        self.weave_a = weave_a
-        self.weave_b = weave_b
-
-
 class VersionedFileError(BzrError):
 
     _fmt = "Versioned file error"
@@ -1734,59 +1613,6 @@
     _fmt = "Working tree has conflicts."
 
 
-class ConfigContentError(BzrError):
-
-    _fmt = "Config file %(filename)s is not UTF-8 encoded\n"
-
-    def __init__(self, filename):
-        BzrError.__init__(self)
-        self.filename = filename
-
-
-class ParseConfigError(BzrError):
-
-    _fmt = "Error(s) parsing config file %(filename)s:\n%(errors)s"
-
-    def __init__(self, errors, filename):
-        BzrError.__init__(self)
-        self.filename = filename
-        self.errors = '\n'.join(e.msg for e in errors)
-
-
-class ConfigOptionValueError(BzrError):
-
-    _fmt = ('Bad value "%(value)s" for option "%(name)s".\n'
-            'See ``brz help %(name)s``')
-
-    def __init__(self, name, value):
-        BzrError.__init__(self, name=name, value=value)
-
-
-class NoEmailInUsername(BzrError):
-
-    _fmt = "%(username)r does not seem to contain a reasonable email address"
-
-    def __init__(self, username):
-        BzrError.__init__(self)
-        self.username = username
-
-
-class SigningFailed(BzrError):
-
-    _fmt = 'Failed to GPG sign data with command "%(command_line)s"'
-
-    def __init__(self, command_line):
-        BzrError.__init__(self, command_line=command_line)
-
-
-class SignatureVerificationFailed(BzrError):
-
-    _fmt = 'Failed to verify GPG signature data with error "%(error)s"'
-
-    def __init__(self, error):
-        BzrError.__init__(self, error=error)
-
-
 class DependencyNotPresent(BzrError):
 
     _fmt = 'Unable to import library "%(library)s": %(error)s'
@@ -1795,14 +1621,6 @@
         BzrError.__init__(self, library=library, error=error)
 
 
-class GpgmeNotInstalled(DependencyNotPresent):
-
-    _fmt = 'python-gpgme is not installed, it is needed to verify signatures'
-
-    def __init__(self, error):
-        DependencyNotPresent.__init__(self, 'gpgme', error)
-
-
 class WorkingTreeNotRevision(BzrError):
 
     _fmt = ("The working tree for %(basedir)s has changed since"
@@ -2582,52 +2400,6 @@
         self.tag_name = tag_name
 
 
-class MalformedBugIdentifier(BzrError):
-
-    _fmt = ('Did not understand bug identifier %(bug_id)s: %(reason)s. '
-            'See "brz help bugs" for more information on this feature.')
-
-    def __init__(self, bug_id, reason):
-        self.bug_id = bug_id
-        self.reason = reason
-
-
-class InvalidBugTrackerURL(BzrError):
-
-    _fmt = ("The URL for bug tracker \"%(abbreviation)s\" doesn't "
-            "contain {id}: %(url)s")
-
-    def __init__(self, abbreviation, url):
-        self.abbreviation = abbreviation
-        self.url = url
-
-
-class UnknownBugTrackerAbbreviation(BzrError):
-
-    _fmt = ("Cannot find registered bug tracker called %(abbreviation)s "
-            "on %(branch)s")
-
-    def __init__(self, abbreviation, branch):
-        self.abbreviation = abbreviation
-        self.branch = branch
-
-
-class InvalidLineInBugsProperty(BzrError):
-
-    _fmt = ("Invalid line in bugs property: '%(line)s'")
-
-    def __init__(self, line):
-        self.line = line
-
-
-class InvalidBugStatus(BzrError):
-
-    _fmt = ("Invalid bug status: '%(status)s'")
-
-    def __init__(self, status):
-        self.status = status
-
-
 class UnexpectedSmartServerResponse(BzrError):
 
     _fmt = "Could not understand response from smart server: %(response_tuple)r"
@@ -2973,22 +2745,6 @@
         self.user_encoding = get_user_encoding()
 
 
-class NoSuchConfig(BzrError):
-
-    _fmt = ('The "%(config_id)s" configuration does not exist.')
-
-    def __init__(self, config_id):
-        BzrError.__init__(self, config_id=config_id)
-
-
-class NoSuchConfigOption(BzrError):
-
-    _fmt = ('The "%(option_name)s" configuration option does not exist.')
-
-    def __init__(self, option_name):
-        BzrError.__init__(self, option_name=option_name)
-
-
 class NoSuchAlias(BzrError):
 
     _fmt = ('The alias "%(alias_name)s" does not exist.')
@@ -3180,16 +2936,6 @@
         self.target_branch = target_branch
 
 
-class FileTimestampUnavailable(BzrError):
-
-    _fmt = "The filestamp for %(path)s is not available."
-
-    internal_error = True
-
-    def __init__(self, path):
-        self.path = path
-
-
 class NoColocatedBranchSupport(BzrError):
 
     _fmt = ("%(controldir)r does not support co-located branches.")
@@ -3198,21 +2944,6 @@
         self.controldir = controldir
 
 
-class NoWhoami(BzrError):
-
-    _fmt = ('Unable to determine your name.\n'
-        "Please, set your name with the 'whoami' command.\n"
-        'E.g. brz whoami "Your Name <name@example.com>"')
-
-
-class InvalidPattern(BzrError):
-
-    _fmt = ('Invalid pattern(s) found. %(msg)s')
-
-    def __init__(self, msg):
-        self.msg = msg
-
-
 class RecursiveBind(BzrError):
 
     _fmt = ('Branch "%(branch_url)s" appears to be bound to itself. '
@@ -3222,44 +2953,6 @@
         self.branch_url = branch_url
 
 
-# FIXME: I would prefer to define the config related exception classes in
-# config.py but the lazy import mechanism proscribes this -- vila 20101222
-class OptionExpansionLoop(BzrError):
-
-    _fmt = 'Loop involving %(refs)r while expanding "%(string)s".'
-
-    def __init__(self, string, refs):
-        self.string = string
-        self.refs = '->'.join(refs)
-
-
-class ExpandingUnknownOption(BzrError):
-
-    _fmt = 'Option "%(name)s" is not defined while expanding "%(string)s".'
-
-    def __init__(self, name, string):
-        self.name = name
-        self.string = string
-
-
-class IllegalOptionName(BzrError):
-
-    _fmt = 'Option "%(name)s" is not allowed.'
-
-    def __init__(self, name):
-        self.name = name
-
-
-class NoCompatibleInter(BzrError):
-
-    _fmt = ('No compatible object available for operations from %(source)r '
-            'to %(target)r.')
-
-    def __init__(self, source, target):
-        self.source = source
-        self.target = target
-
-
 class HpssVfsRequestNotAllowed(BzrError):
 
     _fmt = ("VFS requests over the smart server are not allowed. Encountered: "
@@ -3282,66 +2975,6 @@
         self.format = format
 
 
-class MissingFeature(BzrError):
-
-    _fmt = ("Missing feature %(feature)s not provided by this "
-            "version of Bazaar or any plugin.")
-
-    def __init__(self, feature):
-        self.feature = feature
-
-
-class PatchSyntax(BzrError):
-    """Base class for patch syntax errors."""
-
-
-class BinaryFiles(BzrError):
-
-    _fmt = 'Binary files section encountered.'
-
-    def __init__(self, orig_name, mod_name):
-        self.orig_name = orig_name
-        self.mod_name = mod_name
-
-
-class MalformedPatchHeader(PatchSyntax):
-
-    _fmt = "Malformed patch header.  %(desc)s\n%(line)r"
-
-    def __init__(self, desc, line):
-        self.desc = desc
-        self.line = line
-
-
-class MalformedHunkHeader(PatchSyntax):
-
-    _fmt = "Malformed hunk header.  %(desc)s\n%(line)r"
-
-    def __init__(self, desc, line):
-        self.desc = desc
-        self.line = line
-
-
-class MalformedLine(PatchSyntax):
-
-    _fmt = "Malformed line.  %(desc)s\n%(line)r"
-
-    def __init__(self, desc, line):
-        self.desc = desc
-        self.line = line
-
-
-class PatchConflict(BzrError):
-
-    _fmt = ('Text contents mismatch at line %(line_no)d.  Original has '
-            '"%(orig_line)s", but patch says it should be "%(patch_line)s"')
-
-    def __init__(self, line_no, orig_line, patch_line):
-        self.line_no = line_no
-        self.orig_line = orig_line.rstrip('\n')
-        self.patch_line = patch_line.rstrip('\n')
-
-
 class FeatureAlreadyRegistered(BzrError):
 
     _fmt = 'The feature %(feature)s has already been registered.'

=== modified file 'breezy/export_pot.py'
--- breezy/export_pot.py	2017-06-05 20:48:31 +0000
+++ breezy/export_pot.py	2017-06-17 12:27:49 +0000
@@ -251,6 +251,8 @@
         _write_command_help(exporter, command)
 
     plugins = breezy.global_state.plugins
+    if plugin_name is not None and plugin_name not in breezy.global_state.plugins:
+        raise errors.BzrError(gettext('Plugin %s is not loaded' % plugin_name))
     core_plugins = set(name for name in plugins
         if plugins[name].path().startswith(breezy.__path__[0]))
     # plugins

=== modified file 'breezy/fifo_cache.py'
--- breezy/fifo_cache.py	2017-06-05 22:13:49 +0000
+++ breezy/fifo_cache.py	2017-06-13 02:58:12 +0000
@@ -41,16 +41,7 @@
 
     def __delitem__(self, key):
         # Remove the key from an arbitrary location in the queue
-        remove = getattr(self._queue, 'remove', None)
-        # Python2.5's has deque.remove, but Python2.4 does not
-        if remove is not None:
-            remove(key)
-        else:
-            # TODO: It would probably be faster to pop()/popleft() until we get to the
-            #       key, and then insert those back into the queue. We know
-            #       the key should only be present in one position, and we
-            #       wouldn't need to rebuild the whole queue.
-            self._queue = deque([k for k in self._queue if k != key])
+        self._queue.remove(key)
         self._remove(key)
 
     def add(self, key, value, cleanup=None):

=== modified file 'breezy/generate_ids.py'
--- breezy/generate_ids.py	2017-06-10 01:57:00 +0000
+++ breezy/generate_ids.py	2017-07-21 13:20:17 +0000
@@ -24,12 +24,12 @@
 
 from breezy import (
     config,
-    errors,
     osutils,
     )
 """)
 
 from . import (
+    errors,
     lazy_regex,
     )
 
@@ -103,7 +103,7 @@
     """
     try:
         user_or_email = config.extract_email_address(username)
-    except errors.NoEmailInUsername:
+    except config.NoEmailInUsername:
         user_or_email = username
 
     user_or_email = user_or_email.lower()

=== modified file 'breezy/globbing.py'
--- breezy/globbing.py	2017-05-22 00:56:52 +0000
+++ breezy/globbing.py	2017-07-10 23:47:19 +0000
@@ -242,7 +242,7 @@
                 match = regex.match(filename)
                 if match:
                     return patterns[match.lastindex -1]
-        except errors.InvalidPattern as e:
+        except lazy_regex.InvalidPattern as e:
             # We can't show the default e.msg to the user as thats for
             # the combined pattern we sent to regex. Instead we indicate to
             # the user that an ignore file needs fixing.
@@ -286,7 +286,7 @@
         try:
             re_obj = lazy_regex.lazy_compile(tpattern, re.UNICODE)
             re_obj.search("") # force compile
-        except errors.InvalidPattern as e:
+        except lazy_regex.InvalidPattern as e:
             result = False
         return result
 

=== modified file 'breezy/gpg.py'
--- breezy/gpg.py	2017-06-05 20:48:31 +0000
+++ breezy/gpg.py	2017-07-04 20:03:11 +0000
@@ -29,7 +29,6 @@
 
 from breezy import (
     config,
-    errors,
     trace,
     ui,
     )
@@ -39,6 +38,9 @@
     )
 """)
 
+from . import (
+    errors,
+    )
 from .sixish import (
     BytesIO,
     )
@@ -51,6 +53,30 @@
 SIGNATURE_EXPIRED = 4
 
 
+class GpgNotInstalled(errors.DependencyNotPresent):
+
+    _fmt = 'python-gpg is not installed, it is needed to verify signatures'
+
+    def __init__(self, error):
+        errors.DependencyNotPresent.__init__(self, 'gpg', error)
+
+
+class SigningFailed(errors.BzrError):
+
+    _fmt = 'Failed to GPG sign data: "%(error)s"'
+
+    def __init__(self, error):
+        errors.BzrError.__init__(self, error=error)
+
+
+class SignatureVerificationFailed(errors.BzrError):
+
+    _fmt = 'Failed to verify GPG signature data with error "%(error)s"'
+
+    def __init__(self, error):
+        errors.BzrError.__init__(self, error=error)
+
+
 def bulk_verify_signatures(repository, revids, strategy,
         process_events_callback=None):
     """Do verifications on a set of revisions
@@ -101,10 +127,10 @@
         """Real strategies take a configuration."""
 
     def sign(self, content):
-        raise errors.SigningFailed('Signing is disabled.')
+        raise SigningFailed('Signing is disabled.')
 
     def verify(self, content, testament):
-        raise errors.SignatureVerificationFailed('Signature verification is \
+        raise SignatureVerificationFailed('Signature verification is \
 disabled.')
 
     def set_acceptable_keys(self, command_line_input):
@@ -162,11 +188,32 @@
     def __init__(self, config_stack):
         self._config_stack = config_stack
         try:
-            import gpgme
-            self.context = gpgme.Context()
+            import gpg
+            self.context = gpg.Context()
         except ImportError as error:
             pass # can't use verify()
 
+        self.context.signers = self._get_signing_keys()
+
+    def _get_signing_keys(self):
+        import gpg
+        keyname = self._config_stack.get('gpg_signing_key')
+        if keyname:
+            try:
+                return [self.context.get_key(keyname)]
+            except gpg.errors.KeyNotFound:
+                pass
+
+        if keyname is None or keyname == 'default':
+            # 'default' or not setting gpg_signing_key at all means we should
+            # use the user email address
+            keyname = config.extract_email_address(self._config_stack.get('email'))
+        possible_keys = self.context.keylist(keyname, secret=True)
+        try:
+            return [possible_keys.next()]
+        except StopIteration:
+            return []
+
     @staticmethod
     def verify_signatures_available():
         """
@@ -175,55 +222,24 @@
         :return: boolean if this strategy can verify signatures
         """
         try:
-            import gpgme
+            import gpg
             return True
         except ImportError as error:
             return False
 
-    def _command_line(self):
-        key = self._config_stack.get('gpg_signing_key')
-        if key is None or key == 'default':
-            # 'default' or not setting gpg_signing_key at all means we should
-            # use the user email address
-            key = config.extract_email_address(self._config_stack.get('email'))
-        return [self._config_stack.get('gpg_signing_command'), '--clearsign',
-                '-u', key]
-
     def sign(self, content):
+        import gpg
         if isinstance(content, unicode):
             raise errors.BzrBadParameterUnicode('content')
-        ui.ui_factory.clear_term()
 
-        preexec_fn = _set_gpg_tty
-        if sys.platform == 'win32':
-            # Win32 doesn't support preexec_fn, but wouldn't support TTY anyway.
-            preexec_fn = None
+        plain_text = gpg.Data(content)
         try:
-            process = subprocess.Popen(self._command_line(),
-                                       stdin=subprocess.PIPE,
-                                       stdout=subprocess.PIPE,
-                                       preexec_fn=preexec_fn)
-            try:
-                result = process.communicate(content)[0]
-                if process.returncode is None:
-                    process.wait()
-                if process.returncode != 0:
-                    raise errors.SigningFailed(self._command_line())
-                return result
-            except OSError as e:
-                if e.errno == errno.EPIPE:
-                    raise errors.SigningFailed(self._command_line())
-                else:
-                    raise
-        except ValueError:
-            # bad subprocess parameters, should never happen.
-            raise
-        except OSError as e:
-            if e.errno == errno.ENOENT:
-                # gpg is not installed
-                raise errors.SigningFailed(self._command_line())
-            else:
-                raise
+            output, result = self.context.sign(
+                plain_text, mode=gpg.constants.sig.mode.CLEAR)
+        except gpg.errors.GPGMEError as error:
+            raise SigningFailed(str(error))
+
+        return output
 
     def verify(self, content, testament):
         """Check content has a valid signature.
@@ -234,74 +250,74 @@
         :return: SIGNATURE_VALID or a failed SIGNATURE_ value, key uid if valid
         """
         try:
-            import gpgme
+            import gpg
         except ImportError as error:
-            raise errors.GpgmeNotInstalled(error)
+            raise errors.GpgNotInstalled(error)
 
-        signature = BytesIO(content)
-        plain_output = BytesIO()
+        signature = gpg.Data(content)
+        sink = gpg.Data()
         try:
-            result = self.context.verify(signature, None, plain_output)
-        except gpgme.GpgmeError as error:
-            raise errors.SignatureVerificationFailed(error[2])
+            plain_output, result = self.context.verify(signature)
+        except gpg.errors.BadSignatures as error:
+            fingerprint = error.result.signatures[0].fpr
+            if error.result.signatures[0].summary & gpg.constants.SIGSUM_KEY_EXPIRED:
+                expires = self.context.get_key(error.result.signatures[0].fpr).subkeys[0].expires
+                if expires > error.result.signatures[0].timestamp:
+                    # The expired key was not expired at time of signing.
+                    # test_verify_expired_but_valid()
+                    return SIGNATURE_EXPIRED, fingerprint[-8:]
+                else:
+                    # I can't work out how to create a test where the signature
+                    # was expired at the time of signing.
+                    return SIGNATURE_NOT_VALID, None
+
+            # GPG does not know this key.
+            # test_verify_unknown_key()
+            if error.result.signatures[0].summary & gpg.constants.SIGSUM_KEY_MISSING:
+                return SIGNATURE_KEY_MISSING, fingerprint[-8:]
+
+            return SIGNATURE_NOT_VALID, None
+        except gpg.errors.GPGMEError as error:
+            raise SignatureVerificationFailed(error[2])
 
         # No result if input is invalid.
         # test_verify_invalid()
-        if len(result) == 0:
+        if len(result.signatures) == 0:
             return SIGNATURE_NOT_VALID, None
         # User has specified a list of acceptable keys, check our result is in
         # it.  test_verify_unacceptable_key()
-        fingerprint = result[0].fpr
+        fingerprint = result.signatures[0].fpr
         if self.acceptable_keys is not None:
             if not fingerprint in self.acceptable_keys:
                 return SIGNATURE_KEY_MISSING, fingerprint[-8:]
         # Check the signature actually matches the testament.
         # test_verify_bad_testament()
-        if testament != plain_output.getvalue():
+        if testament != plain_output:
             return SIGNATURE_NOT_VALID, None
-        # Yay gpgme set the valid bit.
+        # Yay gpg set the valid bit.
         # Can't write a test for this one as you can't set a key to be
-        # trusted using gpgme.
-        if result[0].summary & gpgme.SIGSUM_VALID:
+        # trusted using gpg.
+        if result.signatures[0].summary & gpg.constants.SIGSUM_VALID:
             key = self.context.get_key(fingerprint)
             name = key.uids[0].name
             email = key.uids[0].email
             return SIGNATURE_VALID, name + " <" + email + ">"
         # Sigsum_red indicates a problem, unfortunatly I have not been able
         # to write any tests which actually set this.
-        if result[0].summary & gpgme.SIGSUM_RED:
+        if result.signatures[0].summary & gpg.constants.SIGSUM_RED:
             return SIGNATURE_NOT_VALID, None
-        # GPG does not know this key.
-        # test_verify_unknown_key()
-        if result[0].summary & gpgme.SIGSUM_KEY_MISSING:
-            return SIGNATURE_KEY_MISSING, fingerprint[-8:]
         # Summary isn't set if sig is valid but key is untrusted but if user
         # has explicity set the key as acceptable we can validate it.
-        if result[0].summary == 0 and self.acceptable_keys is not None:
+        if result.signatures[0].summary == 0 and self.acceptable_keys is not None:
             if fingerprint in self.acceptable_keys:
                 # test_verify_untrusted_but_accepted()
                 return SIGNATURE_VALID, None
         # test_verify_valid_but_untrusted()
-        if result[0].summary == 0 and self.acceptable_keys is None:
-            return SIGNATURE_NOT_VALID, None
-        if result[0].summary & gpgme.SIGSUM_KEY_EXPIRED:
-            expires = self.context.get_key(result[0].fpr).subkeys[0].expires
-            if expires > result[0].timestamp:
-                # The expired key was not expired at time of signing.
-                # test_verify_expired_but_valid()
-                return SIGNATURE_EXPIRED, fingerprint[-8:]
-            else:
-                # I can't work out how to create a test where the signature
-                # was expired at the time of signing.
-                return SIGNATURE_NOT_VALID, None
-        # A signature from a revoked key gets this.
-        # test_verify_revoked_signature()
-        if ((result[0].summary & gpgme.SIGSUM_SYS_ERROR
-             or result[0].status.strerror == 'Certificate revoked')):
+        if result.signatures[0].summary == 0 and self.acceptable_keys is None:
             return SIGNATURE_NOT_VALID, None
         # Other error types such as revoked keys should (I think) be caught by
         # SIGSUM_RED so anything else means something is buggy.
-        raise errors.SignatureVerificationFailed(
+        raise SignatureVerificationFailed(
             "Unknown GnuPG key verification result")
 
     def set_acceptable_keys(self, command_line_input):

=== modified file 'breezy/help.py'
--- breezy/help.py	2017-06-05 20:48:31 +0000
+++ breezy/help.py	2017-07-14 20:37:44 +0000
@@ -36,6 +36,15 @@
     )
 
 
+class NoHelpTopic(errors.BzrError):
+
+    _fmt = ("No help could be found for '%(topic)s'. "
+        "Please use 'brz help topics' to obtain a list of topics.")
+
+    def __init__(self, topic):
+        self.topic = topic
+
+
 def help(topic=None, outfile=None):
     """Write the help for the specific topic to outfile"""
     if outfile is None:
@@ -52,7 +61,7 @@
                 topic_obj.get_help_topic()))
         source = topics[0][1]
         outfile.write(source.get_help_text(shadowed_terms))
-    except errors.NoHelpTopic:
+    except NoHelpTopic:
         if alias is None:
             raise
 
@@ -162,6 +171,6 @@
         for index in self.search_path:
             result.extend([(index, _topic) for _topic in index.get_topics(topic)])
         if not result:
-            raise errors.NoHelpTopic(topic)
+            raise NoHelpTopic(topic)
         else:
             return result

=== modified file 'breezy/help_topics/__init__.py'
--- breezy/help_topics/__init__.py	2017-06-05 20:48:31 +0000
+++ breezy/help_topics/__init__.py	2017-06-15 00:39:26 +0000
@@ -791,6 +791,8 @@
                         'Options to show or record debug information')
 topic_registry.register('log-formats', _load_from_file,
                         'Details on the logging formats available')
+topic_registry.register('missing-extensions', _load_from_file,
+                        'What to do when compiled extensions are missing')
 topic_registry.register('url-special-chars', _load_from_file,
                         'Special character handling in URLs')
 

=== modified file 'breezy/help_topics/en/configuration.txt'
--- breezy/help_topics/en/configuration.txt	2017-05-21 18:10:28 +0000
+++ breezy/help_topics/en/configuration.txt	2017-07-04 20:03:11 +0000
@@ -498,16 +498,6 @@
     This section only applies to the branch at this directory and not
     branches below it.
 
-gpg_signing_command
-~~~~~~~~~~~~~~~~~~~
-
-(Default: "gpg"). Which program should be used to sign and check revisions.
-For example::
-
-    gpg_signing_command = /usr/bin/gnpg
-
-The specified command must accept the options "--clearsign" and "-u <email>".
-
 bzr_remote_path
 ~~~~~~~~~~~~~~~
 
@@ -600,6 +590,10 @@
     whether the format deprecation warning is shown on repositories that are
     using deprecated formats.
 
+* ``insecure_permissions``:
+    whether a warning is shown if ``authentication.conf`` can be read
+    by other users.
+
 default_format
 ~~~~~~~~~~~~~~
 

=== added file 'breezy/help_topics/en/missing-extensions.txt'
--- breezy/help_topics/en/missing-extensions.txt	1970-01-01 00:00:00 +0000
+++ breezy/help_topics/en/missing-extensions.txt	2017-06-15 00:50:22 +0000
@@ -0,0 +1,26 @@
+Missing Extensions
+==================
+
+Breezy gives a warning if it failed to load some compiled extensions. (See
+http://bazaarvcs.wordpress.com/2009/09/17/extension-warnings/ for background.)
+
+This may indicate either that there's been a problem installing or packaging
+Breezy, or it may just be a case where you don't want to use the compiled
+extensions.
+
+1. If you installed Breezy from an official binary package or a distribution
+and you're seeing this warning, please report a bug.
+
+2. If you're running Breezy from a source tree or tarball, please run 'make' in
+that directory to compile the extensions. You should do this every time you
+update the tree. To do this you will need some build dependencies installed
+including make, a C compiler, and the Python header files.
+
+3. If you can't or don't want to build the extensions, add this line to
+bazaar.conf (whose location is shown by 'brz --version')::
+
+  suppress_warnings=missing_extensions
+
+Breezy will work normally but some operations may be slower.
+
+The specific modules that failed, and why, are listed in .brz.log.

=== modified file 'breezy/info.py'
--- breezy/info.py	2017-06-10 16:40:42 +0000
+++ breezy/info.py	2017-07-15 13:23:08 +0000
@@ -60,7 +60,7 @@
             return
         try:
             path = urlutils.local_path_from_url(url)
-        except errors.InvalidURL:
+        except urlutils.InvalidURL:
             self.locs.append((label, url))
         else:
             self.add_path(label, path)

=== modified file 'breezy/inter.py'
--- breezy/inter.py	2017-05-22 00:56:52 +0000
+++ breezy/inter.py	2017-07-10 23:56:33 +0000
@@ -18,7 +18,17 @@
 
 from __future__ import absolute_import
 
-from .errors import NoCompatibleInter
+from .errors import BzrError
+
+
+class NoCompatibleInter(BzrError):
+
+    _fmt = ('No compatible object available for operations from %(source)r '
+            'to %(target)r.')
+
+    def __init__(self, source, target):
+        self.source = source
+        self.target = target
 
 
 class InterObject(object):

=== modified file 'breezy/lazy_regex.py'
--- breezy/lazy_regex.py	2017-05-22 00:56:52 +0000
+++ breezy/lazy_regex.py	2017-07-10 23:47:19 +0000
@@ -31,6 +31,14 @@
 from . import errors
 
 
+class InvalidPattern(errors.BzrError):
+
+    _fmt = ('Invalid pattern(s) found. %(msg)s')
+
+    def __init__(self, msg):
+        self.msg = msg
+
+
 class LazyRegex(object):
     """A proxy around a real regex, which won't be compiled until accessed."""
 
@@ -71,7 +79,7 @@
         except re.error as e:
             # raise InvalidPattern instead of re.error as this gives a
             # cleaner message to the user.
-            raise errors.InvalidPattern('"' + args[0] + '" ' +str(e))
+            raise InvalidPattern('"' + args[0] + '" ' +str(e))
 
     def __getstate__(self):
         """Return the state to use when pickling."""

=== modified file 'breezy/lockdir.py'
--- breezy/lockdir.py	2017-05-22 00:56:52 +0000
+++ breezy/lockdir.py	2017-07-21 13:20:17 +0000
@@ -136,9 +136,13 @@
         ResourceBusy,
         TransportError,
         )
+from .i18n import gettext
+from .osutils import format_delta, rand_chars, get_host_name
+from .sixish import (
+    PY3,
+    text_type,
+    )
 from .trace import mutter, note
-from .osutils import format_delta, rand_chars, get_host_name
-from .i18n import gettext
 
 from .lazy_import import lazy_import
 lazy_import(globals(), """
@@ -301,7 +305,7 @@
                     ui.ui_factory.show_user_warning(
                         'locks_steal_dead',
                         lock_url=urlutils.join(self.transport.base, self.path),
-                        other_holder_info=unicode(other_holder))
+                        other_holder_info=text_type(other_holder))
                     self.force_break(other_holder)
                     self._trace("stole lock from dead holder")
                     return
@@ -408,7 +412,7 @@
             if ui.ui_factory.confirm_action(
                 u"Break %(lock_info)s",
                 'breezy.lockdir.break',
-                dict(lock_info=unicode(holder_info))):
+                dict(lock_info=text_type(holder_info))):
                 result = self.force_break(holder_info)
                 ui.ui_factory.show_message(
                     "Broke lock %s" % result.lock_url)
@@ -739,6 +743,9 @@
             u'held by %(user)s on %(hostname)s (process #%(pid)s), '
             u'acquired %(time_ago)s') % d)
 
+    if PY3:
+        __str__ = __unicode__
+
     def to_readable_dict(self):
         """Turn the holder info into a dict of human-readable attributes.
 
@@ -804,11 +811,17 @@
         else:
             return cls(stanza.as_dict())
 
-    def __cmp__(self, other):
-        """Value comparison of lock holders."""
-        return (
-            cmp(type(self), type(other))
-            or cmp(self.info_dict, other.info_dict))
+    def __hash__(self):
+        return id(self)
+
+    def __eq__(self, other):
+        """Equality check for lock holders."""
+        if type(self) != type(other):
+            return False
+        return self.info_dict == other.info_dict
+
+    def __ne__(self, other):
+        return not self == other
 
     def is_locked_by_this_process(self):
         """True if this process seems to be the current lock holder."""
@@ -859,5 +872,5 @@
     """
     try:
         return config.GlobalStack().get('email')
-    except errors.NoWhoami:
+    except config.NoWhoami:
         return osutils.getuser_unicode()

=== modified file 'breezy/log.py'
--- breezy/log.py	2017-06-05 20:48:31 +0000
+++ breezy/log.py	2017-06-22 22:44:36 +0000
@@ -409,8 +409,12 @@
 
         # Find and print the interesting revisions
         generator = self._generator_factory(self.branch, rqst)
-        for lr in generator.iter_log_revisions():
-            lf.log_revision(lr)
+        try:
+            for lr in generator.iter_log_revisions():
+                lf.log_revision(lr)
+        except errors.GhostRevisionUnusableHere:
+            raise errors.BzrCommandError(
+                    gettext('Further revision history missing.'))
         lf.show_advice()
 
     def _generator_factory(self, branch, rqst):
@@ -456,6 +460,8 @@
                     continue
                 if omit_merges and len(rev.parent_ids) > 1:
                     continue
+                if rev is None:
+                    raise errors.GhostRevisionUnusableHere(rev_id)
                 if diff_type is None:
                     diff = None
                 else:
@@ -723,6 +729,7 @@
     :param exclude_common_ancestry: Whether the start_rev_id should be part of
         the iterated revisions.
     :return: An iterator of (revision_id, dotted_revno, merge_depth) tuples.
+        dotted_revno will be None for ghosts
     :raises _StartNotLinearAncestor: if a start_rev_id is specified but
         is not found walking the left-hand history
     """
@@ -731,27 +738,44 @@
     graph = repo.get_graph()
     if start_rev_id is None and end_rev_id is None:
         cur_revno = br_revno
-        for revision_id in graph.iter_lefthand_ancestry(br_rev_id,
-            (_mod_revision.NULL_REVISION,)):
-            yield revision_id, str(cur_revno), 0
-            cur_revno -= 1
+        graph_iter = graph.iter_lefthand_ancestry(br_rev_id,
+            (_mod_revision.NULL_REVISION,))
+        while True:
+            try:
+                revision_id = next(graph_iter)
+            except errors.RevisionNotPresent as e:
+                # Oops, a ghost.
+                yield e.revision_id, None, None
+                break
+            else:
+                yield revision_id, str(cur_revno), 0
+                cur_revno -= 1
     else:
         if end_rev_id is None:
             end_rev_id = br_rev_id
         found_start = start_rev_id is None
-        for revision_id in graph.iter_lefthand_ancestry(end_rev_id,
-                (_mod_revision.NULL_REVISION,)):
-            revno_str = _compute_revno_str(branch, revision_id)
-            if not found_start and revision_id == start_rev_id:
-                if not exclude_common_ancestry:
+        graph_iter = graph.iter_lefthand_ancestry(end_rev_id,
+            (_mod_revision.NULL_REVISION,))
+        while True:
+            try:
+                revision_id = next(graph_iter)
+            except StopIteration:
+                break
+            except errors.RevisionNotPresent as e:
+                # Oops, a ghost.
+                yield e.revision_id, None, None
+                break
+            else:
+                revno_str = _compute_revno_str(branch, revision_id)
+                if not found_start and revision_id == start_rev_id:
+                    if not exclude_common_ancestry:
+                        yield revision_id, revno_str, 0
+                    found_start = True
+                    break
+                else:
                     yield revision_id, revno_str, 0
-                found_start = True
-                break
-            else:
-                yield revision_id, revno_str, 0
-        else:
-            if not found_start:
-                raise _StartNotLinearAncestor()
+        if not found_start:
+            raise _StartNotLinearAncestor()
 
 
 def _graph_view_revisions(branch, start_rev_id, end_rev_id,
@@ -861,7 +885,7 @@
     :return: An iterator over lists of ((rev_id, revno, merge_depth), rev,
         delta).
     """
-    if match is None:
+    if not match:
         return log_rev_iterator
     searchRE = [(k, [re.compile(x, re.IGNORECASE) for x in v])
                 for k, v in match.items()]
@@ -998,10 +1022,8 @@
     for revs in log_rev_iterator:
         # r = revision_id, n = revno, d = merge depth
         revision_ids = [view[0] for view, _, _ in revs]
-        revisions = repository.get_revisions(revision_ids)
-        revs = [(rev[0], revision, rev[2]) for rev, revision in
-            zip(revs, revisions)]
-        yield revs
+        revisions = dict(repository.iter_revisions(revision_ids))
+        yield [(rev[0], revisions[rev[0][0]], rev[2]) for rev in revs]
 
 
 def _make_batch_filter(branch, generate_delta, search, log_rev_iterator):

=== modified file 'breezy/merge.py'
--- breezy/merge.py	2017-06-10 01:39:46 +0000
+++ breezy/merge.py	2017-06-24 13:32:35 +0000
@@ -280,7 +280,7 @@
     hooks = MergeHooks()
 
     def __init__(self, this_branch, other_tree=None, base_tree=None,
-                 this_tree=None, pb=None, change_reporter=None,
+                 this_tree=None, change_reporter=None,
                  recurse='down', revision_graph=None):
         object.__init__(self)
         self.this_branch = this_branch
@@ -299,8 +299,6 @@
         self.interesting_files = None
         self.show_base = False
         self.reprocess = False
-        if pb is not None:
-            warnings.warn("pb parameter to Merger() is deprecated and ignored")
         self.pp = None
         self.recurse = recurse
         self.change_reporter = change_reporter
@@ -354,30 +352,28 @@
                                       _set_base_is_other_ancestor)
 
     @staticmethod
-    def from_uncommitted(tree, other_tree, pb=None, base_tree=None):
+    def from_uncommitted(tree, other_tree, base_tree=None):
         """Return a Merger for uncommitted changes in other_tree.
 
         :param tree: The tree to merge into
         :param other_tree: The tree to get uncommitted changes from
-        :param pb: A progress indicator
         :param base_tree: The basis to use for the merge.  If unspecified,
             other_tree.basis_tree() will be used.
         """
         if base_tree is None:
             base_tree = other_tree.basis_tree()
-        merger = Merger(tree.branch, other_tree, base_tree, tree, pb)
+        merger = Merger(tree.branch, other_tree, base_tree, tree)
         merger.base_rev_id = merger.base_tree.get_revision_id()
         merger.other_rev_id = None
         merger.other_basis = merger.base_rev_id
         return merger
 
     @classmethod
-    def from_mergeable(klass, tree, mergeable, pb):
+    def from_mergeable(klass, tree, mergeable):
         """Return a Merger for a bundle or merge directive.
 
         :param tree: The tree to merge changes into
         :param mergeable: A merge directive or bundle
-        :param pb: A progress indicator
         """
         mergeable.install_revisions(tree.branch.repository)
         base_revision_id, other_revision_id, verified =\
@@ -390,18 +386,17 @@
                 base_revision_id = None
             else:
                 trace.warning('Performing cherrypick')
-        merger = klass.from_revision_ids(pb, tree, other_revision_id,
+        merger = klass.from_revision_ids(tree, other_revision_id,
                                          base_revision_id, revision_graph=
                                          revision_graph)
         return merger, verified
 
     @staticmethod
-    def from_revision_ids(pb, tree, other, base=None, other_branch=None,
+    def from_revision_ids(tree, other, base=None, other_branch=None,
                           base_branch=None, revision_graph=None,
                           tree_branch=None):
         """Return a Merger for revision-ids.
 
-        :param pb: A progress indicator
         :param tree: The tree to merge changes into
         :param other: The revision-id to use as OTHER
         :param base: The revision-id to use as BASE.  If not specified, will
@@ -417,7 +412,7 @@
         """
         if tree_branch is None:
             tree_branch = tree.branch
-        merger = Merger(tree_branch, this_tree=tree, pb=pb,
+        merger = Merger(tree_branch, this_tree=tree,
                         revision_graph=revision_graph)
         if other_branch is None:
             other_branch = tree.branch
@@ -642,8 +637,7 @@
         if self._is_criss_cross and getattr(self.merge_type,
                                             'supports_lca_trees', False):
             kwargs['lca_trees'] = self._lca_trees
-        return self.merge_type(pb=None,
-                               change_reporter=self.change_reporter,
+        return self.merge_type(change_reporter=self.change_reporter,
                                **kwargs)
 
     def _do_merge_to(self):
@@ -725,8 +719,7 @@
 
     def __init__(self, working_tree, this_tree, base_tree, other_tree,
                  interesting_ids=None, reprocess=False, show_base=False,
-                 pb=None, pp=None, change_reporter=None,
-                 interesting_files=None, do_merge=True,
+                 change_reporter=None, interesting_files=None, do_merge=True,
                  cherrypick=False, lca_trees=None, this_branch=None,
                  other_branch=None):
         """Initialize the merger object and perform the merge.
@@ -744,8 +737,6 @@
         :param: reprocess If True, perform conflict-reduction processing.
         :param show_base: If True, show the base revision in text conflicts.
             (incompatible with reprocess)
-        :param pb: ignored
-        :param pp: A ProgressPhase object
         :param change_reporter: An object that should report changes made
         :param interesting_files: The tree-relative paths of files that should
             participate in the merge.  If these paths refer to directories,
@@ -785,10 +776,6 @@
         self.cherrypick = cherrypick
         if do_merge:
             self.do_merge()
-        if pp is not None:
-            warnings.warn("pp argument to Merge3Merger is deprecated")
-        if pb is not None:
-            warnings.warn("pb argument to Merge3Merger is deprecated")
 
     def do_merge(self):
         operation = cleanup.OperationWithCleanups(self._do_merge)
@@ -1922,7 +1909,6 @@
                 other_rev_id=None,
                 interesting_files=None,
                 this_tree=None,
-                pb=None,
                 change_reporter=None):
     """Primary interface for merging.
 
@@ -1935,7 +1921,7 @@
         raise errors.BzrError("breezy.merge.merge_inner requires a this_tree "
                               "parameter")
     merger = Merger(this_branch, other_tree, base_tree, this_tree=this_tree,
-                    pb=pb, change_reporter=change_reporter)
+                    change_reporter=change_reporter)
     merger.backup_files = backup_files
     merger.merge_type = merge_type
     merger.interesting_ids = interesting_ids

=== modified file 'breezy/missing.py'
--- breezy/missing.py	2017-05-30 19:16:23 +0000
+++ breezy/missing.py	2017-06-24 13:32:35 +0000
@@ -30,14 +30,7 @@
 
     if rev_tag_dict is None:
         rev_tag_dict = {}
-    for rev in revisions:
-        # We need the following for backward compatibilty (hopefully
-        # this will be deprecated soon :-/) -- vila 080911
-        if len(rev) == 2:
-            revno, rev_id = rev
-            merge_depth = 0
-        else:
-            revno, rev_id, merge_depth = rev
+    for revno, rev_id, merge_depth in revisions:
         rev = revision_source.get_revision(rev_id)
         if verbose:
             delta = revision_source.get_revision_delta(rev_id)
@@ -97,7 +90,7 @@
     :param tip: The tip of mainline
     :param backward: Show oldest versions first when True, newest versions
         first when False.
-    :return: [(revno, revision_id)] for all revisions in ancestry that
+    :return: [(revno, revision_id, 0)] for all revisions in ancestry that
         are left-hand parents from tip, or None if ancestry is None.
     """
     if ancestry is None:
@@ -117,7 +110,7 @@
         parents = parent_map.get(cur)
         if not parents:
             break # Ghost, we are done
-        mainline.append((str(cur_revno), cur))
+        mainline.append((str(cur_revno), cur, 0))
         cur = parents[0]
         cur_revno -= 1
     if not backward:

=== modified file 'breezy/mutabletree.py'
--- breezy/mutabletree.py	2017-06-10 23:37:36 +0000
+++ breezy/mutabletree.py	2017-06-14 23:29:06 +0000
@@ -33,6 +33,7 @@
 
 from .decorators import needs_read_lock, needs_write_lock
 from .sixish import (
+    text_type,
     viewvalues,
     )
 
@@ -106,12 +107,12 @@
 
         TODO: Perhaps callback with the ids and paths as they're added.
         """
-        if isinstance(files, basestring):
+        if isinstance(files, (str, text_type)):
             # XXX: Passing a single string is inconsistent and should be
             # deprecated.
-            if not (ids is None or isinstance(ids, basestring)):
+            if not (ids is None or isinstance(ids, (str, text_type))):
                 raise AssertionError()
-            if not (kinds is None or isinstance(kinds, basestring)):
+            if not (kinds is None or isinstance(kinds, (str, text_type))):
                 raise AssertionError()
             files = [files]
             if ids is not None:

=== modified file 'breezy/option.py'
--- breezy/option.py	2017-05-22 00:56:52 +0000
+++ breezy/option.py	2017-07-15 15:22:22 +0000
@@ -25,13 +25,13 @@
 from .lazy_import import lazy_import
 lazy_import(globals(), """
 from breezy import (
-    errors,
     revisionspec,
     i18n,
     )
 """)
 
 from . import (
+    errors,
     registry as _mod_registry,
     )
 from .sixish import (
@@ -39,6 +39,14 @@
     )
 
 
+class BadOptionValue(errors.BzrError):
+
+    _fmt = """Bad value "%(value)s" for option "%(name)s"."""
+
+    def __init__(self, name, value):
+        errors.BzrError.__init__(self, name=name, value=value)
+
+
 def _parse_revision_str(revstr):
     """This handles a revision string -> revno.
 
@@ -306,7 +314,7 @@
     def validate_value(self, value):
         """Validate a value name"""
         if value not in self.registry:
-            raise errors.BadOptionValue(self.name, value)
+            raise BadOptionValue(self.name, value)
 
     def convert(self, value):
         """Convert a value name into an output type"""

=== modified file 'breezy/osutils.py'
--- breezy/osutils.py	2017-06-11 01:19:16 +0000
+++ breezy/osutils.py	2017-06-15 23:20:39 +0000
@@ -1076,13 +1076,13 @@
 def report_extension_load_failures():
     if not _extension_load_failures:
         return
-    if config.GlobalStack().get('ignore_missing_extensions'):
+    if config.GlobalConfig().suppress_warning('missing_extensions'):
         return
     # the warnings framework should by default show this only once
     from .trace import warning
     warning(
         "brz: warning: some compiled extensions could not be loaded; "
-        "see <https://answers.launchpad.net/bzr/+faq/703>")
+        "see ``brz help missing-extensions``")
     # we no longer show the specific missing extensions here, because it makes
     # the message too long and scary - see
     # https://bugs.launchpad.net/bzr/+bug/430529
@@ -1391,7 +1391,7 @@
     :return: None or a utf8 revision id.
     """
     if (unicode_or_utf8_string is None
-        or unicode_or_utf8_string.__class__ == str):
+        or unicode_or_utf8_string.__class__ == bytes):
         return unicode_or_utf8_string
     raise TypeError('Unicode revision ids are no longer supported. '
                     'Revision id generators should be creating utf8 revision '
@@ -1409,7 +1409,7 @@
     :return: None or a utf8 file id.
     """
     if (unicode_or_utf8_string is None
-        or unicode_or_utf8_string.__class__ == str):
+        or unicode_or_utf8_string.__class__ == bytes):
         return unicode_or_utf8_string
     raise TypeError('Unicode file ids are no longer supported. '
                     'File id generators should be creating utf8 file ids.')
@@ -2327,13 +2327,11 @@
 
     concurrency = os.environ.get('BRZ_CONCURRENCY', None)
     if concurrency is None:
+        import multiprocessing
         try:
-            import multiprocessing
             concurrency = multiprocessing.cpu_count()
-        except (ImportError, NotImplementedError):
-            # multiprocessing is only available on Python >= 2.6
-            # and multiprocessing.cpu_count() isn't implemented on all
-            # platforms
+        except NotImplementedError:
+            # multiprocessing.cpu_count() isn't implemented on all platforms
             try:
                 concurrency = _local_concurrency()
             except (OSError, IOError):

=== modified file 'breezy/patches.py'
--- breezy/patches.py	2017-05-25 01:35:55 +0000
+++ breezy/patches.py	2017-07-10 20:13:57 +0000
@@ -18,12 +18,7 @@
 from __future__ import absolute_import
 
 from .errors import (
-    BinaryFiles,
-    MalformedHunkHeader,
-    MalformedLine,
-    MalformedPatchHeader,
-    PatchConflict,
-    PatchSyntax,
+    BzrError,
     )
 
 import re
@@ -32,6 +27,57 @@
 binary_files_re = 'Binary files (.*) and (.*) differ\n'
 
 
+class PatchSyntax(BzrError):
+    """Base class for patch syntax errors."""
+
+
+class BinaryFiles(BzrError):
+
+    _fmt = 'Binary files section encountered.'
+
+    def __init__(self, orig_name, mod_name):
+        self.orig_name = orig_name
+        self.mod_name = mod_name
+
+
+class MalformedPatchHeader(PatchSyntax):
+
+    _fmt = "Malformed patch header.  %(desc)s\n%(line)r"
+
+    def __init__(self, desc, line):
+        self.desc = desc
+        self.line = line
+
+
+class MalformedLine(PatchSyntax):
+
+    _fmt = "Malformed line.  %(desc)s\n%(line)r"
+
+    def __init__(self, desc, line):
+        self.desc = desc
+        self.line = line
+
+
+class PatchConflict(BzrError):
+
+    _fmt = ('Text contents mismatch at line %(line_no)d.  Original has '
+            '"%(orig_line)s", but patch says it should be "%(patch_line)s"')
+
+    def __init__(self, line_no, orig_line, patch_line):
+        self.line_no = line_no
+        self.orig_line = orig_line.rstrip('\n')
+        self.patch_line = patch_line.rstrip('\n')
+
+
+class MalformedHunkHeader(PatchSyntax):
+
+    _fmt = "Malformed hunk header.  %(desc)s\n%(line)r"
+
+    def __init__(self, desc, line):
+        self.desc = desc
+        self.line = line
+
+
 def get_patch_names(iter_lines):
     line = next(iter_lines)
     try:

=== modified file 'breezy/plugins/bash_completion/tests/test_bashcomp.py'
--- breezy/plugins/bash_completion/tests/test_bashcomp.py	2017-05-22 00:56:52 +0000
+++ breezy/plugins/bash_completion/tests/test_bashcomp.py	2017-06-19 20:34:29 +0000
@@ -120,7 +120,7 @@
 
     def test_init_opts(self):
         self.complete(['brz', 'init', '-'])
-        self.assertCompletionContains('-h', '--2a', '--format=2a')
+        self.assertCompletionContains('-h', '--format=2a')
 
     def test_global_opts(self):
         self.complete(['brz', '-', 'init'], cword=1)

=== modified file 'breezy/plugins/email/emailer.py'
--- breezy/plugins/email/emailer.py	2017-06-10 00:35:59 +0000
+++ breezy/plugins/email/emailer.py	2017-06-14 23:29:06 +0000
@@ -242,8 +242,6 @@
         subject = self.subject()
         from_addr = self.from_address()
         to_addrs = self.to()
-        if isinstance(to_addrs, basestring):
-            to_addrs = [to_addrs]
         header = self.extra_headers()
 
         msg = EmailMessage(from_addr, to_addrs, subject, body)

=== modified file 'breezy/plugins/fastimport/revision_store.py'
--- breezy/plugins/fastimport/revision_store.py	2017-06-10 01:39:46 +0000
+++ breezy/plugins/fastimport/revision_store.py	2017-06-20 02:08:23 +0000
@@ -415,12 +415,9 @@
         # This is a duplicate of Builder.commit() since we already have the
         # Revision object, and we *don't* want to call commit_write_group()
         rev.inv_sha1 = builder.inv_sha1
-        try:
-            config = builder._config_stack
-        except AttributeError: # bzr < 2.5
-            config = builder._config
+        config = builder._config_stack
         builder.repository.add_revision(builder._new_revision_id, rev,
-            builder.new_inventory)
+            builder.revision_tree().root_inventory)
         if self._graph is not None:
             # TODO: Use StaticTuple and .intern() for these things
             self._graph.add_node(builder._new_revision_id, rev.parent_ids)

=== modified file 'breezy/plugins/grep/grep.py'
--- breezy/plugins/grep/grep.py	2017-06-07 00:24:29 +0000
+++ breezy/plugins/grep/grep.py	2017-06-13 01:12:46 +0000
@@ -653,15 +653,7 @@
 
     if opts.files_with_matches or opts.files_without_match:
         if opts.fixed_string:
-            if sys.platform > (2, 5):
-                found = pattern in file_text
-            else:
-                for line in file_text.splitlines():
-                    if pattern in line:
-                        found = True
-                        break
-                else:
-                    found = False
+            found = pattern in file_text
         else:
             search = opts.patternc.search
             if "$" not in pattern:

=== modified file 'breezy/plugins/launchpad/cmds.py'
--- breezy/plugins/launchpad/cmds.py	2017-06-02 01:01:21 +0000
+++ breezy/plugins/launchpad/cmds.py	2017-07-15 13:23:08 +0000
@@ -28,7 +28,6 @@
     )
 from ...errors import (
     BzrCommandError,
-    InvalidURL,
     NotBranchError,
     )
 from ...i18n import gettext
@@ -65,6 +64,7 @@
 
     def _get_web_url(self, service, location):
         from .lp_registration import (
+            InvalidURL,
             NotLaunchpadBranch)
         for branch_url in self._possible_locations(location):
             try:

=== modified file 'breezy/plugins/launchpad/lp_directory.py'
--- breezy/plugins/launchpad/lp_directory.py	2017-05-22 00:56:52 +0000
+++ breezy/plugins/launchpad/lp_directory.py	2017-07-15 13:23:08 +0000
@@ -30,7 +30,10 @@
 from ...i18n import gettext
 
 from .lp_registration import (
-    LaunchpadService, ResolveLaunchpadPathRequest)
+    InvalidURL,
+    LaunchpadService,
+    ResolveLaunchpadPathRequest,
+    )
 from .account import get_lp_login
 
 
@@ -99,7 +102,7 @@
         try:
             result = resolve.submit(service)
         except xmlrpclib.Fault as fault:
-            raise errors.InvalidURL(
+            raise InvalidURL(
                 path=url, extra=fault.faultString)
         return result
 
@@ -132,7 +135,7 @@
             else:
                 # There are either 0 or > 2 path parts, neither of which is
                 # supported for these schemes.
-                raise errors.InvalidURL('Bad path: %s' % url)
+                raise InvalidURL('Bad path: %s' % url)
             # Expand any series shortcuts, but keep unknown series.
             series = distro_series.get(series, series)
             # Hack the url and let the following do the final resolution.
@@ -146,7 +149,7 @@
     def _expand_user(self, path, url, lp_login):
         if path.startswith('~/'):
             if lp_login is None:
-                raise errors.InvalidURL(path=url,
+                raise InvalidURL(path=url,
                     extra='Cannot resolve "~" to your username.'
                           ' See "bzr help launchpad-login"')
             path = '~' + lp_login + path[1:]
@@ -199,7 +202,7 @@
                 else:
                     break
         else:
-            raise errors.InvalidURL(path=url, extra='no supported schemes')
+            raise InvalidURL(path=url, extra='no supported schemes')
         return url
 
 

=== modified file 'breezy/plugins/launchpad/lp_registration.py'
--- breezy/plugins/launchpad/lp_registration.py	2017-06-05 20:48:31 +0000
+++ breezy/plugins/launchpad/lp_registration.py	2017-07-15 13:23:08 +0000
@@ -37,6 +37,12 @@
 export BRZ_LP_XMLRPC_URL=http://xmlrpc.staging.launchpad.net/bazaar/
 '''
 
+
+class InvalidURL(errors.PathError):
+
+    _fmt = 'Invalid url supplied to transport: "%(path)s"%(extra)s'
+
+
 class InvalidLaunchpadInstance(errors.BzrError):
 
     _fmt = "%(lp_instance)s is not a valid Launchpad instance."
@@ -142,7 +148,7 @@
         if lp_instance == '':
             lp_instance = None
         elif lp_instance not in cls.LAUNCHPAD_INSTANCE:
-            raise errors.InvalidURL(path=url)
+            raise InvalidURL(url)
         return cls(lp_instance=lp_instance, **kwargs)
 
     def get_proxy(self):
@@ -191,7 +197,7 @@
             try:
                 result = resolve.submit(self)
             except xmlrpclib.Fault as fault:
-                raise errors.InvalidURL(branch_url, str(fault))
+                raise InvalidURL(branch_url, str(fault))
             branch_url = result['urls'][0]
             path = urlsplit(branch_url)[2]
         else:
@@ -205,7 +211,7 @@
     def get_web_url_from_branch_url(self, branch_url, _request_factory=None):
         """Get the Launchpad web URL for the given branch URL.
 
-        :raise errors.InvalidURL: if 'branch_url' cannot be identified as a
+        :raise InvalidURL: if 'branch_url' cannot be identified as a
             Launchpad branch URL.
         :return: The URL of the branch on Launchpad.
         """
@@ -245,8 +251,7 @@
 
     def __init__(self, path):
         if not path:
-            raise errors.InvalidURL(path=path,
-                                    extra="You must specify a project.")
+            raise InvalidURL(url=path, extra="You must specify a project.")
         self.path = path
 
     def _request_params(self):

=== modified file 'breezy/plugins/launchpad/test_lp_directory.py'
--- breezy/plugins/launchpad/test_lp_directory.py	2017-05-23 14:08:03 +0000
+++ breezy/plugins/launchpad/test_lp_directory.py	2017-07-15 13:23:08 +0000
@@ -129,7 +129,7 @@
         :seealso: http://pad.lv/843900
         """
         # This ought to be natty-updates.
-        self.assertRaises(errors.InvalidURL,
+        self.assertRaises(lp_registration.InvalidURL,
             self.assertResolve,
             '',
             'ubuntu:natty/updates/smartpm')
@@ -287,7 +287,7 @@
             self, 'apt', dict(urls=[
                     'bad-scheme://bazaar.launchpad.net/~apt/apt/devel']))
         directory = LaunchpadDirectory()
-        self.assertRaises(errors.InvalidURL,
+        self.assertRaises(lp_registration.InvalidURL,
                           directory._resolve, 'lp:///apt', factory)
 
     def test_directory_fault(self):
@@ -297,7 +297,7 @@
             raise xmlrpclib.Fault(42, 'something went wrong')
         factory.submit = submit
         directory = LaunchpadDirectory()
-        self.assertRaises(errors.InvalidURL,
+        self.assertRaises(lp_registration.InvalidURL,
                           directory._resolve, 'lp:///apt', factory)
 
     def test_skip_bzr_ssh_launchpad_net_when_anonymous(self):
@@ -350,7 +350,7 @@
     # TODO: check we get an error if the url is unreasonable
     def test_error_for_bad_url(self):
         directory = LaunchpadDirectory()
-        self.assertRaises(errors.InvalidURL,
+        self.assertRaises(lp_registration.InvalidURL,
             directory._resolve, 'lp://ratotehunoahu')
 
     def test_resolve_tilde_to_user(self):
@@ -373,7 +373,7 @@
                     'bzr+ssh://bazaar.launchpad.net/~username/apt/test']))
         self.assertIs(None, get_lp_login())
         directory = LaunchpadDirectory()
-        self.assertRaises(errors.InvalidURL,
+        self.assertRaises(lp_registration.InvalidURL,
                           directory._resolve, 'lp:~/apt/test', factory)
 
 
@@ -545,15 +545,15 @@
     # Bogus distro.
 
     def test_bogus_distro(self):
-        self.assertRaises(errors.InvalidURL,
+        self.assertRaises(lp_registration.InvalidURL,
                           self.directory._resolve, 'gentoo:foo')
 
     def test_trick_bogus_distro_u(self):
-        self.assertRaises(errors.InvalidURL,
+        self.assertRaises(lp_registration.InvalidURL,
                           self.directory._resolve, 'utube:foo')
 
     def test_trick_bogus_distro_d(self):
-        self.assertRaises(errors.InvalidURL,
+        self.assertRaises(lp_registration.InvalidURL,
                           self.directory._resolve, 'debuntu:foo')
 
     def test_missing_ubuntu_distroseries_without_project(self):

=== modified file 'breezy/plugins/launchpad/test_lp_service.py'
--- breezy/plugins/launchpad/test_lp_service.py	2017-05-22 00:56:52 +0000
+++ breezy/plugins/launchpad/test_lp_service.py	2017-07-15 13:23:08 +0000
@@ -21,7 +21,11 @@
 
 from ... import errors
 from .lp_registration import (
-    InvalidLaunchpadInstance, LaunchpadService, NotLaunchpadBranch)
+    InvalidURL,
+    InvalidLaunchpadInstance,
+    LaunchpadService,
+    NotLaunchpadBranch,
+    )
 from .test_lp_directory import FakeResolveFactory
 from ...tests import TestCase
 
@@ -156,7 +160,7 @@
             raise xmlrpclib.Fault(42, 'something went wrong')
         factory.submit = submit
         self.assertRaises(
-            errors.InvalidURL, service.get_web_url_from_branch_url, 'lp:foo',
+            InvalidURL, service.get_web_url_from_branch_url, 'lp:foo',
             factory)
 
     def test_staging_url(self):

=== modified file 'breezy/plugins/stats/cmds.py'
--- breezy/plugins/stats/cmds.py	2017-06-05 21:24:34 +0000
+++ breezy/plugins/stats/cmds.py	2017-06-22 01:28:40 +0000
@@ -148,8 +148,8 @@
     pb = ui.ui_factory.nested_progress_bar()
     try:
         trace.note('getting revisions')
-        revisions = a_repo.get_revisions(revids)
-        for count, rev in enumerate(revisions):
+        revisions = a_repo.iter_revisions(revids)
+        for count, (revid, rev) in enumerate(revisions):
             pb.update('checking', count, len(revids))
             for author in rev.get_apparent_authors():
                 # XXX: There is a chance sometimes with svn imports that the

=== modified file 'breezy/plugins/weave_fmt/__init__.py'
--- breezy/plugins/weave_fmt/__init__.py	2017-06-11 20:33:20 +0000
+++ breezy/plugins/weave_fmt/__init__.py	2017-06-14 22:34:05 +0000
@@ -33,7 +33,7 @@
 from ...bzr import (
     serializer,
     )
-from ...bzr.bzrdir import (
+from ...bzr import (
     BzrProber,
     register_metadir,
     )

=== modified file 'breezy/plugins/weave_fmt/repository.py'
--- breezy/plugins/weave_fmt/repository.py	2017-06-11 20:33:20 +0000
+++ breezy/plugins/weave_fmt/repository.py	2017-06-22 01:06:22 +0000
@@ -61,6 +61,7 @@
     )
 from ...sixish import (
     BytesIO,
+    text_type,
     )
 from .store.text import TextStore
 from ...bzr.versionedfile import (
@@ -88,7 +89,7 @@
         return xml5.serializer_v5
 
     def _escape(self, file_or_path):
-        if not isinstance(file_or_path, basestring):
+        if not isinstance(file_or_path, (str, text_type)):
             file_or_path = '/'.join(file_or_path)
         if file_or_path == '':
             return u''
@@ -165,11 +166,6 @@
         self.start_write_group()
         return result
 
-    @needs_read_lock
-    def get_revisions(self, revision_ids):
-        revs = self._get_revisions(revision_ids)
-        return revs
-
     def _inventory_add_lines(self, revision_id, parents, lines,
         check_content=True):
         """Store lines in inv_vf and return the sha1 of the inventory."""

=== modified file 'breezy/plugins/weave_fmt/test_bzrdir.py'
--- breezy/plugins/weave_fmt/test_bzrdir.py	2017-06-11 01:22:16 +0000
+++ breezy/plugins/weave_fmt/test_bzrdir.py	2017-06-20 21:55:37 +0000
@@ -39,7 +39,7 @@
 from ...osutils import (
     getcwd,
     )
-from ...tests.test_bundle import V4BundleTester
+from ...tests import test_bundle
 from ...tests.test_sftp_transport import TestCaseWithSFTPServer
 from ...tests import (
     TestCaseWithTransport,
@@ -580,7 +580,7 @@
         self.assertEqual('', err)
 
 
-class V4WeaveBundleTester(V4BundleTester):
+class V4WeaveBundleTester(test_bundle.V4BundleTester):
 
     def bzrdir_format(self):
         return 'metaweave'

=== modified file 'breezy/python-compat.h'
--- breezy/python-compat.h	2017-06-05 23:59:08 +0000
+++ breezy/python-compat.h	2017-06-22 00:40:51 +0000
@@ -25,6 +25,37 @@
 #ifndef _BZR_PYTHON_COMPAT_H
 #define _BZR_PYTHON_COMPAT_H
 
+#if PY_MAJOR_VERSION >= 3
+
+#define PyInt_FromSsize_t PyLong_FromSsize_t
+
+/* In Python 3 the Py_TPFLAGS_CHECKTYPES behaviour is on by default */
+#define Py_TPFLAGS_CHECKTYPES 0
+
+#define PYMOD_ERROR NULL
+#define PYMOD_SUCCESS(val) val
+#define PYMOD_INIT_FUNC(name) PyMODINIT_FUNC PyInit_##name(void)
+#define PYMOD_CREATE(ob, name, doc, methods) do { \
+    static struct PyModuleDef moduledef = { \
+        PyModuleDef_HEAD_INIT, name, doc, -1, methods \
+    }; \
+    ob = PyModule_Create(&moduledef); \
+    } while(0)
+
+#else
+
+#define PyBytes_Type PyString_Type
+#define PyBytes_CheckExact PyString_CheckExact
+
+#define PYMOD_ERROR
+#define PYMOD_SUCCESS(val)
+#define PYMOD_INIT_FUNC(name) void init##name(void)
+#define PYMOD_CREATE(ob, name, doc, methods) do { \
+    ob = Py_InitModule3(name, methods, doc); \
+    } while(0)
+
+#endif
+
 #if defined(_WIN32) || defined(WIN32)
     /* Defining WIN32_LEAN_AND_MEAN makes including windows quite a bit
      * lighter weight.

=== modified file 'breezy/repository.py'
--- breezy/repository.py	2017-06-11 13:48:12 +0000
+++ breezy/repository.py	2017-06-22 01:06:22 +0000
@@ -74,8 +74,6 @@
 
     # all clients should supply tree roots.
     record_root_entry = True
-    # whether this commit builder supports the record_entry_contents interface
-    supports_record_entry_contents = False
     # whether this commit builder will automatically update the branch that is
     # being committed to
     updates_branch = False
@@ -201,15 +199,6 @@
         else:
             self.random_revid = False
 
-    def will_record_deletes(self):
-        """Tell the commit builder that deletes are being notified.
-
-        This enables the accumulation of an inventory delta; for the resulting
-        commit to be valid, deletes against the basis MUST be recorded via
-        builder.record_delete().
-        """
-        raise NotImplementedError(self.will_record_deletes)
-
     def record_iter_changes(self, tree, basis_revision_id, iter_changes):
         """Record a new tree via iter_changes.
 
@@ -833,11 +822,29 @@
 
     def get_revisions(self, revision_ids):
         """Get many revisions at once.
-        
-        Repositories that need to check data on every revision read should 
+
+        Repositories that need to check data on every revision read should
         subclass this method.
         """
-        raise NotImplementedError(self.get_revisions)
+        revs = {}
+        for revid, rev in self.iter_revisions(revision_ids):
+            if rev is None:
+                raise errors.NoSuchRevision(self, revid)
+            revs[revid] = rev
+        return [revs[revid] for revid in revision_ids]
+
+    def iter_revisions(self, revision_ids):
+        """Iterate over revision objects.
+
+        :param revision_ids: An iterable of revisions to examine. None may be
+            passed to request all revisions known to the repository. Note that
+            not all repositories can find unreferenced revisions; for those
+            repositories only referenced ones will be returned.
+        :return: An iterator of (revid, revision) tuples. Absent revisions (
+            those asked for but not available) are returned as (revid, None).
+            N.B.: Revisions are not necessarily yielded in order.
+        """
+        raise NotImplementedError(self.iter_revisions)
 
     def get_deltas_for_revisions(self, revisions, specific_fileids=None):
         """Produce a generator of revision deltas.

=== modified file 'breezy/revision.py'
--- breezy/revision.py	2017-06-11 01:19:16 +0000
+++ breezy/revision.py	2017-07-10 20:12:25 +0000
@@ -152,9 +152,9 @@
             try:
                 url, status = line.split(None, 2)
             except ValueError:
-                raise errors.InvalidLineInBugsProperty(line)
+                raise bugtracker.InvalidLineInBugsProperty(line)
             if status not in bugtracker.ALLOWED_BUG_STATUSES:
-                raise errors.InvalidBugStatus(status)
+                raise bugtracker.InvalidBugStatus(status)
             yield url, status
 
 

=== modified file 'breezy/revisionspec.py'
--- breezy/revisionspec.py	2017-05-30 19:16:23 +0000
+++ breezy/revisionspec.py	2017-06-15 22:34:55 +0000
@@ -38,6 +38,9 @@
     registry,
     trace,
     )
+from .sixish import (
+    text_type,
+    )
 
 
 class RevisionInfo(object):
@@ -154,11 +157,10 @@
         :return: A RevisionSpec object that understands how to parse the
             supplied notation.
         """
-        if not isinstance(spec, (type(None), basestring)):
-            raise TypeError('error')
-
         if spec is None:
             return RevisionSpec(None, _internal=True)
+        if not isinstance(spec, (str, text_type)):
+            raise TypeError("revision spec needs to be text")
         match = revspec_registry.get_prefix(spec)
         if match is not None:
             spectype, specsuffix = match

=== modified file 'breezy/shelf.py'
--- breezy/shelf.py	2017-06-08 23:30:31 +0000
+++ breezy/shelf.py	2017-06-24 13:32:35 +0000
@@ -344,11 +344,11 @@
         tt.deserialize(records)
         return klass(tree, base_tree, tt, metadata.get('message'))
 
-    def make_merger(self, task=None):
+    def make_merger(self):
         """Return a merger that can unshelve the changes."""
         target_tree = self.transform.get_preview_tree()
         merger = merge.Merger.from_uncommitted(self.tree, target_tree,
-            task, self.base_tree)
+            self.base_tree)
         merger.merge_type = merge.Merge3Merger
         return merger
 

=== modified file 'breezy/shelf_ui.py'
--- breezy/shelf_ui.py	2017-05-22 00:56:52 +0000
+++ breezy/shelf_ui.py	2017-06-24 13:32:35 +0000
@@ -466,7 +466,7 @@
                 if unshelver.message is not None:
                     trace.note(gettext('Message: %s') % unshelver.message)
                 change_reporter = delta._ChangeReporter()
-                merger = unshelver.make_merger(None)
+                merger = unshelver.make_merger()
                 merger.change_reporter = change_reporter
                 if self.apply_changes:
                     merger.do_merge()

=== modified file 'breezy/status.py'
--- breezy/status.py	2017-05-25 01:35:55 +0000
+++ breezy/status.py	2017-06-22 01:28:40 +0000
@@ -297,7 +297,7 @@
     log_formatter = log.LineLogFormatter(to_file)
     for merge in pending:
         try:
-            rev = branch.repository.get_revisions([merge])[0]
+            rev = branch.repository.get_revision(merge)
         except errors.NoSuchRevision:
             # If we are missing a revision, just print out the revision id
             to_file.write(first_prefix + '(ghost) ' + merge + '\n')
@@ -316,19 +316,7 @@
         merge_extra.discard(_mod_revision.NULL_REVISION)
 
         # Get a handle to all of the revisions we will need
-        try:
-            revisions = dict((rev.revision_id, rev) for rev in
-                             branch.repository.get_revisions(merge_extra))
-        except errors.NoSuchRevision:
-            # One of the sub nodes is a ghost, check each one
-            revisions = {}
-            for revision_id in merge_extra:
-                try:
-                    rev = branch.repository.get_revisions([revision_id])[0]
-                except errors.NoSuchRevision:
-                    revisions[revision_id] = None
-                else:
-                    revisions[revision_id] = rev
+        revisions = dict(branch.repository.iter_revisions(merge_extra))
 
         # Display the revisions brought in by this merge.
         rev_id_iterator = _get_sorted_revisions(merge, merge_extra,

=== modified file 'breezy/tests/__init__.py'
--- breezy/tests/__init__.py	2017-06-11 20:33:20 +0000
+++ breezy/tests/__init__.py	2017-07-02 13:44:33 +0000
@@ -599,15 +599,10 @@
 
     def __init__(self, stream, descriptions, verbosity,
                  bench_history=None,
-                 pb=None,
                  strict=None,
                  ):
         ExtendedTestResult.__init__(self, stream, descriptions, verbosity,
             bench_history, strict)
-        # We no longer pass them around, but just rely on the UIFactory stack
-        # for state
-        if pb is not None:
-            warnings.warn("Passing pb to TextTestResult is deprecated")
         self.pb = self.ui.nested_progress_bar()
         self.pb.show_pct = False
         self.pb.show_spinner = False
@@ -794,11 +789,7 @@
         # to encode using ascii.
         new_encoding = osutils.get_terminal_encoding()
         codec = codecs.lookup(new_encoding)
-        if isinstance(codec, tuple):
-            # Python 2.4
-            encode = codec[0]
-        else:
-            encode = codec.encode
+        encode = codec.encode
         # GZ 2010-09-08: Really we don't want to be writing arbitrary bytes,
         #                so should swap to the plain codecs.StreamWriter
         stream = osutils.UnicodeOrBytesToBytesWriter(encode, stream,
@@ -1042,7 +1033,7 @@
                                   % (counter_name,))
         _counters[counter_name] = 0
         self.addDetail(counter_name, content.Content(content.UTF8_TEXT,
-            lambda: ['%d' % (_counters[counter_name],)]))
+            lambda: [b'%d' % (_counters[counter_name],)]))
         def increment_counter(*args, **kwargs):
             _counters[counter_name] += 1
         label = 'count %s calls' % (counter_name,)
@@ -1560,7 +1551,7 @@
 
     def assertPathDoesNotExist(self, path):
         """Fail if path or paths, which may be abs or relative, exist."""
-        if not isinstance(path, basestring):
+        if not isinstance(path, (str, text_type)):
             for p in path:
                 self.assertPathDoesNotExist(p)
         else:
@@ -1889,8 +1880,8 @@
         self._benchcalls.
         """
         if self._benchtime is None:
-            self.addDetail('benchtime', content.Content(content.ContentType(
-                "text", "plain"), lambda:[str(self._benchtime)]))
+            self.addDetail('benchtime', content.Content(content.UTF8_TEXT,
+                lambda:[str(self._benchtime).encode('utf-8')]))
             self._benchtime = 0
         start = time.time()
         try:
@@ -2091,7 +2082,7 @@
         if len(args) == 1:
             if isinstance(args[0], list):
                 args = args[0]
-            elif isinstance(args[0], basestring):
+            elif isinstance(args[0], (str, text_type)):
                 args = list(shlex.split(args[0]))
         else:
             raise ValueError("passing varargs to run_bzr_subprocess")
@@ -2253,9 +2244,9 @@
         if retcode is not None and retcode != process.returncode:
             if process_args is None:
                 process_args = "(unknown args)"
-            trace.mutter('Output of brz %s:\n%s', process_args, out)
-            trace.mutter('Error for brz %s:\n%s', process_args, err)
-            self.fail('Command brz %s failed with retcode %s != %s'
+            trace.mutter('Output of brz %r:\n%s', process_args, out)
+            trace.mutter('Error for brz %r:\n%s', process_args, err)
+            self.fail('Command brz %r failed with retcode %d != %d'
                       % (process_args, retcode, process.returncode))
         return [out, err]
 
@@ -2289,7 +2280,7 @@
         if not callable(a_callable):
             raise ValueError("a_callable must be callable.")
         if stdin is None:
-            stdin = BytesIO("")
+            stdin = BytesIO(b"")
         if stdout is None:
             if getattr(self, "_log_file", None) is not None:
                 stdout = self._log_file
@@ -2848,18 +2839,18 @@
         if transport is None or transport.is_readonly():
             transport = _mod_transport.get_transport_from_path(".")
         for name in shape:
-            self.assertIsInstance(name, basestring)
+            self.assertIsInstance(name, (str, text_type))
             if name[-1] == '/':
                 transport.mkdir(urlutils.escape(name[:-1]))
             else:
                 if line_endings == 'binary':
-                    end = '\n'
+                    end = b'\n'
                 elif line_endings == 'native':
-                    end = os.linesep
+                    end = os.linesep.encode('ascii')
                 else:
                     raise errors.BzrError(
                         'Invalid line ending request %r' % line_endings)
-                content = "contents of %s%s" % (name.encode('utf-8'), end)
+                content = b"contents of %s%s" % (name.encode('utf-8'), end)
                 transport.put_bytes_non_atomic(urlutils.escape(name), content)
 
     build_tree_contents = staticmethod(treeshape.build_tree_contents)
@@ -2868,7 +2859,7 @@
         """Assert whether path or paths are in the WorkingTree"""
         if tree is None:
             tree = workingtree.WorkingTree.open(root_path)
-        if not isinstance(path, basestring):
+        if not isinstance(path, (str, text_type)):
             for p in path:
                 self.assertInWorkingTree(p, tree=tree)
         else:
@@ -2879,7 +2870,7 @@
         """Assert whether path or paths are not in the WorkingTree"""
         if tree is None:
             tree = workingtree.WorkingTree.open(root_path)
-        if not isinstance(path, basestring):
+        if not isinstance(path, (str, text_type)):
             for p in path:
                 self.assertNotInWorkingTree(p,tree=tree)
         else:
@@ -2998,7 +2989,8 @@
         There is no point in forcing them to duplicate the extension related
         warning.
         """
-        config.GlobalStack().set('ignore_missing_extensions', True)
+        config.GlobalConfig().set_user_option(
+            'suppress_warnings', 'missing_extensions')
 
 
 class ChrootedTestCase(TestCaseWithTransport):

=== modified file 'breezy/tests/blackbox/test_aliases.py'
--- breezy/tests/blackbox/test_aliases.py	2017-05-21 18:10:28 +0000
+++ breezy/tests/blackbox/test_aliases.py	2017-07-20 00:00:04 +0000
@@ -65,7 +65,8 @@
         # So we need to catch the output as well
         self.assertEqual(bzr_catch_error('--no-aliases c a',
                                           retcode=None),
-                          'brz: ERROR: unknown command "c"\n')
+                          'brz: ERROR: unknown command "c". '
+                          'Perhaps you meant "ci"\n')
 
         bzr('c -r1 -r2', retcode=3)
         bzr('c1 -r1 -r2', retcode=3)

=== modified file 'breezy/tests/blackbox/test_branch.py'
--- breezy/tests/blackbox/test_branch.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/blackbox/test_branch.py	2017-06-13 00:21:36 +0000
@@ -614,22 +614,6 @@
         self.assertFalse(t.has('remote/file'))
 
 
-class TestDeprecatedAliases(tests.TestCaseWithTransport):
-
-    def test_deprecated_aliases(self):
-        """brz branch can be called clone or get, but those names are
-        deprecated.
-
-        See bug 506265.
-        """
-        for command in ['clone', 'get']:
-            run_script(self, """
-            $ brz %(command)s A B
-            2>The command 'brz %(command)s' has been deprecated in brz 2.4. Please use 'brz branch' instead.
-            2>brz: ERROR: Not a branch...
-            """ % locals())
-
-
 class TestBranchParentLocation(test_switch.TestSwitchParentLocationBase):
 
     def _checkout_and_branch(self, option=''):

=== modified file 'breezy/tests/blackbox/test_exceptions.py'
--- breezy/tests/blackbox/test_exceptions.py	2017-06-11 01:22:16 +0000
+++ breezy/tests/blackbox/test_exceptions.py	2017-06-14 22:34:05 +0000
@@ -20,6 +20,7 @@
 import re
 
 from breezy import (
+    bzr,
     config,
     controldir,
     errors,
@@ -27,9 +28,6 @@
     repository,
     tests,
     )
-from breezy.bzr import (
-    bzrdir,
-    )
 from breezy.bzr.groupcompress_repo import RepositoryFormat2a
 
 
@@ -104,7 +102,7 @@
             TestObsoleteRepoFormat)
         repository.format_registry.register(TestObsoleteRepoFormat)
         self.addCleanup(controldir.format_registry.remove, "testobsolete")
-        bzrdir.register_metadir(controldir.format_registry, "testobsolete",
+        bzr.register_metadir(controldir.format_registry, "testobsolete",
             "breezy.tests.blackbox.test_exceptions.TestObsoleteRepoFormat",
             branch_format='breezy.bzr.branch.BzrBranchFormat7',
             tree_format='breezy.bzr.workingtree_4.WorkingTreeFormat6',

=== modified file 'breezy/tests/blackbox/test_export_pot.py'
--- breezy/tests/blackbox/test_export_pot.py	2017-05-21 18:10:28 +0000
+++ breezy/tests/blackbox/test_export_pot.py	2017-06-17 12:27:49 +0000
@@ -20,7 +20,7 @@
 
 from breezy import ignores, osutils
 from breezy.tests import TestCaseWithMemoryTransport
-from breezy.tests.features import ModuleAvailableFeature
+from breezy.tests.features import PluginLoadedFeature
 
 class TestExportPot(TestCaseWithMemoryTransport):
 
@@ -30,8 +30,13 @@
         self.assertContainsRe(out, "help of 'change' option\n"\
                                    "msgid \"Select changes introduced by the specified revision.")
 
+    def test_export_pot_plugin_unknown(self):
+        out, err = self.run_bzr("export-pot --plugin=lalalala", retcode=3)
+        self.assertContainsRe(
+             err, 'ERROR: Plugin lalalala is not loaded')
+
     def test_export_pot_plugin(self):
-        self.requireFeature(ModuleAvailableFeature('breezy.plugins.launchpad'))
+        self.requireFeature(PluginLoadedFeature('launchpad'))
         out, err = self.run_bzr("export-pot --plugin=launchpad")
         self.assertContainsRe(err, 'Exporting messages from plugin command: launchpad-login in launchpad')
         self.assertContainsRe(out, 'msgid "Show or set the Launchpad user ID."')

=== modified file 'breezy/tests/blackbox/test_init.py'
--- breezy/tests/blackbox/test_init.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/blackbox/test_init.py	2017-06-19 20:34:29 +0000
@@ -187,7 +187,7 @@
 
     def test_init(self):
         # init on a remote url should succeed.
-        out, err = self.run_bzr(['init', '--pack-0.92', self.get_url()])
+        out, err = self.run_bzr(['init', '--format=pack-0.92', self.get_url()])
         self.assertEqual(out,
             """Created a standalone branch (format: pack-0.92)\n""")
         self.assertEqual('', err)
@@ -213,14 +213,14 @@
         self.run_bzr_error(['Already a branch'], ['init', self.get_url()])
 
     def test_init_append_revisions_only(self):
-        self.run_bzr('init --dirstate-tags normal_branch6')
+        self.run_bzr('init --format=dirstate-tags normal_branch6')
         branch = _mod_branch.Branch.open('normal_branch6')
         self.assertEqual(None, branch.get_append_revisions_only())
-        self.run_bzr('init --append-revisions-only --dirstate-tags branch6')
+        self.run_bzr('init --append-revisions-only --format=dirstate-tags branch6')
         branch = _mod_branch.Branch.open('branch6')
         self.assertEqual(True, branch.get_append_revisions_only())
         self.run_bzr_error(['cannot be set to append-revisions-only'],
-                           'init --append-revisions-only --knit knit')
+                           'init --append-revisions-only --format=knit knit')
 
     def test_init_without_username(self):
         """Ensure init works if username is not set.

=== modified file 'breezy/tests/blackbox/test_log.py'
--- breezy/tests/blackbox/test_log.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/blackbox/test_log.py	2017-07-04 20:03:11 +0000
@@ -17,6 +17,9 @@
 
 """Black-box tests for brz log."""
 
+from __future__ import absolute_import
+
+
 import os
 
 from breezy import (
@@ -474,7 +477,7 @@
 class TestLogSignatures(TestLog):
 
     def test_log_with_signatures(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
 
         tree = self.make_linear_branch(format='dirstate-tags')
 
@@ -482,7 +485,7 @@
         self.assertTrue('signature: no signature' in log)
 
     def test_log_without_signatures(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
 
         tree = self.make_linear_branch(format='dirstate-tags')
 
@@ -995,16 +998,18 @@
         self.assertLogRevnos([], ["2", "1"])
 
     def test_log_range_open_begin(self):
-        self.knownFailure("log with ghosts fails. bug #726466")
         (stdout, stderr) = self.run_bzr(['log', '-r..2'], retcode=3)
         self.assertEqual(["2", "1"],
                          [r.revno for r in self.get_captured_revisions()])
-        self.assertEqual("brz: ERROR: Further revision history missing.", stderr)
+        self.assertEqual("brz: ERROR: Further revision history missing.\n",
+                stderr)
 
     def test_log_range_open_end(self):
         self.assertLogRevnos(["-r1.."], ["2", "1"])
 
+
 class TestLogMatch(TestLogWithLogCatcher):
+
     def prepare_tree(self):
         tree = self.make_branch_and_tree('')
         self.build_tree(
@@ -1013,7 +1018,7 @@
         tree.commit(message='message1', committer='committer1', authors=['author1'])
         tree.add('goodbye.txt')
         tree.commit(message='message2', committer='committer2', authors=['author2'])
-    
+
     def test_message(self):
         self.prepare_tree()
         self.assertLogRevnos(["-m", "message1"], ["1"])

=== modified file 'breezy/tests/blackbox/test_resolve.py'
--- breezy/tests/blackbox/test_resolve.py	2017-05-21 18:10:28 +0000
+++ breezy/tests/blackbox/test_resolve.py	2017-06-13 10:58:47 +0000
@@ -84,7 +84,7 @@
 $ cd a
 $ brz commit -m one
 $ cd ..
-$ brz clone a b
+$ brz branch a b
 $ echo bar > b/dir/file
 $ cd a
 $ rm -r dir

=== modified file 'breezy/tests/blackbox/test_upgrade.py'
--- breezy/tests/blackbox/test_upgrade.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/blackbox/test_upgrade.py	2017-06-19 20:34:29 +0000
@@ -19,6 +19,7 @@
 import stat
 
 from breezy import (
+    bzr,
     controldir,
     lockable_files,
     ui,
@@ -130,9 +131,9 @@
 
     def test_upgrade_control_dir(self):
         old_format = OldBzrDirFormat()
-        self.addCleanup(bzrdir.BzrProber.formats.remove,
+        self.addCleanup(bzr.BzrProber.formats.remove,
             old_format.get_format_string())
-        bzrdir.BzrProber.formats.register(old_format.get_format_string(),
+        bzr.BzrProber.formats.register(old_format.get_format_string(),
             old_format)
         self.addCleanup(controldir.ControlDirFormat._set_default_format,
                         controldir.ControlDirFormat.get_default_format())
@@ -278,7 +279,7 @@
 
     def test_recommend_upgrade_wt4(self):
         # using a deprecated format gives a warning
-        self.run_bzr('init --knit a')
+        self.run_bzr('init --format=knit a')
         out, err = self.run_bzr('status a')
         self.assertContainsRe(err, 'brz upgrade .*[/\\\\]a')
 
@@ -286,7 +287,7 @@
         # we should only get a recommendation to upgrade when we're accessing
         # the actual workingtree, not when we only open a bzrdir that contains
         # an old workngtree
-        self.run_bzr('init --knit a')
+        self.run_bzr('init --format=knit a')
         out, err = self.run_bzr('revno a')
         if err.find('upgrade') > -1:
             self.fail("message shouldn't suggest upgrade:\n%s" % err)

=== modified file 'breezy/tests/blackbox/test_whoami.py'
--- breezy/tests/blackbox/test_whoami.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/blackbox/test_whoami.py	2017-07-21 13:20:17 +0000
@@ -142,7 +142,7 @@
         self.overrideEnv('EMAIL', None)
         self.overrideAttr(config, '_auto_user_id', lambda: (None, None))
         global_conf = config.GlobalStack()
-        self.assertRaises(errors.NoWhoami, global_conf.get, 'email')
+        self.assertRaises(config.NoWhoami, global_conf.get, 'email')
 
     def test_whoami_nonbranch_directory(self):
         """Test --directory mentioning a non-branch directory."""

=== modified file 'breezy/tests/features.py'
--- breezy/tests/features.py	2017-06-08 00:00:28 +0000
+++ breezy/tests/features.py	2017-07-04 20:03:11 +0000
@@ -17,6 +17,8 @@
 """A collection of commonly used 'Features' to optionally run tests.
 """
 
+from __future__ import absolute_import
+
 import os
 import subprocess
 import stat
@@ -188,6 +190,33 @@
         return self.module_name
 
 
+class PluginLoadedFeature(Feature):
+    """Check whether a plugin with specific name is loaded.
+
+    This is different from ModuleAvailableFeature, because
+    plugins can be available but explicitly disabled
+    (e.g. through BRZ_DISABLE_PLUGINS=blah).
+
+    :ivar plugin_name: The name of the plugin
+    """
+
+    def __init__(self, plugin_name):
+        super(PluginLoadedFeature, self).__init__()
+        self.plugin_name = plugin_name
+
+    def _probe(self):
+        import breezy
+        return self.plugin_name in breezy.global_state.plugins
+
+    @property
+    def plugin(self):
+        import breezy
+        return breezy.global_state.plugins.get(self.plugin_name)
+
+    def feature_name(self):
+        return '%s plugin' % self.plugin_name
+
+
 class _HTTPSServerFeature(Feature):
     """Some tests want an https Server, check if one is available.
 
@@ -346,7 +375,7 @@
 not_running_as_root = _NotRunningAsRoot()
 
 apport = ModuleAvailableFeature('apport')
-gpgme = ModuleAvailableFeature('gpgme')
+gpg = ModuleAvailableFeature('gpg')
 lzma = ModuleAvailableFeature('lzma')
 meliae = ModuleAvailableFeature('meliae.scanner')
 paramiko = ModuleAvailableFeature('paramiko')

=== modified file 'breezy/tests/matchers.py'
--- breezy/tests/matchers.py	2017-06-11 20:15:04 +0000
+++ breezy/tests/matchers.py	2017-06-14 23:29:06 +0000
@@ -44,6 +44,9 @@
 from breezy.bzr.smart.request import request_handlers as smart_request_handlers
 from breezy.bzr.smart import vfs
 """)
+from ..sixish import (
+    text_type,
+    )
 
 from testtools.matchers import Equals, Mismatch, Matcher
 
@@ -156,7 +159,7 @@
         """
         directories = []
         for entry in entries:
-            if isinstance(entry, basestring):
+            if isinstance(entry, (str, text_type)):
                 path = entry
             else:
                 path = entry[0]
@@ -176,7 +179,7 @@
 
     def match(self, tree):
         actual = list(self.get_tree_layout(tree))
-        if self.entries and isinstance(self.entries[0], basestring):
+        if self.entries and isinstance(self.entries[0], (str, text_type)):
             actual = [path for (path, fileid) in actual]
         if not tree.has_versioned_directories():
             entries = list(self._strip_unreferenced_directories(self.entries))

=== modified file 'breezy/tests/per_branch/test_branch.py'
--- breezy/tests/per_branch/test_branch.py	2017-06-11 14:07:05 +0000
+++ breezy/tests/per_branch/test_branch.py	2017-06-24 14:00:24 +0000
@@ -38,6 +38,9 @@
     branch as _mod_bzrbranch,
     remote,
     )
+from breezy.sixish import (
+    text_type,
+    )
 from breezy.tests import (
     per_branch,
     )
@@ -281,12 +284,12 @@
         branch = self.make_branch('bzr.dev')
         # An implicit nick name is set; what it is exactly depends on the
         # format.
-        self.assertIsInstance(branch.nick, basestring)
+        self.assertIsInstance(branch.nick, text_type)
         # Set the branch nick explicitly.
-        branch.nick = "Aaron's branch"
+        branch.nick = u"Aaron's branch"
         # Because the nick has been set explicitly, the nick is now always
         # "Aaron's branch".
-        self.assertEqual(branch.nick, "Aaron's branch")
+        self.assertEqual(branch.nick, u"Aaron's branch")
         branch.nick = u"\u1234"
         self.assertEqual(branch.nick, u"\u1234")
 
@@ -1025,7 +1028,7 @@
         checkout.commit('bar')
         tree.lock_write()
         self.addCleanup(tree.unlock)
-        merger = merge.Merger.from_revision_ids(None, tree,
+        merger = merge.Merger.from_revision_ids(tree,
                                                 branch.last_revision(),
                                                 other_branch=branch)
         merger.merge_type = merge.Merge3Merger

=== modified file 'breezy/tests/per_branch/test_parent.py'
--- breezy/tests/per_branch/test_parent.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/per_branch/test_parent.py	2017-07-15 13:23:08 +0000
@@ -59,7 +59,7 @@
         self.assertEqual(path, b.get_parent())
 
 
-        self.assertRaises(breezy.errors.InvalidURL, b.set_parent, u'\xb5')
+        self.assertRaises(urlutils.InvalidURL, b.set_parent, u'\xb5')
         b.set_parent(urlutils.escape(u'\xb5'))
         self.assertEqual('%C2%B5', b._get_parent_location())
 

=== modified file 'breezy/tests/per_merger.py'
--- breezy/tests/per_merger.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/per_merger.py	2017-06-24 13:32:35 +0000
@@ -44,7 +44,7 @@
 class TestMergeImplementation(TestCaseWithTransport):
 
     def do_merge(self, target_tree, source_tree, **kwargs):
-        merger = _mod_merge.Merger.from_revision_ids(None,
+        merger = _mod_merge.Merger.from_revision_ids(
             target_tree, source_tree.last_revision(),
             other_branch=source_tree.branch)
         merger.merge_type=self.merge_type

=== modified file 'breezy/tests/per_repository/test_commit_builder.py'
--- breezy/tests/per_repository/test_commit_builder.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/per_repository/test_commit_builder.py	2017-07-21 13:20:17 +0000
@@ -47,34 +47,6 @@
         branch.repository.commit_write_group()
         branch.repository.unlock()
 
-    def record_root(self, builder, tree):
-        if builder.record_root_entry is True:
-            tree.lock_read()
-            try:
-                ie = tree.root_inventory.root
-            finally:
-                tree.unlock()
-            parent_tree = tree.branch.repository.revision_tree(
-                              _mod_revision.NULL_REVISION)
-            parent_invs = []
-            builder.record_entry_contents(ie, parent_invs, '', tree,
-                tree.path_content_summary(''))
-
-    def test_finish_inventory_with_record_root(self):
-        tree = self.make_branch_and_tree(".")
-        tree.lock_write()
-        try:
-            builder = tree.branch.get_commit_builder([])
-            if not builder.supports_record_entry_contents:
-                raise tests.TestNotApplicable("CommitBuilder doesn't support "
-                    "record_entry_contents")
-            repo = tree.branch.repository
-            self.record_root(builder, tree)
-            builder.finish_inventory()
-            repo.commit_write_group()
-        finally:
-            tree.unlock()
-
     def test_finish_inventory_record_iter_changes(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
@@ -92,20 +64,6 @@
         finally:
             tree.unlock()
 
-    def test_abort_record_entry_contents(self):
-        tree = self.make_branch_and_tree(".")
-        tree.lock_write()
-        try:
-            builder = tree.branch.get_commit_builder([])
-            if not builder.supports_record_entry_contents:
-                raise tests.TestNotApplicable("CommitBuilder doesn't support "
-                    "record_entry_contents")
-            self.record_root(builder, tree)
-            builder.finish_inventory()
-            builder.abort()
-        finally:
-            tree.unlock()
-
     def test_abort_record_iter_changes(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
@@ -165,41 +123,6 @@
         actually_updated_branch = (tree.branch.last_revision() == rev_id)
         self.assertEqual(actually_updated_branch, will_update_branch)
 
-    def test_commit_with_revision_id_record_entry_contents(self):
-        tree = self.make_branch_and_tree(".")
-        tree.lock_write()
-        try:
-            # use a unicode revision id to test more corner cases.
-            # The repository layer is meant to handle this.
-            revision_id = u'\xc8abc'.encode('utf8')
-            try:
-                try:
-                    builder = tree.branch.get_commit_builder([],
-                        revision_id=revision_id)
-                except errors.NonAsciiRevisionId:
-                    revision_id = 'abc'
-                    builder = tree.branch.get_commit_builder([],
-                        revision_id=revision_id)
-            except errors.CannotSetRevisionId:
-                # This format doesn't support supplied revision ids
-                return
-            if not builder.supports_record_entry_contents:
-                raise tests.TestNotApplicable("CommitBuilder doesn't support "
-                    "record_entry_contents")
-            self.assertFalse(builder.random_revid)
-            self.record_root(builder, tree)
-            builder.finish_inventory()
-            self.assertEqual(revision_id, builder.commit('foo bar'))
-        finally:
-            tree.unlock()
-        self.assertTrue(tree.branch.repository.has_revision(revision_id))
-        # the revision id must be set on the inventory when saving it. This
-        # does not precisely test that - a repository that wants to can add it
-        # on deserialisation, but thats all the current contract guarantees
-        # anyway.
-        self.assertEqual(revision_id,
-            tree.branch.repository.get_inventory(revision_id).revision_id)
-
     def test_commit_with_revision_id_record_iter_changes(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
@@ -256,65 +179,6 @@
         finally:
             tree.unlock()
 
-    def test_commit_without_root_or_record_iter_changes_errors(self):
-        tree = self.make_branch_and_tree(".")
-        tree.lock_write()
-        try:
-            self.build_tree(['foo'])
-            tree.add('foo', 'foo-id')
-            builder = tree.branch.get_commit_builder([])
-            if not builder.supports_record_entry_contents:
-                raise tests.TestNotApplicable("CommitBuilder doesn't support "
-                    "record_entry_contents")
-            entry = tree.root_inventory['foo-id']
-            self.assertRaises(errors.RootMissing,
-                builder.record_entry_contents, entry, [], 'foo', tree,
-                    tree.path_content_summary('foo'))
-            builder.abort()
-        finally:
-            tree.unlock()
-
-    def test_commit_unchanged_root_record_entry_contents(self):
-        tree = self.make_branch_and_tree(".")
-        old_revision_id = tree.commit('')
-        tree.lock_write()
-        parent_tree = tree.basis_tree()
-        parent_tree.lock_read()
-        self.addCleanup(parent_tree.unlock)
-        builder = tree.branch.get_commit_builder([old_revision_id])
-        try:
-            if not builder.supports_record_entry_contents:
-                raise tests.TestNotApplicable("CommitBuilder doesn't support "
-                    "record_entry_contents")
-            builder.will_record_deletes()
-            ie = inventory.make_entry('directory', '', None,
-                    tree.get_root_id())
-            delta, version_recorded, fs_hash = builder.record_entry_contents(
-                ie, [parent_tree.root_inventory], '', tree,
-                tree.path_content_summary(''))
-            # Regardless of repository root behaviour we should consider this a
-            # pointless commit.
-            self.assertFalse(builder.any_changes())
-            self.assertFalse(version_recorded)
-            # if the repository format recorded a new root revision, that
-            # should be in the delta
-            got_new_revision = ie.revision != old_revision_id
-            if got_new_revision:
-                self.assertEqual(('', '', ie.file_id, ie), delta)
-                # The delta should be tracked
-                self.assertEqual(delta, builder.get_basis_delta()[-1])
-            else:
-                self.assertEqual(None, delta)
-            # Directories do not get hashed.
-            self.assertEqual(None, fs_hash)
-            builder.abort()
-        except:
-            builder.abort()
-            tree.unlock()
-            raise
-        else:
-            tree.unlock()
-
     def test_commit_unchanged_root_record_iter_changes(self):
         tree = self.make_branch_and_tree(".")
         old_revision_id = tree.commit('')
@@ -339,124 +203,6 @@
         finally:
             tree.unlock()
 
-    def test_commit_record_entry_contents(self):
-        tree = self.make_branch_and_tree(".")
-        tree.lock_write()
-        try:
-            builder = tree.branch.get_commit_builder([])
-            if not builder.supports_record_entry_contents:
-                raise tests.TestNotApplicable("CommitBuilder doesn't "
-                    "support record_entry_contents")
-            self.record_root(builder, tree)
-            builder.finish_inventory()
-            rev_id = builder.commit('foo bar')
-        finally:
-            tree.unlock()
-        self.assertNotEqual(None, rev_id)
-        self.assertTrue(tree.branch.repository.has_revision(rev_id))
-        # the revision id must be set on the inventory when saving it. This does not
-        # precisely test that - a repository that wants to can add it on deserialisation,
-        # but thats all the current contract guarantees anyway.
-        self.assertEqual(rev_id, tree.branch.repository.get_inventory(rev_id).revision_id)
-
-    def test_get_basis_delta(self):
-        tree = self.make_branch_and_tree(".")
-        self.build_tree(["foo"])
-        tree.add(["foo"], ["foo-id"])
-        old_revision_id = tree.commit("added foo")
-        tree.lock_write()
-        try:
-            self.build_tree(['bar'])
-            tree.add(['bar'], ['bar-id'])
-            basis = tree.branch.repository.revision_tree(old_revision_id)
-            basis.lock_read()
-            self.addCleanup(basis.unlock)
-            builder = tree.branch.get_commit_builder([old_revision_id])
-            total_delta = []
-            try:
-                if not builder.supports_record_entry_contents:
-                    raise tests.TestNotApplicable("CommitBuilder doesn't "
-                        "support record_entry_contents")
-                parent_invs = [basis.root_inventory]
-                builder.will_record_deletes()
-                if builder.record_root_entry:
-                    ie = basis.root_inventory.root.copy()
-                    delta, _, _ = builder.record_entry_contents(ie, parent_invs,
-                        '', tree, tree.path_content_summary(''))
-                    if delta is not None:
-                        total_delta.append(delta)
-                delta = builder.record_delete("foo", "foo-id")
-                total_delta.append(delta)
-                new_bar = inventory.make_entry('file', 'bar',
-                    parent_id=tree.get_root_id(), file_id='bar-id')
-                delta, _, _ = builder.record_entry_contents(new_bar, parent_invs,
-                    'bar', tree, tree.path_content_summary('bar'))
-                total_delta.append(delta)
-                # All actions should have been recorded in the basis_delta
-                self.assertEqual(total_delta, builder.get_basis_delta())
-                builder.finish_inventory()
-                builder.commit('delete foo, add bar')
-            except:
-                tree.branch.repository.abort_write_group()
-                raise
-        finally:
-            tree.unlock()
-
-    def test_get_basis_delta_without_notification(self):
-        tree = self.make_branch_and_tree(".")
-        old_revision_id = tree.commit('')
-        tree.lock_write()
-        try:
-            parent_tree = tree.basis_tree()
-            parent_tree.lock_read()
-            self.addCleanup(parent_tree.unlock)
-            builder = tree.branch.get_commit_builder([old_revision_id])
-            # It is an error to expect builder.get_basis_delta() to be correct,
-            # if you have not also called will_record_deletes() to indicate you
-            # will be calling record_delete() when appropriate
-            self.assertRaises(AssertionError, builder.get_basis_delta)
-            tree.branch.repository.abort_write_group()
-        finally:
-            tree.unlock()
-
-    def test_record_delete(self):
-        tree = self.make_branch_and_tree(".")
-        self.build_tree(["foo"])
-        tree.add(["foo"], ["foo-id"])
-        rev_id = tree.commit("added foo")
-        # Remove the inventory details for foo-id, because
-        # record_entry_contents ends up copying root verbatim.
-        tree.unversion(["foo-id"])
-        tree.lock_write()
-        try:
-            basis = tree.branch.repository.revision_tree(rev_id)
-            builder = tree.branch.get_commit_builder([rev_id])
-            try:
-                if not builder.supports_record_entry_contents:
-                    raise tests.TestNotApplicable("CommitBuilder doesn't "
-                        "support record_entry_contents")
-                builder.will_record_deletes()
-                if builder.record_root_entry is True:
-                    parent_invs = [basis.root_inventory]
-                    del basis.root_inventory.root.children['foo']
-                    builder.record_entry_contents(basis.root_inventory.root,
-                        parent_invs, '', tree, tree.path_content_summary(''))
-                # the delta should be returned, and recorded in _basis_delta
-                delta = builder.record_delete("foo", "foo-id")
-                self.assertEqual(("foo", None, "foo-id", None), delta)
-                self.assertEqual(delta, builder.get_basis_delta()[-1])
-                builder.finish_inventory()
-                rev_id2 = builder.commit('delete foo')
-            except:
-                tree.branch.repository.abort_write_group()
-                raise
-        finally:
-            tree.unlock()
-        rev_tree = builder.revision_tree()
-        rev_tree.lock_read()
-        self.addCleanup(rev_tree.unlock)
-        self.assertFalse(rev_tree.path2id('foo'))
-
     def test_record_delete_record_iter_changes(self):
         tree = self.make_branch_and_tree(".")
         self.build_tree(["foo"])
@@ -466,7 +212,6 @@
         try:
             builder = tree.branch.get_commit_builder([rev_id])
             try:
-                builder.will_record_deletes()
                 delete_change = ('foo-id', ('foo', None), True, (True, False),
                     (tree.path2id(''), None), ('foo', None), ('file', None),
                     (False, None))
@@ -487,45 +232,6 @@
         self.addCleanup(rev_tree.unlock)
         self.assertFalse(rev_tree.path2id('foo'))
 
-    def test_record_delete_without_notification(self):
-        tree = self.make_branch_and_tree(".")
-        self.build_tree(["foo"])
-        tree.add(["foo"], ["foo-id"])
-        rev_id = tree.commit("added foo")
-        tree.lock_write()
-        try:
-            builder = tree.branch.get_commit_builder([rev_id])
-            try:
-                if not builder.supports_record_entry_contents:
-                    raise tests.TestNotApplicable("CommitBuilder doesn't "
-                        "support record_entry_contents")
-                self.record_root(builder, tree)
-                self.assertRaises(AssertionError,
-                    builder.record_delete, "foo", "foo-id")
-            finally:
-                tree.branch.repository.abort_write_group()
-        finally:
-            tree.unlock()
-
-    def test_revision_tree_record_entry_contents(self):
-        tree = self.make_branch_and_tree(".")
-        tree.lock_write()
-        try:
-            builder = tree.branch.get_commit_builder([])
-            if not builder.supports_record_entry_contents:
-                raise tests.TestNotApplicable("CommitBuilder doesn't "
-                    "support record_entry_contents")
-            self.record_root(builder, tree)
-            builder.finish_inventory()
-            rev_id = builder.commit('foo bar')
-        finally:
-            tree.unlock()
-        rev_tree = builder.revision_tree()
-        # Just a couple simple tests to ensure that it actually follows
-        # the RevisionTree api.
-        self.assertEqual(rev_id, rev_tree.get_revision_id())
-        self.assertEqual([], rev_tree.get_parent_ids())
-
     def test_revision_tree_record_iter_changes(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
@@ -595,8 +301,6 @@
 
     def _commit_check_unchanged(self, tree, name, file_id, mini_commit=None):
         rev1 = tree.commit('')
-        if mini_commit is None:
-            mini_commit = self.mini_commit
         rev2 = mini_commit(tree, name, name, False, False)
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
         self.assertEqual(rev1, tree1.get_file_revision(file_id))
@@ -609,12 +313,6 @@
         # committing without changing a dir does not change the last modified.
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir/'])
-        self._add_commit_check_unchanged(tree, 'dir')
-
-    def test_last_modified_revision_after_commit_dir_unchanged_ric(self):
-        # committing without changing a dir does not change the last modified.
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['dir/'])
         self._add_commit_check_unchanged(tree, 'dir',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -640,12 +338,6 @@
         # committing without changing a file does not change the last modified.
         tree = self.make_branch_and_tree('.')
         self.build_tree(['file'])
-        self._add_commit_check_unchanged(tree, 'file')
-
-    def test_last_modified_revision_after_commit_file_unchanged_ric(self):
-        # committing without changing a file does not change the last modified.
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['file'])
         self._add_commit_check_unchanged(tree, 'file',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -654,13 +346,6 @@
         self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
-        self._add_commit_check_unchanged(tree, 'link')
-
-    def test_last_modified_revision_after_commit_link_unchanged_ric(self):
-        # committing without changing a link does not change the last modified.
-        self.requireFeature(features.SymlinkFeature)
-        tree = self.make_branch_and_tree('.')
-        os.symlink('target', 'link')
         self._add_commit_check_unchanged(tree, 'link',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -672,18 +357,6 @@
         try:
             tree.add_reference(subtree)
             self._commit_check_unchanged(tree, 'reference',
-                subtree.get_root_id())
-        except errors.UnsupportedOperation:
-            return
-
-    def test_last_modified_revision_after_commit_reference_unchanged_ric(self):
-        # committing without changing a subtree does not change the last
-        # modified.
-        tree = self.make_branch_and_tree('.')
-        subtree = self.make_reference('reference')
-        try:
-            tree.add_reference(subtree)
-            self._commit_check_unchanged(tree, 'reference',
                 subtree.get_root_id(),
                 mini_commit=self.mini_commit_record_iter_changes)
         except errors.UnsupportedOperation:
@@ -707,12 +380,6 @@
         # renaming a dir changes the last modified.
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir/'])
-        self._add_commit_renamed_check_changed(tree, 'dir')
-
-    def test_last_modified_revision_after_rename_dir_changes_ric(self):
-        # renaming a dir changes the last modified.
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['dir/'])
         self._add_commit_renamed_check_changed(tree, 'dir',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -721,13 +388,6 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['file'])
         self._add_commit_renamed_check_changed(tree, 'file',
-            expect_fs_hash=True)
-
-    def test_last_modified_revision_after_rename_file_changes_ric(self):
-        # renaming a file changes the last modified.
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['file'])
-        self._add_commit_renamed_check_changed(tree, 'file',
             expect_fs_hash=True,
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -736,13 +396,6 @@
         self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
-        self._add_commit_renamed_check_changed(tree, 'link')
-
-    def test_last_modified_revision_after_rename_link_changes_ric(self):
-        # renaming a link changes the last modified.
-        self.requireFeature(features.SymlinkFeature)
-        tree = self.make_branch_and_tree('.')
-        os.symlink('target', 'link')
         self._add_commit_renamed_check_changed(tree, 'link',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -753,17 +406,6 @@
         try:
             tree.add_reference(subtree)
             self._commit_renamed_check_changed(tree, 'reference',
-                subtree.get_root_id())
-        except errors.UnsupportedOperation:
-            return
-
-    def test_last_modified_revision_after_rename_ref_changes_ric(self):
-        # renaming a reference changes the last modified.
-        tree = self.make_branch_and_tree('.')
-        subtree = self.make_reference('reference')
-        try:
-            tree.add_reference(subtree)
-            self._commit_renamed_check_changed(tree, 'reference',
                 subtree.get_root_id(),
                 mini_commit=self.mini_commit_record_iter_changes)
         except errors.UnsupportedOperation:
@@ -782,12 +424,6 @@
         # reparenting a dir changes the last modified.
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir/'])
-        self._add_commit_reparent_check_changed(tree, 'dir')
-
-    def test_last_modified_revision_after_reparent_dir_changes_ric(self):
-        # reparenting a dir changes the last modified.
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['dir/'])
         self._add_commit_reparent_check_changed(tree, 'dir',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -796,13 +432,6 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['file'])
         self._add_commit_reparent_check_changed(tree, 'file',
-            expect_fs_hash=True)
-
-    def test_last_modified_revision_after_reparent_file_changes_ric(self):
-        # reparenting a file changes the last modified.
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['file'])
-        self._add_commit_reparent_check_changed(tree, 'file',
             expect_fs_hash=True,
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -811,13 +440,6 @@
         self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
-        self._add_commit_reparent_check_changed(tree, 'link')
-
-    def test_last_modified_revision_after_reparent_link_changes_ric(self):
-        # reparenting a link changes the last modified.
-        self.requireFeature(features.SymlinkFeature)
-        tree = self.make_branch_and_tree('.')
-        os.symlink('target', 'link')
         self._add_commit_reparent_check_changed(tree, 'link',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -834,8 +456,6 @@
         expect_fs_hash=False, mini_commit=None):
         rev1 = tree.commit('')
         changer()
-        if mini_commit is None:
-            mini_commit = self.mini_commit
         rev2 = mini_commit(tree, name, tree.id2path(file_id),
             expect_fs_hash=expect_fs_hash)
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
@@ -846,98 +466,6 @@
         expected_graph[(file_id, rev2)] = ((file_id, rev1),)
         self.assertFileGraph(expected_graph, tree, (file_id, rev2))
 
-    def mini_commit(self, tree, name, new_name, records_version=True,
-        delta_against_basis=True, expect_fs_hash=False):
-        """Perform a miniature commit looking for record entry results.
-
-        :param tree: The tree to commit.
-        :param name: The path in the basis tree of the tree being committed.
-        :param new_name: The path in the tree being committed.
-        :param records_version: True if the commit of new_name is expected to
-            record a new version.
-        :param delta_against_basis: True of the commit of new_name is expected
-            to have a delta against the basis.
-        :param expect_fs_hash: True or false to indicate whether we expect a
-            file hash to be returned from the record_entry_contents call.
-        """
-        tree.lock_write()
-        try:
-            # mini manual commit here so we can check the return of
-            # record_entry_contents.
-            parent_ids = tree.get_parent_ids()
-            builder = tree.branch.get_commit_builder(parent_ids)
-            try:
-                if not builder.supports_record_entry_contents:
-                    raise tests.TestNotApplicable("CommitBuilder doesn't "
-                        "support record_entry_contents")
-                builder.will_record_deletes()
-                parent_tree = tree.basis_tree()
-                parent_tree.lock_read()
-                self.addCleanup(parent_tree.unlock)
-                parent_invs = [parent_tree.root_inventory]
-                for parent_id in parent_ids[1:]:
-                    parent_invs.append(tree.branch.repository.revision_tree(
-                        parent_id).root_inventory)
-                # root
-                builder.record_entry_contents(
-                    inventory.make_entry('directory', '', None,
-                        tree.get_root_id()), parent_invs, '', tree,
-                        tree.path_content_summary(''))
-                def commit_id(file_id):
-                    old_ie = tree.root_inventory[file_id]
-                    path = tree.id2path(file_id)
-                    ie = inventory.make_entry(tree.kind(file_id), old_ie.name,
-                        old_ie.parent_id, file_id)
-                    content_summary = tree.path_content_summary(path)
-                    if content_summary[0] == 'tree-reference':
-                        content_summary = content_summary[:3] + (
-                            tree.get_reference_revision(file_id),)
-                    return builder.record_entry_contents(ie, parent_invs, path,
-                        tree, content_summary)
-
-                file_id = tree.path2id(new_name)
-                parent_id = tree.root_inventory[file_id].parent_id
-                if parent_id != tree.get_root_id():
-                    commit_id(parent_id)
-                # because a change of some sort is meant to have occurred,
-                # recording the entry must return True.
-                delta, version_recorded, fs_hash = commit_id(file_id)
-                if records_version:
-                    self.assertTrue(version_recorded)
-                else:
-                    self.assertFalse(version_recorded)
-                if expect_fs_hash:
-                    tree_file_stat = tree.get_file_with_stat(file_id)
-                    tree_file_stat[0].close()
-                    self.assertEqual(2, len(fs_hash))
-                    self.assertEqual(tree.get_file_sha1(file_id), fs_hash[0])
-                    self.assertEqualStat(tree_file_stat[1], fs_hash[1])
-                else:
-                    self.assertEqual(None, fs_hash)
-                new_entry = builder.new_inventory[file_id]
-                if delta_against_basis:
-                    expected_delta = (name, new_name, file_id, new_entry)
-                    # The delta should be recorded
-                    self.assertEqual(expected_delta,
-                        builder.get_basis_delta()[-1])
-                else:
-                    expected_delta = None
-                self.assertEqual(expected_delta, delta)
-                builder.finish_inventory()
-            except:
-                builder.abort()
-                raise
-            else:
-                rev2 = builder.commit('')
-        except:
-            tree.unlock()
-            raise
-        try:
-            tree.set_parent_ids([rev2])
-        finally:
-            tree.unlock()
-        return rev2
-
     def mini_commit_record_iter_changes(self, tree, name, new_name,
         records_version=True, delta_against_basis=True, expect_fs_hash=False):
         """Perform a miniature commit looking for record entry results.
@@ -957,10 +485,9 @@
         tree.lock_write()
         try:
             # mini manual commit here so we can check the return of
-            # record_entry_contents.
+            # record_iter_changes
             parent_ids = tree.get_parent_ids()
             builder = tree.branch.get_commit_builder(parent_ids)
-            builder.will_record_deletes()
             parent_tree = tree.basis_tree()
             parent_tree.lock_read()
             self.addCleanup(parent_tree.unlock)
@@ -982,14 +509,12 @@
                 self.assertEqualStat(result[2][1], tree_file_stat[1])
             else:
                 self.assertEqual([], result)
-            self.assertIs(None, builder.new_inventory)
             builder.finish_inventory()
             if tree.branch.repository._format.supports_full_versioned_files:
                 inv_key = (builder._new_revision_id,)
                 inv_sha1 = tree.branch.repository.inventories.get_sha1s(
                                 [inv_key])[inv_key]
                 self.assertEqual(inv_sha1, builder.inv_sha1)
-            self.assertIs(None, builder.new_inventory)
             rev2 = builder.commit('')
             delta = builder.get_basis_delta()
             delta_dict = dict((change[2], change) for change in delta)
@@ -1033,29 +558,10 @@
         def change_file():
             tree.put_file_bytes_non_atomic('fileid', 'new content')
         self._add_commit_change_check_changed(tree, 'file', change_file,
-            expect_fs_hash=True)
-
-    def test_last_modified_revision_after_content_file_changes_ric(self):
-        # altering a file changes the last modified.
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['file'])
-        def change_file():
-            tree.put_file_bytes_non_atomic('fileid', 'new content')
-        self._add_commit_change_check_changed(tree, 'file', change_file,
             expect_fs_hash=True,
             mini_commit=self.mini_commit_record_iter_changes)
 
-    def test_last_modified_revision_after_content_link_changes(self):
-        # changing a link changes the last modified.
-        self.requireFeature(features.SymlinkFeature)
-        tree = self.make_branch_and_tree('.')
-        os.symlink('target', 'link')
-        def change_link():
-            os.unlink('link')
-            os.symlink('newtarget', 'link')
-        self._add_commit_change_check_changed(tree, 'link', change_link)
-
-    def _test_last_mod_rev_after_content_link_changes_ric(
+    def _test_last_mod_rev_after_content_link_changes(
         self, link, target, newtarget, file_id=None):
         if file_id is None:
             file_id = link
@@ -1071,13 +577,13 @@
             mini_commit=self.mini_commit_record_iter_changes,
             file_id=file_id)
 
-    def test_last_modified_rev_after_content_link_changes_ric(self):
-        self._test_last_mod_rev_after_content_link_changes_ric(
+    def test_last_modified_rev_after_content_link_changes(self):
+        self._test_last_mod_rev_after_content_link_changes(
             'link', 'target', 'newtarget')
 
-    def test_last_modified_rev_after_content_unicode_link_changes_ric(self):
+    def test_last_modified_rev_after_content_unicode_link_changes(self):
         self.requireFeature(features.UnicodeFilenameFeature)
-        self._test_last_mod_rev_after_content_link_changes_ric(
+        self._test_last_mod_rev_after_content_link_changes(
             u'li\u1234nk', u'targ\N{Euro Sign}t', u'n\N{Euro Sign}wtarget',
 
             file_id=u'li\u1234nk'.encode('UTF-8'))
@@ -1099,8 +605,6 @@
         rev2 = self._rename_in_tree(tree1, name)
         rev3 = self._rename_in_tree(tree2, name)
         tree1.merge_from_branch(tree2.branch)
-        if mini_commit is None:
-            mini_commit = self.mini_commit
         rev4 = mini_commit(tree1, 'new_' + name, 'new_' + name,
             expect_fs_hash=expect_fs_hash)
         tree3, = self._get_revtrees(tree1, [rev4])
@@ -1117,12 +621,6 @@
         # merge a dir changes the last modified.
         tree1 = self.make_branch_and_tree('t1')
         self.build_tree(['t1/dir/'])
-        self._commit_sprout_rename_merge(tree1, 'dir')
-
-    def test_last_modified_revision_after_merge_dir_changes_ric(self):
-        # merge a dir changes the last modified.
-        tree1 = self.make_branch_and_tree('t1')
-        self.build_tree(['t1/dir/'])
         self._commit_sprout_rename_merge(tree1, 'dir',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -1130,12 +628,6 @@
         # merge a file changes the last modified.
         tree1 = self.make_branch_and_tree('t1')
         self.build_tree(['t1/file'])
-        self._commit_sprout_rename_merge(tree1, 'file', expect_fs_hash=True)
-
-    def test_last_modified_revision_after_merge_file_changes_ric(self):
-        # merge a file changes the last modified.
-        tree1 = self.make_branch_and_tree('t1')
-        self.build_tree(['t1/file'])
         self._commit_sprout_rename_merge(tree1, 'file', expect_fs_hash=True,
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -1144,13 +636,6 @@
         self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
-        self._commit_sprout_rename_merge(tree1, 'link')
-
-    def test_last_modified_revision_after_merge_link_changes_ric(self):
-        # merge a link changes the last modified.
-        self.requireFeature(features.SymlinkFeature)
-        tree1 = self.make_branch_and_tree('t1')
-        os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge(tree1, 'link',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -1164,8 +649,6 @@
         # change on the other side to merge back
         rev2 = self._rename_in_tree(tree2, name)
         tree1.merge_from_branch(tree2.branch)
-        if mini_commit is None:
-            mini_commit = self.mini_commit
         def _check_graph(in_tree, changed_in_tree):
             rev3 = mini_commit(in_tree, name, 'new_' + name, False,
                 delta_against_basis=changed_in_tree)
@@ -1197,8 +680,6 @@
         tree2.add(['name'], [file_id])
         rev2 = tree2.commit('')
         tree1.merge_from_branch(tree2.branch)
-        if mini_commit is None:
-            mini_commit = self.mini_commit
         rev3 = mini_commit(tree1, None, 'name', False)
         tree3, = self._get_revtrees(tree1, [rev2])
         # in rev2, name should be only changed in rev2
@@ -1211,12 +692,6 @@
         # merge a dir that changed preserves the last modified.
         tree1 = self.make_branch_and_tree('t1')
         self.build_tree(['t1/dir/'])
-        self._commit_sprout_rename_merge_converged(tree1, 'dir')
-
-    def test_last_modified_revision_after_converged_merge_dir_unchanged_ric(self):
-        # merge a dir that changed preserves the last modified.
-        tree1 = self.make_branch_and_tree('t1')
-        self.build_tree(['t1/dir/'])
         self._commit_sprout_rename_merge_converged(tree1, 'dir',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -1224,12 +699,6 @@
         # merge a file that changed preserves the last modified.
         tree1 = self.make_branch_and_tree('t1')
         self.build_tree(['t1/file'])
-        self._commit_sprout_rename_merge_converged(tree1, 'file')
-
-    def test_last_modified_revision_after_converged_merge_file_unchanged_ric(self):
-        # merge a file that changed preserves the last modified.
-        tree1 = self.make_branch_and_tree('t1')
-        self.build_tree(['t1/file'])
         self._commit_sprout_rename_merge_converged(tree1, 'file',
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -1238,46 +707,24 @@
         self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
-        self._commit_sprout_rename_merge_converged(tree1, 'link')
-
-    def test_last_modified_revision_after_converged_merge_link_unchanged_ric(self):
-        # merge a link that changed preserves the last modified.
-        self.requireFeature(features.SymlinkFeature)
-        tree1 = self.make_branch_and_tree('t1')
-        os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge_converged(tree1, 'link',
             mini_commit=self.mini_commit_record_iter_changes)
 
     def test_last_modified_revision_after_merge_new_dir_unchanged(self):
         # merge a new dir does not change the last modified.
         tree1 = self.make_branch_and_tree('t1')
-        self._commit_sprout_make_merge(tree1, self.make_dir)
-
-    def test_last_modified_revision_after_merge_new_dir_unchanged_ric(self):
-        # merge a new dir does not change the last modified.
-        tree1 = self.make_branch_and_tree('t1')
         self._commit_sprout_make_merge(tree1, self.make_dir,
             mini_commit=self.mini_commit_record_iter_changes)
 
     def test_last_modified_revision_after_merge_new_file_unchanged(self):
         # merge a new file does not change the last modified.
         tree1 = self.make_branch_and_tree('t1')
-        self._commit_sprout_make_merge(tree1, self.make_file)
-
-    def test_last_modified_revision_after_merge_new_file_unchanged_ric(self):
-        # merge a new file does not change the last modified.
-        tree1 = self.make_branch_and_tree('t1')
         self._commit_sprout_make_merge(tree1, self.make_file,
             mini_commit=self.mini_commit_record_iter_changes)
 
     def test_last_modified_revision_after_merge_new_link_unchanged(self):
         # merge a new link does not change the last modified.
         tree1 = self.make_branch_and_tree('t1')
-        self._commit_sprout_make_merge(tree1, self.make_link)
-
-    def test_last_modified_revision_after_merge_new_link_unchanged_ric(self):
-        # merge a new link does not change the last modified.
-        tree1 = self.make_branch_and_tree('t1')
         self._commit_sprout_make_merge(tree1, self.make_link,
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -1313,10 +760,6 @@
             expect_fs_hash=expect_fs_hash, mini_commit=mini_commit)
 
     def test_last_modified_dir_file(self):
-        self._check_kind_change(self.make_dir, self.make_file,
-            expect_fs_hash=True)
-
-    def test_last_modified_dir_file_ric(self):
         try:
             self._check_kind_change(self.make_dir, self.make_file,
                 expect_fs_hash=True,
@@ -1327,9 +770,6 @@
                 "directory to file")
 
     def test_last_modified_dir_link(self):
-        self._check_kind_change(self.make_dir, self.make_link)
-
-    def test_last_modified_dir_link_ric(self):
         try:
             self._check_kind_change(self.make_dir, self.make_link,
                 mini_commit=self.mini_commit_record_iter_changes)
@@ -1340,31 +780,18 @@
 
     def test_last_modified_link_file(self):
         self._check_kind_change(self.make_link, self.make_file,
-            expect_fs_hash=True)
-
-    def test_last_modified_link_file_ric(self):
-        self._check_kind_change(self.make_link, self.make_file,
             expect_fs_hash=True,
             mini_commit=self.mini_commit_record_iter_changes)
 
     def test_last_modified_link_dir(self):
-        self._check_kind_change(self.make_link, self.make_dir)
-
-    def test_last_modified_link_dir_ric(self):
         self._check_kind_change(self.make_link, self.make_dir,
             mini_commit=self.mini_commit_record_iter_changes)
 
     def test_last_modified_file_dir(self):
-        self._check_kind_change(self.make_file, self.make_dir)
-
-    def test_last_modified_file_dir_ric(self):
         self._check_kind_change(self.make_file, self.make_dir,
             mini_commit=self.mini_commit_record_iter_changes)
 
     def test_last_modified_file_link(self):
-        self._check_kind_change(self.make_file, self.make_link)
-
-    def test_last_modified_file_link_ric(self):
         self._check_kind_change(self.make_file, self.make_link,
             mini_commit=self.mini_commit_record_iter_changes)
 
@@ -1428,7 +855,7 @@
         tree.lock_write()
         try:
             # Make sure no username is available.
-            self.assertRaises(errors.NoWhoami, tree.branch.get_commit_builder,
+            self.assertRaises(config.NoWhoami, tree.branch.get_commit_builder,
                               [])
             builder = tree.branch.get_commit_builder(
                 [], committer='me@example.com')

=== modified file 'breezy/tests/per_repository/test_repository.py'
--- breezy/tests/per_repository/test_repository.py	2017-06-11 14:07:05 +0000
+++ breezy/tests/per_repository/test_repository.py	2017-06-22 01:06:22 +0000
@@ -424,6 +424,25 @@
             self.assertEqual(revision.revision_id, revision_id)
             self.assertEqual(revision, repo.get_revision(revision_id))
 
+    def test_iter_revisions(self):
+        tree = self.make_branch_and_tree('.')
+        tree.commit('initial empty commit', rev_id='a-rev',
+                    allow_pointless=True)
+        tree.commit('second empty commit', rev_id='b-rev',
+                    allow_pointless=True)
+        tree.commit('third empty commit', rev_id='c-rev',
+                    allow_pointless=True)
+        repo = tree.branch.repository
+        revision_ids = ['a-rev', 'c-rev', 'b-rev', 'd-rev']
+        revid_with_rev = repo.iter_revisions(revision_ids)
+        self.assertEqual(
+            set((revid, rev.revision_id if rev is not None else None)
+                for (revid, rev) in revid_with_rev),
+            {('a-rev', 'a-rev'),
+             ('b-rev', 'b-rev'),
+             ('c-rev', 'c-rev'),
+             ('d-rev', None)})
+
     def test_root_entry_has_revision(self):
         tree = self.make_branch_and_tree('.')
         tree.commit('message', rev_id='rev_id')

=== modified file 'breezy/tests/per_transport.py'
--- breezy/tests/per_transport.py	2017-06-11 20:15:04 +0000
+++ breezy/tests/per_transport.py	2017-07-15 13:23:08 +0000
@@ -34,7 +34,6 @@
     )
 from ..errors import (ConnectionError,
                       FileExists,
-                      InvalidURL,
                       NoSuchFile,
                       PathError,
                       TransportNotPossible,
@@ -1554,7 +1553,7 @@
 
         # A plain unicode string is not a valid url
         for fname in files:
-            self.assertRaises(InvalidURL, t.get, fname)
+            self.assertRaises(urlutils.InvalidURL, t.get, fname)
 
         for fname in files:
             fname_utf8 = fname.encode('utf-8')

=== modified file 'breezy/tests/per_versionedfile.py'
--- breezy/tests/per_versionedfile.py	2017-06-08 23:30:31 +0000
+++ breezy/tests/per_versionedfile.py	2017-07-12 00:35:04 +0000
@@ -65,7 +65,10 @@
     VirtualVersionedFiles,
     make_versioned_files_factory,
     )
-from ..bzr.weave import WeaveFile
+from ..bzr.weave import (
+    WeaveFile,
+    WeaveInvalidChecksum,
+    )
 from ..bzr.weavefile import write_weave
 from .scenarios import load_tests_apply_scenarios
 
@@ -616,16 +619,16 @@
         w = self.get_file_corrupted_text()
 
         self.assertEqual('hello\n', w.get_text('v1'))
-        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
-        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
-        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+        self.assertRaises(WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(WeaveInvalidChecksum, w.check)
 
         w = self.get_file_corrupted_checksum()
 
         self.assertEqual('hello\n', w.get_text('v1'))
-        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
-        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
-        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+        self.assertRaises(WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(WeaveInvalidChecksum, w.check)
 
     def get_file_corrupted_text(self):
         """Return a versioned file with corrupt text but valid metadata."""

=== modified file 'breezy/tests/per_workingtree/test_commit.py'
--- breezy/tests/per_workingtree/test_commit.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/per_workingtree/test_commit.py	2017-06-20 00:26:58 +0000
@@ -420,6 +420,7 @@
         if not tree.supports_tree_reference():
             # inapplicable test.
             return
+        self.knownFailure('nested trees don\'t work well with iter_changes')
         subtree = self.make_branch_and_tree('subtree')
         tree.add(['subtree'])
         self.build_tree(['subtree/file'])

=== modified file 'breezy/tests/per_workingtree/test_get_file_mtime.py'
--- breezy/tests/per_workingtree/test_get_file_mtime.py	2017-05-21 18:10:28 +0000
+++ breezy/tests/per_workingtree/test_get_file_mtime.py	2017-07-14 20:34:39 +0000
@@ -19,6 +19,7 @@
 import os
 
 from breezy import errors
+from breezy.tree import FileTimestampUnavailable
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -106,7 +107,7 @@
         os.remove('tree/one')
         tree.lock_read()
         try:
-            self.assertRaises(errors.FileTimestampUnavailable,
+            self.assertRaises(FileTimestampUnavailable,
                 tree.get_file_mtime, file_id='one-id')
         finally:
             tree.unlock()

=== modified file 'breezy/tests/test__simple_set.py'
--- breezy/tests/test__simple_set.py	2017-05-25 01:35:55 +0000
+++ breezy/tests/test__simple_set.py	2017-06-18 10:15:11 +0000
@@ -35,7 +35,7 @@
     """A simple object which has a fixed hash value.
 
     We could have used an 'int', but it turns out that Int objects don't
-    implement tp_richcompare...
+    implement tp_richcompare in Python 2.
     """
 
     def __init__(self, the_hash):
@@ -69,12 +69,16 @@
     def __eq__(self, other):
         raise RuntimeError('I refuse to play nice')
 
+    __hash__ = _Hashable.__hash__
+
 
 class _NoImplementCompare(_Hashable):
 
     def __eq__(self, other):
         return NotImplemented
 
+    __hash__ = _Hashable.__hash__
+
 
 # Even though this is an extension, we don't permute the tests for a python
 # version. As the plain python version is just a dict or set
@@ -87,14 +91,6 @@
     _test_needs_features = [compiled_simpleset_feature]
     module = _simple_set_pyx
 
-    def assertIn(self, obj, container):
-        self.assertTrue(obj in container,
-            '%s not found in %s' % (obj, container))
-
-    def assertNotIn(self, obj, container):
-        self.assertTrue(obj not in container,
-            'We found %s in %s' % (obj, container))
-
     def assertFillState(self, used, fill, mask, obj):
         self.assertEqual((used, fill, mask), (obj.used, obj.fill, obj.mask))
 
@@ -281,9 +277,10 @@
 
     def test__resize(self):
         obj = self.module.SimpleSet()
-        k1 = ('foo',)
-        k2 = ('bar',)
-        k3 = ('baz',)
+        # Need objects with exact hash as checking offset of <null> later
+        k1 = _Hashable(501)
+        k2 = _Hashable(591)
+        k3 = _Hashable(2051)
         obj.add(k1)
         obj.add(k2)
         obj.add(k3)
@@ -373,7 +370,7 @@
             all.add(key)
         self.assertEqual(sorted([k1, k2, k3]), sorted(all))
         iterator = iter(obj)
-        next(iterator)
+        self.assertIn(next(iterator), all)
         obj.add(('foo',))
         # Set changed size
         self.assertRaises(RuntimeError, next, iterator)

=== modified file 'breezy/tests/test__static_tuple.py'
--- breezy/tests/test__static_tuple.py	2017-05-23 14:08:03 +0000
+++ breezy/tests/test__static_tuple.py	2017-06-27 00:57:14 +0000
@@ -16,7 +16,11 @@
 
 """Tests for the StaticTuple type."""
 
-import cPickle
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
+import operator
 import sys
 
 from breezy import (
@@ -26,6 +30,10 @@
     static_tuple,
     tests,
     )
+from breezy.sixish import (
+    PY3,
+    text_type,
+    )
 from breezy.tests import (
     features,
     )
@@ -213,6 +221,8 @@
         self.assertRaises(TypeError, self.module.StaticTuple, subint(2))
 
     def test_holds_long(self):
+        if PY3:
+            self.skipTest("No long type on Python 3")
         k1 = self.module.StaticTuple(2**65)
         class sublong(long):
             pass
@@ -225,15 +235,15 @@
             pass
         self.assertRaises(TypeError, self.module.StaticTuple, subfloat(1.5))
 
-    def test_holds_str(self):
-        k1 = self.module.StaticTuple('astring')
-        class substr(str):
+    def test_holds_bytes(self):
+        k1 = self.module.StaticTuple(b'astring')
+        class substr(bytes):
             pass
-        self.assertRaises(TypeError, self.module.StaticTuple, substr('a'))
+        self.assertRaises(TypeError, self.module.StaticTuple, substr(b'a'))
 
     def test_holds_unicode(self):
         k1 = self.module.StaticTuple(u'\xb5')
-        class subunicode(unicode):
+        class subunicode(text_type):
             pass
         self.assertRaises(TypeError, self.module.StaticTuple,
                           subunicode(u'\xb5'))
@@ -278,15 +288,24 @@
         k6 = self.module.StaticTuple(k3, k4)
         self.assertCompareEqual(k5, k6)
 
-    def assertCompareDifferent(self, k_small, k_big):
+    def check_strict_compare(self, k1, k2, mismatched_types):
+        """True if on Python 3 and stricter comparison semantics are used."""
+        if PY3 and mismatched_types:
+            for op in ("ge", "gt", "le", "lt"):
+                self.assertRaises(TypeError, getattr(operator, op), k1, k2)
+            return True
+        return False
+
+    def assertCompareDifferent(self, k_small, k_big, mismatched_types=False):
         self.assertFalse(k_small == k_big)
-        self.assertFalse(k_small >= k_big)
-        self.assertFalse(k_small > k_big)
         self.assertTrue(k_small != k_big)
-        self.assertTrue(k_small <= k_big)
-        self.assertTrue(k_small < k_big)
+        if not self.check_strict_compare(k_small, k_big, mismatched_types):
+            self.assertFalse(k_small >= k_big)
+            self.assertFalse(k_small > k_big)
+            self.assertTrue(k_small <= k_big)
+            self.assertTrue(k_small < k_big)
 
-    def assertCompareNoRelation(self, k1, k2):
+    def assertCompareNoRelation(self, k1, k2, mismatched_types=False):
         """Run the comparison operators, make sure they do something.
 
         However, we don't actually care what comes first or second. This is
@@ -295,20 +314,21 @@
         """
         self.assertFalse(k1 == k2)
         self.assertTrue(k1 != k2)
-        # Do the comparison, but we don't care about the result
-        k1 >= k2
-        k1 > k2
-        k1 <= k2
-        k1 < k2
+        if not self.check_strict_compare(k1, k2, mismatched_types):
+            # Do the comparison, but we don't care about the result
+            k1 >= k2
+            k1 > k2
+            k1 <= k2
+            k1 < k2
 
     def test_compare_vs_none(self):
         k1 = self.module.StaticTuple('baz', 'bing')
-        self.assertCompareDifferent(None, k1)
+        self.assertCompareDifferent(None, k1, mismatched_types=True)
     
     def test_compare_cross_class(self):
         k1 = self.module.StaticTuple('baz', 'bing')
-        self.assertCompareNoRelation(10, k1)
-        self.assertCompareNoRelation('baz', k1)
+        self.assertCompareNoRelation(10, k1, mismatched_types=True)
+        self.assertCompareNoRelation('baz', k1, mismatched_types=True)
 
     def test_compare_all_different_same_width(self):
         k1 = self.module.StaticTuple('baz', 'bing')
@@ -335,8 +355,8 @@
         k4 = self.module.StaticTuple(k1, k2)
         self.assertCompareDifferent(k3, k4)
         k5 = self.module.StaticTuple('foo', None)
-        self.assertCompareDifferent(k5, k1)
-        self.assertCompareDifferent(k5, k2)
+        self.assertCompareDifferent(k5, k1, mismatched_types=True)
+        self.assertCompareDifferent(k5, k2, mismatched_types=True)
 
     def test_compare_diff_width(self):
         k1 = self.module.StaticTuple('foo')
@@ -350,13 +370,13 @@
         k1 = self.module.StaticTuple('foo', 'bar')
         k2 = self.module.StaticTuple('foo', 1, None, u'\xb5', 1.2, 2**65, True,
                                      k1)
-        self.assertCompareNoRelation(k1, k2)
+        self.assertCompareNoRelation(k1, k2, mismatched_types=True)
         k3 = self.module.StaticTuple('foo')
         self.assertCompareDifferent(k3, k1)
         k4 = self.module.StaticTuple(None)
-        self.assertCompareDifferent(k4, k1)
+        self.assertCompareDifferent(k4, k1, mismatched_types=True)
         k5 = self.module.StaticTuple(1)
-        self.assertCompareNoRelation(k1, k5)
+        self.assertCompareNoRelation(k1, k5, mismatched_types=True)
 
     def test_compare_to_tuples(self):
         k1 = self.module.StaticTuple('foo')
@@ -372,7 +392,7 @@
         self.assertCompareDifferent(('foo',), k2)
         self.assertCompareDifferent(('foo', 'aaa'), k2)
         self.assertCompareDifferent(('baz', 'bing'), k2)
-        self.assertCompareDifferent(('foo', 10), k2)
+        self.assertCompareDifferent(('foo', 10), k2, mismatched_types=True)
 
         k3 = self.module.StaticTuple(k1, k2)
         self.assertCompareEqual(k3, (('foo',), ('foo', 'bar')))
@@ -388,7 +408,7 @@
         # This requires comparing a StaticTuple to a 'string', and then
         # interpreting that value in the next higher StaticTuple. This used to
         # generate a PyErr_BadIternalCall. We now fall back to *something*.
-        self.assertCompareNoRelation(k1, k2)
+        self.assertCompareNoRelation(k1, k2, mismatched_types=True)
 
     def test_hash(self):
         k = self.module.StaticTuple('foo')
@@ -416,16 +436,8 @@
         k = self.module.StaticTuple('foo', 'bar', 'baz', 'bing')
         self.assertEqual(('foo', 'bar'), k[:2])
         self.assertEqual(('baz',), k[2:-1])
-        try:
-            val = k[::2]
-        except TypeError:
-            # C implementation raises a TypeError, we don't need the
-            # implementation yet, so allow this to pass
-            pass
-        else:
-            # Python implementation uses a regular Tuple, so make sure it gives
-            # the right result
-            self.assertEqual(('foo', 'baz'), val)
+        self.assertEqual(('foo', 'baz',), k[::2])
+        self.assertRaises(TypeError, k.__getitem__, 'not_slice')
 
     def test_referents(self):
         # We implement tp_traverse so that things like 'meliae' can measure the
@@ -582,20 +594,20 @@
 
     def test_pickle(self):
         st = self.module.StaticTuple('foo', 'bar')
-        pickled = cPickle.dumps(st)
-        unpickled = cPickle.loads(pickled)
+        pickled = pickle.dumps(st)
+        unpickled = pickle.loads(pickled)
         self.assertEqual(unpickled, st)
 
     def test_pickle_empty(self):
         st = self.module.StaticTuple()
-        pickled = cPickle.dumps(st)
-        unpickled = cPickle.loads(pickled)
+        pickled = pickle.dumps(st)
+        unpickled = pickle.loads(pickled)
         self.assertIs(st, unpickled)
 
     def test_pickle_nested(self):
         st = self.module.StaticTuple('foo', self.module.StaticTuple('bar'))
-        pickled = cPickle.dumps(st)
-        unpickled = cPickle.loads(pickled)
+        pickled = pickle.dumps(st)
+        unpickled = pickle.loads(pickled)
         self.assertEqual(unpickled, st)
 
     def test_static_tuple_thunk(self):

=== modified file 'breezy/tests/test_btree_index.py'
--- breezy/tests/test_btree_index.py	2017-06-10 01:39:46 +0000
+++ breezy/tests/test_btree_index.py	2017-07-02 14:35:49 +0000
@@ -66,17 +66,19 @@
 
     def make_nodes(self, count, key_elements, reference_lists):
         """Generate count*key_elements sample nodes."""
+        def _pos_to_key(pos, lead=b""):
+            return (lead + (b"%d" % pos) * 40,)
         keys = []
         for prefix_pos in range(key_elements):
             if key_elements - 1:
-                prefix = (str(prefix_pos) * 40,)
+                prefix = _pos_to_key(prefix_pos)
             else:
                 prefix = ()
             for pos in range(count):
                 # TODO: This creates odd keys. When count == 100,000, it
                 #       creates a 240 byte key
-                key = prefix + (str(pos) * 40,)
-                value = "value:%s" % pos
+                key = prefix + _pos_to_key(pos)
+                value = b"value:%d" % pos
                 if reference_lists:
                     # generate some references
                     refs = []
@@ -89,10 +91,10 @@
                         for ref_pos in range(list_pos + pos % 2):
                             if pos % 2:
                                 # refer to a nearby key
-                                refs[-1].append(prefix + ("ref" + str(pos - 1) * 40,))
+                                refs[-1].append(prefix + _pos_to_key(pos - 1, b"ref"))
                             else:
                                 # serial of this ref in the ref list
-                                refs[-1].append(prefix + ("ref" + str(ref_pos) * 40,))
+                                refs[-1].append(prefix + _pos_to_key(ref_pos, b"ref"))
                         refs[-1] = tuple(refs[-1])
                     refs = tuple(refs)
                 else:
@@ -131,8 +133,8 @@
         content = temp_file.read()
         del temp_file
         self.assertEqual(
-            "B+Tree Graph Index 2\nnode_ref_lists=0\nkey_elements=1\nlen=0\n"
-            "row_lengths=\n",
+            b"B+Tree Graph Index 2\nnode_ref_lists=0\nkey_elements=1\nlen=0\n"
+            b"row_lengths=\n",
             content)
 
     def test_empty_2_1(self):
@@ -142,8 +144,8 @@
         content = temp_file.read()
         del temp_file
         self.assertEqual(
-            "B+Tree Graph Index 2\nnode_ref_lists=1\nkey_elements=2\nlen=0\n"
-            "row_lengths=\n",
+            b"B+Tree Graph Index 2\nnode_ref_lists=1\nkey_elements=2\nlen=0\n"
+            b"row_lengths=\n",
             content)
 
     def test_root_leaf_1_0(self):
@@ -157,17 +159,17 @@
         del temp_file
         self.assertEqual(131, len(content))
         self.assertEqual(
-            "B+Tree Graph Index 2\nnode_ref_lists=0\nkey_elements=1\nlen=5\n"
-            "row_lengths=1\n",
+            b"B+Tree Graph Index 2\nnode_ref_lists=0\nkey_elements=1\nlen=5\n"
+            b"row_lengths=1\n",
             content[:73])
         node_content = content[73:]
         node_bytes = zlib.decompress(node_content)
-        expected_node = ("type=leaf\n"
-            "0000000000000000000000000000000000000000\x00\x00value:0\n"
-            "1111111111111111111111111111111111111111\x00\x00value:1\n"
-            "2222222222222222222222222222222222222222\x00\x00value:2\n"
-            "3333333333333333333333333333333333333333\x00\x00value:3\n"
-            "4444444444444444444444444444444444444444\x00\x00value:4\n")
+        expected_node = (b"type=leaf\n"
+            b"0000000000000000000000000000000000000000\x00\x00value:0\n"
+            b"1111111111111111111111111111111111111111\x00\x00value:1\n"
+            b"2222222222222222222222222222222222222222\x00\x00value:2\n"
+            b"3333333333333333333333333333333333333333\x00\x00value:3\n"
+            b"4444444444444444444444444444444444444444\x00\x00value:4\n")
         self.assertEqual(expected_node, node_bytes)
 
     def test_root_leaf_2_2(self):
@@ -181,24 +183,24 @@
         del temp_file
         self.assertEqual(238, len(content))
         self.assertEqual(
-            "B+Tree Graph Index 2\nnode_ref_lists=2\nkey_elements=2\nlen=10\n"
-            "row_lengths=1\n",
+            b"B+Tree Graph Index 2\nnode_ref_lists=2\nkey_elements=2\nlen=10\n"
+            b"row_lengths=1\n",
             content[:74])
         node_content = content[74:]
         node_bytes = zlib.decompress(node_content)
         expected_node = (
-            "type=leaf\n"
-            "0000000000000000000000000000000000000000\x000000000000000000000000000000000000000000\x00\t0000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\x00value:0\n"
-            "0000000000000000000000000000000000000000\x001111111111111111111111111111111111111111\x000000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\t0000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\r0000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\x00value:1\n"
-            "0000000000000000000000000000000000000000\x002222222222222222222222222222222222222222\x00\t0000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\x00value:2\n"
-            "0000000000000000000000000000000000000000\x003333333333333333333333333333333333333333\x000000000000000000000000000000000000000000\x00ref2222222222222222222222222222222222222222\t0000000000000000000000000000000000000000\x00ref2222222222222222222222222222222222222222\r0000000000000000000000000000000000000000\x00ref2222222222222222222222222222222222222222\x00value:3\n"
-            "0000000000000000000000000000000000000000\x004444444444444444444444444444444444444444\x00\t0000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\x00value:4\n"
-            "1111111111111111111111111111111111111111\x000000000000000000000000000000000000000000\x00\t1111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\x00value:0\n"
-            "1111111111111111111111111111111111111111\x001111111111111111111111111111111111111111\x001111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\t1111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\r1111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\x00value:1\n"
-            "1111111111111111111111111111111111111111\x002222222222222222222222222222222222222222\x00\t1111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\x00value:2\n"
-            "1111111111111111111111111111111111111111\x003333333333333333333333333333333333333333\x001111111111111111111111111111111111111111\x00ref2222222222222222222222222222222222222222\t1111111111111111111111111111111111111111\x00ref2222222222222222222222222222222222222222\r1111111111111111111111111111111111111111\x00ref2222222222222222222222222222222222222222\x00value:3\n"
-            "1111111111111111111111111111111111111111\x004444444444444444444444444444444444444444\x00\t1111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\x00value:4\n"
-            ""
+            b"type=leaf\n"
+            b"0000000000000000000000000000000000000000\x000000000000000000000000000000000000000000\x00\t0000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\x00value:0\n"
+            b"0000000000000000000000000000000000000000\x001111111111111111111111111111111111111111\x000000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\t0000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\r0000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\x00value:1\n"
+            b"0000000000000000000000000000000000000000\x002222222222222222222222222222222222222222\x00\t0000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\x00value:2\n"
+            b"0000000000000000000000000000000000000000\x003333333333333333333333333333333333333333\x000000000000000000000000000000000000000000\x00ref2222222222222222222222222222222222222222\t0000000000000000000000000000000000000000\x00ref2222222222222222222222222222222222222222\r0000000000000000000000000000000000000000\x00ref2222222222222222222222222222222222222222\x00value:3\n"
+            b"0000000000000000000000000000000000000000\x004444444444444444444444444444444444444444\x00\t0000000000000000000000000000000000000000\x00ref0000000000000000000000000000000000000000\x00value:4\n"
+            b"1111111111111111111111111111111111111111\x000000000000000000000000000000000000000000\x00\t1111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\x00value:0\n"
+            b"1111111111111111111111111111111111111111\x001111111111111111111111111111111111111111\x001111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\t1111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\r1111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\x00value:1\n"
+            b"1111111111111111111111111111111111111111\x002222222222222222222222222222222222222222\x00\t1111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\x00value:2\n"
+            b"1111111111111111111111111111111111111111\x003333333333333333333333333333333333333333\x001111111111111111111111111111111111111111\x00ref2222222222222222222222222222222222222222\t1111111111111111111111111111111111111111\x00ref2222222222222222222222222222222222222222\r1111111111111111111111111111111111111111\x00ref2222222222222222222222222222222222222222\x00value:3\n"
+            b"1111111111111111111111111111111111111111\x004444444444444444444444444444444444444444\x00\t1111111111111111111111111111111111111111\x00ref0000000000000000000000000000000000000000\x00value:4\n"
+            b""
             )
         self.assertEqual(expected_node, node_bytes)
 
@@ -213,17 +215,17 @@
         del temp_file
         self.assertEqualApproxCompressed(9283, len(content))
         self.assertEqual(
-            "B+Tree Graph Index 2\nnode_ref_lists=0\nkey_elements=1\nlen=400\n"
-            "row_lengths=1,2\n",
+            b"B+Tree Graph Index 2\nnode_ref_lists=0\nkey_elements=1\nlen=400\n"
+            b"row_lengths=1,2\n",
             content[:77])
         root = content[77:4096]
         leaf1 = content[4096:8192]
         leaf2 = content[8192:]
         root_bytes = zlib.decompress(root)
         expected_root = (
-            "type=internal\n"
-            "offset=0\n"
-            ) + ("307" * 40) + "\n"
+            b"type=internal\n"
+            b"offset=0\n"
+            ) + (b"307" * 40) + b"\n"
         self.assertEqual(expected_root, root_bytes)
         # We already know serialisation works for leaves, check key selection:
         leaf1_bytes = zlib.decompress(leaf1)
@@ -247,8 +249,8 @@
         del temp_file
         self.assertEqualApproxCompressed(155, len(content))
         self.assertEqual(
-            "B+Tree Graph Index 2\nnode_ref_lists=0\nkey_elements=1\nlen=10\n"
-            "row_lengths=1\n",
+            b"B+Tree Graph Index 2\nnode_ref_lists=0\nkey_elements=1\nlen=10\n"
+            b"row_lengths=1\n",
             content[:74])
         # Check thelast page is well formed
         leaf2 = content[74:]
@@ -269,8 +271,8 @@
         del temp_file
         self.assertEqualApproxCompressed(9283, len(content))
         self.assertEqual(
-            "B+Tree Graph Index 2\nnode_ref_lists=0\nkey_elements=1\nlen=400\n"
-            "row_lengths=1,2\n",
+            b"B+Tree Graph Index 2\nnode_ref_lists=0\nkey_elements=1\nlen=400\n"
+            b"row_lengths=1,2\n",
             content[:77])
         # Check the last page is well formed
         leaf2 = content[8192:]
@@ -328,8 +330,8 @@
         del temp_file
         self.assertEqualApproxCompressed(12643, len(content))
         self.assertEqual(
-            "B+Tree Graph Index 2\nnode_ref_lists=2\nkey_elements=2\nlen=200\n"
-            "row_lengths=1,3\n",
+            b"B+Tree Graph Index 2\nnode_ref_lists=2\nkey_elements=2\nlen=200\n"
+            b"row_lengths=1,3\n",
             content[:77])
         root = content[77:4096]
         leaf1 = content[4096:8192]
@@ -337,10 +339,10 @@
         leaf3 = content[12288:]
         root_bytes = zlib.decompress(root)
         expected_root = (
-            "type=internal\n"
-            "offset=0\n"
-            + ("0" * 40) + "\x00" + ("91" * 40) + "\n"
-            + ("1" * 40) + "\x00" + ("81" * 40) + "\n"
+            b"type=internal\n"
+            b"offset=0\n"
+            + (b"0" * 40) + b"\x00" + (b"91" * 40) + b"\n"
+            + (b"1" * 40) + b"\x00" + (b"81" * 40) + b"\n"
             )
         self.assertEqual(expected_root, root_bytes)
         # We assume the other leaf nodes have been written correctly - layering
@@ -635,7 +637,7 @@
         content = temp_file.read()
         del temp_file
         size = len(content)
-        transport.put_bytes('index', (' '*offset)+content)
+        transport.put_bytes('index', (b' '*offset)+content)
         return btree_index.BTreeGraphIndex(transport, 'index', size=size,
                                            offset=offset)
 
@@ -734,14 +736,14 @@
         self.assertEqual(200, index.key_count())
 
     def test__read_nodes_no_size_one_page_reads_once(self):
-        self.make_index(nodes=[(('key',), 'value', ())])
+        self.make_index(nodes=[((b'key',), b'value', ())])
         trans = transport.get_transport_from_url('trace+' + self.get_url())
         index = btree_index.BTreeGraphIndex(trans, 'index', None)
         del trans._activity[:]
         nodes = dict(index._read_nodes([0]))
         self.assertEqual({0}, set(nodes))
         node = nodes[0]
-        self.assertEqual([('key',)], node.all_keys())
+        self.assertEqual([(b'key',)], node.all_keys())
         self.assertEqual([('get', 'index')], trans._activity)
 
     def test__read_nodes_no_size_multiple_pages(self):
@@ -847,17 +849,17 @@
     def test_key_too_big(self):
         # the size that matters here is the _compressed_ size of the key, so we can't
         # do a simple character repeat.
-        bigKey = ''.join(map(repr, range(btree_index._PAGE_SIZE)))
+        bigKey = b''.join(b'%d' % n for n in range(btree_index._PAGE_SIZE))
         self.assertRaises(errors.BadIndexKey,
                           self.make_index,
-                          nodes=[((bigKey,), 'value', ())])
-        
+                          nodes=[((bigKey,), b'value', ())])
+
     def test_iter_all_only_root_no_size(self):
-        self.make_index(nodes=[(('key',), 'value', ())])
+        self.make_index(nodes=[((b'key',), b'value', ())])
         t = transport.get_transport_from_url('trace+' + self.get_url(''))
         index = btree_index.BTreeGraphIndex(t, 'index', None)
         del t._activity[:]
-        self.assertEqual([(('key',), 'value')],
+        self.assertEqual([((b'key',), b'value')],
                          [x[1:] for x in index.iter_all_entries()])
         self.assertEqual([('get', 'index')], t._activity)
 
@@ -908,14 +910,6 @@
             self.assertEqualDiff(pprint.pformat(expected),
                                  pprint.pformat(t._activity))
 
-    def _test_iter_entries_references_resolved(self):
-        index = self.make_index(1, nodes=[
-            (('name', ), 'data', ([('ref', ), ('ref', )], )),
-            (('ref', ), 'refdata', ([], ))])
-        self.assertEqual({(index, ('name', ), 'data', ((('ref',),('ref',)),)),
-            (index, ('ref', ), 'refdata', ((), ))},
-            set(index.iter_entries([('name',), ('ref',)])))
-
     def test_iter_entries_references_2_refs_resolved(self):
         # iterating some entries reads just the pages needed. For now, to
         # get it working and start measuring, only 4K pages are read.
@@ -953,52 +947,56 @@
     def test_iter_key_prefix_wrong_length(self):
         index = self.make_index()
         self.assertRaises(errors.BadIndexKey, list,
-            index.iter_entries_prefix([('foo', None)]))
+            index.iter_entries_prefix([(b'foo', None)]))
         index = self.make_index(key_elements=2)
         self.assertRaises(errors.BadIndexKey, list,
-            index.iter_entries_prefix([('foo', )]))
+            index.iter_entries_prefix([(b'foo', )]))
         self.assertRaises(errors.BadIndexKey, list,
-            index.iter_entries_prefix([('foo', None, None)]))
+            index.iter_entries_prefix([(b'foo', None, None)]))
 
     def test_iter_key_prefix_1_key_element_no_refs(self):
-        index = self.make_index( nodes=[
-            (('name', ), 'data', ()),
-            (('ref', ), 'refdata', ())])
-        self.assertEqual({(index, ('name', ), 'data'),
-            (index, ('ref', ), 'refdata')},
-            set(index.iter_entries_prefix([('name', ), ('ref', )])))
+        index = self.make_index(nodes=[
+            ((b'name', ), b'data', ()),
+            ((b'ref', ), b'refdata', ())])
+        self.assertEqual({(index, (b'name', ), b'data'),
+            (index, (b'ref', ), b'refdata')},
+            set(index.iter_entries_prefix([(b'name', ), (b'ref', )])))
 
     def test_iter_key_prefix_1_key_element_refs(self):
         index = self.make_index(1, nodes=[
-            (('name', ), 'data', ([('ref', )], )),
-            (('ref', ), 'refdata', ([], ))])
-        self.assertEqual({(index, ('name', ), 'data', ((('ref',),),)),
-            (index, ('ref', ), 'refdata', ((), ))},
-            set(index.iter_entries_prefix([('name', ), ('ref', )])))
+            ((b'name', ), b'data', ([(b'ref', )], )),
+            ((b'ref', ), b'refdata', ([], ))])
+        self.assertEqual({(index, (b'name', ), b'data', (((b'ref',),),)),
+            (index, (b'ref', ), b'refdata', ((), ))},
+            set(index.iter_entries_prefix([(b'name', ), (b'ref', )])))
 
     def test_iter_key_prefix_2_key_element_no_refs(self):
         index = self.make_index(key_elements=2, nodes=[
-            (('name', 'fin1'), 'data', ()),
-            (('name', 'fin2'), 'beta', ()),
-            (('ref', 'erence'), 'refdata', ())])
-        self.assertEqual({(index, ('name', 'fin1'), 'data'),
-            (index, ('ref', 'erence'), 'refdata')},
-            set(index.iter_entries_prefix([('name', 'fin1'), ('ref', 'erence')])))
-        self.assertEqual({(index, ('name', 'fin1'), 'data'),
-            (index, ('name', 'fin2'), 'beta')},
-            set(index.iter_entries_prefix([('name', None)])))
+            ((b'name', b'fin1'), b'data', ()),
+            ((b'name', b'fin2'), b'beta', ()),
+            ((b'ref', b'erence'), b'refdata', ())])
+        self.assertEqual({(index, (b'name', b'fin1'), b'data'),
+            (index, (b'ref', b'erence'), b'refdata')},
+            set(index.iter_entries_prefix(
+                [(b'name', b'fin1'), (b'ref', b'erence')])))
+        self.assertEqual({(index, (b'name', b'fin1'), b'data'),
+            (index, (b'name', b'fin2'), b'beta')},
+            set(index.iter_entries_prefix([(b'name', None)])))
 
     def test_iter_key_prefix_2_key_element_refs(self):
         index = self.make_index(1, key_elements=2, nodes=[
-            (('name', 'fin1'), 'data', ([('ref', 'erence')], )),
-            (('name', 'fin2'), 'beta', ([], )),
-            (('ref', 'erence'), 'refdata', ([], ))])
-        self.assertEqual({(index, ('name', 'fin1'), 'data', ((('ref', 'erence'),),)),
-            (index, ('ref', 'erence'), 'refdata', ((), ))},
-            set(index.iter_entries_prefix([('name', 'fin1'), ('ref', 'erence')])))
-        self.assertEqual({(index, ('name', 'fin1'), 'data', ((('ref', 'erence'),),)),
-            (index, ('name', 'fin2'), 'beta', ((), ))},
-            set(index.iter_entries_prefix([('name', None)])))
+            ((b'name', b'fin1'), b'data', ([(b'ref', b'erence')], )),
+            ((b'name', b'fin2'), b'beta', ([], )),
+            ((b'ref', b'erence'), b'refdata', ([], ))])
+        self.assertEqual(
+            {(index, (b'name', b'fin1'), b'data', (((b'ref', b'erence'),),)),
+                (index, (b'ref', b'erence'), b'refdata', ((), ))},
+            set(index.iter_entries_prefix(
+                [(b'name', b'fin1'), (b'ref', b'erence')])))
+        self.assertEqual(
+            {(index, (b'name', b'fin1'), b'data', (((b'ref', b'erence'),),)),
+                (index, (b'name', b'fin2'), b'beta', ((), ))},
+            set(index.iter_entries_prefix([(b'name', None)])))
 
     # XXX: external_references tests are duplicated in test_index.  We
     # probably should have per_graph_index tests...
@@ -1008,35 +1006,35 @@
 
     def test_external_references_no_results(self):
         index = self.make_index(ref_lists=1, nodes=[
-            (('key',), 'value', ([],))])
+            ((b'key',), b'value', ([],))])
         self.assertEqual(set(), index.external_references(0))
 
     def test_external_references_missing_ref(self):
-        missing_key = ('missing',)
+        missing_key = (b'missing',)
         index = self.make_index(ref_lists=1, nodes=[
-            (('key',), 'value', ([missing_key],))])
+            ((b'key',), b'value', ([missing_key],))])
         self.assertEqual({missing_key}, index.external_references(0))
 
     def test_external_references_multiple_ref_lists(self):
-        missing_key = ('missing',)
+        missing_key = (b'missing',)
         index = self.make_index(ref_lists=2, nodes=[
-            (('key',), 'value', ([], [missing_key]))])
+            ((b'key',), b'value', ([], [missing_key]))])
         self.assertEqual(set([]), index.external_references(0))
         self.assertEqual({missing_key}, index.external_references(1))
 
     def test_external_references_two_records(self):
         index = self.make_index(ref_lists=1, nodes=[
-            (('key-1',), 'value', ([('key-2',)],)),
-            (('key-2',), 'value', ([],)),
+            ((b'key-1',), b'value', ([(b'key-2',)],)),
+            ((b'key-2',), b'value', ([],)),
             ])
         self.assertEqual(set([]), index.external_references(0))
 
     def test__find_ancestors_one_page(self):
-        key1 = ('key-1',)
-        key2 = ('key-2',)
+        key1 = (b'key-1',)
+        key2 = (b'key-2',)
         index = self.make_index(ref_lists=1, key_elements=1, nodes=[
-            (key1, 'value', ([key2],)),
-            (key2, 'value', ([],)),
+            (key1, b'value', ([key2],)),
+            (key2, b'value', ([],)),
             ])
         parent_map = {}
         missing_keys = set()
@@ -1046,12 +1044,12 @@
         self.assertEqual(set(), search_keys)
 
     def test__find_ancestors_one_page_w_missing(self):
-        key1 = ('key-1',)
-        key2 = ('key-2',)
-        key3 = ('key-3',)
+        key1 = (b'key-1',)
+        key2 = (b'key-2',)
+        key3 = (b'key-3',)
         index = self.make_index(ref_lists=1, key_elements=1, nodes=[
-            (key1, 'value', ([key2],)),
-            (key2, 'value', ([],)),
+            (key1, b'value', ([key2],)),
+            (key2, b'value', ([],)),
             ])
         parent_map = {}
         missing_keys = set()
@@ -1064,12 +1062,12 @@
         self.assertEqual(set(), search_keys)
 
     def test__find_ancestors_one_parent_missing(self):
-        key1 = ('key-1',)
-        key2 = ('key-2',)
-        key3 = ('key-3',)
+        key1 = (b'key-1',)
+        key2 = (b'key-2',)
+        key3 = (b'key-3',)
         index = self.make_index(ref_lists=1, key_elements=1, nodes=[
-            (key1, 'value', ([key2],)),
-            (key2, 'value', ([key3],)),
+            (key1, b'value', ([key2],)),
+            (key2, b'value', ([key3],)),
             ])
         parent_map = {}
         missing_keys = set()
@@ -1089,13 +1087,13 @@
         self.assertEqual(set([]), search_keys)
 
     def test__find_ancestors_dont_search_known(self):
-        key1 = ('key-1',)
-        key2 = ('key-2',)
-        key3 = ('key-3',)
+        key1 = (b'key-1',)
+        key2 = (b'key-2',)
+        key3 = (b'key-3',)
         index = self.make_index(ref_lists=1, key_elements=1, nodes=[
-            (key1, 'value', ([key2],)),
-            (key2, 'value', ([key3],)),
-            (key3, 'value', ([],)),
+            (key1, b'value', ([key2],)),
+            (key2, b'value', ([key3],)),
+            (key3, b'value', ([],)),
             ])
         # We already know about key2, so we won't try to search for key3
         parent_map = {key2: (key3,)}
@@ -1114,11 +1112,11 @@
         ref_lists = ((),)
         rev_keys = []
         for i in range(400):
-            rev_id = '%s-%s-%s' % (email,
+            rev_id = ('%s-%s-%s' % (email,
                                    osutils.compact_date(start_time + i),
-                                   osutils.rand_chars(16))
+                                   osutils.rand_chars(16))).encode('ascii')
             rev_key = (rev_id,)
-            nodes.append((rev_key, 'value', ref_lists))
+            nodes.append((rev_key, b'value', ref_lists))
             # We have a ref 'list' of length 1, with a list of parents, with 1
             # parent which is a key
             ref_lists = ((rev_key,),)
@@ -1212,113 +1210,113 @@
         self.overrideAttr(btree_index, '_btree_serializer', self.parse_btree)
 
     def test_LeafNode_1_0(self):
-        node_bytes = ("type=leaf\n"
-            "0000000000000000000000000000000000000000\x00\x00value:0\n"
-            "1111111111111111111111111111111111111111\x00\x00value:1\n"
-            "2222222222222222222222222222222222222222\x00\x00value:2\n"
-            "3333333333333333333333333333333333333333\x00\x00value:3\n"
-            "4444444444444444444444444444444444444444\x00\x00value:4\n")
+        node_bytes = (b"type=leaf\n"
+            b"0000000000000000000000000000000000000000\x00\x00value:0\n"
+            b"1111111111111111111111111111111111111111\x00\x00value:1\n"
+            b"2222222222222222222222222222222222222222\x00\x00value:2\n"
+            b"3333333333333333333333333333333333333333\x00\x00value:3\n"
+            b"4444444444444444444444444444444444444444\x00\x00value:4\n")
         node = btree_index._LeafNode(node_bytes, 1, 0)
         # We do direct access, or don't care about order, to leaf nodes most of
         # the time, so a dict is useful:
         self.assertEqual({
-            ("0000000000000000000000000000000000000000",): ("value:0", ()),
-            ("1111111111111111111111111111111111111111",): ("value:1", ()),
-            ("2222222222222222222222222222222222222222",): ("value:2", ()),
-            ("3333333333333333333333333333333333333333",): ("value:3", ()),
-            ("4444444444444444444444444444444444444444",): ("value:4", ()),
+            (b"0000000000000000000000000000000000000000",): (b"value:0", ()),
+            (b"1111111111111111111111111111111111111111",): (b"value:1", ()),
+            (b"2222222222222222222222222222222222222222",): (b"value:2", ()),
+            (b"3333333333333333333333333333333333333333",): (b"value:3", ()),
+            (b"4444444444444444444444444444444444444444",): (b"value:4", ()),
             }, dict(node.all_items()))
 
     def test_LeafNode_2_2(self):
-        node_bytes = ("type=leaf\n"
-            "00\x0000\x00\t00\x00ref00\x00value:0\n"
-            "00\x0011\x0000\x00ref00\t00\x00ref00\r01\x00ref01\x00value:1\n"
-            "11\x0033\x0011\x00ref22\t11\x00ref22\r11\x00ref22\x00value:3\n"
-            "11\x0044\x00\t11\x00ref00\x00value:4\n"
-            ""
+        node_bytes = (b"type=leaf\n"
+            b"00\x0000\x00\t00\x00ref00\x00value:0\n"
+            b"00\x0011\x0000\x00ref00\t00\x00ref00\r01\x00ref01\x00value:1\n"
+            b"11\x0033\x0011\x00ref22\t11\x00ref22\r11\x00ref22\x00value:3\n"
+            b"11\x0044\x00\t11\x00ref00\x00value:4\n"
+            b""
             )
         node = btree_index._LeafNode(node_bytes, 2, 2)
         # We do direct access, or don't care about order, to leaf nodes most of
         # the time, so a dict is useful:
         self.assertEqual({
-            ('00', '00'): ('value:0', ((), (('00', 'ref00'),))),
-            ('00', '11'): ('value:1',
-                ((('00', 'ref00'),), (('00', 'ref00'), ('01', 'ref01')))),
-            ('11', '33'): ('value:3',
-                ((('11', 'ref22'),), (('11', 'ref22'), ('11', 'ref22')))),
-            ('11', '44'): ('value:4', ((), (('11', 'ref00'),)))
+            (b'00', b'00'): (b'value:0', ((), ((b'00', b'ref00'),))),
+            (b'00', b'11'): (b'value:1', (((b'00', b'ref00'),),
+                ((b'00', b'ref00'), (b'01', b'ref01')))),
+            (b'11', b'33'): (b'value:3', (((b'11', b'ref22'),),
+                ((b'11', b'ref22'), (b'11', b'ref22')))),
+            (b'11', b'44'): (b'value:4', ((), ((b'11', b'ref00'),)))
             }, dict(node.all_items()))
 
     def test_InternalNode_1(self):
-        node_bytes = ("type=internal\n"
-            "offset=1\n"
-            "0000000000000000000000000000000000000000\n"
-            "1111111111111111111111111111111111111111\n"
-            "2222222222222222222222222222222222222222\n"
-            "3333333333333333333333333333333333333333\n"
-            "4444444444444444444444444444444444444444\n"
+        node_bytes = (b"type=internal\n"
+            b"offset=1\n"
+            b"0000000000000000000000000000000000000000\n"
+            b"1111111111111111111111111111111111111111\n"
+            b"2222222222222222222222222222222222222222\n"
+            b"3333333333333333333333333333333333333333\n"
+            b"4444444444444444444444444444444444444444\n"
             )
         node = btree_index._InternalNode(node_bytes)
         # We want to bisect to find the right children from this node, so a
         # vector is most useful.
         self.assertEqual([
-            ("0000000000000000000000000000000000000000",),
-            ("1111111111111111111111111111111111111111",),
-            ("2222222222222222222222222222222222222222",),
-            ("3333333333333333333333333333333333333333",),
-            ("4444444444444444444444444444444444444444",),
+            (b"0000000000000000000000000000000000000000",),
+            (b"1111111111111111111111111111111111111111",),
+            (b"2222222222222222222222222222222222222222",),
+            (b"3333333333333333333333333333333333333333",),
+            (b"4444444444444444444444444444444444444444",),
             ], node.keys)
         self.assertEqual(1, node.offset)
 
     def test_LeafNode_2_2(self):
-        node_bytes = ("type=leaf\n"
-            "00\x0000\x00\t00\x00ref00\x00value:0\n"
-            "00\x0011\x0000\x00ref00\t00\x00ref00\r01\x00ref01\x00value:1\n"
-            "11\x0033\x0011\x00ref22\t11\x00ref22\r11\x00ref22\x00value:3\n"
-            "11\x0044\x00\t11\x00ref00\x00value:4\n"
-            ""
+        node_bytes = (b"type=leaf\n"
+            b"00\x0000\x00\t00\x00ref00\x00value:0\n"
+            b"00\x0011\x0000\x00ref00\t00\x00ref00\r01\x00ref01\x00value:1\n"
+            b"11\x0033\x0011\x00ref22\t11\x00ref22\r11\x00ref22\x00value:3\n"
+            b"11\x0044\x00\t11\x00ref00\x00value:4\n"
+            b""
             )
         node = btree_index._LeafNode(node_bytes, 2, 2)
         # We do direct access, or don't care about order, to leaf nodes most of
         # the time, so a dict is useful:
         self.assertEqual({
-            ('00', '00'): ('value:0', ((), (('00', 'ref00'),))),
-            ('00', '11'): ('value:1',
-                ((('00', 'ref00'),), (('00', 'ref00'), ('01', 'ref01')))),
-            ('11', '33'): ('value:3',
-                ((('11', 'ref22'),), (('11', 'ref22'), ('11', 'ref22')))),
-            ('11', '44'): ('value:4', ((), (('11', 'ref00'),)))
+            (b'00', b'00'): (b'value:0', ((), ((b'00', b'ref00'),))),
+            (b'00', b'11'): (b'value:1', (((b'00', b'ref00'),),
+                ((b'00', b'ref00'), (b'01', b'ref01')))),
+            (b'11', b'33'): (b'value:3', (((b'11', b'ref22'),),
+                ((b'11', b'ref22'), (b'11', b'ref22')))),
+            (b'11', b'44'): (b'value:4', ((), ((b'11', b'ref00'),)))
             }, dict(node.all_items()))
 
     def assertFlattened(self, expected, key, value, refs):
         flat_key, flat_line = self.parse_btree._flatten_node(
             (None, key, value, refs), bool(refs))
-        self.assertEqual('\x00'.join(key), flat_key)
+        self.assertEqual(b'\x00'.join(key), flat_key)
         self.assertEqual(expected, flat_line)
 
     def test__flatten_node(self):
-        self.assertFlattened('key\0\0value\n', ('key',), 'value', [])
-        self.assertFlattened('key\0tuple\0\0value str\n',
-                             ('key', 'tuple'), 'value str', [])
-        self.assertFlattened('key\0tuple\0triple\0\0value str\n',
-                             ('key', 'tuple', 'triple'), 'value str', [])
-        self.assertFlattened('k\0t\0s\0ref\0value str\n',
-                             ('k', 't', 's'), 'value str', [[('ref',)]])
-        self.assertFlattened('key\0tuple\0ref\0key\0value str\n',
-                             ('key', 'tuple'), 'value str', [[('ref', 'key')]])
-        self.assertFlattened("00\x0000\x00\t00\x00ref00\x00value:0\n",
-            ('00', '00'), 'value:0', ((), (('00', 'ref00'),)))
-        self.assertFlattened(
-            "00\x0011\x0000\x00ref00\t00\x00ref00\r01\x00ref01\x00value:1\n",
-            ('00', '11'), 'value:1',
-                ((('00', 'ref00'),), (('00', 'ref00'), ('01', 'ref01'))))
-        self.assertFlattened(
-            "11\x0033\x0011\x00ref22\t11\x00ref22\r11\x00ref22\x00value:3\n",
-            ('11', '33'), 'value:3',
-                ((('11', 'ref22'),), (('11', 'ref22'), ('11', 'ref22'))))
-        self.assertFlattened(
-            "11\x0044\x00\t11\x00ref00\x00value:4\n",
-            ('11', '44'), 'value:4', ((), (('11', 'ref00'),)))
+        self.assertFlattened(b'key\0\0value\n', (b'key',), b'value', [])
+        self.assertFlattened(b'key\0tuple\0\0value str\n',
+            (b'key', b'tuple'), b'value str', [])
+        self.assertFlattened(b'key\0tuple\0triple\0\0value str\n',
+            (b'key', b'tuple', b'triple'), b'value str', [])
+        self.assertFlattened(b'k\0t\0s\0ref\0value str\n',
+            (b'k', b't', b's'), b'value str', [[(b'ref',)]])
+        self.assertFlattened(b'key\0tuple\0ref\0key\0value str\n',
+            (b'key', b'tuple'), b'value str', [[(b'ref', b'key')]])
+        self.assertFlattened(b"00\x0000\x00\t00\x00ref00\x00value:0\n",
+            (b'00', b'00'), b'value:0', ((), ((b'00', b'ref00'),)))
+        self.assertFlattened(
+            b"00\x0011\x0000\x00ref00\t00\x00ref00\r01\x00ref01\x00value:1\n",
+            (b'00', b'11'), b'value:1',
+                (((b'00', b'ref00'),), ((b'00', b'ref00'), (b'01', b'ref01'))))
+        self.assertFlattened(
+            b"11\x0033\x0011\x00ref22\t11\x00ref22\r11\x00ref22\x00value:3\n",
+            (b'11', b'33'), b'value:3',
+                (((b'11', b'ref22'),), ((b'11', b'ref22'), (b'11', b'ref22'))))
+        self.assertFlattened(
+            b"11\x0044\x00\t11\x00ref00\x00value:4\n",
+            (b'11', b'44'), b'value:4', ((), ((b'11', b'ref00'),)))
 
 
 class TestCompiledBtree(tests.TestCase):
@@ -1394,7 +1392,7 @@
         index._key_count = key_count
         index._row_lengths = row_lengths
         index._compute_row_offsets()
-        index._root_node = btree_index._InternalNode('internal\noffset=0\n')
+        index._root_node = btree_index._InternalNode(b'internal\noffset=0\n')
         self.set_cached_offsets(index, cached_offsets)
 
     def make_100_node_index(self):

=== modified file 'breezy/tests/test_bugtracker.py'
--- breezy/tests/test_bugtracker.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/test_bugtracker.py	2017-07-10 20:12:25 +0000
@@ -15,10 +15,37 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-from .. import bugtracker, errors, urlutils
+from .. import bugtracker, urlutils
 from . import TestCase, TestCaseWithMemoryTransport
 
 
+class ErrorsTest(TestCaseWithMemoryTransport):
+
+    def test_unknown_bug_tracker_abbreviation(self):
+        """Test the formatting of UnknownBugTrackerAbbreviation."""
+        branch = self.make_branch('some_branch')
+        error = bugtracker.UnknownBugTrackerAbbreviation('xxx', branch)
+        self.assertEqual(
+            "Cannot find registered bug tracker called xxx on %s" % branch,
+            str(error))
+
+    def test_malformed_bug_identifier(self):
+        """Test the formatting of MalformedBugIdentifier."""
+        error = bugtracker.MalformedBugIdentifier(
+            'bogus', 'reason for bogosity')
+        self.assertEqual(
+            'Did not understand bug identifier bogus: reason for bogosity. '
+            'See "brz help bugs" for more information on this feature.',
+            str(error))
+
+    def test_incorrect_url(self):
+        err = bugtracker.InvalidBugTrackerURL('foo', 'http://bug.com/')
+        self.assertEqual(
+            ("The URL for bug tracker \"foo\" doesn't contain {id}: "
+             "http://bug.com/"),
+            str(err))
+
+
 class TestGetBugURL(TestCaseWithMemoryTransport):
     """Tests for bugtracker.get_bug_url"""
 
@@ -54,7 +81,7 @@
     def test_unrecognized_abbreviation_raises_error(self):
         """If the abbreviation is unrecognized, then raise an error."""
         branch = self.make_branch('some_branch')
-        self.assertRaises(errors.UnknownBugTrackerAbbreviation,
+        self.assertRaises(bugtracker.UnknownBugTrackerAbbreviation,
                           bugtracker.get_bug_url, 'xxx', branch, '1234')
         self.assertEqual([('get', 'xxx', branch)], self.tracker_type.log)
 
@@ -130,7 +157,8 @@
         config = branch.get_config()
         config.set_user_option('bugtracker_foo_url', 'http://bugs.com/view.html')
         tracker = bugtracker.tracker_registry.get_tracker('foo', branch)
-        self.assertRaises(errors.InvalidBugTrackerURL, tracker.get_bug_url, '1234')
+        self.assertRaises(bugtracker.InvalidBugTrackerURL, tracker.get_bug_url,
+                '1234')
 
 
 class TestUniqueIntegerBugTracker(TestCaseWithMemoryTransport):
@@ -179,7 +207,7 @@
         tracker = bugtracker.UniqueIntegerBugTracker('xxx',
                 'http://bugs.com/')
         self.assertRaises(
-            errors.MalformedBugIdentifier, tracker.check_bug_id, 'red')
+            bugtracker.MalformedBugIdentifier, tracker.check_bug_id, 'red')
 
 class TestURLParametrizedBugTracker(TestCaseWithMemoryTransport):
     """Tests for URLParametrizedBugTracker."""
@@ -230,7 +258,7 @@
         should raise an error.
         """
         self.assertRaises(
-            errors.MalformedBugIdentifier, self.tracker.get_bug_url, 'bad')
+            bugtracker.MalformedBugIdentifier, self.tracker.get_bug_url, 'bad')
 
 
 class TestPropertyEncoding(TestCase):

=== modified file 'breezy/tests/test_bzrdir.py'
--- breezy/tests/test_bzrdir.py	2017-06-11 14:07:05 +0000
+++ breezy/tests/test_bzrdir.py	2017-07-14 20:49:41 +0000
@@ -25,6 +25,7 @@
 
 from .. import (
     branch,
+    bzr,
     config,
     controldir,
     errors,
@@ -103,18 +104,18 @@
         my_format_registry.register_lazy('lazy', 'breezy.tests.test_bzrdir',
             'DeprecatedBzrDirFormat', 'Format registered lazily',
             deprecated=True)
-        bzrdir.register_metadir(my_format_registry, 'knit',
+        bzr.register_metadir(my_format_registry, 'knit',
             'breezy.bzr.knitrepo.RepositoryFormatKnit1',
             'Format using knits',
             )
         my_format_registry.set_default('knit')
-        bzrdir.register_metadir(my_format_registry,
+        bzr.register_metadir(my_format_registry,
             'branch6',
             'breezy.bzr.knitrepo.RepositoryFormatKnit3',
             'Experimental successor to knit.  Use at your own risk.',
             branch_format='breezy.bzr.branch.BzrBranchFormat6',
             experimental=True)
-        bzrdir.register_metadir(my_format_registry,
+        bzr.register_metadir(my_format_registry,
             'hidden format',
             'breezy.bzr.knitrepo.RepositoryFormatKnit3',
             'Experimental successor to knit.  Use at your own risk.',
@@ -281,13 +282,13 @@
     def test_find_format(self):
         # is the right format object found for a branch?
         # create a branch with a few known format objects.
-        bzrdir.BzrProber.formats.register(BzrDirFormatTest1.get_format_string(),
+        bzr.BzrProber.formats.register(BzrDirFormatTest1.get_format_string(),
             BzrDirFormatTest1())
-        self.addCleanup(bzrdir.BzrProber.formats.remove,
+        self.addCleanup(bzr.BzrProber.formats.remove,
             BzrDirFormatTest1.get_format_string())
-        bzrdir.BzrProber.formats.register(BzrDirFormatTest2.get_format_string(),
+        bzr.BzrProber.formats.register(BzrDirFormatTest2.get_format_string(),
             BzrDirFormatTest2())
-        self.addCleanup(bzrdir.BzrProber.formats.remove,
+        self.addCleanup(bzr.BzrProber.formats.remove,
             BzrDirFormatTest2.get_format_string())
         t = self.get_transport()
         self.build_tree(["foo/", "bar/"], transport=t)
@@ -318,7 +319,7 @@
         # make a bzrdir
         format.initialize(url)
         # register a format for it.
-        bzrdir.BzrProber.formats.register(format.get_format_string(), format)
+        bzr.BzrProber.formats.register(format.get_format_string(), format)
         # which bzrdir.Open will refuse (not supported)
         self.assertRaises(UnsupportedFormatError, bzrdir.BzrDir.open, url)
         # which bzrdir.open_containing will refuse (not supported)
@@ -327,7 +328,7 @@
         t = _mod_transport.get_transport_from_url(url)
         self.assertEqual(format.open(t), bzrdir.BzrDir.open_unsupported(url))
         # unregister the format
-        bzrdir.BzrProber.formats.remove(format.get_format_string())
+        bzr.BzrProber.formats.remove(format.get_format_string())
         # now open_downlevel should fail too.
         self.assertRaises(UnknownFormatError, bzrdir.BzrDir.open_unsupported, url)
 
@@ -1031,7 +1032,7 @@
     def test_with_features(self):
         tree = self.make_branch_and_tree('tree', format='2a')
         tree.controldir.update_feature_flags({"bar": "required"})
-        self.assertRaises(errors.MissingFeature, bzrdir.BzrDir.open, 'tree')
+        self.assertRaises(bzrdir.MissingFeature, bzrdir.BzrDir.open, 'tree')
         bzrdir.BzrDirMetaFormat1.register_feature('bar')
         self.addCleanup(bzrdir.BzrDirMetaFormat1.unregister_feature, 'bar')
         dir = bzrdir.BzrDir.open('tree')
@@ -1535,7 +1536,7 @@
         # Optional, so trigger an exception
         format = SampleBzrFormat()
         format.features = {"nested-trees": "required"}
-        self.assertRaises(errors.MissingFeature, format.check_support_status,
+        self.assertRaises(bzrdir.MissingFeature, format.check_support_status,
             True)
         self.addCleanup(SampleBzrFormat.unregister_feature, "nested-trees")
         SampleBzrFormat.register_feature("nested-trees")
@@ -1545,7 +1546,7 @@
         # treat unknown necessity as required
         format = SampleBzrFormat()
         format.features = {"nested-trees": "unknown"}
-        self.assertRaises(errors.MissingFeature, format.check_support_status,
+        self.assertRaises(bzrdir.MissingFeature, format.check_support_status,
             True)
         self.addCleanup(SampleBzrFormat.unregister_feature, "nested-trees")
         SampleBzrFormat.register_feature("nested-trees")

=== modified file 'breezy/tests/test_chunk_writer.py'
--- breezy/tests/test_chunk_writer.py	2017-06-04 18:09:30 +0000
+++ breezy/tests/test_chunk_writer.py	2017-07-02 13:39:38 +0000
@@ -26,15 +26,15 @@
 class TestWriter(TestCaseWithTransport):
 
     def check_chunk(self, bytes_list, size):
-        bytes = ''.join(bytes_list)
-        self.assertEqual(size, len(bytes))
-        return zlib.decompress(bytes)
+        data = b''.join(bytes_list)
+        self.assertEqual(size, len(data))
+        return zlib.decompress(data)
 
     def test_chunk_writer_empty(self):
         writer = chunk_writer.ChunkWriter(4096)
         bytes_list, unused, padding = writer.finish()
         node_bytes = self.check_chunk(bytes_list, 4096)
-        self.assertEqual("", node_bytes)
+        self.assertEqual(b"", node_bytes)
         self.assertEqual(None, unused)
         # Only a zlib header.
         self.assertEqual(4088, padding)
@@ -59,22 +59,27 @@
 
     def test_some_data(self):
         writer = chunk_writer.ChunkWriter(4096)
-        writer.write("foo bar baz quux\n")
+        writer.write(b"foo bar baz quux\n")
         bytes_list, unused, padding = writer.finish()
         node_bytes = self.check_chunk(bytes_list, 4096)
-        self.assertEqual("foo bar baz quux\n", node_bytes)
+        self.assertEqual(b"foo bar baz quux\n", node_bytes)
         self.assertEqual(None, unused)
         # More than just the header..
         self.assertEqual(4073, padding)
 
+    @staticmethod
+    def _make_lines():
+        lines = []
+        for group in range(48):
+            offset = group * 50
+            numbers = list(range(offset, offset + 50))
+            # Create a line with this group
+            lines.append(b''.join(b'%d' % n for n in numbers) + b'\n')
+        return lines
+
     def test_too_much_data_does_not_exceed_size(self):
         # Generate enough data to exceed 4K
-        lines = []
-        for group in range(48):
-            offset = group * 50
-            numbers = list(range(offset, offset + 50))
-            # Create a line with this group
-            lines.append(''.join(map(str, numbers)) + '\n')
+        lines = self._make_lines()
         writer = chunk_writer.ChunkWriter(4096)
         for idx, line in enumerate(lines):
             if writer.write(line):
@@ -83,19 +88,14 @@
         bytes_list, unused, _ = writer.finish()
         node_bytes = self.check_chunk(bytes_list, 4096)
         # the first 46 lines should have been added
-        expected_bytes = ''.join(lines[:46])
+        expected_bytes = b''.join(lines[:46])
         self.assertEqualDiff(expected_bytes, node_bytes)
         # And the line that failed should have been saved for us
         self.assertEqual(lines[46], unused)
 
     def test_too_much_data_preserves_reserve_space(self):
         # Generate enough data to exceed 4K
-        lines = []
-        for group in range(48):
-            offset = group * 50
-            numbers = list(range(offset, offset + 50))
-            # Create a line with this group
-            lines.append(''.join(map(str, numbers)) + '\n')
+        lines = self._make_lines()
         writer = chunk_writer.ChunkWriter(4096, 256)
         for idx, line in enumerate(lines):
             if writer.write(line):
@@ -103,11 +103,11 @@
                 break
         else:
             self.fail('We were able to write all lines')
-        self.assertFalse(writer.write("A"*256, reserved=True))
+        self.assertFalse(writer.write(b"A"*256, reserved=True))
         bytes_list, unused, _ = writer.finish()
         node_bytes = self.check_chunk(bytes_list, 4096)
         # the first 44 lines should have been added
-        expected_bytes = ''.join(lines[:44]) + "A"*256
+        expected_bytes = b''.join(lines[:44]) + b"A"*256
         self.assertEqualDiff(expected_bytes, node_bytes)
         # And the line that failed should have been saved for us
         self.assertEqual(lines[44], unused)

=== modified file 'breezy/tests/test_commands.py'
--- breezy/tests/test_commands.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/test_commands.py	2017-07-20 00:00:04 +0000
@@ -309,6 +309,25 @@
         self.assertIsInstance(hook_calls[0][1], builtins.cmd_info)
 
 
+class TestCommandNotFound(tests.TestCase):
+
+    def setUp(self):
+        super(TestCommandNotFound, self).setUp()
+        commands._register_builtin_commands()
+        commands.install_bzr_command_hooks()
+
+    def test_not_found_no_suggestion(self):
+        e = self.assertRaises(errors.BzrCommandError,
+            commands.get_cmd_object, 'idontexistand')
+        self.assertEqual('unknown command "idontexistand"', str(e))
+
+    def test_not_found_with_suggestion(self):
+        e = self.assertRaises(errors.BzrCommandError,
+            commands.get_cmd_object, 'statue')
+        self.assertEqual('unknown command "statue". Perhaps you meant "status"',
+            str(e))
+
+
 class TestGetMissingCommandHook(tests.TestCase):
 
     def hook_missing(self):
@@ -429,7 +448,7 @@
         def pre_command(cmd):
             hook_calls.append('pre')
             # verify that all subclasses of BzrCommandError caught too
-            raise errors.BzrOptionError()
+            raise commands.BzrOptionError()
 
         def post_command(cmd, e):
             self.fail('post_command should not be called')
@@ -449,3 +468,20 @@
                           commands.run_bzr, [u'rocks'])
         self.assertEqual(['pre'], hook_calls)
 
+
+class GuessCommandTests(tests.TestCase):
+
+    def setUp(self):
+        super(GuessCommandTests, self).setUp()
+        commands._register_builtin_commands()
+        commands.install_bzr_command_hooks()
+
+    def test_guess_override(self):
+        self.assertEqual('ci', commands.guess_command('ic'))
+
+    def test_guess(self):
+        commands.get_cmd_object('status')
+        self.assertEqual('status', commands.guess_command('statue'))
+
+    def test_none(self):
+        self.assertIs(None, commands.guess_command('nothingisevenclose'))

=== modified file 'breezy/tests/test_commit.py'
--- breezy/tests/test_commit.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/test_commit.py	2017-07-04 20:03:11 +0000
@@ -25,14 +25,18 @@
     )
 from ..branch import Branch
 from ..bzr.bzrdir import BzrDirMetaFormat1
-from ..commit import Commit, NullCommitReporter
+from ..commit import (
+    Commit,
+    NullCommitReporter,
+    filter_excluded,
+    )
 from ..errors import (
     PointlessCommit,
     BzrError,
-    SigningFailed,
     LockContention,
     )
 from . import (
+    TestCase,
     TestCaseWithTransport,
     test_foreign,
     )
@@ -48,7 +52,6 @@
 
     def __init__(self):
         super(MustSignConfig, self).__init__('''
-gpg_signing_command=cat -
 create_signatures=always
 ''')
 
@@ -426,7 +429,6 @@
             # monkey patch gpg signing mechanism
             breezy.gpg.GPGStrategy = breezy.gpg.LoopbackGPGStrategy
             conf = config.MemoryStack('''
-gpg_signing_command=cat -
 create_signatures=always
 ''')
             commit.Commit(config_stack=conf).commit(
@@ -452,10 +454,9 @@
             # monkey patch gpg signing mechanism
             breezy.gpg.GPGStrategy = breezy.gpg.DisabledGPGStrategy
             conf = config.MemoryStack('''
-gpg_signing_command=cat -
 create_signatures=always
 ''')
-            self.assertRaises(SigningFailed,
+            self.assertRaises(breezy.gpg.SigningFailed,
                               commit.Commit(config_stack=conf).commit,
                               message="base",
                               allow_pointless=True,
@@ -832,3 +833,35 @@
         tree2 = branch.create_checkout('repo/tree2')
         tree2.commit('message', rev_id='rev1')
         self.assertTrue(tree2.branch.repository.has_revision('rev1'))
+
+
+class FilterExcludedTests(TestCase):
+
+    def test_add_file_not_excluded(self):
+        changes = [
+            ('fid', (None, 'newpath'),
+             0, (False, False), ('pid', 'pid'), ('newpath', 'newpath'),
+             ('file', 'file'), (True, True))]
+        self.assertEqual(changes, list(filter_excluded(changes, ['otherpath'])))
+
+    def test_add_file_excluded(self):
+        changes = [
+            ('fid', (None, 'newpath'),
+             0, (False, False), ('pid', 'pid'), ('newpath', 'newpath'),
+             ('file', 'file'), (True, True))]
+        self.assertEqual([], list(filter_excluded(changes, ['newpath'])))
+
+    def test_delete_file_excluded(self):
+        changes = [
+            ('fid', ('somepath', None),
+             0, (False, None), ('pid', None), ('newpath', None),
+             ('file', None), (True, None))]
+        self.assertEqual([], list(filter_excluded(changes, ['somepath'])))
+
+    def test_move_from_or_to_excluded(self):
+        changes = [
+            ('fid', ('oldpath', 'newpath'),
+             0, (False, False), ('pid', 'pid'), ('oldpath', 'newpath'),
+             ('file', 'file'), (True, True))]
+        self.assertEqual([], list(filter_excluded(changes, ['oldpath'])))
+        self.assertEqual([], list(filter_excluded(changes, ['newpath'])))

=== modified file 'breezy/tests/test_config.py'
--- breezy/tests/test_config.py	2017-06-11 14:07:05 +0000
+++ breezy/tests/test_config.py	2017-07-15 15:58:12 +0000
@@ -176,7 +176,6 @@
 email=Erik B\u00e5gfors <erik@bagfors.nu>
 editor=vim
 change_editor=vimdiff -of @new_path @old_path
-gpg_signing_command=gnome-gpg
 gpg_signing_key=DD4D5088
 log_format=short
 validate_signatures_in_log=true
@@ -232,7 +231,6 @@
 # test trailing / matching with no children
 [/a/]
 check_signatures=check-available
-gpg_signing_command=false
 gpg_signing_key=default
 user_local_option=local
 # test trailing / matching
@@ -656,7 +654,7 @@
 
     def test_unknown_ref(self):
         c = self.get_config('')
-        self.assertRaises(errors.ExpandingUnknownOption,
+        self.assertRaises(config.ExpandingUnknownOption,
                           c.expand_options, '{foo}')
 
     def test_indirect_ref(self):
@@ -675,14 +673,15 @@
 
     def test_simple_loop(self):
         c = self.get_config('foo={foo}')
-        self.assertRaises(errors.OptionExpansionLoop, c.expand_options, '{foo}')
+        self.assertRaises(config.OptionExpansionLoop, c.expand_options,
+                          '{foo}')
 
     def test_indirect_loop(self):
         c = self.get_config('''
 foo={bar}
 bar={baz}
 baz={foo}''')
-        e = self.assertRaises(errors.OptionExpansionLoop,
+        e = self.assertRaises(config.OptionExpansionLoop,
                               c.expand_options, '{foo}')
         self.assertEqual('foo->bar->baz', e.refs)
         self.assertEqual('{foo}', e.string)
@@ -741,7 +740,7 @@
 [/another/branch/path]
 bar = {foo}/2
 ''')
-        self.assertRaises(errors.ExpandingUnknownOption,
+        self.assertRaises(config.ExpandingUnknownOption,
                           c.get_user_option, 'bar', expand=True)
 
     def test_cross_related_sections(self):
@@ -956,7 +955,7 @@
         self.assertEqual(True, suppress_warning('b'))
 
 
-class TestGetConfig(tests.TestCase):
+class TestGetConfig(tests.TestCaseInTempDir):
 
     def test_constructs(self):
         config.GlobalConfig()
@@ -1536,7 +1535,7 @@
     def test_extract_email_address(self):
         self.assertEqual('jane@test.com',
                          config.extract_email_address('Jane <jane@test.com>'))
-        self.assertRaises(errors.NoEmailInUsername,
+        self.assertRaises(config.NoEmailInUsername,
                           config.extract_email_address, 'Jane Tester')
 
     def test_parse_username(self):
@@ -1597,14 +1596,14 @@
         t = self.get_transport()
         t.put_bytes('foo.conf', 'user=foo\n#\xff\n')
         conf = config.TransportConfig(t, 'foo.conf')
-        self.assertRaises(errors.ConfigContentError, conf._get_configobj)
+        self.assertRaises(config.ConfigContentError, conf._get_configobj)
 
     def test_load_erroneous_content(self):
         """Ensure we display a proper error on content that can't be parsed."""
         t = self.get_transport()
         t.put_bytes('foo.conf', '[open_section\n')
         conf = config.TransportConfig(t, 'foo.conf')
-        self.assertRaises(errors.ParseConfigError, conf._get_configobj)
+        self.assertRaises(config.ParseConfigError, conf._get_configobj)
 
     def test_load_permission_denied(self):
         """Ensure we get an empty config file if the file is inaccessible."""
@@ -2014,7 +2013,7 @@
             warnings[0])
 
     def assertCallsError(self, opt, value):
-        self.assertRaises(errors.ConfigOptionValueError,
+        self.assertRaises(config.ConfigOptionValueError,
                           opt.convert_from_unicode, None, value)
 
     def assertConvertInvalid(self, opt, invalid_value):
@@ -2181,9 +2180,9 @@
         self.assertEqual('A simple option', self.registry.get_help('foo'))
 
     def test_dont_register_illegal_name(self):
-        self.assertRaises(errors.IllegalOptionName,
+        self.assertRaises(config.IllegalOptionName,
                           self.registry.register, config.Option(' foo'))
-        self.assertRaises(errors.IllegalOptionName,
+        self.assertRaises(config.IllegalOptionName,
                           self.registry.register, config.Option('bar,'))
 
     lazy_option = config.Option('lazy_foo', help='Lazy help')
@@ -2204,10 +2203,10 @@
         # the option name which indirectly requires that the option name is a
         # valid python identifier. We violate that rule here (using a key that
         # doesn't match the option name) to test the option name checking.
-        self.assertRaises(errors.IllegalOptionName,
+        self.assertRaises(config.IllegalOptionName,
                           self.registry.register_lazy, ' foo', self.__module__,
                           'TestOptionRegistry.lazy_option')
-        self.assertRaises(errors.IllegalOptionName,
+        self.assertRaises(config.IllegalOptionName,
                           self.registry.register_lazy, '1,2', self.__module__,
                           'TestOptionRegistry.lazy_option')
 
@@ -2521,14 +2520,14 @@
         t = self.get_transport()
         t.put_bytes('foo.conf', 'user=foo\n#%s\n' % (self.invalid_utf8_char,))
         store = config.TransportIniFileStore(t, 'foo.conf')
-        self.assertRaises(errors.ConfigContentError, store.load)
+        self.assertRaises(config.ConfigContentError, store.load)
 
     def test_load_erroneous_content(self):
         """Ensure we display a proper error on content that can't be parsed."""
         t = self.get_transport()
         t.put_bytes('foo.conf', '[open_section\n')
         store = config.TransportIniFileStore(t, 'foo.conf')
-        self.assertRaises(errors.ParseConfigError, store.load)
+        self.assertRaises(config.ParseConfigError, store.load)
 
     def test_load_permission_denied(self):
         """Ensure we get warned when trying to load an inaccessible file."""
@@ -2578,14 +2577,14 @@
         with open('foo.conf', 'wb') as f:
             f.write('user=foo\n#%s\n' % (self.invalid_utf8_char,))
         conf = config.IniBasedConfig(file_name='foo.conf')
-        self.assertRaises(errors.ConfigContentError, conf._get_parser)
+        self.assertRaises(config.ConfigContentError, conf._get_parser)
 
     def test_load_erroneous_content(self):
         """Ensure we display a proper error on content that can't be parsed."""
         with open('foo.conf', 'wb') as f:
             f.write('[open_section\n')
         conf = config.IniBasedConfig(file_name='foo.conf')
-        self.assertRaises(errors.ParseConfigError, conf._get_parser)
+        self.assertRaises(config.ParseConfigError, conf._get_parser)
 
 
 class TestMutableStore(TestStore):
@@ -2870,7 +2869,7 @@
         store = config.TransportIniFileStore(self.get_transport(), 'foo.conf')
         self.assertEqual(False, store.is_loaded())
         exc = self.assertRaises(
-            errors.ParseConfigError, store._load_from_string,
+            config.ParseConfigError, store._load_from_string,
             'this is invalid !')
         self.assertEndsWith(exc.filename, 'foo.conf')
         # And the load failed
@@ -3670,7 +3669,7 @@
         self.assertExpansion('xxx', '{foo}')
 
     def test_unknown_ref(self):
-        self.assertRaises(errors.ExpandingUnknownOption,
+        self.assertRaises(config.ExpandingUnknownOption,
                           self.conf.expand_options, '{foo}')
 
     def test_illegal_def_is_ignored(self):
@@ -3694,7 +3693,7 @@
 
     def test_simple_loop(self):
         self.conf.store._load_from_string('foo={foo}')
-        self.assertRaises(errors.OptionExpansionLoop,
+        self.assertRaises(config.OptionExpansionLoop,
                           self.conf.expand_options, '{foo}')
 
     def test_indirect_loop(self):
@@ -3702,7 +3701,7 @@
 foo={bar}
 bar={baz}
 baz={foo}''')
-        e = self.assertRaises(errors.OptionExpansionLoop,
+        e = self.assertRaises(config.OptionExpansionLoop,
                               self.conf.expand_options, '{foo}')
         self.assertEqual('foo->bar->baz', e.refs)
         self.assertEqual('{foo}', e.string)
@@ -3773,7 +3772,7 @@
 [/another/branch/path]
 bar = {foo}/2
 ''')
-        self.assertRaises(errors.ExpandingUnknownOption,
+        self.assertRaises(config.ExpandingUnknownOption,
                           c.get, 'bar', expand=True)
 
     def test_cross_related_sections(self):
@@ -3861,7 +3860,7 @@
 ''')
         g_store.save()
         stack = config.LocationStack('/home/user/project/branch')
-        self.assertRaises(errors.ExpandingUnknownOption,
+        self.assertRaises(config.ExpandingUnknownOption,
                           stack.get, 'gfoo', expand=True)
 
     def test_expand_local_option_locally(self):
@@ -4114,6 +4113,38 @@
         self.assertIs(g1.store, g2.store)
 
 
+class TestAuthenticationConfigFilePermissions(tests.TestCaseInTempDir):
+    """Test warning for permissions of authentication.conf."""
+
+    def setUp(self):
+        super(TestAuthenticationConfigFilePermissions, self).setUp()
+        self.path = osutils.pathjoin(self.test_dir, 'authentication.conf')
+        with open(self.path, 'w') as f:
+            f.write(b"""[broken]
+scheme=ftp
+user=joe
+port=port # Error: Not an int
+""")
+        self.overrideAttr(config, 'authentication_config_filename',
+            lambda: self.path)
+        osutils.chmod_if_possible(self.path, 0o755)
+
+    def test_check_warning(self):
+        conf = config.AuthenticationConfig()
+        self.assertEqual(conf._filename, self.path)
+        self.assertContainsRe(self.get_log(),
+            'Saved passwords may be accessible by other users.')
+
+    def test_check_suppressed_warning(self):
+        global_config = config.GlobalConfig()
+        global_config.set_user_option('suppress_warnings',
+            'insecure_permissions')
+        conf = config.AuthenticationConfig()
+        self.assertEqual(conf._filename, self.path)
+        self.assertNotContainsRe(self.get_log(),
+            'Saved passwords may be accessible by other users.')
+
+
 class TestAuthenticationConfigFile(tests.TestCase):
     """Test the authentication.conf file matching"""
 
@@ -4136,7 +4167,7 @@
 
     def test_non_utf8_config(self):
         conf = config.AuthenticationConfig(_file=BytesIO(b'foo = bar\xff'))
-        self.assertRaises(errors.ConfigContentError, conf._get_config)
+        self.assertRaises(config.ConfigContentError, conf._get_config)
 
     def test_missing_auth_section_header(self):
         conf = config.AuthenticationConfig(_file=BytesIO(b'foo = bar'))
@@ -4144,7 +4175,7 @@
 
     def test_auth_section_header_not_closed(self):
         conf = config.AuthenticationConfig(_file=BytesIO(b'[DEF'))
-        self.assertRaises(errors.ParseConfigError, conf._get_config)
+        self.assertRaises(config.ParseConfigError, conf._get_config)
 
     def test_auth_value_not_boolean(self):
         conf = config.AuthenticationConfig(_file=BytesIO(b"""\
@@ -4350,7 +4381,7 @@
         self.assertEqual(CREDENTIALS, credentials)
 
 
-class TestAuthenticationConfig(tests.TestCase):
+class TestAuthenticationConfig(tests.TestCaseInTempDir):
     """Test AuthenticationConfig behaviour"""
 
     def _check_default_password_prompt(self, expected_prompt_format, scheme,
@@ -4735,5 +4766,5 @@
 
     def test_unknown(self):
         conf = config.MemoryStack('mail_client=firebird')
-        self.assertRaises(errors.ConfigOptionValueError, conf.get,
+        self.assertRaises(config.ConfigOptionValueError, conf.get,
                 'mail_client')

=== modified file 'breezy/tests/test_conflicts.py'
--- breezy/tests/test_conflicts.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/test_conflicts.py	2017-07-14 21:15:49 +0000
@@ -1178,7 +1178,7 @@
         return self.parser.parse_args(args)
 
     def test_unknown_action(self):
-        self.assertRaises(errors.BadOptionValue,
+        self.assertRaises(option.BadOptionValue,
                           self.parse, ['--action', 'take-me-to-the-moon'])
 
     def test_done(self):

=== modified file 'breezy/tests/test_controldir.py'
--- breezy/tests/test_controldir.py	2017-06-11 00:27:48 +0000
+++ breezy/tests/test_controldir.py	2017-06-13 10:58:02 +0000
@@ -85,14 +85,6 @@
         self.assertIsInstance(formats[0], SampleExtraComponentFormat)
 
 
-class TestControlDirFormatDeprecated(tests.TestCaseWithTransport):
-    """Tests for removed registration method in the ControlDirFormat facility."""
-
-    def test_register_format(self):
-        self.assertRaises(errors.BzrError,
-            controldir.ControlDirFormat.register_format, object())
-
-
 class TestProber(tests.TestCaseWithTransport):
     """Per-prober tests."""
 

=== modified file 'breezy/tests/test_errors.py'
--- breezy/tests/test_errors.py	2017-06-10 18:39:27 +0000
+++ breezy/tests/test_errors.py	2017-07-21 14:18:24 +0000
@@ -202,12 +202,6 @@
             "smart protocol.",
             str(error))
 
-    def test_no_help_topic(self):
-        error = errors.NoHelpTopic("topic")
-        self.assertEqualDiff("No help could be found for 'topic'. "
-            "Please use 'brz help topics' to obtain a list of topics.",
-            str(error))
-
     def test_no_such_id(self):
         error = errors.NoSuchId("atree", "anid")
         self.assertEqualDiff("The file id \"anid\" is not present in the tree "
@@ -413,14 +407,6 @@
             "location specified in the merge directive is not a branch: "
             "foo.", str(error))
 
-    def test_malformed_bug_identifier(self):
-        """Test the formatting of MalformedBugIdentifier."""
-        error = errors.MalformedBugIdentifier('bogus', 'reason for bogosity')
-        self.assertEqual(
-            'Did not understand bug identifier bogus: reason for bogosity. '
-            'See "brz help bugs" for more information on this feature.',
-            str(error))
-
     def test_unexpected_smart_server_response(self):
         e = errors.UnexpectedSmartServerResponse(('not yes',))
         self.assertEqual(
@@ -505,18 +491,11 @@
 
     def test_invalid_url_join(self):
         """Test the formatting of InvalidURLJoin."""
-        e = errors.InvalidURLJoin('Reason', 'base path', ('args',))
+        e = urlutils.InvalidURLJoin('Reason', 'base path', ('args',))
         self.assertEqual(
             "Invalid URL join request: Reason: 'base path' + ('args',)",
             str(e))
 
-    def test_incorrect_url(self):
-        err = errors.InvalidBugTrackerURL('foo', 'http://bug.com/')
-        self.assertEqual(
-            ("The URL for bug tracker \"foo\" doesn't contain {id}: "
-             "http://bug.com/"),
-            str(err))
-
     def test_unable_encode_path(self):
         err = errors.UnableEncodePath('foo', 'executable')
         self.assertEqual("Unable to encode executable path 'foo' in "
@@ -619,11 +598,6 @@
         err = errors.NotBranchError('path', controldir=FakeBzrDir())
         self.assertEqual('Not a branch: "path": NotBranchError.', str(err))
 
-    def test_invalid_pattern(self):
-        error = errors.InvalidPattern('Bad pattern msg.')
-        self.assertEqualDiff("Invalid pattern(s) found. Bad pattern msg.",
-            str(error))
-
     def test_recursive_bind(self):
         error = errors.RecursiveBind('foo_bar_branch')
         msg = ('Branch "foo_bar_branch" appears to be bound to itself. '
@@ -689,11 +663,6 @@
             str(e),
             r'Cannot bind address "example\.com:22":.*Permission denied')
 
-    def test_file_timestamp_unavailable(self):
-        e = errors.FileTimestampUnavailable("/path/foo")
-        self.assertEqual("The filestamp for /path/foo is not available.",
-                         str(e))
-
     def test_transform_rename_failed(self):
         e = errors.TransformRenameFailed(u"from", u"to", "readonly file", 2)
         self.assertEqual(
@@ -725,14 +694,6 @@
                              repo.controldir.root_transport.base,
                              str(error))
 
-    def test_unknown_bug_tracker_abbreviation(self):
-        """Test the formatting of UnknownBugTrackerAbbreviation."""
-        branch = self.make_branch('some_branch')
-        error = errors.UnknownBugTrackerAbbreviation('xxx', branch)
-        self.assertEqual(
-            "Cannot find registered bug tracker called xxx on %s" % branch,
-            str(error))
-
     def test_not_branch_bzrdir_with_repo(self):
         controldir = self.make_repository('repo').controldir
         err = errors.NotBranchError('path', controldir=controldir)

=== modified file 'breezy/tests/test_features.py'
--- breezy/tests/test_features.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/test_features.py	2017-06-17 14:28:26 +0000
@@ -18,6 +18,7 @@
 
 import sys
 
+import breezy
 from .. import (
     symbol_versioning,
     tests,
@@ -128,6 +129,24 @@
         self.assertIs(None, feature.module)
 
 
+class TestPluginLoadedFeature(tests.TestCase):
+
+    def test_available_plugin(self):
+        if not breezy.global_state.plugins:
+            self.skipTest('no plugins available to test with')
+        a_plugin_name = next(iter(breezy.global_state.plugins))
+        feature = features.PluginLoadedFeature(a_plugin_name)
+        self.assertEqual(a_plugin_name, feature.plugin_name)
+        self.assertEqual(a_plugin_name + ' plugin', str(feature))
+        self.assertTrue(feature.available())
+
+    def test_unavailable_plugin(self):
+        feature = features.PluginLoadedFeature('idontexist')
+        self.assertEqual('idontexist plugin', str(feature))
+        self.assertFalse(feature.available())
+        self.assertIs(None, feature.plugin)
+
+
 class TestUnicodeFilenameFeature(tests.TestCase):
 
     def test_probe_passes(self):

=== modified file 'breezy/tests/test_foreign.py'
--- breezy/tests/test_foreign.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/test_foreign.py	2017-06-20 01:48:45 +0000
@@ -203,28 +203,24 @@
                     parent_revids = []
                 else:
                     parent_revids = [parent_revid]
-                builder = self.target.get_commit_builder(parent_revids, 
+                builder = self.target.get_commit_builder(parent_revids,
                         self.target.get_config_stack(), rev.timestamp,
                         rev.timezone, rev.committer, rev.properties,
                         new_revid)
                 try:
                     parent_tree = self.target.repository.revision_tree(
                         parent_revid)
-                    for path, ie in tree.iter_entries_by_dir():
-                        new_ie = ie.copy()
-                        new_ie.revision = None
-                        builder.record_entry_contents(new_ie, 
-                            [parent_tree.root_inventory],
-                            path, tree, 
-                            (ie.kind, ie.text_size, ie.executable, ie.text_sha1))
+                    iter_changes = tree.iter_changes(parent_tree)
+                    list(builder.record_iter_changes(
+                        tree, parent_revid, iter_changes))
                     builder.finish_inventory()
                 except:
                     builder.abort()
                     raise
                 revidmap[revid] = builder.commit(rev.message)
-                self.target.set_last_revision_info(parent_revno+1, 
+                self.target.set_last_revision_info(parent_revno+1,
                     revidmap[revid])
-                trace.mutter('lossily pushed revision %s -> %s', 
+                trace.mutter('lossily pushed revision %s -> %s',
                     revid, revidmap[revid])
         finally:
             self.source.unlock()

=== modified file 'breezy/tests/test_globbing.py'
--- breezy/tests/test_globbing.py	2017-06-04 18:09:30 +0000
+++ breezy/tests/test_globbing.py	2017-07-10 23:47:19 +0000
@@ -17,7 +17,7 @@
 
 import re
 
-from .. import errors
+from .. import errors, lazy_regex
 from ..globbing import (
     Globster,
     ExceptionGlobster,
@@ -316,7 +316,7 @@
         """Ensure that globster handles bad patterns cleanly."""
         patterns = [u'RE:[', u'/home/foo', u'RE:*.cpp']
         g = Globster(patterns)
-        e = self.assertRaises(errors.InvalidPattern, g.match, 'filename')
+        e = self.assertRaises(lazy_regex.InvalidPattern, g.match, 'filename')
         self.assertContainsRe(e.msg,
             "File.*ignore.*contains error.*RE:\[.*RE:\*\.cpp", flags=re.DOTALL)
 

=== modified file 'breezy/tests/test_gpg.py'
--- breezy/tests/test_gpg.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/test_gpg.py	2017-07-04 20:03:11 +0000
@@ -43,90 +43,17 @@
         if content is None:
             content = '''
 gpg_signing_key=amy@example.com
-gpg_signing_command=false'''
+'''
         super(FakeConfig, self).__init__(content)
 
 
-class TestCommandLine(tests.TestCase):
-
-    def setUp(self):
-        super(TestCommandLine, self).setUp()
-        self.my_gpg = gpg.GPGStrategy(FakeConfig())
-
-    def test_signing_command_line(self):
-        self.assertEqual(['false',  '--clearsign', '-u', 'amy@example.com'],
-                         self.my_gpg._command_line())
-
-    def test_signing_command_line_from_default(self):
-        # Using 'default' for gpg_signing_key will use the mail part of 'email'
-        my_gpg = gpg.GPGStrategy(FakeConfig('''
-email=Amy <amy@example.com>
-gpg_signing_key=default
-gpg_signing_command=false'''))
-        self.assertEqual(['false',  '--clearsign', '-u', 'amy@example.com'],
-                         my_gpg._command_line())
-
-    def test_signing_command_line_from_email(self):
-        # Not setting gpg_signing_key will use the mail part of 'email'
-        my_gpg = gpg.GPGStrategy(FakeConfig('''
-email=Amy <amy@example.com>
-gpg_signing_command=false'''))
-        self.assertEqual(['false',  '--clearsign', '-u', 'amy@example.com'],
-                         my_gpg._command_line())
-
-    def test_checks_return_code(self):
-        # This test needs a unix like platform - one with 'false' to run.
-        # if you have one, please make this work :)
-        self.assertRaises(errors.SigningFailed, self.my_gpg.sign, 'content')
-
-    def assertProduces(self, content):
-        # This needs a 'cat' command or similar to work.
-        if sys.platform == 'win32':
-            # Windows doesn't come with cat, and we don't require it
-            # so lets try using python instead.
-            # But stupid windows and line-ending conversions.
-            # It is too much work to make sys.stdout be in binary mode.
-            # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/65443
-            self.my_gpg._command_line = lambda:[sys.executable, '-c',
-                    'import sys; sys.stdout.write(sys.stdin.read())']
-            new_content = content.replace('\n', '\r\n')
-
-            self.assertEqual(new_content, self.my_gpg.sign(content))
-        else:
-            self.my_gpg._command_line = lambda:['cat', '-']
-            self.assertEqual(content, self.my_gpg.sign(content))
-
-    def test_returns_output(self):
-        content = "some content\nwith newlines\n"
-        self.assertProduces(content)
-
-    def test_clears_progress(self):
-        content = "some content\nwith newlines\n"
-        old_clear_term = ui.ui_factory.clear_term
-        clear_term_called = []
-        def clear_term():
-            old_clear_term()
-            clear_term_called.append(True)
-        ui.ui_factory.clear_term = clear_term
-        try:
-            self.assertProduces(content)
-        finally:
-            ui.ui_factory.clear_term = old_clear_term
-        self.assertEqual([True], clear_term_called)
-
-    def test_aborts_on_unicode(self):
-        """You can't sign Unicode text; it must be encoded first."""
-        self.assertRaises(errors.BzrBadParameterUnicode,
-                          self.assertProduces, u'foo')
-
-
 class TestVerify(TestCase):
 
     def import_keys(self):
-        import gpgme
-        context = gpgme.Context()
+        import gpg
+        context = gpg.Context()
 
-        key = BytesIO(b"""-----BEGIN PGP PUBLIC KEY BLOCK-----
+        key = gpg.Data(b"""-----BEGIN PGP PUBLIC KEY BLOCK-----
 Version: GnuPG v1.4.11 (GNU/Linux)
 
 mQENBE343IgBCADwzPW7kmKb2bjB+UU+1ER/ABMZspvtoZMPusUw7bk6coXHF/0W
@@ -158,7 +85,7 @@
 -----END PGP PUBLIC KEY BLOCK-----
 """)
 
-        secret_key = BytesIO(b"""-----BEGIN PGP PRIVATE KEY BLOCK-----
+        secret_key = gpg.Data(b"""-----BEGIN PGP PRIVATE KEY BLOCK-----
 Version: GnuPG v1.4.11 (GNU/Linux)
 
 lQOYBE343IgBCADwzPW7kmKb2bjB+UU+1ER/ABMZspvtoZMPusUw7bk6coXHF/0W
@@ -217,7 +144,7 @@
 -----END PGP PRIVATE KEY BLOCK-----
 """)
 
-        revoked_key = BytesIO(b"""-----BEGIN PGP PUBLIC KEY BLOCK-----
+        revoked_key = gpg.Data(b"""-----BEGIN PGP PUBLIC KEY BLOCK-----
 Version: GnuPG v1.4.11 (GNU/Linux)
 
 mI0ETjlW5gEEAOb/6P+TVM59E897wRtatxys2BhsHCXM4T7xjIiANfDwejDdifqh
@@ -242,7 +169,7 @@
 -----END PGP PUBLIC KEY BLOCK-----
 """)
 
-        expired_key = BytesIO(b"""-----BEGIN PGP PUBLIC KEY BLOCK-----
+        expired_key = gpg.Data(b"""-----BEGIN PGP PUBLIC KEY BLOCK-----
 Version: GnuPG v1.4.11 (GNU/Linux)
 
 mI0ETjZ6PAEEALkR4GcFQidCCxV7pgQwQd5MZua0YO2l92fVqHX+PhnZ6egCLKdD
@@ -263,14 +190,14 @@
 =p0gt
 -----END PGP PUBLIC KEY BLOCK-----
 """)
-        context.import_(key)
-        context.import_(secret_key)
-        context.import_(revoked_key)
-        context.import_(expired_key)
+        context.op_import(key)
+        context.op_import(secret_key)
+        context.op_import(revoked_key)
+        context.op_import(expired_key)
 
     def test_verify_untrusted_but_accepted(self):
         #untrusted by gpg but listed as acceptable_keys by user
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         self.import_keys()
 
         content = """-----BEGIN PGP SIGNED MESSAGE-----
@@ -301,7 +228,7 @@
                             plain))
 
     def test_verify_unacceptable_key(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         self.import_keys()
 
         content = """-----BEGIN PGP SIGNED MESSAGE-----
@@ -332,7 +259,7 @@
                          my_gpg.verify(content, plain))
 
     def test_verify_valid_but_untrusted(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         self.import_keys()
 
         content = """-----BEGIN PGP SIGNED MESSAGE-----
@@ -362,7 +289,7 @@
                             plain))
 
     def test_verify_bad_testament(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         self.import_keys()
 
         content = """-----BEGIN PGP SIGNED MESSAGE-----
@@ -394,7 +321,7 @@
 
 
     def test_verify_revoked_signature(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         self.import_keys()
 
         content = """-----BEGIN PGP SIGNED MESSAGE-----
@@ -418,7 +345,7 @@
                             plain))
 
     def test_verify_invalid(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         self.import_keys()
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
@@ -443,7 +370,7 @@
                             my_gpg.verify(content, plain))
 
     def test_verify_expired_but_valid(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         self.import_keys()
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
@@ -470,7 +397,7 @@
                             my_gpg.verify(content, plain))
 
     def test_verify_unknown_key(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         self.import_keys()
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
@@ -494,7 +421,7 @@
                             my_gpg.verify(content, plain))
 
     def test_set_acceptable_keys(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         self.import_keys()
         my_gpg = gpg.GPGStrategy(FakeConfig())
         my_gpg.set_acceptable_keys("bazaar@example.com")
@@ -502,7 +429,7 @@
                          [u'B5DEED5FCB15DAE6ECEF919587681B1EE3080E45'])
 
     def test_set_acceptable_keys_from_config(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         self.import_keys()
         my_gpg = gpg.GPGStrategy(FakeConfig(
                 'acceptable_keys=bazaar@example.com'))
@@ -511,7 +438,7 @@
                          [u'B5DEED5FCB15DAE6ECEF919587681B1EE3080E45'])
 
     def test_set_acceptable_keys_unknown(self):
-        self.requireFeature(features.gpgme)
+        self.requireFeature(features.gpg)
         my_gpg = gpg.GPGStrategy(FakeConfig())
         self.notes = []
         def note(*args):
@@ -526,10 +453,10 @@
 class TestDisabled(TestCase):
 
     def test_sign(self):
-        self.assertRaises(errors.SigningFailed,
+        self.assertRaises(gpg.SigningFailed,
                           gpg.DisabledGPGStrategy(None).sign, 'content')
 
     def test_verify(self):
-        self.assertRaises(errors.SignatureVerificationFailed,
+        self.assertRaises(gpg.SignatureVerificationFailed,
                           gpg.DisabledGPGStrategy(None).verify, 'content',
                           'testament')

=== modified file 'breezy/tests/test_help.py'
--- breezy/tests/test_help.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/test_help.py	2017-07-14 20:37:44 +0000
@@ -34,6 +34,15 @@
 import re
 
 
+class ErrorTests(tests.TestCase):
+
+    def test_no_help_topic(self):
+        error = help.NoHelpTopic("topic")
+        self.assertEqualDiff("No help could be found for 'topic'. "
+            "Please use 'brz help topics' to obtain a list of topics.",
+            str(error))
+
+
 class TestCommandHelp(tests.TestCase):
     """Tests for help on commands."""
 
@@ -660,7 +669,7 @@
         """Searching for an unknown topic should raise NoHelpTopic."""
         indices = help.HelpIndices()
         indices.search_path = []
-        error = self.assertRaises(errors.NoHelpTopic, indices.search, 'foo')
+        error = self.assertRaises(help.NoHelpTopic, indices.search, 'foo')
         self.assertEqual('foo', error.topic)
 
     def test_search_calls_get_topic(self):

=== modified file 'breezy/tests/test_http.py'
--- breezy/tests/test_http.py	2017-06-11 14:07:05 +0000
+++ breezy/tests/test_http.py	2017-07-15 15:20:13 +0000
@@ -41,6 +41,7 @@
     trace,
     transport,
     ui,
+    urlutils,
     )
 from ..bzr import (
     remote as _mod_remote,
@@ -381,7 +382,7 @@
     def test_invalid_http_urls(self):
         """Trap invalid construction of urls"""
         self._transport('http://example.com/bzr/bzr.dev/')
-        self.assertRaises(errors.InvalidURL,
+        self.assertRaises(urlutils.InvalidURL,
                           self._transport,
                           'http://http://example.com/bzr/bzr.dev/')
 
@@ -1144,7 +1145,7 @@
     def test_invalid_proxy(self):
         """A proxy env variable without scheme"""
         self.overrideEnv('http_proxy', 'host:1234')
-        self.assertRaises(errors.InvalidURL, self._proxied_request)
+        self.assertRaises(urlutils.InvalidURL, self._proxied_request)
 
     def test_evaluate_proxy_bypass_true(self):
         """The host is not proxied"""
@@ -1242,7 +1243,7 @@
 
     def test_http_proxy_without_scheme(self):
         self.overrideEnv('http_proxy', self.server_host_port)
-        self.assertRaises(errors.InvalidURL, self.assertProxied)
+        self.assertRaises(urlutils.InvalidURL, self.assertProxied)
 
 
 class TestRanges(http_utils.TestCaseWithWebserver):

=== modified file 'breezy/tests/test_https_urllib.py'
--- breezy/tests/test_https_urllib.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/test_https_urllib.py	2017-07-21 13:20:17 +0000
@@ -25,9 +25,6 @@
     config,
     trace,
 )
-from ..errors import (
-    ConfigOptionValueError,
-)
 from .. import tests
 from ..transport.http import _urllib2_wrappers
 from ..transport.http._urllib2_wrappers import ssl
@@ -77,7 +74,8 @@
         stack = config.MemoryStack("ssl.cert_reqs = required\n")
         self.assertEqual(ssl.CERT_REQUIRED, stack.get("ssl.cert_reqs"))
         stack = config.MemoryStack("ssl.cert_reqs = invalid\n")
-        self.assertRaises(ConfigOptionValueError, stack.get, "ssl.cert_reqs")
+        self.assertRaises(config.ConfigOptionValueError, stack.get,
+                          "ssl.cert_reqs")
 
 
 class MatchHostnameTests(tests.TestCase):

=== modified file 'breezy/tests/test_knit.py'
--- breezy/tests/test_knit.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/test_knit.py	2017-06-13 01:08:59 +0000
@@ -1196,16 +1196,7 @@
             "b option 0 1 4 :"  # We don't have a 4th record
             ])
         index = self.get_knit_index(transport, 'filename', 'r')
-        try:
-            self.assertRaises(errors.KnitCorrupt, index.keys)
-        except TypeError as e:
-            if (str(e) == ('exceptions must be strings, classes, or instances,'
-                           ' not exceptions.IndexError')):
-                self.knownFailure('Pyrex <0.9.5 fails with TypeError when'
-                                  ' raising new style exceptions with python'
-                                  ' >=2.5')
-            else:
-                raise
+        self.assertRaises(errors.KnitCorrupt, index.keys)
 
     def test_corrupted_parent(self):
         transport = MockTransport([
@@ -1215,16 +1206,7 @@
             "c option 0 1 1v :", # Can't have a parent of '1v'
             ])
         index = self.get_knit_index(transport, 'filename', 'r')
-        try:
-            self.assertRaises(errors.KnitCorrupt, index.keys)
-        except TypeError as e:
-            if (str(e) == ('exceptions must be strings, classes, or instances,'
-                           ' not exceptions.ValueError')):
-                self.knownFailure('Pyrex <0.9.5 fails with TypeError when'
-                                  ' raising new style exceptions with python'
-                                  ' >=2.5')
-            else:
-                raise
+        self.assertRaises(errors.KnitCorrupt, index.keys)
 
     def test_corrupted_parent_in_list(self):
         transport = MockTransport([
@@ -1234,16 +1216,7 @@
             "c option 0 1 1 v :", # Can't have a parent of 'v'
             ])
         index = self.get_knit_index(transport, 'filename', 'r')
-        try:
-            self.assertRaises(errors.KnitCorrupt, index.keys)
-        except TypeError as e:
-            if (str(e) == ('exceptions must be strings, classes, or instances,'
-                           ' not exceptions.ValueError')):
-                self.knownFailure('Pyrex <0.9.5 fails with TypeError when'
-                                  ' raising new style exceptions with python'
-                                  ' >=2.5')
-            else:
-                raise
+        self.assertRaises(errors.KnitCorrupt, index.keys)
 
     def test_invalid_position(self):
         transport = MockTransport([
@@ -1251,16 +1224,7 @@
             "a option 1v 1 :",
             ])
         index = self.get_knit_index(transport, 'filename', 'r')
-        try:
-            self.assertRaises(errors.KnitCorrupt, index.keys)
-        except TypeError as e:
-            if (str(e) == ('exceptions must be strings, classes, or instances,'
-                           ' not exceptions.ValueError')):
-                self.knownFailure('Pyrex <0.9.5 fails with TypeError when'
-                                  ' raising new style exceptions with python'
-                                  ' >=2.5')
-            else:
-                raise
+        self.assertRaises(errors.KnitCorrupt, index.keys)
 
     def test_invalid_size(self):
         transport = MockTransport([
@@ -1268,16 +1232,7 @@
             "a option 1 1v :",
             ])
         index = self.get_knit_index(transport, 'filename', 'r')
-        try:
-            self.assertRaises(errors.KnitCorrupt, index.keys)
-        except TypeError as e:
-            if (str(e) == ('exceptions must be strings, classes, or instances,'
-                           ' not exceptions.ValueError')):
-                self.knownFailure('Pyrex <0.9.5 fails with TypeError when'
-                                  ' raising new style exceptions with python'
-                                  ' >=2.5')
-            else:
-                raise
+        self.assertRaises(errors.KnitCorrupt, index.keys)
 
     def test_scan_unvalidated_index_not_implemented(self):
         transport = MockTransport()

=== modified file 'breezy/tests/test_lazy_regex.py'
--- breezy/tests/test_lazy_regex.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/test_lazy_regex.py	2017-07-10 23:47:19 +0000
@@ -26,6 +26,14 @@
     )
 
 
+class ErrorTests(tests.TestCase):
+
+    def test_invalid_pattern(self):
+        error = lazy_regex.InvalidPattern('Bad pattern msg.')
+        self.assertEqualDiff("Invalid pattern(s) found. Bad pattern msg.",
+            str(error))
+
+
 class InstrumentedLazyRegex(lazy_regex.LazyRegex):
     """Keep track of actions on the lazy regex"""
 
@@ -71,7 +79,7 @@
         p = lazy_regex.lazy_compile('RE:[')
         # As p.match is lazy, we make it into a lambda so its handled
         # by assertRaises correctly.
-        e = self.assertRaises(errors.InvalidPattern, lambda: p.match('foo'))
+        e = self.assertRaises(lazy_regex.InvalidPattern, lambda: p.match('foo'))
         self.assertEqual(e.msg, '"RE:[" unexpected end of regular expression')
 
 

=== modified file 'breezy/tests/test_lockdir.py'
--- breezy/tests/test_lockdir.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/test_lockdir.py	2017-06-16 18:15:52 +0000
@@ -40,6 +40,9 @@
     LockDir,
     LockHeldInfo,
     )
+from ..sixish import (
+    text_type,
+    )
 from . import (
     features,
     TestCase,
@@ -346,7 +349,7 @@
         ld2 = self.get_lock()
         ld.create()
         ld.lock_write()
-        ld.transport.put_bytes_non_atomic('test_lock/held/info', '\0')
+        ld.transport.put_bytes_non_atomic('test_lock/held/info', b'\0')
 
         class LoggingUIFactory(breezy.ui.SilentUIFactory):
             def __init__(self):
@@ -508,7 +511,7 @@
         t = self.get_transport()
         t.mkdir('test_lock')
         t.mkdir('test_lock/held')
-        t.put_bytes('test_lock/held/info', '')
+        t.put_bytes('test_lock/held/info', b'')
         lf = LockDir(t, 'test_lock')
         info = lf.peek()
         formatted_info = info.to_readable_dict()
@@ -526,7 +529,7 @@
         t = self.get_transport()
         t.mkdir('test_lock')
         t.mkdir('test_lock/held')
-        t.put_bytes('test_lock/held/info', '\0')
+        t.put_bytes('test_lock/held/info', b'\0')
         lf = LockDir(t, 'test_lock')
         self.assertRaises(errors.LockCorrupt, lf.peek)
         # Currently attempt_lock gives LockContention, but LockCorrupt would be
@@ -665,7 +668,7 @@
 
     def test_unicode(self):
         info = LockHeldInfo.for_this_process(None)
-        self.assertContainsRe(unicode(info),
+        self.assertContainsRe(text_type(info),
             r'held by .* on .* \(process #\d+\), acquired .* ago')
 
     def test_is_locked_by_this_process(self):

=== modified file 'breezy/tests/test_merge.py'
--- breezy/tests/test_merge.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/test_merge.py	2017-06-24 16:32:30 +0000
@@ -226,6 +226,8 @@
             tree_a.conflicts())
 
     def test_nested_merge(self):
+        self.knownFailure(
+            'iter_changes doesn\'t work with changes in nested trees')
         tree = self.make_branch_and_tree('tree',
             format='development-subtree')
         sub_tree = self.make_branch_and_tree('tree/sub-tree',
@@ -371,7 +373,7 @@
 
     def test_weave_cherrypick(self):
         this_tree, other_tree = self.prepare_cherrypick()
-        merger = _mod_merge.Merger.from_revision_ids(None,
+        merger = _mod_merge.Merger.from_revision_ids(
             this_tree, 'rev3b', 'rev2b', other_tree.branch)
         merger.merge_type = _mod_merge.WeaveMerger
         merger.do_merge()
@@ -379,14 +381,14 @@
 
     def test_weave_cannot_reverse_cherrypick(self):
         this_tree, other_tree = self.prepare_cherrypick()
-        merger = _mod_merge.Merger.from_revision_ids(None,
+        merger = _mod_merge.Merger.from_revision_ids(
             this_tree, 'rev2b', 'rev3b', other_tree.branch)
         merger.merge_type = _mod_merge.WeaveMerger
         self.assertRaises(errors.CannotReverseCherrypick, merger.do_merge)
 
     def test_merge3_can_reverse_cherrypick(self):
         this_tree, other_tree = self.prepare_cherrypick()
-        merger = _mod_merge.Merger.from_revision_ids(None,
+        merger = _mod_merge.Merger.from_revision_ids(
             this_tree, 'rev2b', 'rev3b', other_tree.branch)
         merger.merge_type = _mod_merge.Merge3Merger
         merger.do_merge()
@@ -404,7 +406,7 @@
         this_tree.lock_write()
         self.addCleanup(this_tree.unlock)
 
-        merger = _mod_merge.Merger.from_revision_ids(None,
+        merger = _mod_merge.Merger.from_revision_ids(
             this_tree, 'rev3b', 'rev2b', other_tree.branch)
         merger.merge_type = _mod_merge.Merge3Merger
         merger.do_merge()
@@ -422,7 +424,7 @@
         self.build_tree(['a'])
         tree.add('a')
         first_rev = tree.commit("added a")
-        merger = _mod_merge.Merger.from_revision_ids(None, tree,
+        merger = _mod_merge.Merger.from_revision_ids(tree,
                                           _mod_revision.NULL_REVISION,
                                           first_rev)
         merger.merge_type = _mod_merge.Merge3Merger
@@ -442,7 +444,7 @@
         other_tree.commit('rev2', rev_id='rev2b')
         this_tree.lock_write()
         self.addCleanup(this_tree.unlock)
-        merger = _mod_merge.Merger.from_revision_ids(None,
+        merger = _mod_merge.Merger.from_revision_ids(
             this_tree, 'rev2b', other_branch=other_tree.branch)
         merger.merge_type = _mod_merge.Merge3Merger
         tree_merger = merger.make_merger()
@@ -465,7 +467,7 @@
         other_tree.commit('rev2', rev_id='rev2b')
         this_tree.lock_write()
         self.addCleanup(this_tree.unlock)
-        merger = _mod_merge.Merger.from_revision_ids(None,
+        merger = _mod_merge.Merger.from_revision_ids(
             this_tree, 'rev2b', other_branch=other_tree.branch)
         merger.merge_type = _mod_merge.Merge3Merger
         tree_merger = merger.make_merger()
@@ -495,7 +497,7 @@
         other_tree.commit('rev2', rev_id='rev2b')
         this_tree.lock_write()
         self.addCleanup(this_tree.unlock)
-        merger = _mod_merge.Merger.from_revision_ids(None,
+        merger = _mod_merge.Merger.from_revision_ids(
             this_tree, 'rev2b', other_branch=other_tree.branch)
         merger.merge_type = _mod_merge.Merge3Merger
         tree_merger = merger.make_merger()
@@ -514,7 +516,7 @@
         tree.add('a')
         first_rev = tree.commit("added a")
         old_root_id = tree.get_root_id()
-        merger = _mod_merge.Merger.from_revision_ids(None, tree,
+        merger = _mod_merge.Merger.from_revision_ids(tree,
                                           _mod_revision.NULL_REVISION,
                                           first_rev)
         merger.merge_type = _mod_merge.Merge3Merger
@@ -1268,7 +1270,7 @@
         mem_tree = memorytree.MemoryTree.create_on_branch(builder.get_branch())
         mem_tree.lock_write()
         self.addCleanup(mem_tree.unlock)
-        merger = _mod_merge.Merger.from_revision_ids(None,
+        merger = _mod_merge.Merger.from_revision_ids(
             mem_tree, other_revision_id)
         merger.set_interesting_files(interesting_files)
         # It seems there is no matching function for set_interesting_ids
@@ -2070,7 +2072,7 @@
 
     def do_merge(self, builder, other_revision_id):
         wt = self.get_wt_from_builder(builder)
-        merger = _mod_merge.Merger.from_revision_ids(None,
+        merger = _mod_merge.Merger.from_revision_ids(
             wt, other_revision_id)
         merger.merge_type = _mod_merge.Merge3Merger
         return wt, merger.do_merge()
@@ -2336,8 +2338,7 @@
         wt.commit('D merges B & C', rev_id='D-id')
         self.assertEqual('barry', wt.id2path('foo-id'))
         # Check the output of the Merger object directly
-        merger = _mod_merge.Merger.from_revision_ids(None,
-            wt, 'F-id')
+        merger = _mod_merge.Merger.from_revision_ids(wt, 'F-id')
         merger.merge_type = _mod_merge.Merge3Merger
         merge_obj = merger.make_merger()
         root_id = wt.path2id('')
@@ -2392,8 +2393,7 @@
         wt.commit('F foo => bing', rev_id='F-id')
 
         # Check the output of the Merger object directly
-        merger = _mod_merge.Merger.from_revision_ids(None,
-            wt, 'E-id')
+        merger = _mod_merge.Merger.from_revision_ids(wt, 'E-id')
         merger.merge_type = _mod_merge.Merge3Merger
         merge_obj = merger.make_merger()
         # Nothing interesting happened in OTHER relative to BASE
@@ -2443,8 +2443,7 @@
         list(wt.iter_changes(wt.basis_tree()))
         wt.commit('D merges B & C, makes it a file', rev_id='D-id')
 
-        merger = _mod_merge.Merger.from_revision_ids(None,
-            wt, 'E-id')
+        merger = _mod_merge.Merger.from_revision_ids(wt, 'E-id')
         merger.merge_type = _mod_merge.Merge3Merger
         merge_obj = merger.make_merger()
         entries = list(merge_obj._entries_lca())
@@ -2658,8 +2657,7 @@
         wt.branch.set_last_revision_info(2, 'B-id')
         wt.commit('D', rev_id='D-id', recursive=None)
 
-        merger = _mod_merge.Merger.from_revision_ids(None,
-            wt, 'E-id')
+        merger = _mod_merge.Merger.from_revision_ids(wt, 'E-id')
         merger.merge_type = _mod_merge.Merge3Merger
         merge_obj = merger.make_merger()
         entries = list(merge_obj._entries_lca())
@@ -2695,8 +2693,7 @@
         wt.branch.set_last_revision_info(2, 'B-id')
         wt.commit('D', rev_id='D-id', recursive=None)
 
-        merger = _mod_merge.Merger.from_revision_ids(None,
-            wt, 'E-id')
+        merger = _mod_merge.Merger.from_revision_ids(wt, 'E-id')
         merger.merge_type = _mod_merge.Merge3Merger
         merge_obj = merger.make_merger()
         entries = list(merge_obj._entries_lca())
@@ -2735,8 +2732,7 @@
         wt.branch.set_last_revision_info(2, 'B-id')
         wt.commit('D', rev_id='D-id', recursive=None)
 
-        merger = _mod_merge.Merger.from_revision_ids(None,
-            wt, 'E-id')
+        merger = _mod_merge.Merger.from_revision_ids(wt, 'E-id')
         merger.merge_type = _mod_merge.Merge3Merger
         merge_obj = merger.make_merger()
         entries = list(merge_obj._entries_lca())
@@ -2780,8 +2776,7 @@
         wt.branch.set_last_revision_info(2, 'B-id')
         wt.commit('D', rev_id='D-id', recursive=None)
 
-        merger = _mod_merge.Merger.from_revision_ids(None,
-            wt, 'E-id')
+        merger = _mod_merge.Merger.from_revision_ids(wt, 'E-id')
         merger.merge_type = _mod_merge.Merge3Merger
         merge_obj = merger.make_merger()
         entries = list(merge_obj._entries_lca())

=== modified file 'breezy/tests/test_merge_core.py'
--- breezy/tests/test_merge_core.py	2017-06-10 00:52:08 +0000
+++ breezy/tests/test_merge_core.py	2017-06-24 16:32:30 +0000
@@ -773,14 +773,14 @@
     def test_from_revision_ids(self):
         this, other = self.set_up_trees()
         self.assertRaises(errors.NoSuchRevision, Merger.from_revision_ids,
-                          None, this, 'rev2b')
+                          this, 'rev2b')
         this.lock_write()
         self.addCleanup(this.unlock)
-        merger = Merger.from_revision_ids(None, this,
+        merger = Merger.from_revision_ids(this,
             'rev2b', other_branch=other.branch)
         self.assertEqual('rev2b', merger.other_rev_id)
         self.assertEqual('rev1', merger.base_rev_id)
-        merger = Merger.from_revision_ids(None, this,
+        merger = Merger.from_revision_ids(this,
             'rev2b', 'rev2a', other_branch=other.branch)
         self.assertEqual('rev2a', merger.base_rev_id)
 
@@ -804,17 +804,14 @@
             other.branch.repository, 'rev3', 0, 0, 'this')
         other.lock_read()
         self.addCleanup(other.unlock)
-        merger, verified = Merger.from_mergeable(this, md,
-            None)
+        merger, verified = Merger.from_mergeable(this, md)
         md.patch = None
-        merger, verified = Merger.from_mergeable(this, md,
-            None)
+        merger, verified = Merger.from_mergeable(this, md)
         self.assertEqual('inapplicable', verified)
         self.assertEqual('rev3', merger.other_rev_id)
         self.assertEqual('rev1', merger.base_rev_id)
         md.base_revision_id = 'rev2b'
-        merger, verified = Merger.from_mergeable(this, md,
-            None)
+        merger, verified = Merger.from_mergeable(this, md)
         self.assertEqual('rev2b', merger.base_rev_id)
 
     def test_from_mergeable_old_merge_directive(self):
@@ -823,7 +820,6 @@
         self.addCleanup(other.unlock)
         md = merge_directive.MergeDirective.from_objects(
             other.branch.repository, 'rev3', 0, 0, 'this')
-        merger, verified = Merger.from_mergeable(this, md,
-            None)
+        merger, verified = Merger.from_mergeable(this, md)
         self.assertEqual('rev3', merger.other_rev_id)
         self.assertEqual('rev1', merger.base_rev_id)

=== modified file 'breezy/tests/test_merge_directive.py'
--- breezy/tests/test_merge_directive.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/test_merge_directive.py	2017-07-04 20:03:11 +0000
@@ -464,8 +464,6 @@
         class FakeBranch(object):
             def get_config_stack(self):
                 return self
-            def gpg_signing_command(self):
-                return 'loopback'
         md = self.make_merge_directive('example:', 'sha', time, timezone,
             'http://example.com', source_branch="http://example.org",
             patch='booga', patch_type='diff')

=== modified file 'breezy/tests/test_missing.py'
--- breezy/tests/test_missing.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/test_missing.py	2017-06-24 16:32:30 +0000
@@ -41,26 +41,26 @@
         merger = merger_tree.branch
         self.assertUnmerged(([], []), original, puller)
         original_tree.commit('a', rev_id='a')
-        self.assertUnmerged(([('1', 'a')], []), original, puller)
+        self.assertUnmerged(([('1', 'a', 0)], []), original, puller)
         puller_tree.pull(original)
         self.assertUnmerged(([], []), original, puller)
         merger_tree.pull(original)
         original_tree.commit('b', rev_id='b')
         original_tree.commit('c', rev_id='c')
-        self.assertUnmerged(([('2', 'b'), ('3', 'c')], []),
+        self.assertUnmerged(([('2', 'b', 0), ('3', 'c', 0)], []),
                             original, puller)
-        self.assertUnmerged(([('3', 'c'), ('2', 'b')], []),
+        self.assertUnmerged(([('3', 'c', 0), ('2', 'b', 0)], []),
                             original, puller, backward=True)
 
         puller_tree.pull(original)
         self.assertUnmerged(([], []), original, puller)
-        self.assertUnmerged(([('2', 'b'), ('3', 'c')], []),
+        self.assertUnmerged(([('2', 'b', 0), ('3', 'c', 0)], []),
                             original, merger)
         merger_tree.merge_from_branch(original)
-        self.assertUnmerged(([('2', 'b'), ('3', 'c')], []),
+        self.assertUnmerged(([('2', 'b', 0), ('3', 'c', 0)], []),
                             original, merger)
         merger_tree.commit('d', rev_id='d')
-        self.assertUnmerged(([], [('2', 'd')]), original, merger)
+        self.assertUnmerged(([], [('2', 'd', 0)]), original, merger)
 
     def test_iter_log_revisions(self):
         base_tree = self.make_branch_and_tree('base')
@@ -159,17 +159,17 @@
         rev1 = tree.commit('one')
         tree2 = tree.controldir.sprout('tree2').open_workingtree()
         rev2 = tree2.commit('two')
-        self.assertUnmerged([], [('2', rev2)], tree.branch, tree2.branch)
-        self.assertUnmerged([('2', rev2)], [], tree2.branch, tree.branch)
+        self.assertUnmerged([], [('2', rev2, 0)], tree.branch, tree2.branch)
+        self.assertUnmerged([('2', rev2, 0)], [], tree2.branch, tree.branch)
 
     def test_restrict(self):
         tree = self.make_branch_and_tree('tree')
         rev1 = tree.commit('one')
         tree2 = tree.controldir.sprout('tree2').open_workingtree()
         rev2 = tree2.commit('two')
-        self.assertUnmerged([], [('2', rev2)], tree.branch, tree2.branch)
+        self.assertUnmerged([], [('2', rev2, 0)], tree.branch, tree2.branch)
         self.assertUnmerged([], None, tree.branch, tree2.branch, 'local')
-        self.assertUnmerged(None, [('2', rev2)], tree.branch, tree2.branch,
+        self.assertUnmerged(None, [('2', rev2, 0)], tree.branch, tree2.branch,
                                                'remote')
 
     def test_merged(self):
@@ -181,8 +181,8 @@
         tree.merge_from_branch(tree2.branch)
         rev4 = tree.commit('four')
 
-        self.assertUnmerged([('2', rev4)], [], tree.branch, tree2.branch)
-        self.assertUnmerged([('2', rev4)], [], tree.branch, tree2.branch,
+        self.assertUnmerged([('2', rev4, 0)], [], tree.branch, tree2.branch)
+        self.assertUnmerged([('2', rev4, 0)], [], tree.branch, tree2.branch,
             local_revid_range=(rev4, rev4))
         self.assertUnmerged([], [], tree.branch, tree2.branch,
             local_revid_range=(rev1, rev1))
@@ -239,8 +239,8 @@
         lrevid2 = local.commit('two')
         lrevid3 = local.commit('three')
         lrevid4 = local.commit('four')
-        local_extra = [('2', lrevid2), ('3', lrevid3), ('4', lrevid4)]
-        remote_extra = [('2', rrevid2), ('3', rrevid3), ('4', rrevid4)]
+        local_extra = [('2', lrevid2, 0), ('3', lrevid3, 0), ('4', lrevid4, 0)]
+        remote_extra = [('2', rrevid2, 0), ('3', rrevid3, 0), ('4', rrevid4, 0)]
 
         # control
         self.assertUnmerged(local_extra, remote_extra,
@@ -250,32 +250,32 @@
             remote_revid_range=(None, None))
 
         # exclude local revisions
-        self.assertUnmerged([('2', lrevid2)], remote_extra,
+        self.assertUnmerged([('2', lrevid2, 0)], remote_extra,
             local.branch, remote.branch, local_revid_range=(lrevid2, lrevid2))
-        self.assertUnmerged([('2', lrevid2), ('3', lrevid3)], remote_extra,
+        self.assertUnmerged([('2', lrevid2, 0), ('3', lrevid3, 0)], remote_extra,
             local.branch, remote.branch, local_revid_range=(lrevid2, lrevid3))
-        self.assertUnmerged([('2', lrevid2), ('3', lrevid3)], None,
+        self.assertUnmerged([('2', lrevid2, 0), ('3', lrevid3, 0)], None,
             local.branch, remote.branch, 'local',
             local_revid_range=(lrevid2, lrevid3))
 
         # exclude remote revisions
-        self.assertUnmerged(local_extra, [('2', rrevid2)],
+        self.assertUnmerged(local_extra, [('2', rrevid2, 0)],
             local.branch, remote.branch, remote_revid_range=(None, rrevid2))
-        self.assertUnmerged(local_extra, [('2', rrevid2)],
+        self.assertUnmerged(local_extra, [('2', rrevid2, 0)],
             local.branch, remote.branch, remote_revid_range=(lrevid1, rrevid2))
-        self.assertUnmerged(local_extra, [('2', rrevid2)],
+        self.assertUnmerged(local_extra, [('2', rrevid2, 0)],
             local.branch, remote.branch, remote_revid_range=(rrevid2, rrevid2))
-        self.assertUnmerged(local_extra, [('2', rrevid2), ('3', rrevid3)],
+        self.assertUnmerged(local_extra, [('2', rrevid2, 0), ('3', rrevid3, 0)],
             local.branch, remote.branch, remote_revid_range=(None, rrevid3))
-        self.assertUnmerged(local_extra, [('2', rrevid2), ('3', rrevid3)],
+        self.assertUnmerged(local_extra, [('2', rrevid2, 0), ('3', rrevid3, 0)],
             local.branch, remote.branch, remote_revid_range=(rrevid2, rrevid3))
-        self.assertUnmerged(local_extra, [('3', rrevid3)],
+        self.assertUnmerged(local_extra, [('3', rrevid3, 0)],
             local.branch, remote.branch, remote_revid_range=(rrevid3, rrevid3))
-        self.assertUnmerged(None, [('2', rrevid2), ('3', rrevid3)],
+        self.assertUnmerged(None, [('2', rrevid2, 0), ('3', rrevid3, 0)],
             local.branch, remote.branch, 'remote',
             remote_revid_range=(rrevid2, rrevid3))
 
         # exclude local and remote revisions
-        self.assertUnmerged([('3', lrevid3)], [('3', rrevid3)],
+        self.assertUnmerged([('3', lrevid3, 0)], [('3', rrevid3, 0)],
             local.branch, remote.branch, local_revid_range=(lrevid3, lrevid3),
             remote_revid_range=(rrevid3, rrevid3))

=== modified file 'breezy/tests/test_options.py'
--- breezy/tests/test_options.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/test_options.py	2017-07-14 21:15:49 +0000
@@ -17,15 +17,13 @@
 import re
 
 from .. import (
+    bzr,
     commands,
     controldir,
     errors,
     option,
     registry,
     )
-from ..bzr import (
-    bzrdir,
-    )
 from ..builtins import cmd_commit
 from ..commands import parse_args
 from . import TestCase
@@ -120,9 +118,9 @@
 
     def test_registry_conversion(self):
         registry = controldir.ControlDirFormatRegistry()
-        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
-        bzrdir.register_metadir(registry, 'two', 'RepositoryFormatKnit1', 'two help')
-        bzrdir.register_metadir(registry, 'hidden', 'RepositoryFormatKnit1',
+        bzr.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzr.register_metadir(registry, 'two', 'RepositoryFormatKnit1', 'two help')
+        bzr.register_metadir(registry, 'hidden', 'RepositoryFormatKnit1',
             'two help', hidden=True)
         registry.set_default('one')
         options = [option.RegistryOption('format', '', registry, str)]
@@ -130,7 +128,7 @@
         self.assertEqual({'format':'one'}, opts)
         opts, args = self.parse(options, ['--format', 'two'])
         self.assertEqual({'format':'two'}, opts)
-        self.assertRaises(errors.BadOptionValue, self.parse, options,
+        self.assertRaises(option.BadOptionValue, self.parse, options,
                           ['--format', 'three'])
         self.assertRaises(errors.BzrCommandError, self.parse, options,
                           ['--two'])
@@ -174,7 +172,7 @@
         opts, args = self.parse(options, ['--format', 'knit'])
         self.assertEqual({'format': 'knit'}, opts)
         self.assertRaises(
-            errors.BadOptionValue, self.parse, options, ['--format', 'BAD'])
+            option.BadOptionValue, self.parse, options, ['--format', 'BAD'])
 
     def test_from_kwargs(self):
         my_option = option.RegistryOption.from_kwargs('my-option',
@@ -190,12 +188,12 @@
 
     def test_help(self):
         registry = controldir.ControlDirFormatRegistry()
-        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
-        bzrdir.register_metadir(registry, 'two',
+        bzr.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzr.register_metadir(registry, 'two',
             'breezy.bzr.knitrepo.RepositoryFormatKnit1',
             'two help',
             )
-        bzrdir.register_metadir(registry, 'hidden', 'RepositoryFormat7', 'hidden help',
+        bzr.register_metadir(registry, 'hidden', 'RepositoryFormat7', 'hidden help',
             hidden=True)
         registry.set_default('one')
         options = [option.RegistryOption('format', 'format help', registry,
@@ -218,8 +216,8 @@
         self.assertEqual(list(opt.iter_switches()),
                          [('hello', None, 'GAR', 'fg')])
         registry = controldir.ControlDirFormatRegistry()
-        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
-        bzrdir.register_metadir(registry, 'two',
+        bzr.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzr.register_metadir(registry, 'two',
                 'breezy.bzr.knitrepo.RepositoryFormatKnit1',
                 'two help',
                 )
@@ -379,9 +377,9 @@
 
     def test_is_hidden(self):
         registry = controldir.ControlDirFormatRegistry()
-        bzrdir.register_metadir(registry, 'hidden', 'HiddenFormat',
+        bzr.register_metadir(registry, 'hidden', 'HiddenFormat',
             'hidden help text', hidden=True)
-        bzrdir.register_metadir(registry, 'visible', 'VisibleFormat',
+        bzr.register_metadir(registry, 'visible', 'VisibleFormat',
             'visible help text', hidden=False)
         format = option.RegistryOption('format', '', registry, str)
         self.assertTrue(format.is_hidden('hidden'))

=== modified file 'breezy/tests/test_osutils.py'
--- breezy/tests/test_osutils.py	2017-06-05 23:54:45 +0000
+++ breezy/tests/test_osutils.py	2017-06-15 00:39:26 +0000
@@ -2008,7 +2008,7 @@
         self.assertContainsRe(
             log.getvalue(),
             r"brz: warning: some compiled extensions could not be loaded; "
-            "see <https://answers\.launchpad\.net/bzr/\+faq/703>\n"
+            "see ``brz help missing-extensions``\n"
             )
 
 

=== modified file 'breezy/tests/test_remote.py'
--- breezy/tests/test_remote.py	2017-06-11 20:15:04 +0000
+++ breezy/tests/test_remote.py	2017-06-15 00:58:22 +0000
@@ -49,6 +49,8 @@
 from ..bzr.bzrdir import (
     BzrDir,
     BzrDirFormat,
+    )
+from ..bzr import (
     RemoteBzrProber,
     )
 from ..bzr.chk_serializer import chk_bencode_serializer
@@ -127,7 +129,7 @@
     def test_find_correct_format(self):
         """Should open a RemoteBzrDir over a RemoteTransport"""
         fmt = BzrDirFormat.find_format(self.transport)
-        self.assertTrue(bzrdir.RemoteBzrProber
+        self.assertTrue(RemoteBzrProber
                         in controldir.ControlDirFormat._server_probers)
         self.assertIsInstance(fmt, RemoteBzrDirFormat)
 

=== modified file 'breezy/tests/test_repository.py'
--- breezy/tests/test_repository.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/test_repository.py	2017-07-14 20:49:41 +0000
@@ -173,7 +173,7 @@
         found_format = bzrrepository.RepositoryFormatMetaDir.find_format(tree.controldir)
         self.assertIsInstance(found_format, bzrrepository.RepositoryFormatMetaDir)
         self.assertEqual(found_format.features.get("name"), "necessity")
-        self.assertRaises(errors.MissingFeature, found_format.check_support_status,
+        self.assertRaises(bzrdir.MissingFeature, found_format.check_support_status,
             True)
         self.addCleanup(bzrrepository.RepositoryFormatMetaDir.unregister_feature,
             "name")
@@ -1721,5 +1721,5 @@
         repo = self.make_repository('.')
         repo.lock_write()
         repo._format.features["makes-cheese-sandwich"] = "required"
-        self.assertRaises(errors.MissingFeature,
+        self.assertRaises(bzrdir.MissingFeature,
             repo._format.check_support_status, False)

=== modified file 'breezy/tests/test_revision.py'
--- breezy/tests/test_revision.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/test_revision.py	2017-07-10 20:12:25 +0000
@@ -21,10 +21,6 @@
     bugtracker,
     revision,
     )
-from breezy.errors import (
-    InvalidBugStatus,
-    InvalidLineInBugsProperty,
-    )
 from breezy.revision import NULL_REVISION
 from breezy.tests import TestCase, TestCaseWithTransport
 from breezy.tests.matchers import MatchesAncestry
@@ -244,14 +240,16 @@
     def test_no_status(self):
         r = revision.Revision(
             '1', properties={'bugs': 'http://example.com/bugs/1'})
-        self.assertRaises(InvalidLineInBugsProperty, list, r.iter_bugs())
+        self.assertRaises(bugtracker.InvalidLineInBugsProperty, list,
+                r.iter_bugs())
 
     def test_too_much_information(self):
         r = revision.Revision(
             '1', properties={'bugs': 'http://example.com/bugs/1 fixed bar'})
-        self.assertRaises(InvalidLineInBugsProperty, list, r.iter_bugs())
+        self.assertRaises(bugtracker.InvalidLineInBugsProperty, list,
+                r.iter_bugs())
 
     def test_invalid_status(self):
         r = revision.Revision(
             '1', properties={'bugs': 'http://example.com/bugs/1 faxed'})
-        self.assertRaises(InvalidBugStatus, list, r.iter_bugs())
+        self.assertRaises(bugtracker.InvalidBugStatus, list, r.iter_bugs())

=== modified file 'breezy/tests/test_revisiontree.py'
--- breezy/tests/test_revisiontree.py	2017-05-25 01:35:55 +0000
+++ breezy/tests/test_revisiontree.py	2017-07-14 20:34:39 +0000
@@ -21,6 +21,7 @@
     errors,
     revision,
     )
+from breezy.tree import FileTimestampUnavailable
 from breezy.tests import TestCaseWithTransport
 
 
@@ -76,5 +77,5 @@
     def test_get_file_mtime_ghost(self):
         file_id = next(iter(self.rev_tree.all_file_ids()))
         self.rev_tree.root_inventory[file_id].revision = 'ghostrev'
-        self.assertRaises(errors.FileTimestampUnavailable, 
+        self.assertRaises(FileTimestampUnavailable, 
             self.rev_tree.get_file_mtime, file_id)

=== modified file 'breezy/tests/test_selftest.py'
--- breezy/tests/test_selftest.py	2017-06-11 20:15:04 +0000
+++ breezy/tests/test_selftest.py	2017-06-14 23:29:06 +0000
@@ -64,6 +64,7 @@
     )
 from ..sixish import (
     StringIO,
+    text_type,
     )
 from ..symbol_versioning import (
     deprecated_function,
@@ -1902,7 +1903,7 @@
 
     def test_assert_isinstance(self):
         self.assertIsInstance(2, int)
-        self.assertIsInstance(u'', basestring)
+        self.assertIsInstance(u'', (str, text_type))
         e = self.assertRaises(AssertionError, self.assertIsInstance, None, int)
         self.assertEqual(str(e),
             "None is an instance of <type 'NoneType'> rather than <type 'int'>")

=== modified file 'breezy/tests/test_smart.py'
--- breezy/tests/test_smart.py	2017-06-11 20:15:04 +0000
+++ breezy/tests/test_smart.py	2017-07-15 13:23:08 +0000
@@ -167,7 +167,7 @@
         request = smart_req.SmartServerRequest(transport, 'foo/')
         self.assertEqual('./', request.translate_client_path('foo/'))
         self.assertRaises(
-            errors.InvalidURLJoin, request.translate_client_path, 'foo/..')
+            urlutils.InvalidURLJoin, request.translate_client_path, 'foo/..')
         self.assertRaises(
             errors.PathNotChild, request.translate_client_path, '/')
         self.assertRaises(

=== modified file 'breezy/tests/test_transform.py'
--- breezy/tests/test_transform.py	2017-06-11 02:27:01 +0000
+++ breezy/tests/test_transform.py	2017-06-24 16:32:30 +0000
@@ -3230,7 +3230,7 @@
         preview.delete_contents(file_trans_id)
         preview.create_file('a\nb\n', file_trans_id)
         preview_tree = preview.get_preview_tree()
-        merger = Merger.from_revision_ids(None, preview_tree,
+        merger = Merger.from_revision_ids(preview_tree,
                                           child_tree.branch.last_revision(),
                                           other_branch=child_tree.branch,
                                           tree_branch=work_tree.branch)
@@ -3249,7 +3249,7 @@
         tree2 = self.make_branch_and_tree('tree2')
         tree2.set_root_id('TREE_ROOT')
         merger = Merger.from_uncommitted(tree2, tt.get_preview_tree(),
-                                         None, tree.basis_tree())
+                                         tree.basis_tree())
         merger.merge_type = Merge3Merger
         merger.do_merge()
 
@@ -3265,9 +3265,8 @@
         tt.create_file('baz', trans_id)
         tree2 = tree.controldir.sprout('tree2').open_workingtree()
         self.build_tree_contents([('tree2/foo', 'qux')])
-        pb = None
         merger = Merger.from_uncommitted(tree2, tt.get_preview_tree(),
-                                         pb, tree.basis_tree())
+                                         tree.basis_tree())
         merger.merge_type = Merge3Merger
         merger.do_merge()
 

=== modified file 'breezy/tests/test_transport.py'
--- breezy/tests/test_transport.py	2017-06-04 18:09:30 +0000
+++ breezy/tests/test_transport.py	2017-07-15 13:23:08 +0000
@@ -411,7 +411,7 @@
         self.start_server(server)
         t = transport.get_transport_from_url(server.get_url())
         self.assertRaises(
-            errors.InvalidURLJoin, urlutils.join, t.base, '..')
+            urlutils.InvalidURLJoin, urlutils.join, t.base, '..')
 
 
 class TestChrootServer(tests.TestCase):
@@ -718,7 +718,7 @@
 class TestTransportFromUrl(tests.TestCaseInTempDir):
 
     def test_with_path(self):
-        self.assertRaises(errors.InvalidURL, transport.get_transport_from_url,
+        self.assertRaises(urlutils.InvalidURL, transport.get_transport_from_url,
             self.test_dir)
 
     def test_with_url(self):
@@ -862,7 +862,7 @@
         self.assertEqual(t.base, 'http://ro%62ey@ex%41mple.com:2222/path/')
 
     def test_parse_invalid_url(self):
-        self.assertRaises(errors.InvalidURL,
+        self.assertRaises(urlutils.InvalidURL,
                           transport.ConnectedTransport,
                           'sftp://lily.org:~janneke/public/bzr/gub')
 
@@ -1129,7 +1129,7 @@
         self.assertEqual("http://bar", location_to_url("bar:"))
 
     def test_unicode_url(self):
-        self.assertRaises(errors.InvalidURL, location_to_url,
+        self.assertRaises(urlutils.InvalidURL, location_to_url,
             "http://fo/\xc3\xaf".decode("utf-8"))
 
     def test_unicode_path(self):

=== modified file 'breezy/tests/test_tree.py'
--- breezy/tests/test_tree.py	2017-05-25 01:35:55 +0000
+++ breezy/tests/test_tree.py	2017-07-15 15:22:22 +0000
@@ -21,8 +21,22 @@
     revision,
     tree as _mod_tree,
     )
-from breezy.tests import TestCaseWithTransport
-from breezy.tree import InterTree
+from breezy.tests import (
+    TestCase,
+    TestCaseWithTransport,
+    )
+from breezy.tree import (
+    FileTimestampUnavailable,
+    InterTree,
+    )
+
+
+class ErrorTests(TestCase):
+
+    def test_file_timestamp_unavailable(self):
+        e = FileTimestampUnavailable("/path/foo")
+        self.assertEqual("The filestamp for /path/foo is not available.",
+                         str(e))
 
 
 class TestInterTree(TestCaseWithTransport):

=== modified file 'breezy/tests/test_ui.py'
--- breezy/tests/test_ui.py	2017-06-11 14:07:05 +0000
+++ breezy/tests/test_ui.py	2017-06-14 22:19:52 +0000
@@ -257,11 +257,13 @@
         ui.show_user_warning('cross_format_fetch', from_format=RepositoryFormat2a(),
             to_format=remote_fmt)
         self.assertEqual('', ui.stdout.getvalue())
-        self.assertEqual("Doing on-the-fly conversion from RepositoryFormat2a() to "
-            "RemoteRepositoryFormat(_network_name='Bazaar RepositoryFormatKnitPack5 "
-            "(bzr 1.6)\\n').\nThis may take some time. Upgrade the repositories to "
-            "the same format for better performance.\n",
-            ui.stderr.getvalue())
+        self.assertContainsRe(
+            ui.stderr.getvalue(),
+            "^Doing on-the-fly conversion from RepositoryFormat2a\(\) to "
+                "RemoteRepositoryFormat\(_network_name="
+                "b?'Bazaar RepositoryFormatKnitPack5 \(bzr 1.6\)\\\\n'\)\.\n"
+            "This may take some time. Upgrade the repositories to "
+                "the same format for better performance\.\n$")
         # and now with it suppressed please
         ui = ui_testing.TextUIFactory()
         ui.suppressed_warnings.add('cross_format_fetch')

=== modified file 'breezy/tests/test_url_policy_open.py'
--- breezy/tests/test_url_policy_open.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/test_url_policy_open.py	2017-06-14 22:34:05 +0000
@@ -23,7 +23,7 @@
 from ..bzr.branch import (
     BranchReferenceFormat,
     )
-from ..bzr.bzrdir import (
+from ..bzr import (
     BzrProber,
     )
 from ..controldir import (

=== modified file 'breezy/tests/test_urlutils.py'
--- breezy/tests/test_urlutils.py	2017-06-05 23:54:45 +0000
+++ breezy/tests/test_urlutils.py	2017-07-15 13:23:08 +0000
@@ -21,9 +21,6 @@
 
 from .. import osutils, urlutils, win32utils
 from ..errors import (
-    InvalidURL,
-    InvalidURLJoin,
-    InvalidRebaseURLs,
     PathNotChild,
     )
 from . import features, TestCaseInTempDir, TestCase, TestSkipped
@@ -36,7 +33,8 @@
         # Test breezy.urlutils.split()
         basename = urlutils.basename
         if sys.platform == 'win32':
-            self.assertRaises(InvalidURL, basename, 'file:///path/to/foo')
+            self.assertRaises(urlutils.InvalidURL, basename,
+                    'file:///path/to/foo')
             self.assertEqual('foo', basename('file:///C|/foo'))
             self.assertEqual('foo', basename('file:///C:/foo'))
             self.assertEqual('', basename('file:///C:/'))
@@ -126,8 +124,9 @@
 
         # Normalize verifies URLs when they are not unicode
         # (indicating they did not come from the user)
-        self.assertRaises(InvalidURL, normalize_url, 'http://host/\xb5')
-        self.assertRaises(InvalidURL, normalize_url, 'http://host/ ')
+        self.assertRaises(urlutils.InvalidURL, normalize_url,
+                'http://host/\xb5')
+        self.assertRaises(urlutils.InvalidURL, normalize_url, 'http://host/ ')
 
     def test_url_scheme_re(self):
         # Test paths that may be URLs
@@ -170,7 +169,8 @@
         # Test breezy.urlutils.dirname()
         dirname = urlutils.dirname
         if sys.platform == 'win32':
-            self.assertRaises(InvalidURL, dirname, 'file:///path/to/foo')
+            self.assertRaises(urlutils.InvalidURL, dirname,
+                'file:///path/to/foo')
             self.assertEqual('file:///C|/', dirname('file:///C|/foo'))
             self.assertEqual('file:///C|/', dirname('file:///C|/'))
         else:
@@ -253,12 +253,12 @@
         # Invalid joinings
         # Cannot go above root
         # Implicitly at root:
-        self.assertRaises(InvalidURLJoin, urlutils.join,
+        self.assertRaises(urlutils.InvalidURLJoin, urlutils.join,
                 'http://foo', '../baz')
-        self.assertRaises(InvalidURLJoin, urlutils.join,
+        self.assertRaises(urlutils.InvalidURLJoin, urlutils.join,
                 'http://foo', '/..')
         # Joining from a path explicitly under the root.
-        self.assertRaises(InvalidURLJoin, urlutils.join,
+        self.assertRaises(urlutils.InvalidURLJoin, urlutils.join,
                 'http://foo/a', '../../b')
 
     def test_joinpath(self):
@@ -290,9 +290,12 @@
 
         # Invalid joinings
         # Cannot go above root
-        self.assertRaises(InvalidURLJoin, urlutils.joinpath, '/', '../baz')
-        self.assertRaises(InvalidURLJoin, urlutils.joinpath, '/', '..')
-        self.assertRaises(InvalidURLJoin, urlutils.joinpath, '/', '/..')
+        self.assertRaises(urlutils.InvalidURLJoin, urlutils.joinpath, '/',
+                '../baz')
+        self.assertRaises(urlutils.InvalidURLJoin, urlutils.joinpath, '/',
+                '..')
+        self.assertRaises(urlutils.InvalidURLJoin, urlutils.joinpath, '/',
+                '/..')
 
     def test_join_segment_parameters_raw(self):
         join_segment_parameters_raw = urlutils.join_segment_parameters_raw
@@ -300,7 +303,7 @@
             join_segment_parameters_raw("/somedir/path"))
         self.assertEqual("/somedir/path,rawdata", 
             join_segment_parameters_raw("/somedir/path", "rawdata"))
-        self.assertRaises(InvalidURLJoin,
+        self.assertRaises(urlutils.InvalidURLJoin,
             join_segment_parameters_raw, "/somedir/path",
                 "rawdata1,rawdata2,rawdata3")
         self.assertEqual("/somedir/path,bla,bar",
@@ -317,10 +320,10 @@
             join_segment_parameters("/somedir/path", {}))
         self.assertEqual("/somedir/path,key1=val1", 
             join_segment_parameters("/somedir/path", {"key1": "val1"}))
-        self.assertRaises(InvalidURLJoin,
+        self.assertRaises(urlutils.InvalidURLJoin,
             join_segment_parameters, "/somedir/path",
             {"branch": "brr,brr,brr"})
-        self.assertRaises(InvalidURLJoin,
+        self.assertRaises(urlutils.InvalidURLJoin,
             join_segment_parameters, "/somedir/path", {"key1=val1": "val2"})
         self.assertEqual("/somedir/path,key1=val1,key2=val2",
             join_segment_parameters("/somedir/path", {
@@ -380,9 +383,9 @@
         self.assertEqual(u'/path/to/r\xe4ksm\xf6rg\xe5s',
             from_url('file://localhost/path/to/r%c3%a4ksm%c3%b6rg%c3%a5s'))
 
-        self.assertRaises(InvalidURL, from_url, '/path/to/foo')
+        self.assertRaises(urlutils.InvalidURL, from_url, '/path/to/foo')
         self.assertRaises(
-            InvalidURL, from_url,
+            urlutils.InvalidURL, from_url,
             'file://remotehost/path/to/r%c3%a4ksm%c3%b6rg%c3%a5s')
 
     def test_win32_local_path_to_url(self):
@@ -438,11 +441,11 @@
         self.assertEqual('C:/path/to/foo',
             from_url('file:///C|/path/to/foo,branch=foo'))
 
-        self.assertRaises(InvalidURL, from_url, 'file:///C:')
-        self.assertRaises(InvalidURL, from_url, 'file:///c')
-        self.assertRaises(InvalidURL, from_url, '/path/to/foo')
+        self.assertRaises(urlutils.InvalidURL, from_url, 'file:///C:')
+        self.assertRaises(urlutils.InvalidURL, from_url, 'file:///c')
+        self.assertRaises(urlutils.InvalidURL, from_url, '/path/to/foo')
         # Not a valid _win32 url, no drive letter
-        self.assertRaises(InvalidURL, from_url, 'file:///path/to/foo')
+        self.assertRaises(urlutils.InvalidURL, from_url, 'file:///path/to/foo')
 
     def test_win32_unc_path_from_url(self):
         from_url = urlutils._win32_local_path_from_url
@@ -452,29 +455,29 @@
         # despite IE allows 2, 4, 5 and 6 slashes in URL to another machine
         # we want to use only 2 slashes
         # Firefox understand only 5 slashes in URL, but it's ugly
-        self.assertRaises(InvalidURL, from_url, 'file:////HOST/path')
-        self.assertRaises(InvalidURL, from_url, 'file://///HOST/path')
-        self.assertRaises(InvalidURL, from_url, 'file://////HOST/path')
+        self.assertRaises(urlutils.InvalidURL, from_url, 'file:////HOST/path')
+        self.assertRaises(urlutils.InvalidURL, from_url, 'file://///HOST/path')
+        self.assertRaises(urlutils.InvalidURL, from_url, 'file://////HOST/path')
         # check for file://C:/ instead of file:///C:/
-        self.assertRaises(InvalidURL, from_url, 'file://C:/path')
+        self.assertRaises(urlutils.InvalidURL, from_url, 'file://C:/path')
 
     def test_win32_extract_drive_letter(self):
         extract = urlutils._win32_extract_drive_letter
         self.assertEqual(('file:///C:', '/foo'), extract('file://', '/C:/foo'))
         self.assertEqual(('file:///d|', '/path'), extract('file://', '/d|/path'))
-        self.assertRaises(InvalidURL, extract, 'file://', '/path')
+        self.assertRaises(urlutils.InvalidURL, extract, 'file://', '/path')
         # Root drives without slash treated as invalid, see bug #841322
         self.assertEqual(('file:///C:', '/'), extract('file://', '/C:/'))
-        self.assertRaises(InvalidURL, extract, 'file://', '/C:')
+        self.assertRaises(urlutils.InvalidURL, extract, 'file://', '/C:')
         # Invalid without drive separator or following forward slash
-        self.assertRaises(InvalidURL, extract, 'file://', '/C')
-        self.assertRaises(InvalidURL, extract, 'file://', '/C:ool')
+        self.assertRaises(urlutils.InvalidURL, extract, 'file://', '/C')
+        self.assertRaises(urlutils.InvalidURL, extract, 'file://', '/C:ool')
 
     def test_split(self):
         # Test breezy.urlutils.split()
         split = urlutils.split
         if sys.platform == 'win32':
-            self.assertRaises(InvalidURL, split, 'file:///path/to/foo')
+            self.assertRaises(urlutils.InvalidURL, split, 'file:///path/to/foo')
             self.assertEqual(('file:///C|/', 'foo'), split('file:///C|/foo'))
             self.assertEqual(('file:///C:/', ''), split('file:///C:/'))
         else:
@@ -656,9 +659,9 @@
         self.assertEqual('%', urlutils.unescape('%25'))
         self.assertEqual(u'\xe5', urlutils.unescape('%C3%A5'))
 
-        self.assertRaises(InvalidURL, urlutils.unescape, u'\xe5')
-        self.assertRaises(InvalidURL, urlutils.unescape, '\xe5')
-        self.assertRaises(InvalidURL, urlutils.unescape, '%E5')
+        self.assertRaises(urlutils.InvalidURL, urlutils.unescape, u'\xe5')
+        self.assertRaises(urlutils.InvalidURL, urlutils.unescape, '\xe5')
+        self.assertRaises(urlutils.InvalidURL, urlutils.unescape, '%E5')
 
     def test_escape_unescape(self):
         self.assertEqual(u'\xe5', urlutils.unescape(urlutils.escape(u'\xe5')))
@@ -782,19 +785,19 @@
         self.assertEqual('/foo', result)
 
     def test_different_ports(self):
-        e = self.assertRaises(InvalidRebaseURLs, urlutils.rebase_url,
+        e = self.assertRaises(urlutils.InvalidRebaseURLs, urlutils.rebase_url,
                               'foo', 'http://bar:80', 'http://bar:81')
         self.assertEqual(str(e), "URLs differ by more than path:"
                          " 'http://bar:80' and 'http://bar:81'")
 
     def test_different_hosts(self):
-        e = self.assertRaises(InvalidRebaseURLs, urlutils.rebase_url,
+        e = self.assertRaises(urlutils.InvalidRebaseURLs, urlutils.rebase_url,
                               'foo', 'http://bar', 'http://baz')
         self.assertEqual(str(e), "URLs differ by more than path: 'http://bar'"
                          " and 'http://baz'")
 
     def test_different_protocol(self):
-        e = self.assertRaises(InvalidRebaseURLs, urlutils.rebase_url,
+        e = self.assertRaises(urlutils.InvalidRebaseURLs, urlutils.rebase_url,
                               'foo', 'http://bar', 'ftp://bar')
         self.assertEqual(str(e), "URLs differ by more than path: 'http://bar'"
                          " and 'ftp://bar'")

=== modified file 'breezy/tests/test_weave.py'
--- breezy/tests/test_weave.py	2017-06-08 23:30:31 +0000
+++ breezy/tests/test_weave.py	2017-07-10 22:36:09 +0000
@@ -31,7 +31,7 @@
     BytesIO,
     )
 from . import TestCase, TestCaseInTempDir
-from ..bzr.weave import Weave, WeaveFormatError
+from ..bzr.weave import Weave, WeaveFormatError, WeaveInvalidChecksum
 from ..bzr.weavefile import write_weave, read_weave
 
 
@@ -698,9 +698,9 @@
         w = read_weave(tmpf)
 
         self.assertEqual('hello\n', w.get_text('v1'))
-        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
-        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
-        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+        self.assertRaises(WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(WeaveInvalidChecksum, w.check)
 
         # Change the sha checksum
         tmpf = BytesIO(b'# bzr weave file v5\n'
@@ -711,9 +711,9 @@
         w = read_weave(tmpf)
 
         self.assertEqual('hello\n', w.get_text('v1'))
-        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
-        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
-        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+        self.assertRaises(WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(WeaveInvalidChecksum, w.check)
 
 
 class TestWeave(TestCase):
@@ -766,7 +766,6 @@
     def test_empty_file(self):
         f = open('empty.weave', 'wb+')
         try:
-            self.assertRaises(errors.WeaveFormatError,
-                              read_weave, f)
+            self.assertRaises(WeaveFormatError, read_weave, f)
         finally:
             f.close()

=== modified file 'breezy/tests/test_workingtree.py'
--- breezy/tests/test_workingtree.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/test_workingtree.py	2017-07-14 20:49:41 +0000
@@ -228,7 +228,7 @@
             tree.controldir)
         self.assertIsInstance(found_format, workingtree.WorkingTreeFormat)
         self.assertEqual(found_format.features.get("name"), "necessity")
-        self.assertRaises(errors.MissingFeature, found_format.check_support_status,
+        self.assertRaises(bzrdir.MissingFeature, found_format.check_support_status,
             True)
         self.addCleanup(bzrworkingtree.WorkingTreeFormatMetaDir.unregister_feature,
             "name")

=== modified file 'breezy/tests/ui_testing.py'
--- breezy/tests/ui_testing.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/ui_testing.py	2017-06-14 22:19:52 +0000
@@ -21,6 +21,7 @@
 from .. import (
     ui,
 )
+from ..ui import text as ui_text
 from ..sixish import (
     text_type,
 )
@@ -48,7 +49,7 @@
         return True
 
 
-class TextUIFactory(ui.text.TextUIFactory):
+class TextUIFactory(ui_text.TextUIFactory):
 
     def __init__(self, stdin=None, stdout=None, stderr=None):
         if isinstance(stdin, bytes):

=== modified file 'breezy/transport/__init__.py'
--- breezy/transport/__init__.py	2017-06-11 20:15:04 +0000
+++ breezy/transport/__init__.py	2017-07-15 13:23:08 +0000
@@ -1537,7 +1537,7 @@
         """
         try:
             parsed_url = self._split_url(other_base)
-        except errors.InvalidURL:
+        except urlutils.InvalidURL:
             # No hope in trying to reuse an existing transport for an invalid
             # URL
             return None
@@ -1591,7 +1591,7 @@
         location = location.encode('ascii')
     except UnicodeError:
         if urlutils.is_url(location):
-            raise errors.InvalidURL(path=location,
+            raise urlutils.InvalidURL(path=location,
                 extra='URLs must be properly escaped')
         location = urlutils.local_path_to_url(location)
     if PY3:
@@ -1647,7 +1647,7 @@
                     possible_transports.append(transport)
                 return transport
     if not urlutils.is_url(url):
-        raise errors.InvalidURL(path=url)
+        raise urlutils.InvalidURL(path=url)
     raise errors.UnsupportedProtocol(url, last_err)
 
 

=== modified file 'breezy/transport/gio_transport.py'
--- breezy/transport/gio_transport.py	2017-05-30 19:16:23 +0000
+++ breezy/transport/gio_transport.py	2017-07-15 13:23:08 +0000
@@ -129,7 +129,7 @@
         #so a hardcoded list it is then.
         gio_backends = ['dav', 'file', 'ftp', 'obex', 'sftp', 'ssh', 'smb']
         if scheme not in gio_backends:
-            raise errors.InvalidURL(base,
+            raise urlutils.InvalidURL(base,
                     extra="GIO support is only available for " + \
                     ', '.join(gio_backends))
 
@@ -145,7 +145,7 @@
     def _relpath_to_url(self, relpath):
         full_url = urlutils.join(self.url, relpath)
         if isinstance(full_url, unicode):
-            raise errors.InvalidURL(full_url)
+            raise urlutils.InvalidURL(full_url)
         return full_url
 
     def _get_GIO(self, relpath):

=== modified file 'breezy/transport/http/_urllib2_wrappers.py'
--- breezy/transport/http/_urllib2_wrappers.py	2017-06-05 20:48:31 +0000
+++ breezy/transport/http/_urllib2_wrappers.py	2017-07-15 13:23:08 +0000
@@ -586,7 +586,7 @@
         if not host:
             # Just a bit of paranoia here, this should have been
             # handled in the higher levels
-            raise errors.InvalidURL(request.get_full_url(), 'no host given.')
+            raise urlutils.InvalidURL(request.get_full_url(), 'no host given.')
 
         # We create a connection (but it will not connect until the first
         # request is made)
@@ -597,7 +597,7 @@
                 ca_certs=self.ca_certs)
         except httplib.InvalidURL as exception:
             # There is only one occurrence of InvalidURL in httplib
-            raise errors.InvalidURL(request.get_full_url(),
+            raise urlutils.InvalidURL(request.get_full_url(),
                                     extra='nonnumeric port')
 
         return connection
@@ -1127,7 +1127,7 @@
 
         parsed_url = transport.ConnectedTransport._split_url(proxy)
         if not parsed_url.host:
-            raise errors.InvalidURL(proxy, 'No host component')
+            raise urlutils.InvalidURL(proxy, 'No host component')
 
         if request.proxy_auth == {}:
             # No proxy auth parameter are available, we are handling the first

=== modified file 'breezy/tree.py'
--- breezy/tree.py	2017-06-10 23:37:36 +0000
+++ breezy/tree.py	2017-07-14 20:34:39 +0000
@@ -29,9 +29,7 @@
     conflicts as _mod_conflicts,
     debug,
     delta,
-    errors,
     filters,
-    osutils,
     revision as _mod_revision,
     rules,
     trace,
@@ -42,6 +40,10 @@
 from breezy.i18n import gettext
 """)
 
+from . import (
+    errors,
+    osutils,
+    )
 from .decorators import needs_read_lock
 from .inter import InterObject
 from .sixish import (
@@ -49,6 +51,16 @@
     )
 
 
+class FileTimestampUnavailable(errors.BzrError):
+
+    _fmt = "The filestamp for %(path)s is not available."
+
+    internal_error = True
+
+    def __init__(self, path):
+        self.path = path
+
+
 class Tree(object):
     """Abstract file tree.
 

=== modified file 'breezy/ui/text.py'
--- breezy/ui/text.py	2017-05-22 00:56:52 +0000
+++ breezy/ui/text.py	2017-06-14 22:19:52 +0000
@@ -39,6 +39,9 @@
     osutils,
     trace,
     )
+from ..sixish import (
+    text_type,
+    )
 from . import (
     NullProgressView,
     UIFactory,
@@ -304,7 +307,7 @@
         :param kwargs: Dictionary of arguments to insert into the prompt,
             to allow UIs to reformat the prompt.
         """
-        if not isinstance(prompt, unicode):
+        if not isinstance(prompt, text_type):
             raise ValueError("prompt %r not a unicode string" % prompt)
         if kwargs:
             # See <https://launchpad.net/bugs/365891>

=== modified file 'breezy/urlutils.py'
--- breezy/urlutils.py	2017-06-10 01:57:00 +0000
+++ breezy/urlutils.py	2017-07-15 13:23:08 +0000
@@ -27,14 +27,14 @@
 except ImportError:
     from urllib import parse as urlparse
 
+from . import (
+    errors,
+    osutils,
+    )
+
 from .lazy_import import lazy_import
 lazy_import(globals(), """
 from posixpath import split as _posix_split
-
-from breezy import (
-    errors,
-    osutils,
-    )
 """)
 
 from .sixish import (
@@ -43,6 +43,32 @@
     )
 
 
+class InvalidURL(errors.PathError):
+
+    _fmt = 'Invalid url supplied to transport: "%(path)s"%(extra)s'
+
+
+class InvalidURLJoin(errors.PathError):
+
+    _fmt = "Invalid URL join request: %(reason)s: %(base)r + %(join_args)r"
+
+    def __init__(self, reason, base, join_args):
+        self.reason = reason
+        self.base = base
+        self.join_args = join_args
+        errors.PathError.__init__(self, base, reason)
+
+
+class InvalidRebaseURLs(errors.PathError):
+
+    _fmt = "URLs differ by more than path: %(from_)r and %(to)r"
+
+    def __init__(self, from_, to):
+        self.from_ = from_
+        self.to = to
+        errors.PathError.__init__(self, from_, 'URLs differ by more than path.')
+
+
 def basename(url, exclude_trailing_slash=True):
     """Return the last component of a URL.
 
@@ -231,7 +257,7 @@
                 continue
             elif chunk == '..':
                 if path == ['']:
-                    raise errors.InvalidURLJoin('Cannot go above root',
+                    raise InvalidURLJoin('Cannot go above root',
                             base, args)
                 path.pop()
             else:
@@ -250,7 +276,7 @@
     if url.startswith(file_localhost_prefix):
         path = url[len(file_localhost_prefix) - 1:]
     elif not url.startswith('file:///'):
-        raise errors.InvalidURL(
+        raise InvalidURL(
             url, 'local urls must start with file:/// or file://localhost/')
     else:
         path = url[len('file://'):]
@@ -271,7 +297,7 @@
 def _win32_local_path_from_url(url):
     """Convert a url like file:///C:/path/to/foo into C:/path/to/foo"""
     if not url.startswith('file://'):
-        raise errors.InvalidURL(url, 'local urls must start with file:///, '
+        raise InvalidURL(url, 'local urls must start with file:///, '
                                      'UNC path urls must start with file://')
     url = split_segment_parameters_raw(url)[0]
     # We strip off all 3 slashes
@@ -280,7 +306,7 @@
     if not win32_url.startswith('///'):
         if (win32_url[2] == '/'
             or win32_url[3] in '|:'):
-            raise errors.InvalidURL(url, 'Win32 UNC path urls'
+            raise InvalidURL(url, 'Win32 UNC path urls'
                 ' have form file://HOST/path')
         return unescape(win32_url)
 
@@ -294,7 +320,7 @@
                                 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
         or win32_url[4] not in  '|:'
         or win32_url[5] != '/'):
-        raise errors.InvalidURL(url, 'Win32 file urls start with'
+        raise InvalidURL(url, 'Win32 file urls start with'
                 ' file:///x:/, where x is a valid drive letter')
     return win32_url[3].upper() + u':' + unescape(win32_url[5:])
 
@@ -377,7 +403,7 @@
     if not isinstance(url, text_type):
         for c in url:
             if c not in _url_safe_characters:
-                raise errors.InvalidURL(url, 'URLs can only contain specific'
+                raise InvalidURL(url, 'URLs can only contain specific'
                                             ' safe characters (not %r)' % c)
         path = _url_hex_escapes_re.sub(_unescape_safe_chars, path)
         return str(prefix + ''.join(path))
@@ -452,7 +478,7 @@
     # Strip off the drive letter
     # path is currently /C:/foo
     if len(path) < 4 or path[2] not in ':|' or path[3] != '/':
-        raise errors.InvalidURL(url_base + path,
+        raise InvalidURL(url_base + path,
             'win32 file:/// paths need a drive letter')
     url_base += path[0:3] # file:// + /C:
     path = path[3:] # /foo
@@ -544,7 +570,7 @@
         if not isinstance(subsegment, str):
             raise TypeError("Subsegment %r is not a bytestring" % subsegment)
         if "," in subsegment:
-            raise errors.InvalidURLJoin(", exists in subsegments",
+            raise InvalidURLJoin(", exists in subsegments",
                                         base, subsegments)
     return ",".join((base,) + subsegments)
 
@@ -568,7 +594,7 @@
             raise TypeError("parameter value %r for %s is not a bytestring" %
                 (key, value))
         if "=" in key:
-            raise errors.InvalidURLJoin("= exists in parameter key", url,
+            raise InvalidURLJoin("= exists in parameter key", url,
                 parameters)
         new_parameters[key] = value
     return join_segment_parameters_raw(base, 
@@ -640,7 +666,7 @@
         try:
             url = url.encode("ascii")
         except UnicodeError as e:
-            raise errors.InvalidURL(url, 'URL was not a plain ASCII url: %s' % (e,))
+            raise InvalidURL(url, 'URL was not a plain ASCII url: %s' % (e,))
     if PY3:
         unquoted = urlparse.unquote_to_bytes(url)
     else:
@@ -648,7 +674,7 @@
     try:
         unicode_path = unquoted.decode('utf-8')
     except UnicodeError as e:
-        raise errors.InvalidURL(url, 'Unable to encode the URL as utf-8: %s' % (e,))
+        raise InvalidURL(url, 'Unable to encode the URL as utf-8: %s' % (e,))
     return unicode_path
 
 
@@ -778,7 +804,7 @@
     old_parsed = urlparse.urlparse(old_base)
     new_parsed = urlparse.urlparse(new_base)
     if (old_parsed[:2]) != (new_parsed[:2]):
-        raise errors.InvalidRebaseURLs(old_base, new_base)
+        raise InvalidRebaseURLs(old_base, new_base)
     return determine_relative_path(new_parsed[2],
                                    join(old_parsed[2], url))
 
@@ -846,7 +872,7 @@
         """
         # GZ 2017-06-09: Actually validate ascii-ness
         if not isinstance(url, str):
-            raise errors.InvalidURL('should be ascii:\n%r' % url)
+            raise InvalidURL('should be ascii:\n%r' % url)
         (scheme, netloc, path, params,
          query, fragment) = urlparse.urlparse(url, allow_fragments=False)
         user = password = host = port = None
@@ -863,8 +889,8 @@
             try:
                 port = int(port)
             except ValueError:
-                raise errors.InvalidURL('invalid port number %s in url:\n%s' %
-                                        (port, url))
+                raise InvalidURL('invalid port number %s in url:\n%s' %
+                                 (port, url))
         if host != "" and host[0] == '[' and host[-1] == ']': #IPv6
             host = host[1:-1]
 
@@ -905,7 +931,7 @@
         :return: urlencoded string for final path.
         """
         if not isinstance(relpath, str):
-            raise errors.InvalidURL(relpath)
+            raise InvalidURL(relpath)
         relpath = _url_hex_escapes_re.sub(_unescape_safe_chars, relpath)
         if relpath.startswith('/'):
             base_parts = []

=== modified file 'breezy/util/_bencode_py.py'
--- breezy/util/_bencode_py.py	2017-06-10 22:09:37 +0000
+++ breezy/util/_bencode_py.py	2017-06-16 20:54:19 +0000
@@ -49,25 +49,24 @@
         f += 1
         newf = x.index(b'e', f)
         n = int(x[f:newf])
-        if x[f] == b'-':
-            if x[f + 1] == b'0':
-                raise ValueError
-        elif x[f] == b'0' and newf != f+1:
+        if x[f:f+2] == b'-0':
+            raise ValueError
+        elif x[f:f+1] == b'0' and newf != f+1:
             raise ValueError
         return (n, newf+1)
 
     def decode_string(self, x, f):
         colon = x.index(b':', f)
         n = int(x[f:colon])
-        if x[f] == b'0' and colon != f+1:
+        if x[f:f+1] == b'0' and colon != f+1:
             raise ValueError
         colon += 1
         return (x[colon:colon+n], colon+n)
 
     def decode_list(self, x, f):
         r, f = [], f+1
-        while x[f] != b'e':
-            v, f = self.decode_func[x[f]](x, f)
+        while x[f:f+1] != b'e':
+            v, f = self.decode_func[x[f:f+1]](x, f)
             r.append(v)
         if self.yield_tuples:
             r = tuple(r)
@@ -76,12 +75,12 @@
     def decode_dict(self, x, f):
         r, f = {}, f+1
         lastkey = None
-        while x[f] != b'e':
+        while x[f:f+1] != b'e':
             k, f = self.decode_string(x, f)
-            if lastkey >= k:
+            if lastkey is not None and lastkey >= k:
                 raise ValueError
             lastkey = k
-            r[k], f = self.decode_func[x[f]](x, f)
+            r[k], f = self.decode_func[x[f:f+1]](x, f)
         return (r, f + 1)
 
     def bdecode(self, x):
@@ -109,6 +108,7 @@
     def __init__(self, s):
         self.bencoded = s
 
+
 def encode_bencached(x,r):
     r.append(x.bencoded)
 
@@ -121,9 +121,6 @@
 def encode_string(x, r):
     r.extend((int_to_bytes(len(x)), b':', x))
 
-def encode_unicode(x, r):
-    r.extend((int_to_bytes(len(x)), b':', x))
-
 def encode_list(x, r):
     r.append(b'l')
     for i in x:
@@ -145,7 +142,8 @@
     encode_func[long] = encode_int
     int_to_bytes = str
 else:
-    int_to_bytes = lambda n: b"%d" % n
+    def int_to_bytes(n):
+        return b'%d' % n
 encode_func[bytes] = encode_string
 encode_func[list] = encode_list
 encode_func[tuple] = encode_list

=== modified file 'breezy/version_info_formats/format_custom.py'
--- breezy/version_info_formats/format_custom.py	2017-05-21 18:10:28 +0000
+++ breezy/version_info_formats/format_custom.py	2017-06-14 23:29:06 +0000
@@ -75,7 +75,7 @@
                 data = self._data[name]
             except KeyError:
                 raise errors.MissingTemplateVariable(name)
-            if not isinstance(data, basestring):
+            if not isinstance(data, str):
                 data = str(data)
             yield data
 

=== modified file 'breezy/workingtree.py'
--- breezy/workingtree.py	2017-06-11 13:48:12 +0000
+++ breezy/workingtree.py	2017-07-02 21:26:29 +0000
@@ -50,8 +50,6 @@
     errors,
     filters as _mod_filters,
     generate_ids,
-    globbing,
-    ignores,
     merge,
     revision as _mod_revision,
     shelf,
@@ -69,6 +67,9 @@
 from .i18n import gettext
 from . import mutabletree
 from .mutabletree import needs_tree_write_lock
+from .sixish import (
+    text_type,
+    )
 from .trace import mutter, note
 
 
@@ -396,9 +397,6 @@
             return self.branch.repository.revision_tree(
                        _mod_revision.NULL_REVISION)
 
-    def _cleanup(self):
-        self._flush_ignore_list_cache()
-
     def relpath(self, path):
         """Return the local path portion from a given path.
 
@@ -952,7 +950,6 @@
                                 new_basis_tree,
                                 basis_tree,
                                 this_tree=self,
-                                pb=None,
                                 change_reporter=change_reporter,
                                 show_base=show_base)
                     basis_root_id = basis_tree.get_root_id()
@@ -1010,46 +1007,10 @@
             if pat is not None:
                 yield subp, pat
 
-    def get_ignore_list(self):
-        """Return list of ignore patterns.
-
-        Cached in the Tree object after the first call.
-        """
-        ignoreset = getattr(self, '_ignoreset', None)
-        if ignoreset is not None:
-            return ignoreset
-
-        ignore_globs = set()
-        ignore_globs.update(ignores.get_runtime_ignores())
-        ignore_globs.update(ignores.get_user_ignores())
-        if self.has_filename(breezy.IGNORE_FILENAME):
-            f = self.get_file_byname(breezy.IGNORE_FILENAME)
-            try:
-                ignore_globs.update(ignores.parse_ignore_file(f))
-            finally:
-                f.close()
-        self._ignoreset = ignore_globs
-        return ignore_globs
-
-    def _flush_ignore_list_cache(self):
-        """Resets the cached ignore list to force a cache rebuild."""
-        self._ignoreset = None
-        self._ignoreglobster = None
-
     def is_ignored(self, filename):
         r"""Check whether the filename matches an ignore pattern.
-
-        Patterns containing '/' or '\' need to match the whole path;
-        others match against only the last component.  Patterns starting
-        with '!' are ignore exceptions.  Exceptions take precedence
-        over regular patterns and cause the filename to not be ignored.
-
-        If the file is ignored, returns the pattern which caused it to
-        be ignored, otherwise None.  So this can simply be used as a
-        boolean if desired."""
-        if getattr(self, '_ignoreglobster', None) is None:
-            self._ignoreglobster = globbing.ExceptionGlobster(self.get_ignore_list())
-        return self._ignoreglobster.match(filename)
+        """
+        raise NotImplementedError(self.is_ignored)
 
     def kind(self, file_id):
         return osutils.file_kind(self.id2abspath(file_id))
@@ -1157,7 +1118,7 @@
         :force: Delete files and directories, even if they are changed and
             even if the directories are not empty.
         """
-        if isinstance(files, basestring):
+        if isinstance(files, (str, text_type)):
             files = [files]
 
         inv_delta = []

=== modified file 'doc/en/mini-tutorial/index.txt'
--- doc/en/mini-tutorial/index.txt	2017-05-21 13:35:49 +0000
+++ doc/en/mini-tutorial/index.txt	2017-06-19 19:47:45 +0000
@@ -70,7 +70,6 @@
   Location:
     shared repository: sample
   $ brz init sample/trunk
-  $ cd sample/trunk
   Created a repository tree (format: 2a)                                         
   Using shared repository: /home/john/sample/
 
@@ -84,11 +83,16 @@
 and save it.  Then we'll "add" the file, which tells brz we want it to
 track changes::
 
-  brz add test1.txt
+  $ cd sample/trunk
+  # create the file test1.txt now
+  $ brz add test1.txt
+  adding test1.txt
 
 and then commit, which saves a snapshot of all versioned files::
 
-  brz commit -m "Added first line of text"
+  $ brz commit -m "Added first line of text"
+  added test1.txt
+  Committed revision 1.
 
 
 Making changes to your files

=== modified file 'doc/en/release-notes/brz-3.0.txt'
--- doc/en/release-notes/brz-3.0.txt	2017-07-23 19:18:44 +0000
+++ doc/en/release-notes/brz-3.0.txt	2017-07-23 22:06:41 +0000
@@ -44,6 +44,25 @@
  * Support for medusa for FTP tests has been dropped, only
    pyftpdlib is now supported. (Jelmer Vernooĳ)
 
+ * The deprecated ``brz get`` and ``brz clone`` commands have been
+   removed. (Jelmer Vernooĳ)
+
+ * The setting to ignore missing C extensions has been changed from
+   ``ignore_missing_extensions=True`` to
+   ``suppress_warnings=missing_extensions``. (Jelmer Vernooĳ)
+
+ * Remove format names as options to ``brz init`` and ``brz
+   init-repository``.. Now you must use ``brz init --format=...``.
+   This simplifies ``brz init --help``.
+   (Neil Martinsen-Burrell, #330494)
+
+ * ``python-gpg`` is now used for checking GPG signatures rather than
+   ``python-gpgme``. (Jelmer Vernooĳ, #1702308)
+
+ * ``python-gpg`` is now used for signing commits, rather than shelling
+   out to the gnupg command. The ``gpg_signing_command`` option has been
+   removed.  (Jelmer Vernooĳ, #847388)
+
 New Features
 ************
 
@@ -67,7 +86,13 @@
  * The 'fetch-ghosts' command is now bundled with brz.
    Imported from bzrtools by Aaron Bentley. (Jelmer Vernooĳ)
 
- * The 'commitfromnews' plugin is now bundled.  (Jelmer Vernooĳ)
+ * The 'commitfromnews' plugin is now bundled and
+   can be enabled by setting ``commit.template_from_files = NEWS``.
+   (Jelmer Vernooĳ)
+
+ * The functionality from ``bzr-guess`` is now merged into Breezy.
+   It will provide suggestions if the user typoes a command.
+   (Jelmer Vernooĳ)
 
 Improvements
 ************
@@ -86,6 +111,17 @@
 * The ``bisect`` command now works in non-``.bzr`` directories.
   (Jelmer Vernooĳ)
 
+* When creating ``authentication.conf``, umask is now set so only the
+  current user can read the file. Breezy warns if the file is
+  accessible for other users when it starts.
+  (Joke de Buhr, Jelmer Vernooĳ, #475501)
+
+* Support ``brz commit -x`` in combination with iter_changes.
+  (Jelmer Vernooĳ, #796582, #403811, #694946, #268135, #299879)
+
+* Print a proper error when encountering ghost revisions in
+  mainline in ``bzr log``. (Jelmer Vernooĳ, #726466)
+
 Documentation
 *************
 
@@ -95,7 +131,7 @@
 API Changes
 ***********
 
- * bzrlib has been renamed to bzrlib. (Martin Packman, Jelmer Vernooĳ)
+ * bzrlib has been renamed to breezy. (Martin Packman, Jelmer Vernooĳ)
 
  * Signature of ``load_tests`` used by ``TestLoader`` changed to be
    compatible with standard libarary unittest.  (Martin Packman)
@@ -103,6 +139,9 @@
  * All previously deprecated functionality has been removed.
    (Jelmer Vernooĳ)
 
+ * ``CommitBuilder.record_entry_contents`` has been removed.
+   (Jelmer Vernooĳ, #731433, #604953)
+
  * Renamed ``breezy.delta.report_delta`` parameter ``filter=`` to
    ``predicate=``. (Martin Packman)
 
@@ -127,6 +166,12 @@
  * ``ControlDir.find_bzrdirs`` has been renamed to
    ``ControlDir.find_controldirs``. (Jelmer Vernooĳ)
 
+ * ``Repository.get_revisions`` no longer accepts ``None`` as
+   argument. (Jelmer Vernooĳ)
+
+ * A new ``Repository.iter_revisions`` method has been added.
+   (Jelmer Vernooĳ)
+
 Internals
 *********
 

=== modified file 'setup.py'
--- setup.py	2017-06-11 20:31:48 +0000
+++ setup.py	2017-06-27 00:11:29 +0000
@@ -285,6 +285,9 @@
         include_dirs=include_dirs))
 
 
+add_cython_extension('breezy._simple_set_pyx')
+ext_modules.append(Extension('breezy._static_tuple_c',
+                             ['breezy/_static_tuple_c.c']))
 add_cython_extension('breezy._annotator_pyx')
 add_cython_extension('breezy._bencode_pyx')
 add_cython_extension('breezy._chunks_to_lines_pyx')
@@ -303,9 +306,6 @@
 add_cython_extension('breezy.bzr._chk_map_pyx')
 ext_modules.append(Extension('breezy._patiencediff_c',
                              ['breezy/_patiencediff_c.c']))
-add_cython_extension('breezy._simple_set_pyx')
-ext_modules.append(Extension('breezy._static_tuple_c',
-                             ['breezy/_static_tuple_c.c']))
 add_cython_extension('breezy.bzr._btree_serializer_pyx')
 
 
@@ -573,10 +573,7 @@
         includes.append(module)
 
     additional_packages = set()
-    if sys.version.startswith('2.4'):
-        # adding elementtree package
-        additional_packages.add('elementtree')
-    elif sys.version.startswith('2.6') or sys.version.startswith('2.5'):
+    if sys.version.startswith('2.7'):
         additional_packages.add('xml.etree')
     else:
         import warnings

=== modified file 'tools/generate_docs.py'
--- tools/generate_docs.py	2017-05-21 18:10:28 +0000
+++ tools/generate_docs.py	2017-06-14 22:47:53 +0000
@@ -27,8 +27,8 @@
 
 Examples: 
 
-    python2.4 generated-docs.py man
-    python2.4 generated-docs.py bash_completion
+    python generated-docs.py man
+    python generated-docs.py bash_completion
 
 Run "%(prog)s --help" for the option reference.
 """
@@ -38,12 +38,11 @@
 
 sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
 
-import breezy
+# Don't remove the following import, it triggers a format registration that
+# avoid http://pad.lv/956860
+import breezy.bzr
 from breezy import (
     commands,
-    # Don't remove the following import, it triggers a format registration that
-    # avoid http://pad.lv/956860
-    branch,
     doc_generate,
     )
 

